package com.tofly.common.apis;

import com.tofly.common.apis.fallback.RemoteServiceImpl;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.SysTask;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.common.entity.dto.MessageDto;
import com.tofly.common.entity.dto.TaskDto;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.io.Serializable;

/**
 * @author haiqiu
 * @date 2022/8/10
 * 系统推送代办接口调用
 */
@FeignClient(value = "tofly-auth",fallback = RemoteServiceImpl.class)
public interface SysRemoteService {


    /**
     * 消息新增数据
     *
     * @param messageDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping("/sysMessage")
    public ResultData insert(@RequestBody MessageDto messageDto);

    /**
     * 待办新增数据
     *
     * @param taskDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping("/sysTask")
    public ResultData insert(@Validated @RequestBody TaskDto taskDto);

    /**
     * 待办修改数据
     *
     * @param taskDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "待办修改数据")
    @PutMapping("/sysTask")
    public ResultData update(@Validated(value = BaseDto.Update.class) @RequestBody TaskDto taskDto);


    /**
     * 待办查询所有数据
     *
     * @param sysTask 查询实体
     * @return 待办查询所有数据
     */
    @ApiOperation(value = "待办查询所有数据")
    @GetMapping("/sysTask/allList")
    public ResultData allList(SysTask sysTask);

    /**
     * 待办通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "待办通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("/sysTask/{id}")
    public ResultData selectOne(@PathVariable(value = "id") Serializable id);
}
