package com.tofly.common.entity.dto;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
* 消息通知
* @TableName SYS_MESSAGE
*/
@Data
public class MessageDto extends BaseDto implements Serializable {

    /**
     * 主键ID
     */
    @NotNull(message="[ID]不能为空",groups = Update.class)
    @ApiModelProperty("唯一编码")
    private Long id;
    /**
    * 消息主题
    */
    @Size(max= 50,message="编码长度不能超过50")
    @ApiModelProperty("消息主题")
    @Length(max= 50,message="编码长度不能超过50")
    private String topic;
    /**
    * 回复的原消息ID
    */
    @ApiModelProperty("回复的原消息ID")
    private Long replyId;
    /**
    * 消息类型：0普通消息,...待定义
    */
    @Size(max= 10,message="编码长度不能超过10")
    @ApiModelProperty("消息类型：0普通消息,...待定义")
    @Length(max= 10,message="编码长度不能超过10")
    private String type;
    /**
    * 消息链接表名
    */
    @Size(max= 50,message="编码长度不能超过50")
    @ApiModelProperty("消息链接表名")
    @Length(max= 50,message="编码长度不能超过50")
    private String tableName;
    /**
    * 链接ID
    */
    @ApiModelProperty("链接ID")
    private Long tableId;
    /**
    * 路由地址进行跳转
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("路由地址进行跳转")
    @Length(max= 200,message="编码长度不能超过200")
    private String routeIp;
    /**
    * APP跳转路由
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("APP跳转路由")
    @Length(max= 200,message="编码长度不能超过200")
    private String routeIpApp;
    /**
    * 消息发送者
    */
    @ApiModelProperty("消息发送者")
    private Long userPushId;
    /**
    * 消息接收者
    */
    @ApiModelProperty("消息接收者")
    private Long userPullId;
    /**
    * 租户ID
    */
    @ApiModelProperty("租户ID")
    private Long tenantId;
    /**
    * 消息是否接收，0：未接收，1：已接收
    */
    @ApiModelProperty("消息是否接收，0：未接收，1：已接收")
    private Integer flag;
    /**
    * 消息接收时间
    */
    @ApiModelProperty("消息接收时间")
    private Date recipientTime;
    /**
    * 消息详情
    */
    @ApiModelProperty("消息详情")
    private String message;

}
