package com.tofly.common.entity.dto;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
* 系统任务待办
* @TableName SYS_TASK
*/
@Data
public class TaskDto extends BaseDto implements Serializable {

    /**
     * 主键ID
     */
    @NotNull(message="[ID]不能为空",groups = Update.class)
    @ApiModelProperty("主键ID")
    private Long id;
    /**
    * 名称描述
    */
    @Size(max= 50,message="编码长度不能超过50")
    @ApiModelProperty("名称描述")
    @Length(max= 50,message="编码长度不能超过50")
    private String describe;
    /**
    * 待办类型大类，根据发起的任务进行分类，如：1巡检任务、2抢维修任务
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("待办类型大类，根据发起的任务进行分类，如：1巡检任务、2抢维修任务")
    @Length(max= 255,message="编码长度不能超过255")
    private String maxType;
    /**
    * 待办任务子类，如：1_1管线巡查
    */
    @ApiModelProperty("待办任务子类，如：1_1管线巡查")
    private Integer minType;
    /**
    * 级别：0一般、1紧急
    */
    @ApiModelProperty("级别：0一般、1紧急")
    private Integer rank;
    /**
    * 任务链接表名
    */
    @Size(max= 50,message="编码长度不能超过50")
    @ApiModelProperty("任务链接表名")
    @Length(max= 50,message="编码长度不能超过50")
    private String tableName;
    /**
    * 链接ID
    */
    @ApiModelProperty("链接ID")
    private Long tableId;
    /**
    * 路由地址进行跳转
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("路由地址进行跳转")
    @Length(max= 200,message="编码长度不能超过200")
    private String routeIp;
    /**
    * 处理状态：0待完成，1已完成
    */
    @ApiModelProperty("处理状态：0待完成，1已完成")
    private Integer status;
    /**
    * 处理时间
    */
    @ApiModelProperty("处理时间")
    private Date handleTime;
    /**
    * 处理人ID
    */
    @ApiModelProperty("处理人ID")
    private Long handleUser;
    /**
    * 任务开始时间
    */
    @ApiModelProperty("任务开始时间")
    private Date startTime;
    /**
    * 任务结束时间
    */
    @ApiModelProperty("任务结束时间")
    private Date endTime;
    /**
    * APP路由地址进行跳转
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("APP路由地址进行跳转")
    @Length(max= 200,message="编码长度不能超过200")
    private String routeAppIp;
    /**
    * 排序
    */
    @ApiModelProperty("排序")
    private Long sort;
    /**
    * 租户ID
    */
    @ApiModelProperty("租户ID")
    private Long tenantId;

}
