package com.tofly.common.utils;

import cn.hutool.core.collection.CollUtil;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class FileNameCheckUtils {
    /**
     * 校验节点文件
     *
     * @param files     文件集合
     * @param fileNames 节点信息
     * @return 校验结果
     */
    public static String checkNodeFile(List<MultipartFile> files, List<String> fileNames) {
        if (CollUtil.isEmpty(fileNames)) {
            return null;
        }
        if (CollUtil.isNotEmpty(fileNames) && CollUtil.isEmpty(files)) {
            return "上传文件不存在";
        }

        Map<String, MultipartFile> map = files.stream().collect(Collectors.toMap(MultipartFile::getOriginalFilename, v -> v));

        for (String filename : fileNames) {
            if (!map.containsKey(filename)) {
                return "上传文件不存在";
            }
        }
        return null;
    }


    /**
     * 保存的文件集合
     *
     * @param files     文件集合
     * @param fileNames 节点信息
     * @return 保存的文件集合
     */
    public static List<MultipartFile> getNodeFile(List<MultipartFile> files, List<String> fileNames) {
        if (CollUtil.isEmpty(fileNames)) {
            return null;
        }
        if (CollUtil.isNotEmpty(fileNames) && CollUtil.isEmpty(files)) {
            throw new RuntimeException("上传文件不存在");
        }

        Map<String, MultipartFile> map = files.stream().collect(Collectors.toMap(MultipartFile::getOriginalFilename, v -> v));
        List<MultipartFile> list = new ArrayList<>();
        for (String filename : fileNames) {
            if (map.containsKey(filename)) {
                list.add(map.get(filename));
            }
        }
        return list;
    }

    /**
     * 校验节点文件
     *
     * @param map       文件集合
     * @param fileNames 节点信息
     * @return 校验结果
     */
    public static String checkNodeFile(Map<String, MultipartFile> map, List<String> fileNames) {
        if (CollUtil.isEmpty(fileNames)) {
            return null;
        }
        if (CollUtil.isNotEmpty(fileNames) && CollUtil.isEmpty(map)) {
            return "上传文件不存在";
        }

        for (String filename : fileNames) {
            if (!map.containsKey(filename)) {
                return "上传文件不存在";
            }
        }

        return null;
    }


    /**
     * 转换文件集合为Map
     *
     * @param files 文件集合
     * @return Map
     */
    public static Map<String, MultipartFile> getNameFiles(List<MultipartFile> files) {
        if (CollUtil.isEmpty(files)) {
            return null;
        }
        return files.stream().collect(Collectors.toMap(MultipartFile::getOriginalFilename, v -> v));
    }


}
