package com.tofly.auth.config;

import com.tofly.common.core.constant.SecurityConstants;
import com.tofly.common.oauth.auth.ToflyUser;
import com.tofly.common.oauth.compent.ToflyWebResponseExceptionTranslator;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.oauth2.config.annotation.configurers.ClientDetailsServiceConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configuration.AuthorizationServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableAuthorizationServer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerEndpointsConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.ClientDetails;
import org.springframework.security.oauth2.provider.client.BaseClientDetails;
import org.springframework.security.oauth2.provider.client.InMemoryClientDetailsService;
import org.springframework.security.oauth2.provider.token.AuthorizationServerTokenServices;
import org.springframework.security.oauth2.provider.token.DefaultTokenServices;
import org.springframework.security.oauth2.provider.token.TokenEnhancer;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.redis.RedisTokenStore;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Administrator
 * OAuth 授权服务器配置
 * @date 2019/9/9  13:47
 */
@Configuration
@EnableAuthorizationServer
@AllArgsConstructor
@Slf4j
public class AuthorizationServerConfiguration extends AuthorizationServerConfigurerAdapter {

    //private DataSource dataSource;

    private UserDetailsService userDetailsService;
    private RedisConnectionFactory redisConnectionFactory;
    private AuthenticationManager authenticationManager;


    //配置客户端详情信息
    @Override
    public void configure(ClientDetailsServiceConfigurer clients) throws Exception {
//        ToFlyClientDetailsService toFlyClientDetailsService = new ToFlyClientDetailsService(dataSource);
//        toFlyClientDetailsService.setSelectClientDetailsSql(SecurityConstants.DEFAULT_SELECT_STATEMENT);
//        toFlyClientDetailsService.setFindClientDetailsSql(SecurityConstants.DEFAULT_FIND_STATEMENT);
//        clients.withClientDetails(toFlyClientDetailsService);
//        //暂时写死，设置内存型客户端
        ToflyMemoryClientDetailsService clientDetailsService = new ToflyMemoryClientDetailsService();
        ClientDetails clientDetails = new BaseClientDetails();
        ((BaseClientDetails) clientDetails).setClientId("tofly");
        ((BaseClientDetails) clientDetails).setClientSecret("tofly");
        ((BaseClientDetails) clientDetails).setScope(new ArrayList<String>() {{
            add("server");
        }});
        ((BaseClientDetails) clientDetails).setAuthorizedGrantTypes(new ArrayList<String>() {{
            add("password");
            add("authorization_code");
            add("refresh_token");
            add("client_credentials");
        }});
        ((BaseClientDetails) clientDetails).setAutoApproveScopes(new ArrayList<String>() {{
            add("true");
        }});
        Map<String, ClientDetails> map = new HashMap();
        map.put("tofly", clientDetails);
        ((InMemoryClientDetailsService) clientDetailsService).setClientDetailsStore(map);
        clients.withClientDetails(clientDetailsService);

    }

    //配置令牌端点(Token Endpoint)的安全与权限访问
    @Override
    public void configure(AuthorizationServerSecurityConfigurer security) throws Exception {
        security.allowFormAuthenticationForClients()
/*
                .tokenKeyAccess("permitAll()")
*/
                .checkTokenAccess("isAuthenticated()");
    }


    //定义授权和令牌端点和令牌服务
    @Override
    public void configure(AuthorizationServerEndpointsConfigurer endpoints) throws Exception {
        //endpoints.tokenServices(tokenServices());
        endpoints.tokenServices(tokenServices());
        endpoints.allowedTokenEndpointRequestMethods(HttpMethod.GET, HttpMethod.POST)
                .tokenStore(tokenStore())
                .tokenEnhancer(tokenEnhancer())
                .authenticationManager(authenticationManager)
                .userDetailsService(userDetailsService)
                //.reuseRefreshTokens(false)
                .reuseRefreshTokens(true)
                .exceptionTranslator(new ToflyWebResponseExceptionTranslator());

    }


    @Bean
    public TokenStore tokenStore() {
        RedisTokenStore tokenStore = new RedisTokenStore(redisConnectionFactory);
        tokenStore.setPrefix(SecurityConstants.PROJECT_PREFIX + SecurityConstants.OAUTH_PREFIX);
        return tokenStore;
    }


    @Bean
    public TokenEnhancer tokenEnhancer() {
        return (accessToken, authentication) -> {
            final Map<String, Object> additionalInfo = new HashMap<>();
            ToflyUser user = (ToflyUser) authentication.getUserAuthentication().getPrincipal();
            additionalInfo.put(SecurityConstants.DETAILS_LICENSE, SecurityConstants.PROJECT_LICENSE);
            additionalInfo.put(SecurityConstants.username, user.getUsername());
            additionalInfo.put(SecurityConstants.userId, user.getId());

            additionalInfo.put(SecurityConstants.jobNumber, user.getJobNumber());
            additionalInfo.put(SecurityConstants.phone, user.getPhone());
            additionalInfo.put(SecurityConstants.lockFlag, user.getLockFlag().toString());
            additionalInfo.put(SecurityConstants.active, user.getActive().toString());
            additionalInfo.put(SecurityConstants.loginFail, user.getLockFlag().toString());

            additionalInfo.put(SecurityConstants.userLevel, user.getUserLevel() == null ? null : user.getUserLevel().toString());
            additionalInfo.put(SecurityConstants.realName, user.getRealName());
            additionalInfo.put(SecurityConstants.avatar, user.getAvatar());

            additionalInfo.put(SecurityConstants.aliasName, user.getAliasName());
            additionalInfo.put(SecurityConstants.email, user.getEmail());
            additionalInfo.put(SecurityConstants.tenantId, user.getTenantId().toString());

            additionalInfo.put(SecurityConstants.deptIds, user.getDeptIds());
            additionalInfo.put(SecurityConstants.roleIds, user.getRoleIds());
            additionalInfo.put(SecurityConstants.postIds, user.getPostIds());
            additionalInfo.put(SecurityConstants.rolePermissions, user.getRolePermissions());

            additionalInfo.put(SecurityConstants.roles, user.getRoles());
            additionalInfo.put(SecurityConstants.depts, user.getDepts());
            additionalInfo.put(SecurityConstants.posts, user.getPosts());
            additionalInfo.put(SecurityConstants.tenant, user.getTenant());

            ((DefaultOAuth2AccessToken) accessToken).setAdditionalInformation(additionalInfo);
            return accessToken;
        };
    }

    @Primary
    @Bean
    public AuthorizationServerTokenServices tokenServices() {
        DefaultTokenServices defaultTokenServices = new DefaultTokenServices();
        defaultTokenServices.setAccessTokenValiditySeconds(-1);
        defaultTokenServices.setRefreshTokenValiditySeconds(500);
        defaultTokenServices.setSupportRefreshToken(true);
        //defaultTokenServices.setReuseRefreshToken(false);
        defaultTokenServices.setReuseRefreshToken(true);
        defaultTokenServices.setTokenStore(tokenStore());
        defaultTokenServices.setTokenEnhancer(tokenEnhancer());

        defaultTokenServices.setAuthenticationManager(authenticationManager);
        return defaultTokenServices;
    }
}
