package com.tofly.auth.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.constant.CommonConstants;
import com.tofly.common.core.constant.SecurityConstants;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.system.entity.SysUser;
import com.tofly.system.service.SysUserService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.endpoint.TokenEndpoint;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.annotation.*;

import java.security.Principal;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author Administrator
 * @date 2019/10/22  13:10
 */
@RestController
@RequestMapping("/oauth")
@Api(tags="登陆接口")
@Slf4j
@AllArgsConstructor
public class OauthController extends BaseController {

    private final CacheManager cacheManager;

    private final SysUserService sysUserService;
    @Autowired
    private TokenEndpoint tokenEndpoint;
    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private ValueOperations<String,String> valueOperations;

    @GetMapping("/token")
//    @ApiOperation(value = "使用get方式登陆")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "username", value = "用户名", required = true, dataType = "String"),
//            @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String")
//    })
    public ResultData getAccessToken(Principal principal, @ApiParam(hidden = true) @RequestParam Map<String, String> parameters) throws HttpRequestMethodNotSupportedException {
        return decide(custom(tokenEndpoint.getAccessToken(principal, parameters).getBody()),ResultEnum.LOGIN);
    }

    @PostMapping("/token")
    @ApiOperation(value = "使用post方式登陆")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名", required = true, dataType = "String"),
            @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String")
    })
    @ToFlyAppLog(title = "登录",type = CommonConstants.LOG_LOGIN)
    public ResultData postAccessToken(@ApiParam(hidden = true)Principal principal,@ApiParam(hidden = true) @RequestParam Map<String, String> parameters) throws HttpRequestMethodNotSupportedException {
        return decide(custom(tokenEndpoint.postAccessToken(principal, parameters).getBody()),ResultEnum.LOGIN);
    }

    @PostMapping("/login")
    @ApiOperation(value = "使用post方式登陆")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名", required = true, dataType = "String"),
            @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String")
    })
    @ToFlyAppLog(title = "登录",type = CommonConstants.LOG_LOGIN)
    public ResultData login() throws HttpRequestMethodNotSupportedException {
        return null;
    }

    @ApiOperation(value = "解锁用户")
    @PostMapping("/unlock")
    public ResultData unlock(@ApiParam(value = "用户名集合") @RequestParam List<String> usernames){
        Cache cache = cacheManager.getCache(SysConstants.USER_DATA_INFO);
        usernames.forEach(username->{
            cache.put(username,null);
        });
        sysUserService.update(Wrappers.<SysUser>lambdaUpdate()
                .set(SysUser::getLockFlag, SysConstants.UNLOCK)
                .in(SysUser::getUsername));
        return decide("解锁成功");
    }


    @ApiOperation(value = "监测是否为第一次登录")
    @ApiImplicitParam(name = "id",value = "用户ID")
    @GetMapping("/check/{id}")
    public ResultData selectOne(@PathVariable Long id) {
        SysUser sysUser = sysUserService.getById(id);
        sysUser.setPassword(null);
        return decide(sysUser);
    }

    //自定义返回格式
    private Map<String, Object> custom(OAuth2AccessToken accessToken) {
        DefaultOAuth2AccessToken token = (DefaultOAuth2AccessToken) accessToken;
        Map<String, Object> data = new LinkedHashMap(token.getAdditionalInformation());
        String tokenValue=token.getValue();
        data.put("accessToken", token.getValue());
        data.put("expiresIn",token.getExpiresIn());
        if (token.getRefreshToken() != null) {
            data.put("refreshToken", token.getRefreshToken().getValue());
        }

        String o = valueOperations.get(SecurityConstants.REDIS_LOGIN_LOCK);
        log.info("当前用户为：{},获取project为:{},token为：{}",data.get("username"),o,tokenValue);
        if(!StringUtil.isEmpty(o)){
            valueOperations.getOperations().delete(SecurityConstants.REDIS_LOGIN_LOCK);
            valueOperations.set(tokenValue,o);
        }
        // token设置超时时间3600
        String s = SysConstants.USER_TOKEN_INFO+tokenValue;
        redisTemplate.expire(s, 43200, TimeUnit.SECONDS);
        // 设置一个回调Key,失效删除
        redisTemplate.opsForValue().set(s+"_2",data.get("username"));
        Long expire = redisTemplate.opsForValue().getOperations().getExpire(s);
        log.info("当前用户为token设置失效时长为：{}",expire);

        return data;
    }
//
//    private Boolean unlock(String key) {
//        String script = "if redis.call('get', KEYS[1]) == ARGV[1] then return redis.call('del', KEYS[1]) else return 0 end";
//        RedisScript<String> redisScript = new DefaultRedisScript<>(script, String.class);
//        Object result = redisTemplate.execute(redisScript, Collections.singletonList(key));
//        if(SUCCESS.equals(result)) {
//            return true;
//        }
//        return false;
//    }
}
