package com.tofly.auth.oauth;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.auth.controller.OauthController;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.constant.CommonConstants;
import com.tofly.common.core.constant.SecurityConstants;
import com.tofly.common.oauth.auth.*;
import com.tofly.entity.pojo.User;
import com.tofly.entity.pojo.UserInfo;
import com.tofly.system.entity.*;
import com.tofly.system.entity.vo.user.UserVo;
import com.tofly.system.service.*;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Administrator
 * @date 2019/9/6  14:48
 * 自定义用户认证
 */
@Service
@Slf4j
@AllArgsConstructor
public class ToFlyUserDetailsService implements UserDetailsService {

    private final CacheManager cacheManager;
    private final SysUserService sysUserService;
    private final RedisTemplate redisTemplate;

    private final SysMenuService sysMenuService;
    private final SysTenantService sysTenantService;
    private final SysUserDeptService userDeptService;
    private final SysUserRoleService userRoleService;
    private final SysUserPostService userPostService;
    private final SysRoleService sysRoleService;
    private final SysDeptService sysDeptService;
    private final SysPostService sysPostService;
    private final SysRolePermissionService rolePermissionService;

    @Override
    @SneakyThrows
    public UserDetails loadUserByUsername(String username) {
        //首先从缓存中获取，当调用时，我们可以将用户信息存储到缓存，或者程序启动时加载到缓存
        Cache cache = cacheManager.getCache(SysConstants.USER_DATA_INFO);

        if (cache != null && cache.get(username) != null && cache.get(username).get() != null) {
            return (ToflyUser) cache.get(username).get();
        }

        SysUser sysUser = sysUserService.getOne(Wrappers.<SysUser>lambdaQuery()
                .eq(StringUtils.hasText(username), SysUser::getUsername, username));

        UserDetails userDetails = getUserDetails(sysUser, username);
        //放入缓存
        cache.put(username, userDetails);
        return userDetails;
    }

    /**
     * 将用户设置为对应的oauth形式
     *
     * @param sysUser
     * @return
     */
    private UserDetails getUserDetails(SysUser sysUser, String username) {
        if (sysUser == null || sysUser.getUsername() == null) {
            throw new UsernameNotFoundException(username + "用户不存在");
        }
        if (sysUser.getActive().equals(Integer.valueOf(CommonConstants.USER_STUTAS_FAIL))) {
            throw new UserCheckAuthenticationException("用户已处于禁用状态，不能登录");
        }

        if (Integer.valueOf(CommonConstants.USER_LOCK_LOCKED).equals(sysUser.getLockFlag())) {
            throw new UserCheckAuthenticationException("用户已处于锁定状态，不能登录");
        }

        List<Role> roleList = new ArrayList<>();
        List<Dept> deptList = new ArrayList<>();
        List<Post> postList = new ArrayList<>();

        //TODO 查询用户角色和权限菜单等操作
        Tenant tenant = new Tenant();
        if (sysUser.getTenantId() != null) {
            SysTenant sysTenant = sysTenantService.getById(sysUser.getTenantId());
            if (!ObjectUtils.isEmpty(sysTenant)){
                BeanUtils.copyProperties(sysTenant, tenant);
            }
        }
        List<SysMenu> menus = sysMenuService.getUserNameMenu(null, username);
        List<SysDept> depts = sysUserService.getDeptByUserId(sysUser.getId());
        List<Long> deptIds;
        String dept = null;
        if (!CollectionUtils.isEmpty(depts)) {
            deptList = depts.stream().map(data -> {
                Dept vo = new Dept();
                BeanUtils.copyProperties(data, vo);
                return vo;
            }).collect(Collectors.toList());
            deptIds = depts.stream().map(SysDept::getId).collect(Collectors.toList());
            dept = deptIds.toString().replace("[", "").replace("]", "").replaceAll(" ","");
        }

        List<SysRole> roles = sysUserService.getRoleByUserId(sysUser.getId());
        List<Long> roleIds;
        String role = null;
        //用户角色包含的部门公司人员权限
        List<RolePermission> rolePermissions = new ArrayList<>();
        if (!CollectionUtils.isEmpty(roles)) {
            roleList = roles.stream().map(data -> {
                Role vo = new Role();
                BeanUtils.copyProperties(data, vo);
                return vo;
            }).collect(Collectors.toList());
            roleIds = roles.stream().map(SysRole::getId).collect(Collectors.toList());
            role = roleIds.toString().replace("[", "").replace("]", "").replaceAll(" ","");
            //获取用户角色包含的部门公司人员权限
            List<SysRolePermission> sysRolePermissions = rolePermissionService.list(Wrappers.<SysRolePermission>lambdaQuery()
                    .in(SysRolePermission::getRoleId, roleIds));
            if (!CollectionUtils.isEmpty(sysRolePermissions)) {
                rolePermissions = sysRolePermissions.stream().map(sysRolePermission -> {
                    RolePermission rolePermission = new RolePermission();
                    BeanUtils.copyProperties(sysRolePermission, rolePermission);
                    return rolePermission;
                }).collect(Collectors.toList());
            }

        }

        List<SysPost> posts = sysUserService.getPostByUserId(sysUser.getId());
        List<Long> postIds;
        String post = null;
        if (!CollectionUtils.isEmpty(posts)) {
            postList = posts.stream().map(data -> {
                Post vo = new Post();
                BeanUtils.copyProperties(data, vo);
                return vo;
            }).collect(Collectors.toList());
            postIds = posts.stream().map(SysPost::getId).collect(Collectors.toList());
            post = postIds.toString().replace("[", "").replace("]", "").replaceAll(" ","");
        }

        UserInfo userInfo = new UserInfo();
        User user1 = new User();
        BeanUtils.copyProperties(sysUser, user1);
        userInfo.setSysUser(user1);

        List<String> permissions = new ArrayList<>();
        if (!CollectionUtils.isEmpty(menus)) {
            //添加菜单+添加权限
            permissions = menus.stream().filter(sysMenu -> StringUtils.hasText(sysMenu.getPath()))
                    .map(SysMenu::getPath).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(permissions)) {
                //如果没有角色，也需要加一个默认角色用户名保证后续不会出错
                permissions.add(userInfo.getSysUser().getUsername());
            }
        } else {
            //如果没有角色，也需要加一个默认角色用户名保证后续不会出错
            permissions.add(userInfo.getSysUser().getUsername());
        }


        Collection<? extends GrantedAuthority> authorities
                = AuthorityUtils.createAuthorityList(permissions.toArray(new String[0]));


        return new ToflyUser(userInfo.getSysUser().getUsername(), userInfo.getSysUser().getPassword(), userInfo.getSysUser().getActive() != 0
                , true, true, userInfo.getSysUser().getLockFlag() != 0,
                authorities, userInfo.getSysUser().getId(), sysUser.getPhone(),
                userInfo.getSysUser().getAvatar(), userInfo.getSysUser().getLockFlag(), userInfo.getSysUser().getActive(),
                userInfo.getSysUser().getLoginFail(), userInfo.getSysUser().getRealName(),
                userInfo.getSysUser().getJobNumber(), userInfo.getSysUser().getUserLevel(), userInfo.getSysUser().getTenantId(),
                role, dept, post, userInfo.getSysUser().getEmail(), sysUser.getAliasName(), rolePermissions,
                roleList, postList, deptList, tenant);
    }


    class UserCheckAuthenticationException extends AuthenticationException {

        public UserCheckAuthenticationException(String msg, Throwable t) {
            super(msg, t);
        }

        public UserCheckAuthenticationException(String msg) {
            super(msg);
        }
    }
}
