package com.tofly.auth.oauth;

import cn.hutool.extra.servlet.ServletUtil;
import com.tofly.auth.controller.OauthController;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.constant.CommonConstants;
import com.tofly.common.core.constant.SecurityConstants;
import com.tofly.common.core.util.IpUtil;
import com.tofly.common.core.util.SpringContextHolder;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.auth.ToflyUser;
import com.tofly.entity.pojo.User;
import com.tofly.system.entity.SysUser;
import com.tofly.system.service.SysUserService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.redis.RedisTokenStore;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Component
@AllArgsConstructor
@Slf4j
public class ToflyAuthenticationProvider implements AuthenticationProvider {
    private final UserDetailsService userDetailService;


    private final CacheManager cacheManager;

    private final SysUserService sysUserService;

    //private final AuthTokenService authTokenService;

    private final TokenStore tokenStore;

    /**
     * 进行身份认证
     *
     * @param authentication
     * @return
     * @throws AuthenticationException
     */
    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        log.info("order 身份认证 来了。。。。。。。");
        // 获取用户输入的用户名和密码
        String username = authentication.getName();
        if (StringUtil.isEmpty(username)) {
            throw new MyAuthenticationException("用户名不能为空");
        }
        String password = (String) authentication.getCredentials();
        if (StringUtil.isEmpty(password)) {
            throw new MyAuthenticationException("密码不能为空");
        }
        // 获取封装用户信息的对象
        UserDetails userDetails = userDetailService.loadUserByUsername(username);
        // 进行密码的比对

        HttpServletRequest request = ((ServletRequestAttributes) Objects
                .requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
        log.info("ip:{}", IpUtil.getIpAddr(request));

//        if(1==1){
//           throw new MyAuthenticationException("aaaaaaaaaa");
//        }


        log.info("界面传过来的password:{}", password);
        log.info("数据库查询出来的password:{}", userDetails.getPassword());

        BCryptPasswordEncoder bCryptPasswordEncoder = new BCryptPasswordEncoder();
        log.info("用{}加密后的密码为：{}", SecurityConstants.BCRYPT + password, bCryptPasswordEncoder.encode(SecurityConstants.BCRYPT + password));
        boolean flag = bCryptPasswordEncoder.matches(SecurityConstants.BCRYPT + password, userDetails.getPassword());

        //return new UsernamePasswordAuthenticationToken(userDetails,password,userDetails.getAuthorities());

        if (StringUtil.isEmpty(password) && !flag) {
            flag = true;
        }

        ToflyUser toflyUser = null;
        Cache cache = cacheManager.getCache(SysConstants.USER_DATA_INFO);
        if (cache != null && cache.get(username) != null && cache.get(username).get() != null) {
            toflyUser = (ToflyUser) cache.get(username).get();
        }

        User sysUser = new User();
        sysUser.setId(toflyUser.getId());
        Integer pwdt = toflyUser.getLoginFail();
        if (pwdt == null) {
            pwdt = 0;
        }

        //用户级别  用于超级管理员与其他用户区分等  。0 超级管理员，1普通管理员,2其他公司超级管理员,3其他公司管理员
        if (pwdt >= 5) {
            if (1L == toflyUser.getUserLevel() || 3L == toflyUser.getUserLevel()) {
                //设置用户状态为锁定状态
                if (!toflyUser.getLockFlag().equals(SysConstants.LOCK)){
                    SysUser user = new SysUser();
                    user.setId(toflyUser.getId());
                    user.setLockFlag(SysConstants.LOCK);
                    sysUserService.updateById(user);
                }
            } else {
                //未登陆成功，清楚缓存，避免下次已经解锁登录时还是锁定状态
                cache.put(username, null);
            }
            throw new MyAuthenticationException("账户已经被锁定，请联系管理人员");
        }

        // 校验通过
        if (flag) {
            sysUser.setLoginFail(0);
            sysUser.setLoginIp(IpUtil.getIpAddr(request));
            // TODO 暂时关闭登录日志事件
            SpringContextHolder.publishEvent(new UserLoginEvent(sysUser));
            //删除token，实现只能登录一次
            RedisTokenStore redisTokenStore=(RedisTokenStore)tokenStore;
            Collection<OAuth2AccessToken> client_id = redisTokenStore.findTokensByClientIdAndUserName((String)((Map) authentication.getDetails()).get("client_id"), authentication.getName());
            client_id.forEach(tokenStore::removeAccessToken);
            //如果密码错误次数不为0，则将缓存中的次数修改为0
            if (pwdt != 0) {
                ToflyUser toflyUser1 = new ToflyUser(username, toflyUser.getPassword(), true
                        , true, true, true,
                        toflyUser.getAuthorities(), toflyUser.getId(), toflyUser.getPhone(),
                        toflyUser.getAvatar(), toflyUser.getLockFlag(), toflyUser.getActive(), 0, toflyUser.getRealName(),
                        toflyUser.getJobNumber(), toflyUser.getUserLevel(), toflyUser.getTenantId(),
                        toflyUser.getRoleIds(), toflyUser.getDeptIds(), toflyUser.getPostIds(),toflyUser.getEmail(),
                        toflyUser.getAliasName(),toflyUser.getRolePermissions(),toflyUser.getRoles(),
                        toflyUser.getPosts(),toflyUser.getDepts(),toflyUser.getTenant());
                cache.put(username, toflyUser1);
            }
            //登录成功更新下登录时间和登录IP
            SysUser userDb = sysUserService.getById(toflyUser.getId());
            SysUser user = new SysUser();
            user.setId(toflyUser.getId());
            user.setLoginTime(new Date());
            user.setLoginIp(IpUtil.getIpAddr(request));
            user.setUserLevel(userDb.getUserLevel());
            sysUserService.updateById(user);

            //将权限信息也封装进去
            return new UsernamePasswordAuthenticationToken(userDetails, password, userDetails.getAuthorities());
        }


        pwdt++;
        sysUser.setLoginFail(pwdt);
        if (pwdt >= 5) {
            sysUser.setLockFlag(Integer.valueOf(CommonConstants.USER_LOCK_LOCKED));
        }
        toflyUser.setLoginFail(pwdt);
        cache.put(username, toflyUser);


        //cache.put(username,toflyUser1);
        SpringContextHolder.publishEvent(new UserLoginEvent(sysUser));
        //cache.put(username,null);
        String message = pwdt < 5 ? "密码错误。当前密码错误次数是：" + pwdt + ",密码错误5次账户将会被锁定" :
                "账户已经被锁定，请联系管理人员";
        throw new MyAuthenticationException(message);
        //验证失败返回 null
        //return null;

    }

    /**
     * 这个方法 确保返回 true 即可，
     *
     * @param aClass
     * @return
     */
    @Override
    public boolean supports(Class<?> aClass) {
        return true;
    }

    class MyAuthenticationException extends AuthenticationException {

        public MyAuthenticationException(String msg, Throwable t) {
            super(msg, t);
        }

        public MyAuthenticationException(String msg) {
            super(msg);
        }
    }
}