package com.tofly.system.controller;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysApp;
import com.tofly.system.entity.dto.AppDto;
import com.tofly.system.service.SysAppService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统App管理(SysApp)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-01 17:40:26
 */
@Api(tags = "系统App管理")
@RestController
@RequestMapping("sysApp")
public class SysAppController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysAppService sysAppService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysApp 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "releaseTimeStart",value = "发布开始时间",dataType = "string"),
            @ApiImplicitParam(name = "releaseTimeEnd",value = "发布结束时间",dataType = "string")
    })
    @GetMapping
    public ResultData pageList(Page<SysApp> page, SysApp sysApp,String releaseTimeStart,String releaseTimeEnd) {
        return decide(sysAppService.page(page, new QueryWrapper<>(sysApp).lambda()
                .between(StringUtils.hasText(releaseTimeStart) && StringUtils.hasText(releaseTimeEnd),
                        SysApp::getReleaseTime,
                        DateUtil.parse(releaseTimeStart),DateUtil.parse(releaseTimeEnd))));
    }


    /**
     * 查询所有数据
     *
     * @param sysApp 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData list(SysApp sysApp) {
        return decide(sysAppService.list(new QueryWrapper<>(sysApp)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        return decide(sysAppService.get(id));
    }


    /**
     * 获取app新版本
     * @return app
     */
    @ApiOperation(value = "获取App更新版本")
    @GetMapping("/upload/app")
    public ResultData getUpdateApp() {
        return decide(sysAppService.getUpdateApp());
    }

    /**
     * 新增数据
     *
     * @param appDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@RequestPart @Validated AppDto appDto,
                             @RequestPart(required = false) @RequestParam(name = "fileList",required = false) List<MultipartFile> fileList) {
        sysAppService.insert(appDto,fileList);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param appDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@RequestPart @Validated(value = BaseDto.Update.class) AppDto appDto,
                             @RequestPart(required = false) @RequestParam(name = "fileList",required = false) List<MultipartFile> fileList) {
        sysAppService.edit(appDto,fileList);
        return decide(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        sysAppService.delete(idList);
        return decide(ResultEnum.DELETE_SUCCESS);
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
//        sysAppService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
//        return decide(sysAppService.importData(file));
        return decide("导入成功");
    }


    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysApp> page, SysApp sysApp,Boolean isAll) {
//        sysAppService.export(page,sysApp,isAll);
    }
}

