package com.tofly.system.controller;



import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysDept;
import com.tofly.system.entity.SysDict;
import com.tofly.system.entity.dto.DeptDto;
import com.tofly.system.service.SysDeptService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统部门组织(SysDept)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-01 17:40:50
 */
@Api(tags = "系统部门")
@RestController
@RequestMapping("sysDept")
public class SysDeptController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysDeptService sysDeptService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysDept 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page<SysDept> page, SysDept sysDept) {
        return decide(sysDeptService.page(page,new QueryWrapper<>(sysDept).lambda().orderByAsc(SysDept::getSort)));
    }

    /**
     * 所有数据
     *
     * @param sysDept 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData allList(SysDept sysDept) {
        return decide(sysDeptService.list(new QueryWrapper<>(sysDept).lambda().orderByAsc(SysDept::getSort)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        return decide(sysDeptService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param deptDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@Validated @RequestBody DeptDto deptDto) {
        sysDeptService.insert(deptDto);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param deptDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@Validated(value = BaseDto.Update.class) @RequestBody DeptDto deptDto) {
        sysDeptService.edit(deptDto);
        return decide(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        sysDeptService.delete(idList);
        return decide(ResultEnum.DELETE_SUCCESS);
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
//        sysDeptService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
//        return decide(sysDeptService.importData(file));
        return decide("导入成功");
    }


    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysDept> page, SysDept sysDept,Boolean isAll) {
//        sysDeptService.export(page,sysDept,isAll);
    }
}

