package com.tofly.system.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysMenu;
import com.tofly.system.entity.SysMenuTenant;
import com.tofly.system.entity.SysPost;
import com.tofly.system.entity.dto.MenuDto;
import com.tofly.system.service.SysMenuService;
import com.tofly.system.service.SysMenuTenantService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统菜单(SysMenu)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-03 09:04:40
 */
@Api(tags = "系统菜单")
@RestController
@RequestMapping("sysMenu")
public class SysMenuController extends BaseController  {
    /**
     * 服务对象
     */
    @Resource
    private SysMenuService sysMenuService;

    @Resource
    private SysMenuTenantService menuTenantService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysMenu 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page<SysMenu> page, SysMenu sysMenu) {
        Page<SysMenu> menuPage = sysMenuService.page(page, new QueryWrapper<>(sysMenu).lambda().orderByAsc(SysMenu::getSort));
        List<SysMenu> menus = menuTenantService.getMenuTenantsByMenuId(menuPage.getRecords());
        menuPage.setRecords(menus);
        return decide(menuPage);
    }


    /**
     * 查询所有数据
     * @return 查询所有数据获取菜单列表
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData allList(SysMenu sysMenu) {
        List<SysMenu> menus = sysMenuService.list(new QueryWrapper<>(sysMenu).lambda().orderByAsc(SysMenu::getSort));
        return decide(menuTenantService.getMenuTenantsByMenuId(menus));
    }

    /**
     * 根据登录用户获取菜单列表(平级)
     *
     * @param sysMenu 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "根据登录用户获取菜单列表(平级)")
    @GetMapping("/listMenu")
    public ResultData listMenu(SysMenu sysMenu) {
        List<SysMenu> menus = sysMenuService.listMenu(sysMenu);
        return decide(menuTenantService.getMenuTenantsByMenuId(menus));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        return decide(sysMenuService.getById(id));
    }


    /**
     * 根据登录用户获取树形菜单
     * @return 根据登录用户获取菜单
     */
    @ApiOperation(value = "根据登录用户获取树形菜单")
    @GetMapping("tree")
    public ResultData treeMenu(SysMenu sysMenu){
        return decide(sysMenuService.treeMenu(sysMenu),ResultEnum.FIND_SUCCESS);
    }

    /**
     * 新增数据
     *
     * @param menuDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@Validated @RequestBody MenuDto menuDto) {
        sysMenuService.insert(menuDto);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param menuDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@Validated(value = BaseDto.Update.class) @RequestBody MenuDto menuDto) {
        sysMenuService.edit(menuDto);
        return decide(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        sysMenuService.delete(idList);
        return decide(ResultEnum.DELETE_SUCCESS);
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
//        sysMenuService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
//        return decide(sysMenuService.importData(file));
        return decide("导入成功");
    }


     /**
     * 数据导出
     * @param page 分页
     * @param sysMenu 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysMenu> page, SysMenu sysMenu,Boolean isAll) {
//        sysMenuService.export(page,sysMenu,isAll);
    }
}

