package com.tofly.system.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysDict;
import com.tofly.system.entity.SysResource;
import com.tofly.system.entity.dto.ResourceDto;
import com.tofly.system.service.SysResourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统资源(SysResource)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-01 17:41:22
 */
@Api(tags = "系统资源")
@RestController
@RequestMapping("sysResource")
public class SysResourceController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysResourceService sysResourceService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysResource 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page<SysResource> page, SysResource sysResource) {
        return decide(sysResourceService.page(page, Wrappers.query(sysResource).lambda().orderByAsc(SysResource::getSort)));
    }

    /**
     * 查询所有数据
     *
     * @param sysResource 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData list(SysResource sysResource) {
        return decide(sysResourceService.list(Wrappers.query(sysResource).lambda().orderByAsc(SysResource::getSort)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        return decide(sysResourceService.getById(id));
    }


    /**
     * 树型结构全部数据
     *
     * @param sysResource 查询参数
     * @return 树型结构全部数据
     */
    @ApiOperation(value = "树型结构全部数据")
    @GetMapping("tree")
    public ResultData tree(SysResource sysResource) {
        return decide(sysResourceService.tree(sysResource));
    }


    /**
     * 新增数据
     *
     * @param resourceDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@Validated @RequestBody ResourceDto resourceDto) {
        sysResourceService.insert(resourceDto);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param resourceDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@Validated(value = BaseDto.Update.class) @RequestBody ResourceDto resourceDto) {
        sysResourceService.edit(resourceDto);
        return decide(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        for (Long id : idList) {
            if (sysResourceService.count(Wrappers.<SysResource>lambdaQuery().eq(SysResource::getPid, id)) != 0) {
                throw new RuntimeException("所选资源下面包含子集，请先删除子集资源");
            }
        }
        return decide(sysResourceService.removeByIds(idList));
    }
}

