package com.tofly.system.controller;



import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.*;
import com.tofly.system.entity.dto.RoleDto;
import com.tofly.system.service.SysMenuTenantService;
import com.tofly.system.service.SysRoleMenuService;
import com.tofly.system.service.SysRolePermissionService;
import com.tofly.system.service.SysRoleService;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统角色(SysRole)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-02 09:57:37
 */
@Api(tags = "系统角色")
@RestController
@RequestMapping("sysRole")
public class SysRoleController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysRoleService sysRoleService;



    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysRole 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page<SysRole> page, SysRole sysRole) {
        return decide(sysRoleService.page(page, new QueryWrapper<>(sysRole).lambda().orderByAsc(SysRole::getSort)),ResultEnum.FIND_SUCCESS);
    }

    /**
     * 所有数据
     *
     * @param sysRole 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData allList(SysRole sysRole,@ApiParam(value = "角色ID数组集合") @RequestParam(required = false) List<Long> roleIds) {
        return decide(sysRoleService.list(new QueryWrapper<>(sysRole).lambda()
                .in(!CollectionUtils.isEmpty(roleIds),SysRole::getId,roleIds)
                .orderByAsc(SysRole::getSort)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        SysRole sysRole = sysRoleService.get(id);
        return decide(sysRole,ResultEnum.FIND_SUCCESS);
    }

    /**
     * 新增数据
     *
     * @param roleDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@Validated @RequestBody RoleDto roleDto) {
        sysRoleService.insert(roleDto);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param roleDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@Validated(value = BaseDto.Update.class) @RequestBody RoleDto roleDto) {
        sysRoleService.edit(roleDto);
        return decide(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        sysRoleService.delete(idList);
        return decide(ResultEnum.DELETE_SUCCESS);
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
//        sysRoleService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
//        return decide(sysRoleService.importData(file));
        return ResultData.ok();
    }


     /**
     * 数据导出
     * @param page 分页
     * @param sysRole 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysRole> page, SysRole sysRole,Boolean isAll) {
//        sysRoleService.export(page,sysRole,isAll);
    }
}

