package com.tofly.system.controller;



import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.SysTask;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.common.entity.dto.TaskDto;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.system.service.SysTaskService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统任务待办(SysTask)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-03 13:36:38
 */
@Api(tags = "系统任务待办")
@RestController
@RequestMapping("sysTask")
public class SysTaskController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysTaskService sysTaskService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysTask 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page<SysTask> page, SysTask sysTask) {
        Long userId = SecurityUtils.getUserId();
        sysTask.setHandleUser(userId);
        return decide(sysTaskService.page(page, new QueryWrapper<>(sysTask)));
    }

    /**
     * 所有数据
     *
     * @param sysTask 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData allList(SysTask sysTask) {
        return decide(sysTaskService.list(new QueryWrapper<>(sysTask)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        return decide(sysTaskService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param taskDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@Validated @RequestBody TaskDto taskDto) {
        sysTaskService.insert(taskDto);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param taskDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@Validated(value = BaseDto.Update.class) @RequestBody TaskDto taskDto) {
        sysTaskService.edit(taskDto);
        return decide(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        return decide(sysTaskService.removeByIds(idList));
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
//        sysTaskService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
//        return decide(sysTaskService.importData(file));
        return ResultData.ok("导入成功");
    }


     /**
     * 数据导出
     * @param page 分页
     * @param sysTask 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "page", value = "分页模型"),
            @ApiImplicitParam(name = "sysTask", value = "查询模型"),
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysTask> page, SysTask sysTask,Boolean isAll) {
//        sysTaskService.export(page,sysTask,isAll);
    }
}

