package com.tofly.system.controller;



import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysTenant;
import com.tofly.system.entity.dto.TenantDto;
import com.tofly.system.service.SysTenantService;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统公司表(SysTenant)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-01 17:29:04
 */
@Api(tags = "系统公司")
@RestController
@RequestMapping("sysTenant")
public class SysTenantController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysTenantService sysTenantService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysTenant 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page<SysTenant> page, SysTenant sysTenant) {
        return decide(sysTenantService.page(page, new QueryWrapper<>(sysTenant).lambda().orderByAsc(SysTenant::getSort)));
    }

    /**
     * 查询所有数据
     *
     * @param sysTenant 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData list(SysTenant sysTenant) {
        return decide(sysTenantService.allList(sysTenant));
    }

    /**
     * 获取系统组织架构树形
     * @return 获取系统组织架构树形
     */
    @ApiOperation(value = "获取系统组织架构树形")
    @GetMapping("tree")
    public ResultData treeTenant(SysTenant sysTenant,
                                 @ApiParam(value = "是否包含部门,默认不包含false") @RequestParam(defaultValue = "false",required = false) boolean isHaveDept){
        return decide(sysTenantService.treeTenant(sysTenant,isHaveDept),ResultEnum.FIND_SUCCESS);
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        return decide(sysTenantService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param tenantDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@Validated @RequestBody TenantDto tenantDto) {
        sysTenantService.insert(tenantDto);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param tenantDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@Validated(value = BaseDto.Update.class) @RequestBody TenantDto tenantDto) {
        sysTenantService.edit(tenantDto);
        return decide(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        for (Long id : idList) {
            int count = sysTenantService.count(new QueryWrapper<SysTenant>().lambda().eq(SysTenant::getParentId, id));
            if (count!=0){
                throw new RuntimeException("所删除的组织含有下级组织，请先删除下级组织");
            }
        }
        return decide(sysTenantService.removeByIds(idList));
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        sysTenantService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
        sysTenantService.importData(file);
        return ResultData.ok("导入成功");
    }

    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "page", value = "分页模型"),
            @ApiImplicitParam(name = "", value = "查询模型"),
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysTenant> page, SysTenant sysTenant,Boolean isAll) {
        sysTenantService.export(page,sysTenant,isAll);
    }
}

