package com.tofly.system.controller;



import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.system.entity.SysTenantResource;
import com.tofly.system.entity.vo.resource.TenantResourceDto;
import com.tofly.system.service.SysTenantResourceService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * 公司资源关联表(SysTenantResource)表控制层
 *
 * @author HaiQiu
 * @since 2022-09-09 17:26:47
 */
@Api(tags = "公司资源关联表")
@RestController
@RequestMapping("sysTenantResource")
public class SysTenantResourceController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysTenantResourceService sysTenantResourceService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param sysTenantResource 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据",hidden = true)
    @GetMapping("/page")
    public ResultData pageList(Page<SysTenantResource> page, SysTenantResource sysTenantResource) {
        return decide(sysTenantResourceService.pageList(page, sysTenantResource));
    }
    
     /**
     * 查询所有数据
     *
     * @param sysTenantResource 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "所有数据")
    @GetMapping("/all")
    public ResultData all(SysTenantResource sysTenantResource) {
        return decide(sysTenantResourceService.all(sysTenantResource));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据",hidden = true)
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData get(@PathVariable Serializable id) {
        return decide(sysTenantResourceService.get(id));
    }

//    /**
//     * 新增数据
//     *
//     * @param sysTenantResource 实体对象
//     * @return 新增结果
//     */
//    @ApiOperation(value = "新增数据",hidden = true)
//    @PostMapping
//    public ResultData insert(@RequestBody SysTenantResource sysTenantResource) {
//        sysTenantResourceService.insert(sysTenantResource);
//        return decide("新增成功");
//    }


    /**
     * 新增关联数据
     *
     * @param tenantResourceDto 实体对象
     * @return 新增关联数据
     */
    @ApiOperation(value = "新增关联数据")
    @PostMapping
    public ResultData insertBatch(@RequestBody TenantResourceDto tenantResourceDto) {
        checkData(tenantResourceDto);
        sysTenantResourceService.remove(Wrappers.<SysTenantResource>lambdaQuery().eq(SysTenantResource::getTenantId,tenantResourceDto.getTenantId()));
        insertData(tenantResourceDto);
        return decide("新增成功");
    }

    //新增数据
    private void insertData(TenantResourceDto tenantResourceDto) {
        List<SysTenantResource> tenantResources = new ArrayList<>();
        for (Long resourceId : tenantResourceDto.getResourceIds()) {
            SysTenantResource sysTenantResource = new SysTenantResource();
            sysTenantResource.setTenantId(tenantResourceDto.getTenantId());
            sysTenantResource.setResourceId(resourceId);
            tenantResources.add(sysTenantResource);
        }
        sysTenantResourceService.saveBatch(tenantResources);
    }

    //检测数据
    private void checkData(TenantResourceDto tenantResourceDto) {
        if (ObjectUtils.isEmpty(tenantResourceDto) ||
                tenantResourceDto.getTenantId()==null ||
                CollectionUtils.isEmpty(tenantResourceDto.getResourceIds())){
            throw new RuntimeException("参数不完整");
        }
    }

//    /**
//     * 修改关联数据
//     *
//     * @param tenantResourceDto 实体对象
//     * @return 修改关联数据
//     */
//    @ApiOperation(value = "修改关联数据")
//    @PutMapping
//    @Transactional
//    public ResultData editBatch(@RequestBody TenantResourceDto tenantResourceDto) {
//        checkData(tenantResourceDto);
//        sysTenantResourceService.remove(Wrappers.<SysTenantResource>lambdaQuery().eq(SysTenantResource::getTenantId,tenantResourceDto.getTenantId()));
//        insertData(tenantResourceDto);
//        return decide("修改成功");
//    }


//    /**
//     * 修改数据
//     *
//     * @param sysTenantResource 实体对象
//     * @return 修改结果
//     */
//    @ApiOperation(value = "修改数据",hidden = true)
//    @PutMapping
//    public ResultData edit(@RequestBody SysTenantResource sysTenantResource) {
//        sysTenantResourceService.edit(sysTenantResource);
//        return decide("修改成功");
//    }

//    /**
//     * 删除数据
//     *
//     * @param idList 主键结合
//     * @return 删除结果
//     */
//    @ApiOperation(value = "删除数据",hidden = true)
//    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
//    @DeleteMapping
//    public ResultData delete(@RequestParam("idList") List<Long> idList) {
//        sysTenantResourceService.delete(idList);
//        return decide("删除成功");
//    }

    /**
     * 删除关联数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除关联数据")
    @ApiImplicitParam(name = "idList",value = "公司数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        sysTenantResourceService.remove(Wrappers.<SysTenantResource>lambdaQuery().in(SysTenantResource::getTenantId,idList));
        return decide("删除成功");
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载",hidden = true)
    @GetMapping("/template")
    public void template() {
        sysTenantResourceService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入",hidden = true)
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
        sysTenantResourceService.importData(file);
        return decide("导入成功");
    }


     /**
     * 数据导出
     * @param page 分页
     * @param sysTenantResource 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出",hidden = true)
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysTenantResource> page, SysTenantResource sysTenantResource,Boolean isAll) {
        sysTenantResourceService.export(page,sysTenantResource,isAll);
    }
}

