package com.tofly.system.entity;



import java.io.Serializable;
import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.common.constant.SqlCondition;
import com.tofly.common.entity.vo.FileInfo;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * 系统App管理
 * @TableName SYS_APP
 */
@TableName(value ="SYS_APP",resultMap = "BaseResultMap")
@Data
public class SysApp implements Serializable {
    /**
     * 
     */
    @TableId(type = IdType.ASSIGN_ID)
    @ApiModelProperty(value = "ID")
    private Long id;

    /**
     * app名称
     */
    @ApiModelProperty(value = "app名称")
    @TableField(condition = SqlCondition.ORACLE_LIKE)
    private String name;

    /**
     * 类型：1 一体化采集；2  综合业务
     */
    @ApiModelProperty(value = "类型：1 一体化采集；2  综合业务")
    private Integer type;

    /**
     * 文件大小
     */
    @ApiModelProperty(value = "文件大小")
    private String fileSize;

    /**
     * 版本号
     */
    @ApiModelProperty(value = "版本号")
    private String versionNumber;

    /**
     * 说明
     */
    @ApiModelProperty(value = "说明")
    private String detail;

    /**
     * 状态：0未发布，1已发布
     */
    @ApiModelProperty(value = "状态：0未发布，1已发布")
    private Integer status;

    /**
     * 上传人
     */
    @ApiModelProperty(value = "上传人")
    private Long uploadUser;

    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String uploadUserName;


    /**
     * 上传时间
     */
    @ApiModelProperty(value = "上传时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private Date uploadTime;

    /**
     * 发布人
     */
    @ApiModelProperty(value = "发布人")
    private Long releaseUser;

    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String releaseUserName;


    /**
     * 发布时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value = "发布时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private Date releaseTime;

    /**
     * 文件存储路径
     */
    @ApiModelProperty(value = "文件存储路径")
    private String filePath;

    /**
     * 文件原始名称
     */
    @ApiModelProperty(value = "文件原始名称")
    private String fileName;

    /**
     * 描述
     */
    @ApiModelProperty(value = "描述")
    private String describe;

    /**
     * 附件
     */
    @ApiModelProperty(value = "附件")
    private String annex;

    /**
     * 发布方式
     */
    @ApiModelProperty(value = "发布方式")
    private Integer releaseType;

    /**
     * 更新模式
     */
    @ApiModelProperty(value = "更新模式")
    private Integer updateType;

    /**
     * 租户ID
     */
    @ApiModelProperty(value = "租户ID")
    private Long tenantId;

    /**
     * 租户名称
     */
    @TableField(exist = false)
    @ApiModelProperty(value = "租户名称",hidden = true)
    private String tenantName;

    /**
     * 排序
     */
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 逻辑删除，1删除，0不删除
     */
    @TableLogic
    @ApiModelProperty(value = "逻辑删除，1删除，0不删除")
    private Integer deleted;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    @ApiModelProperty(value = "更新时间")
    private Date updateTime;

    /**
     * 创建人
     */
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "创建人")
    private Long createUser;

    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String createUserName;

    /**
     * 修改人
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value = "修改人")
    private Long updateUser;

    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String updateUserName;


    /**
     * 发布范围
     */
    @ApiModelProperty("发布范围")
    @TableField(exist = false)
    private SysAppPermission appPermission;

    /**
     * 文件集合
     */
    @ApiModelProperty("文件集合")
    @TableField(exist = false)
    private List<FileInfo> fileInfos;

    @TableField(exist = false)
    private static final long serialVersionUID = 1L;
}