package com.tofly.system.entity;

import com.baomidou.mybatisplus.annotation.*;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import com.tofly.common.constant.SqlCondition;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 系统菜单
 * @TableName SYS_MENU
 */
@TableName(value ="SYS_MENU",resultMap = "BaseResultMap")
@Data
public class SysMenu implements Serializable {
    /**
     * 主键ID
     */
    @TableId(type = IdType.ASSIGN_ID)
    @ApiModelProperty(value = "主键ID")
    private Long id;

    /**
     * 菜单名称
     */
    @ApiModelProperty(value = "菜单名称")
    @TableField(condition = SqlCondition.ORACLE_LIKE)
    private String name;

    /**
     * 菜单权限标识（保留字段）
     */
    @ApiModelProperty(value = "菜单权限标识（保留字段）")
    private String permission;

    /**
     * 路由地址
     */
    @ApiModelProperty(value = "路由地址")
    private String path;

    /**
     * 父菜单ID
     */
    @ApiModelProperty(value = "父菜单ID")
    private Long parentId;

    /**
     * 默认图标
     */
    @ApiModelProperty(value = "默认图标")
    private String icon;

    /**
     * 排序
     */
    @ApiModelProperty(value = "排序")
    private Integer sort;

    /**
     * 开启关闭 1开启，0关闭
     */
    @ApiModelProperty(value = "开启关闭 1开启，0关闭")
    private Integer keepAlive;

    /**
     * 菜单类型 0菜单 1功能 2工具 3按钮 4App菜单
     */
    @ApiModelProperty(value = "菜单类型 0菜单 1功能 2工具 3按钮 4App菜单")
    private Integer type;
    /**
     * 菜单类型 0菜单 1功能 2工具 3按钮 4App菜单,查询专用，多个英文逗号隔开
     */
    @ApiModelProperty(value = "菜单类型 0菜单 1功能 2工具 3按钮 4App菜单,查询专用，多个英文逗号隔开")
    @TableField(exist = false)
    private String types;

    /**
     * 是否启用禁用 1启用，0禁用
     */
    @ApiModelProperty(value = "是否启用禁用 1启用，0禁用")
    private Integer active;

    /**
     * 菜单编码（自定义菜单）
     */
    @ApiModelProperty(value = "菜单编码（自定义菜单）")
    private String code;

    /**
     * 鼠标经过图标
     */
    @ApiModelProperty(value = "鼠标经过图标")
    private String mouseIcon;

    /**
     * 是否放置在模板内  1放置在模板内，0不放置。
     */
    @ApiModelProperty(value = "是否放置在模板内  1放置在模板内，0不放置。")
    private Integer inpanel;

    /**
     * 放置模板
     */
    @ApiModelProperty(value = "放置模板")
    private String panel;

    /**
     * 是否跟随系统启动 1启动 2不启动
     */
    @ApiModelProperty(value = "是否跟随系统启动 1启动 2不启动")
    private Integer autoRun;

    /**
     * 元标签
     */
    @ApiModelProperty(value = "元标签")
    private String meta;

    /**
     * 组件
     */
    @ApiModelProperty(value = "组件")
    private String component;

    /**
     * 扩展JSON字段
     */
    @ApiModelProperty(value = "扩展JSON字段")
    private String extend;

    /**
     * 重定向地址
     */
    @ApiModelProperty(value = "重定向地址")
    private String redirect;

    /**
     * 请求方法
     */
    @ApiModelProperty(value = "请求方法")
    private String method;

    /**
     * 租户ID
     */
    @ApiModelProperty(value = "租户ID")
    private Long tenantId;

    /**
     * 租户名称
     */
    @TableField(exist = false)
    @ApiModelProperty(value = "租户名称",hidden = true)
    private String tenantName;

    /**
     * 逻辑删除，1删除，0不删除
     */
    @TableLogic
    @ApiModelProperty(value = "逻辑删除，1删除，0不删除")
    private Integer deleted;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value = "更新时间")
    private Date updateTime;

    /**
     * 创建人
     */
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "创建人")
    private Long createUser;

    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String createUserName;

    /**
     * 修改人
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value = "修改人")
    private Long updateUser;

    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String updateUserName;

    @TableField(exist = false)
    @ApiModelProperty(value = "可用公司列表集合")
    private List<SysMenuTenant> menuTenants;

    @TableField(exist = false)
    private static final long serialVersionUID = 1L;
}