package com.tofly.system.entity.dto;

import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysAppPermission;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
* 系统App管理
* @TableName SYS_APP
*/
@Data
public class AppDto extends BaseDto implements Serializable {

    /**
    * 主键ID
    */
    @NotNull(message="[ID]不能为空",groups = Update.class)
    @ApiModelProperty("ID")
    private Long id;
    /**
    * app名称
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("app名称")
    @Length(max= 100,message="编码长度不能超过100")
    private String name;
    /**
    * 类型：1 一体化采集；2  综合业务
    */
    @ApiModelProperty("类型：1 一体化采集；2  综合业务")
    private Integer type;
    /**
    * 文件大小
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("文件大小")
    @Length(max= 200,message="编码长度不能超过200")
    private String fileSize;
    /**
    * 版本号
    */
    @Size(max= 500,message="编码长度不能超过500")
    @ApiModelProperty("版本号")
    @Length(max= 500,message="编码长度不能超过500")
    private String versionNumber;
    /**
    * 说明
    */
    @Size(max= 1000,message="编码长度不能超过1000")
    @ApiModelProperty("说明")
    @Length(max= 1000,message="编码长度不能超过1,000")
    private String detail;
    /**
    * 状态：0未发布，1已发布
    */
    @NotNull(message="[状态：0未发布，1已发布]不能为空")
    @ApiModelProperty("状态：0未发布，1已发布")
    private Integer status;
    /**
    * 上传人
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("上传人")
    @Length(max= 255,message="编码长度不能超过255")
    private String uploadUser;
    /**
    * 上传时间
    */
    @ApiModelProperty("上传时间")
    private Date uploadTime;
    /**
    * 发布人
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("发布人")
    @Length(max= 255,message="编码长度不能超过255")
    private String releaseUser;
    /**
    * 发布时间
    */
    @ApiModelProperty("发布时间")
    private Date releaseTime;
    /**
    * 文件存储路径
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("文件存储路径")
    @Length(max= 255,message="编码长度不能超过255")
    private String filePath;
    /**
    * 文件原始名称
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("文件原始名称")
    @Length(max= 255,message="编码长度不能超过255")
    private String fileName;
    /**
    * 描述
    */
    @Size(max= 500,message="编码长度不能超过500")
    @ApiModelProperty("描述")
    @Length(max= 500,message="编码长度不能超过500")
    private String describe;
    /**
    * 附件
    */
    @Size(max= 4000,message="编码长度不能超过4000")
    @ApiModelProperty("附件")
    @Length(max= 4000,message="编码长度不能超过4000")
    private String annex;
    /**
    * 发布方式
    */
    @ApiModelProperty("发布方式")
    private Integer releaseType;
    /**
    * 更新模式
    */
    @ApiModelProperty("更新模式")
    private Integer updateType;
    /**
    * 租户ID
    */
    @ApiModelProperty("租户ID")
    private Long tenantId;
    /**
    * 排序
    */
    @ApiModelProperty("排序")
    private Long sort;

    /**
     * 发布范围
     */
    @ApiModelProperty("发布范围")
    private SysAppPermission appPermission;

}
