package com.tofly.system.entity.dto;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import javax.validation.constraints.NotNull;

import java.io.Serializable;

import com.tofly.common.entity.dto.BaseDto;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

/**
* 系统部门组织
* @TableName SYS_DEPT
*/
@Data
public class DeptDto extends BaseDto implements Serializable {

    /**
     * 主键ID
     */
    @NotNull(message="[ID]不能为空",groups = Update.class)
    @ApiModelProperty("主键ID")
    private Long id;
    /**
    * 父级ID
    */
    @NotNull(message="[父级ID]不能为空")
    @ApiModelProperty("父级ID")
    private Long pid;
    /**
    * 公司ID
    */
    @ApiModelProperty("公司ID")
    private Long tenantId;
    /**
    * 类型：0缺省值，暂时无意义表示，1建设单位，2施工单位，3监理单位
    */
    @NotNull(message="[类型：0缺省值，暂时无意义表示，1建设单位，2施工单位，3监理单位]不能为空")
    @ApiModelProperty("类型：0缺省值，暂时无意义表示，1建设单位，2施工单位，3监理单位")
    private Integer type;
    /**
    * 组织全称
    */
    @NotBlank(message="[组织全称]不能为空")
    @Size(max= 500,message="编码长度不能超过500")
    @ApiModelProperty("组织全称")
    @Length(max= 500,message="编码长度不能超过500")
    private String name;
    /**
    * 组织简称
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("组织简称")
    @Length(max= 100,message="编码长度不能超过100")
    private String simpleName;
    /**
    * 电话
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("电话")
    @Length(max= 100,message="编码长度不能超过100")
    private String phone;
    /**
    * 编码
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("编码")
    @Length(max= 100,message="编码长度不能超过100")
    private String code;
    /**
    * 描述
    */
    @Size(max= 500,message="编码长度不能超过500")
    @ApiModelProperty("描述")
    @Length(max= 500,message="编码长度不能超过500")
    private String describe;
    /**
    * 排序
    */
    @ApiModelProperty("排序")
    private Long sort;

}
