package com.tofly.system.entity.dto;

import javax.validation.constraints.Size;
import javax.validation.constraints.NotNull;

import java.io.Serializable;
import java.util.List;

import com.tofly.common.entity.dto.BaseDto;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

/**
* 系统菜单
* @TableName SYS_MENU
*/
@Data
public class MenuDto extends BaseDto implements Serializable {

    /**
     * 主键ID
     */
    @NotNull(message="[ID]不能为空",groups = Update.class)
    @ApiModelProperty("主键ID")
    private Long id;
    /**
    * 菜单名称
    */
    @Size(max= 510,message="编码长度不能超过510")
    @ApiModelProperty("菜单名称")
    @Length(max= 510,message="编码长度不能超过510")
    private String name;
    /**
    * 菜单权限标识（保留字段）
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("菜单权限标识（保留字段）")
    @Length(max= 100,message="编码长度不能超过100")
    private String permission;
    /**
    * 路由地址
    */
    @Size(max= 128,message="编码长度不能超过128")
    @ApiModelProperty("路由地址")
    @Length(max= 128,message="编码长度不能超过128")
    private String path;
    /**
    * 父菜单ID
    */
    @ApiModelProperty("父菜单ID")
    private Long parentId;
    /**
    * 默认图标
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("默认图标")
    @Length(max= 200,message="编码长度不能超过200")
    private String icon;
    /**
    * 排序
    */
    @ApiModelProperty("排序")
    private Integer sort;
    /**
    * 开启关闭 1开启，0关闭
    */
    @ApiModelProperty("开启关闭 1开启，0关闭")
    private Integer keepAlive;
    /**
     * 菜单类型 0菜单 1功能 2工具 3按钮 4App菜单
    */
    @ApiModelProperty(value = "菜单类型 0菜单 1功能 2工具 3按钮 4App菜单")
    private Integer type;
    /**
    * 是否启用禁用 1启用，0禁用
    */
    @ApiModelProperty("是否启用禁用 1启用，0禁用")
    private Integer active;
    /**
    * 菜单编码（自定义菜单）
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("菜单编码（自定义菜单）")
    @Length(max= 100,message="编码长度不能超过100")
    private String code;
    /**
    * 鼠标经过图标
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("鼠标经过图标")
    @Length(max= 200,message="编码长度不能超过200")
    private String mouseIcon;
    /**
    * 是否放置在模板内  1放置在模板内，0不放置。
    */
    @ApiModelProperty("是否放置在模板内  1放置在模板内，0不放置。")
    private Integer inpanel;
    /**
    * 放置模板
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("放置模板")
    @Length(max= 100,message="编码长度不能超过100")
    private String panel;
    /**
    * 是否跟随系统启动 1启动 2不启动
    */
    @ApiModelProperty("是否跟随系统启动 1启动 2不启动")
    private Integer autoRun;
    /**
    * 元标签
    */
    @Size(max= 510,message="编码长度不能超过510")
    @ApiModelProperty("元标签")
    @Length(max= 510,message="编码长度不能超过510")
    private String meta;
    /**
    * 组件
    */
    @Size(max= 510,message="编码长度不能超过510")
    @ApiModelProperty("组件")
    @Length(max= 510,message="编码长度不能超过510")
    private String component;
    /**
    * 扩展JSON字段
    */
    @Size(max= 1000,message="编码长度不能超过1000")
    @ApiModelProperty("扩展JSON字段")
    @Length(max= 1000,message="编码长度不能超过1,000")
    private String extend;
    /**
    * 重定向地址
    */
    @Size(max= 510,message="编码长度不能超过510")
    @ApiModelProperty("重定向地址")
    @Length(max= 510,message="编码长度不能超过510")
    private String redirect;
    /**
    * 请求方法
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("请求方法")
    @Length(max= 255,message="编码长度不能超过255")
    private String method;
    /**
    * 租户ID
    */
    @ApiModelProperty("租户ID")
    private Long tenantId;

    /**
     * 多选关联租户ID数组集合
     */
    @ApiModelProperty("多选关联租户ID数组集合")
    private List<Long> tenantIds;
}
