package com.tofly.system.entity.dto;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import javax.validation.constraints.NotNull;

import java.io.Serializable;
import java.util.List;

import com.tofly.common.entity.dto.BaseDto;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

/**
* 系统岗位
* @TableName SYS_POST
*/
@Data
public class PostDto extends BaseDto implements Serializable {

    /**
     * 主键ID
     */
    @NotNull(message="[ID]不能为空",groups = Update.class)
    @ApiModelProperty("主键ID")
    private Long id;
    /**
    * 公司ID
    */
    @ApiModelProperty("公司ID")
    private Long tenantId;

    /**
     * 公司ID批量数组集合，批量添加修改使用
     */
    @ApiModelProperty("公司ID批量数组集合，批量添加修改使用")
    @NotNull(message="[公司ID]不能为空")
    @Size(message = "[公司ID]不能为空")
    private List<Long> tenantIds;
    /**
    * 岗位名称
    */
    @NotBlank(message="[岗位名称]不能为空")
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("岗位名称")
    @Length(max= 200,message="编码长度不能超过200")
    private String name;
    /**
    * 备注
    */
    @Size(max= 500,message="编码长度不能超过500")
    @ApiModelProperty("备注")
    @Length(max= 500,message="编码长度不能超过500")
    private String remark;
    /**
    * 编码
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("编码")
    @Length(max= 100,message="编码长度不能超过100")
    private String code;
    /**
    * 岗位职责描述
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("岗位职责描述")
    @Length(max= 200,message="编码长度不能超过200")
    private String describe;
    /**
    * 排序
    */
    @ApiModelProperty("排序")
    private Long sort;

}
