package com.tofly.system.entity.dto;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.Size;
import javax.validation.constraints.NotNull;

import java.io.Serializable;
import java.util.List;

import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysRolePermission;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

/**
* 系统角色
* @TableName SYS_ROLE
*/
@Data
public class RoleDto extends BaseDto implements Serializable {

    /**
     * 主键ID
     */
    @NotNull(message="[ID]不能为空",groups = Update.class)
    @ApiModelProperty("角色ID")
    private Long id;
    /**
    * 角色名字
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("角色名字")
    @Length(max= 100,message="编码长度不能超过100")
    @NotEmpty(message = "角色名称不能为空")
    private String name;
    /**
    * 角色标识
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("角色标识")
    @Length(max= 100,message="编码长度不能超过100")
    private String code;
    /**
    * 角色描述
    */
    @Size(max= 200,message="编码长度不能超过200")
    @ApiModelProperty("角色描述")
    @Length(max= 200,message="编码长度不能超过200")
    private String describe;
    /**
    * 是否启用禁用 1启用 0禁用
    */
    @ApiModelProperty("是否启用禁用 1启用 0禁用")
    private Integer active;
    /**
    * 租户ID
    */
    @ApiModelProperty("租户ID")
    private Long tenantId;

    /**
     * 公司ID批量数组集合，批量添加修改使用
     */
    @ApiModelProperty("公司ID批量数组集合，批量添加修改使用")
    private List<Long> tenantIds;

    /**
    * 排序
    */
    @ApiModelProperty("排序")
    private Long sort;

    /**
     * 角色操作权限
     */
    @ApiModelProperty("角色操作权限")
    private SysRolePermission sysRolePermission;

    /**
     * 角色菜单权限
     */
    @ApiModelProperty("角色菜单权限")
    private List<Long> menuIds;

}
