package com.tofly.system.entity.dto;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import javax.validation.constraints.NotNull;

import java.io.Serializable;

import java.util.Date;
import java.util.List;

import com.tofly.common.entity.dto.BaseDto;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

/**
* 系统用户
* @TableName SYS_USER
*/
@Data
public class UserDto extends BaseDto implements Serializable {

    /**
    * 用户ID，主键
    */
    @NotNull(message="[用户ID，主键]不能为空",groups = BaseDto.Update.class)
    @ApiModelProperty("用户ID，主键")
    private Long id;
    /**
    * 用户名
    */
    @Pattern(regexp = "[A-Za-z0-9_\\-\\u4e00-\\u9fa5]+",message = "[用户名]不能为特殊字符或中文")
    @NotBlank(message="[用户名]不能为空")
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("用户名")
    @Length(max= 100,message="编码长度不能超过100")
    private String username;
    /**
    * 密码
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("密码")
    @Length(max= 255,message="编码长度不能超过255")
    private String password;

    /**
    * 电话
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("电话")
    @Length(max= 100,message="编码长度不能超过100")
    private String phone;
    /**
    * 头像
    */
    @Size(max= 255,message="编码长度不能超过255")
    @ApiModelProperty("头像")
    @Length(max= 255,message="编码长度不能超过255")
    private String avatar;
    /**
    * 是否锁定，1正常，0锁定
    */
    @ApiModelProperty("是否锁定，1正常，0锁定")
    private Integer lockFlag;
    /**
    * 是否启用禁用，1启用，0禁用
    */
    @ApiModelProperty("是否启用禁用，1启用，0禁用")
    private Integer active;
    /**
    * 扩展功能，微信openid
    */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("扩展功能，微信openid")
    @Length(max= 32,message="编码长度不能超过32")
    private String wxOpenid;
    /**
    * 扩展功能，QQ openid
    */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("扩展功能，QQ openid")
    @Length(max= 32,message="编码长度不能超过32")
    private String qqOpenid;
    /**
    * 用户email
    */
    @Size(max= 50,message="编码长度不能超过50")
    @ApiModelProperty("用户email")
    @Length(max= 50,message="编码长度不能超过50")
    private String email;
    /**
    * 用户姓名
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("用户姓名")
    @Length(max= 100,message="编码长度不能超过100")
    private String realName;

    /**
    * 工号
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("工号")
    @Length(max= 100,message="编码长度不能超过100")
    private String jobNumber;

    /**
    * 别名
    */
    @Size(max= 100,message="编码长度不能超过100")
    @ApiModelProperty("别名")
    @Length(max= 100,message="编码长度不能超过100")
    private String aliasName;
    /**
    * 生日
    */
    @ApiModelProperty("生日")
    private Date birthday;
    /**
    * 用户级别  用于超级管理员与其他用户区分等  。0 超级管理员1普通管理员,2其他公司管理员
    */
    @ApiModelProperty("用户级别  用于超级管理员与其他用户区分等  。0 超级管理员1普通管理员,2其他公司管理员")
    private Integer userLevel;

    /**
    * 用户账号详情
    */
    @Size(max= 510,message="编码长度不能超过510")
    @ApiModelProperty("用户账号详情")
    @Length(max= 510,message="编码长度不能超过510")
    private String details;
    /**
    * 备注
    */
    @Size(max= 2000,message="编码长度不能超过2000")
    @ApiModelProperty("备注")
    @Length(max= 2000,message="编码长度不能超过2,000")
    private String note;

    /**
    * 公司ID
    */
    @NotNull(message="[公司ID]不能为空")
    @ApiModelProperty("公司ID")
    private Long tenantId;
    /**
    * 排序
    */
    @ApiModelProperty("排序")
    private Long sort;

    /**
     * 角色ID集合
     */
    @ApiModelProperty("角色ID集合")
    private List<Long> roleIds;

    /**
     * 部门ID集合
     */
    @ApiModelProperty("部门ID集合")
    private List<Long> deptIds;

    /**
     * 岗位ID集合
     */
    @ApiModelProperty("岗位ID集合")
    private List<Long> postIds;

}
