package com.tofly.system.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.system.entity.SysDept;
import com.tofly.system.entity.SysPost;
import com.tofly.system.entity.SysRole;
import com.tofly.system.entity.SysUser;
import com.tofly.system.entity.dto.UserDto;
import com.tofly.system.entity.query.UserQuery;
import com.tofly.system.entity.vo.user.UserVo;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
* @author haiqiu
* @description 针对表【SYS_USER(系统用户)】的数据库操作Service
* @createDate 2022-07-29 14:23:51
*/
public interface SysUserService extends IService<SysUser> {

    /**
     * 根据ID查询
     * @param id id
     * @return 数据
     */
    UserVo get(Long id);

    /**
     * 新增
     * @param userDto 实体
     * @param avatarFile 文件
     */
    void insert(UserDto userDto, MultipartFile avatarFile);

    /**
     * 修改
     * @param userDto 实体
     * @param avatarFile 文件
     */
    void edit(UserDto userDto, MultipartFile avatarFile);

    /**
     * 批量删除
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     * @param page 分页
     * @param userQuery 查询wrapper
     * @return 数据集合
     */
    Page<UserVo> pageList(Page page, UserQuery userQuery);

    /**
     * 数据导出
     * @param page 分页
     * @param userQuery 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    void export(Page page, UserQuery userQuery, Boolean isAll);

    /**
     * 根据用户ID查询用户的角色列表
     * @param userId 用户ID
     * @return 根据用户ID查询用户的角色列表
     */
    List<SysRole> getRoleByUserId(Long userId);

    /**
     * 根据用户ID查询用户的岗位列表
     * @param userId 用户ID
     * @return 根据用户ID查询用户的岗位列表
     */
    List<SysPost> getPostByUserId(Long userId);

    /**
     * 根据用户ID查询用户的部门列表
     * @param userId 用户ID
     * @return 根据用户ID查询用户的部门列表
     */
    List<SysDept> getDeptByUserId(Long userId);

    /**
     * 查询所有数据
     * @param userQuery 查询条件
     * @return 用户列表
     */
    List<UserVo> allList(UserQuery userQuery);

    /**
     * 修改密码
     * @param oldPwd 旧密码
     * @param newPwd 新密码
     * @param newRePwd 重复新密码
     */
    void updatePwd(String oldPwd, String newPwd, String newRePwd);

    /**
     * 清空用户登录的缓存信息
     * @param userIds 用户ID
     */
    void clearCached(List<Long> userIds);
}
