package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.system.entity.SysDept;
import com.tofly.system.entity.SysRolePermission;
import com.tofly.system.entity.SysUserDept;
import com.tofly.system.entity.dto.DeptDto;
import com.tofly.system.entity.vo.dept.DeptVo;
import com.tofly.system.entity.vo.tenant.TenantVo;
import com.tofly.system.mapper.SysDeptMapper;
import com.tofly.system.service.SysDeptService;
import com.tofly.system.service.SysUserDeptService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author haiqiu
 * @description 针对表【SYS_DEPT(系统部门组织)】的数据库操作Service实现
 * @createDate 2022-08-01 14:50:48
 */
@Service
@AllArgsConstructor
public class SysDeptServiceImpl extends ServiceImpl<SysDeptMapper, SysDept>
        implements SysDeptService {

    private final SysUserDeptService userDeptService;

    @Override
    @Transactional
    public void insert(DeptDto deptDto) {
        setDefaultVal(deptDto);
        SysDept sysDept = new SysDept();
        BeanUtils.copyProperties(deptDto, sysDept);
        if (!save(sysDept)) {
            throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
        }
    }

    @Override
    @Transactional
    public void edit(DeptDto deptDto) {
        setDefaultVal(deptDto);
        SysDept sysDept = new SysDept();
        BeanUtils.copyProperties(deptDto, sysDept);
        if (!updateById(sysDept)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
    }

    /**
     * 自动该账号的公司ID
     *
     * @param deptDto 设置缺省值
     */
    public void setDefaultVal(DeptDto deptDto) {
        if (deptDto.getTenantId() == null) {
            Long tenantId = SecurityUtils.getTenantId();
            deptDto.setTenantId(tenantId);
        }
    }

    @Override
    public void delete(List<Long> idList) {
        for (Long id : idList) {
            if (count(Wrappers.<SysDept>lambdaQuery().eq(SysDept::getPid, id)) != 0) {
                SysDept dept = getById(id);
                throw new RuntimeException(dept.getName() + "下存在子集部门，请先删除子集部门");
            }
        }
        if (!removeByIds(idList)) {
            throw new RuntimeException(ResultEnum.DELETE_FAIL.getMessage());
        }
        //删除关联人员
        userDeptService.remove(Wrappers.<SysUserDept>lambdaQuery().in(SysUserDept::getDeptId, idList));
    }

    @Override
    public List<DeptVo> getChildren(Long treeId) {
        return treeData(treeId, list());
    }

    private List<DeptVo> treeData(Long treeId, List<SysDept> depts) {
        if (CollectionUtils.isEmpty(depts)) {
            return Collections.emptyList();
        }
        List<DeptVo> voList = depts.stream().map(data -> {
            DeptVo deptVo = new DeptVo();
            BeanUtils.copyProperties(data, deptVo);
            return deptVo;
        }).collect(Collectors.toList());
        List<DeptVo> deptVos = new ArrayList<>();
        for (DeptVo dept : voList) {
            if (dept.getId().equals(treeId)) {
                deptVos.add(selectChild(dept, voList));
            }
        }
        return deptVos;
    }

    private DeptVo selectChild(DeptVo deptVo, List<DeptVo> deptVoList) {
        deptVo.setChildren(new ArrayList<>());
        for (DeptVo vo : deptVoList) {
            if (deptVo.getChildren() == null) {
                deptVo.setChildren(new ArrayList<>());
            }
            if (deptVo.getId().equals(vo.getPid())) {
                deptVo.getChildren().add(selectChild(vo, deptVoList));
            }
        }
        return deptVo;
    }

}




