package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.system.entity.SysDict;
import com.tofly.system.entity.dto.DictDto;
import com.tofly.system.service.SysDictService;
import com.tofly.system.mapper.SysDictMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * @author haiqiu
 * @description 针对表【SYS_DICT(系统字典)】的数据库操作Service实现
 * @createDate 2022-08-01 14:50:28
 */
@Service
public class SysDictServiceImpl extends ServiceImpl<SysDictMapper, SysDict>
        implements SysDictService {

    @Override
    public void insert(DictDto dictDto) {
        SysDict sysDict = new SysDict();
        //检测code编码重复问题
        checkCode(dictDto);
        BeanUtils.copyProperties(dictDto, sysDict);
        if (!save(sysDict)) {
            throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
        }
    }

    /**
     * 检测code编码重复问题
     *
     * @param dictDto code编码
     */
    private void checkCode(DictDto dictDto) {
        if (dictDto == null && !StringUtils.hasText(dictDto.getCode())) {
            return;
        }
        //子集不添加code编码
        if (dictDto.getPid() != null && !dictDto.getPid().equals(SysConstants.PID)) {
            dictDto.setCode(null);
            return;
        } else if (dictDto.getPid() == null || dictDto.getPid().equals(SysConstants.PID)) {
            if (StringUtils.isEmpty(dictDto.getCode())) {
                throw new RuntimeException("字典编码不能为空");
            }
        }
        //编码是否存在
        if (dictDto.getId() == null) {
            if (count(Wrappers.<SysDict>lambdaQuery().eq(SysDict::getCode, dictDto.getCode())) != 0) {
                throw new RuntimeException(dictDto.getCode() + "编码已存在");
            }
        } else {
            if (count(Wrappers.<SysDict>lambdaQuery().eq(SysDict::getCode, dictDto.getCode())
                    .ne(SysDict::getId, dictDto.getId())) != 0) {
                throw new RuntimeException(dictDto.getCode() + "编码已存在");
            }
        }
    }

    @Override
    public void edit(DictDto dictDto) {
        //检测code编码重复问题
        checkCode(dictDto);
        SysDict sysDict = new SysDict();
        BeanUtils.copyProperties(dictDto, sysDict);
        if (!updateById(sysDict)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
    }
}




