package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.system.entity.SysPost;
import com.tofly.system.entity.SysTenant;
import com.tofly.system.entity.SysUserPost;
import com.tofly.system.entity.dto.PostDto;
import com.tofly.system.mapper.SysPostMapper;
import com.tofly.system.service.SysPostService;
import com.tofly.system.service.SysTenantService;
import com.tofly.system.service.SysUserPostService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author haiqiu
 * @description 针对表【SYS_POST(系统岗位)】的数据库操作Service实现
 * @createDate 2022-08-01 14:52:05
 */
@Service
@AllArgsConstructor
public class SysPostServiceImpl extends ServiceImpl<SysPostMapper, SysPost>
        implements SysPostService {

    private final SysUserPostService userPostService;

    private final SysTenantService tenantService;

    @Override
    public void insert(PostDto postDto) {
        //批量操作
        batchInsertOrUpdate(postDto);
    }

    private void batchInsertOrUpdate(PostDto postDto) {
        if (postDto == null) {
            throw new RuntimeException("参数不全");
        }
        //批量新增添加公司岗位
        if (!CollectionUtils.isEmpty(postDto.getTenantIds())) {
            List<SysPost> postList = new ArrayList<>();
            for (Long tenantId : postDto.getTenantIds()) {
                //判断是否名称重复
                checkPostName(postDto, tenantId);
                SysPost sysPost = new SysPost();
                BeanUtils.copyProperties(postDto, sysPost);
                sysPost.setTenantId(tenantId);
                postList.add(sysPost);
            }
            if (!saveOrUpdateBatch(postList)) {
                throw new RuntimeException("操作成功");
            }
            //否则就是单条新增修改
        } else {
            //判断是否名称重复
            checkPostName(postDto, postDto.getTenantId());
            SysPost sysPost = new SysPost();
            BeanUtils.copyProperties(postDto, sysPost);
            if (!saveOrUpdate(sysPost)) {
                throw new RuntimeException("操作失败");
            }
        }
    }


    /**
     * 检测岗位名称重复
     * @param postDto 需要添加的数据实体
     * @param tenantId 租户ID
     */
    private void checkPostName(PostDto postDto, Long tenantId) {
        int count;
        if (postDto.getId() == null){
            count = count(Wrappers.<SysPost>lambdaQuery()
                    .eq(StringUtils.hasText(postDto.getName()), SysPost::getName, postDto.getName())
                    .eq(tenantId != null, SysPost::getTenantId, tenantId));
        }else {
            count = count(Wrappers.<SysPost>lambdaQuery()
                    .eq(StringUtils.hasText(postDto.getName()), SysPost::getName, postDto.getName())
                    .eq(tenantId != null, SysPost::getTenantId, tenantId)
                    .ne(SysPost::getId, postDto.getId()));
        }
        if (count!=0){
            SysTenant tenant = tenantService.getById(tenantId);
            throw new RuntimeException(tenant.getName()+"已存在岗位名称："+ postDto.getName());
        }
    }

    @Override
    public void edit(PostDto postDto) {
        //批量操作
        batchInsertOrUpdate(postDto);
    }

    @Transactional
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException(ResultEnum.DELETE_FAIL.getMessage());
        }
        //删除绑定的用户
        userPostService.remove(Wrappers.<SysUserPost>lambdaQuery().in(SysUserPost::getPostId, idList));
    }
}




