package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.system.entity.SysDict;
import com.tofly.system.entity.SysResource;
import com.tofly.system.entity.dto.DictDto;
import com.tofly.system.entity.dto.ResourceDto;
import com.tofly.system.entity.vo.resource.ResourceVo;
import com.tofly.system.mapper.SysDictMapper;
import com.tofly.system.mapper.SysResourceMapper;
import com.tofly.system.service.SysDictService;
import com.tofly.system.service.SysResourceService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author haiqiu
 * @description 针对表【SYS_RESOURCE(系统资源)】的数据库操作Service实现
 * @createDate 2022-08-01 14:50:28
 */
@Service
public class SysResourceServiceImpl extends ServiceImpl<SysResourceMapper, SysResource>
        implements SysResourceService {

    @Override
    public void insert(ResourceDto resourceDto) {
        SysResource sysResource = new SysResource();
        BeanUtils.copyProperties(resourceDto, sysResource);
        if (!save(sysResource)) {
            throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
        }
    }

    @Override
    public void edit(ResourceDto resourceDto) {
        SysResource sysResource = new SysResource();
        BeanUtils.copyProperties(resourceDto, sysResource);
        if (!updateById(sysResource)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
    }

    @Override
    public List<ResourceVo> tree(SysResource sysResource) {
        List<SysResource> resources = list(Wrappers.<SysResource>lambdaQuery(sysResource));
        if (CollectionUtils.isEmpty(resources)){
            return null;
        }
        return getTreeData(resources);
    }

    private List<ResourceVo> getTreeData(List<SysResource> resources) {

        List<ResourceVo> resourceVos = resources.stream().map(sysResource -> {
            ResourceVo resourceVo = new ResourceVo();
            BeanUtils.copyProperties(sysResource, resourceVo);
            return resourceVo;
        }).collect(Collectors.toList());

        List<ResourceVo> resourceVoList = new ArrayList<>();
        for (ResourceVo resource : resourceVos) {
            if (resource.getPid().equals(SysConstants.PID)){
                resourceVoList.add(getChildrenData(resource,resourceVos));
            }
        }
        return resourceVoList;
    }

    private ResourceVo getChildrenData(ResourceVo resourceVo, List<ResourceVo> resources) {
        resourceVo.setChildren(new ArrayList<>());
        for (ResourceVo resource : resources) {
            if (resourceVo.getChildren()==null){
                resourceVo.setChildren(new ArrayList<>());
            }
            if (resource.getPid().equals(resourceVo.getId())){
                resourceVo.getChildren().add(getChildrenData(resource,resources));
            }
        }
        return resourceVo;
    }
}




