package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.system.entity.SysRole;
import com.tofly.system.entity.SysRoleMenu;
import com.tofly.system.entity.SysRolePermission;
import com.tofly.system.entity.SysTenant;
import com.tofly.system.entity.dto.RoleDto;
import com.tofly.system.mapper.SysRoleMapper;
import com.tofly.system.service.SysRoleMenuService;
import com.tofly.system.service.SysRolePermissionService;
import com.tofly.system.service.SysRoleService;
import com.tofly.system.service.SysTenantService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author haiqiu
 * @description 针对表【SYS_ROLE(系统角色)】的数据库操作Service实现
 * @createDate 2022-08-01 14:52:14
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole>
        implements SysRoleService {

    @Resource
    private SysRoleMenuService roleMenuService;

    @Resource
    private SysRolePermissionService rolePermissionService;

    @Resource
    private SysTenantService sysTenantService;

    @Override
    @Transactional
    public void insert(RoleDto roleDto) {
        if (roleDto == null) {
            throw new RuntimeException("参数不全");
        }
        //批量根据公司添加
        if (!CollectionUtils.isEmpty(roleDto.getTenantIds())) {
            for (Long tenantId : roleDto.getTenantIds()) {
                if (count(Wrappers.<SysRole>lambdaQuery().eq(SysRole::getName, roleDto.getName())
                        .eq(tenantId != null, SysRole::getTenantId, tenantId)) != 0) {
                    SysTenant tenant = sysTenantService.getById(tenantId);
                    throw new RuntimeException(tenant.getName() + "系统中，角色名称：" + roleDto.getName() + "已存在");
                }
                SysRole sysRole = new SysRole();
                BeanUtils.copyProperties(roleDto, sysRole);
                sysRole.setTenantId(tenantId);
                if (!save(sysRole)) {
                    throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
                }
                //绑定菜单
                saveMenus(sysRole.getId(), roleDto.getMenuIds());
                //绑定权限
                savePermission(sysRole.getId(), roleDto.getSysRolePermission());
            }
            //否则就是单条添加
        } else {
            if (count(Wrappers.<SysRole>lambdaQuery().eq(SysRole::getName, roleDto.getName())
                    .eq(roleDto.getTenantId() != null, SysRole::getTenantId, roleDto.getTenantId())) != 0) {
                throw new RuntimeException("角色名称：" + roleDto.getName() + "已存在");
            }
            SysRole sysRole = new SysRole();
            BeanUtils.copyProperties(roleDto, sysRole);
            if (!save(sysRole)) {
                throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
            }
            //绑定菜单
            saveMenus(sysRole.getId(), roleDto.getMenuIds());
            //绑定权限
            savePermission(sysRole.getId(), roleDto.getSysRolePermission());
        }
    }

    @Override
    @Transactional
    public void edit(RoleDto roleDto) {
        if (roleDto == null) {
            throw new RuntimeException("参数不全");
        }
        //批量根据公司添加
        if (!CollectionUtils.isEmpty(roleDto.getTenantIds())) {
            for (Long tenantId : roleDto.getTenantIds()) {
                if (count(Wrappers.<SysRole>lambdaQuery().eq(SysRole::getName, roleDto.getName())
                        .eq(tenantId != null, SysRole::getTenantId, tenantId)
                        .ne(SysRole::getId, roleDto.getId())) != 0) {
                    SysTenant tenant = sysTenantService.getById(tenantId);
                    throw new RuntimeException(tenant.getName() + "系统中，角色名称：" + roleDto.getName() + "已存在");
                }
                SysRole sysRole = new SysRole();
                BeanUtils.copyProperties(roleDto, sysRole);
                sysRole.setTenantId(tenantId);
                if (!updateById(sysRole)) {
                    throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
                }
                //绑定菜单
                saveMenus(sysRole.getId(), roleDto.getMenuIds());
                //绑定权限
                savePermission(sysRole.getId(), roleDto.getSysRolePermission());
            }
            //否则就是单条添加
        } else {
            if (count(Wrappers.<SysRole>lambdaQuery().eq(SysRole::getName, roleDto.getName())
                    .eq(roleDto.getTenantId() != null, SysRole::getTenantId, roleDto.getTenantId())
                    .ne(SysRole::getId, roleDto.getId())) != 0) {
                throw new RuntimeException("角色名称：" + roleDto.getName() + "已存在");
            }
            SysRole sysRole = new SysRole();
            BeanUtils.copyProperties(roleDto, sysRole);
            if (!updateById(sysRole)) {
                throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
            }
            //绑定菜单
            saveMenus(sysRole.getId(), roleDto.getMenuIds());
            //绑定权限
            savePermission(sysRole.getId(), roleDto.getSysRolePermission());
        }
    }

    @Transactional
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException(ResultEnum.DELETE_FAIL.getMessage());
        }
        //删除关联权限
        rolePermissionService.remove(Wrappers.<SysRolePermission>lambdaQuery().in(SysRolePermission::getRoleId, idList));
        //删除关联菜单
        roleMenuService.remove(Wrappers.<SysRoleMenu>lambdaQuery().in(SysRoleMenu::getRoleId, idList));
    }

    @Override
    public SysRole get(Serializable id) {
        SysRole sysRole = getById(id);
        //角色绑定菜单
        List<SysRoleMenu> roleMenus = roleMenuService.list(Wrappers.<SysRoleMenu>lambdaQuery()
                .eq(SysRoleMenu::getRoleId, id));
        //角色发布范围
        List<SysRolePermission> rolePermissions = rolePermissionService.list(Wrappers.<SysRolePermission>lambdaQuery()
                .eq(SysRolePermission::getRoleId, id));

        sysRole.setRoleMenus(roleMenus);
        sysRole.setRolePermissions(rolePermissions);
        return sysRole;
    }


    /**
     * 添加关联菜单权限
     *
     * @param roleId  角色ID
     * @param menuIds 菜单ID集合
     */
    public void saveMenus(Long roleId, List<Long> menuIds) {
        //删除
        roleMenuService.remove(Wrappers.<SysRoleMenu>lambdaQuery().eq(SysRoleMenu::getRoleId, roleId));
        if (CollectionUtils.isEmpty(menuIds)) {
            return;
        }
        //新增
        List<SysRoleMenu> roleMenus = menuIds.stream().map(menuId -> {
            SysRoleMenu sysRoleMenu = new SysRoleMenu();
            sysRoleMenu.setMenuId(menuId);
            sysRoleMenu.setRoleId(roleId);
            return sysRoleMenu;
        }).collect(Collectors.toList());
        roleMenuService.saveBatch(roleMenus);

    }

    /**
     * 角色绑定上下级权限
     *
     * @param roleId            角色ID
     * @param sysRolePermission 权限
     */
    public void savePermission(Long roleId, SysRolePermission sysRolePermission) {
        //删除
        rolePermissionService.remove(Wrappers.<SysRolePermission>lambdaQuery().eq(SysRolePermission::getRoleId, roleId));
        if (ObjectUtils.isEmpty(sysRolePermission)) {
            return;
        }
        //新增
        sysRolePermission.setRoleId(roleId);
        rolePermissionService.save(sysRolePermission);
    }
}




