package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.haiqiu.tools.excel.utils.HqExcelUtils;
import com.tofly.common.constant.SysConstants;
import org.springframework.util.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.system.mapper.SysTenantResourceMapper;
import com.tofly.system.entity.SysTenantResource;
import com.tofly.system.service.SysTenantResourceService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 公司资源关联表(SysTenantResource)表服务实现类
 *
 * @author HaiQiu
 * @since 2022-09-09 17:26:47
 */
@Service("sysTenantResourceService")
public class SysTenantResourceServiceImpl extends ServiceImpl<SysTenantResourceMapper, SysTenantResource> implements SysTenantResourceService {

    /**
     * 根据ID查询
     * @param id id
     * @return 数据
     */
    @Override
    public SysTenantResource get(Serializable id){
        return getById(id);
    }

    /**
     * 新增
     * @param sysTenantResource 实体
     */
    @Override
    public void insert(SysTenantResource sysTenantResource){
        if(!save(sysTenantResource)){
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     * @param sysTenantResource 实体
     */
    @Override
    public void edit(SysTenantResource sysTenantResource){
        if(!updateById(sysTenantResource)){
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 批量删除
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList){
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template(){
        SysTenantResource sysTenantResource = new SysTenantResource ();
        HqExcelUtils.createTemplate("SysTenantResource导入", "SysTenantResource列表",
                SysTenantResource.class, Collections.singletonList(sysTenantResource));
    }

    /**
     * 数据导入
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file){
        String filename = file.getOriginalFilename();
        String suffix = filename.substring(filename.lastIndexOf(".") + 1, filename.length() - 1);
        if (!SysConstants.XLSX.equals(suffix)){
            throw new RuntimeException("文件类型必须是xlsx");
        }
        List<SysTenantResource> datas = HqExcelUtils.getExcelToList("SysTenantResource列表", file, SysTenantResource.class);
        if (!CollectionUtils.isEmpty(datas)) {
            saveBatch(datas);
        }
    }

    /**
     * 分页查询
     * @param page 分页
     * @param sysTenantResource 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<SysTenantResource> pageList(Page<SysTenantResource> page, SysTenantResource sysTenantResource){
        return page(page, new QueryWrapper<>(sysTenantResource));
    }

    /**
     * 数据导出
     * @param page 分页
     * @param sysTenantResource 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @Override
    public void export(Page<SysTenantResource> page, SysTenantResource sysTenantResource, Boolean isAll){
        Page<SysTenantResource> pageData = new Page<>();
        if (isAll==null || !isAll){
            pageData = page(page,new QueryWrapper<>(sysTenantResource));
        }else {
            pageData.setRecords(list(new QueryWrapper<>(sysTenantResource)));
        }
        //导出数据
        HqExcelUtils.getListToExcel(SysTenantResource .class,pageData.getRecords(),"sysTenantResource列表");
    }

    /**
     * 查询所有数据
     * @param sysTenantResource 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<SysTenantResource> all(SysTenantResource sysTenantResource){
        return list(new QueryWrapper<>(sysTenantResource));
    }
}

