package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.system.entity.SysDept;
import com.tofly.system.entity.SysTenant;
import com.tofly.system.entity.dto.TenantDto;
import com.tofly.system.entity.vo.dept.DeptVo;
import com.tofly.system.entity.vo.tenant.TenantVo;
import com.tofly.system.mapper.SysTenantMapper;
import com.tofly.system.service.SysDeptService;
import com.tofly.system.service.SysTenantService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.beans.beancontext.BeanContext;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author haiqiu
 * @description 针对表【SYS_TENANT(系统公司表)】的数据库操作Service实现
 * @createDate 2022-08-01 14:52:51
 */
@Service
@AllArgsConstructor
public class SysTenantServiceImpl extends ServiceImpl<SysTenantMapper, SysTenant>
        implements SysTenantService {

    private final SysDeptService sysDeptService;

    @Override
    public void insert(TenantDto tenantDto) {
        SysTenant sysTenant = new SysTenant();
        BeanUtils.copyProperties(tenantDto, sysTenant);
        if (!save(sysTenant)) {
            throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
        }
    }

    @Override
    public void edit(TenantDto tenantDto) {
        SysTenant sysTenant = new SysTenant();
        BeanUtils.copyProperties(tenantDto, sysTenant);
        if (!updateById(sysTenant)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
    }

    @Override
    public void importData(MultipartFile file) {

    }

    @Override
    public void export(Page<SysTenant> page, SysTenant sysTenant, Boolean isAll) {

    }

    @Override
    public void template() {

    }

    @Override
    public List<TenantVo> treeTenant(SysTenant sysTenant, boolean isHaveDept) {
        //查询公司
        List<SysTenant> tenants = allList(sysTenant);
        //是否包含部门,默认不包含false
        if (isHaveDept) {
            //查询全部部门
            List<SysDept> depts = sysDeptService.list();
            //添加部门数据
            List<TenantVo> tenantVos = tenants.stream().map(tenant -> {
                TenantVo tenantVo = new TenantVo();
                BeanUtils.copyProperties(tenant, tenantVo);
                List<SysDept> deptList = depts.stream().filter(sysDept -> sysDept.getTenantId() != null &&
                                sysDept.getTenantId().equals(tenantVo.getId()) &&
                                sysDept.getPid().equals(SysConstants.PID))
                        .collect(Collectors.toList());
                if (CollectionUtils.isEmpty(deptList)) {
                    tenantVo.setDepts(Collections.emptyList());
                } else {
                    //递归获取部门子集
                    List<DeptVo> deptVos = new ArrayList<>();
                    for (SysDept sysDept : deptList) {
                        deptVos.addAll(sysDeptService.getChildren(sysDept.getId()));
                    }
                    tenantVo.setDepts(deptVos);
                }
                return tenantVo;
            }).collect(Collectors.toList());
            //如果数据为空就返回空数组
            if (CollectionUtils.isEmpty(tenantVos)) {
                return Collections.emptyList();
            }
            //返回树形结构
            return treeData(tenantVos);
        }
        //否则不包含部门查询返回
        List<TenantVo> tenantVos = tenants.stream().map(tenant -> {
            TenantVo tenantVo = new TenantVo();
            BeanUtils.copyProperties(tenant, tenantVo);
            return tenantVo;
        }).collect(Collectors.toList());
        //如果数据为空就返回空数组
        if (CollectionUtils.isEmpty(tenantVos)) {
            return Collections.emptyList();
        }
        //返回树形结构
        return treeData(tenantVos);

    }

    @Override
    public List<SysTenant> allList(SysTenant sysTenant) {
        //根据前端传参id进行数据筛选
        Long meId = null;
        if (sysTenant != null && sysTenant.getId() != null) {
            meId = sysTenant.getId();
            sysTenant.setId(null);
        }
        //如果前端没有传id参数进行筛选查询，那么就自动列为顶级公司ID进行查询
        return baseMapper.allListByChildren(new QueryWrapper<>(sysTenant), meId == null ? SysConstants.TOP_TENANT_ID : meId);
    }

    /**
     * 获取递归树形结构
     *
     * @param tenantVos 数据集合
     * @return 获取递归树形结构
     */
    private List<TenantVo> treeData(List<TenantVo> tenantVos) {
        List<TenantVo> voList = new ArrayList<>();
        //根据当前账号获取当前所属的公司和下级公司
        long pid = !CollectionUtils.isEmpty(tenantVos) ? tenantVos.get(0).getId() : 0L;
        for (TenantVo tenantVo : tenantVos) {
            if (tenantVo.getId().equals(pid)) {
                voList.add(selectChild(tenantVo, tenantVos));
            }
        }
        return voList;
    }

    /**
     * 获取子集数据结构
     *
     * @param tenantVo  子节点
     * @param tenantVos 数据集合
     * @return 单节点树形结构
     */
    private TenantVo selectChild(TenantVo tenantVo, List<TenantVo> tenantVos) {
        tenantVo.setChildren(new ArrayList<>());
        for (TenantVo vo : tenantVos) {
            if (tenantVo.getChildren() == null) {
                tenantVo.setChildren(new ArrayList<>());
            }
            if (tenantVo.getId().equals(vo.getParentId())) {
                tenantVo.getChildren().add(selectChild(vo, tenantVos));
            }
        }
        return tenantVo;
    }
}




