/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.workflow.baseconf.entity.ContractConfig;
import com.tofly.workflow.baseconf.entity.ContractConfigField;
import com.tofly.workflow.baseconf.entity.dto.QueryConfigDto;
import com.tofly.workflow.baseconf.entity.vo.ContractConfigFieldBatchVo;
import com.tofly.workflow.baseconf.menum.ContractConfigTypeEnum;
import com.tofly.workflow.baseconf.service.ContractConfigFieldService;
import com.tofly.workflow.baseconf.service.ContractConfigService;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;
import javax.xml.transform.Result;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 合同计算配置信息
 *
 * @author WANGZHUN
 * @date Tue Apr 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/contractconfig")
@Api(tags="合同计算配置信息接口")
public class ContractConfigController {

    private final  ContractConfigService contractConfigService;

    private final ContractConfigFieldService contractConfigFieldService;
    /**
    * 分页查询
    * @param page 分页对象
    * @param contractConfig 合同计算配置信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getContractConfigPage(Page page, @ModelAttribute ContractConfig contractConfig) {
      return ResultData.ok(contractConfigService.page(page, Wrappers.query(contractConfig)));
    }


    @GetMapping("/copy" )
    @ApiOperation(value = "模板配置复制")
    public ResultData copyConfig(Long sourceConfigId,String targetTemplateIds ) {

        if(sourceConfigId==null){
            throw  new ToflyDeniedException("请传入源配置模板");
        }
        if(!StringUtils.isNotBlank(targetTemplateIds)){
            throw  new ToflyDeniedException("请传入目标模板id");
        }
        contractConfigService.copyConfig(sourceConfigId,targetTemplateIds);
        return  ResultData.ok();

    }


    @PostMapping("/getConfigData" )
    @ApiOperation(value = "获取合同自动计算数据")
    public ResultData getConfigData(@RequestBody @Valid QueryConfigDto queryConfigDto ) {
        if(queryConfigDto.getTemplateFieldId()==null ||queryConfigDto.getTemplateFieldId()==0){

            return  ResultData.ok();
        }

        ContractConfig contractConfig = new ContractConfig();
        contractConfig.setTenantId(queryConfigDto.getTenantId());
        contractConfig.setTemplateFieldId(queryConfigDto.getTemplateFieldId());
        return ResultData.ok(contractConfigService.paraReplace(queryConfigDto.getProcessId(),
                contractConfig,queryConfigDto.getTfProjectBaseW()));
    }
    /**
     * 单一字段值获取
     */
    @PostMapping("/getFieldTarget" )
    @ApiOperation(value = "获取单个字段自动计算")
    public ResultData getFieldTarget(@RequestBody @Valid QueryConfigDto queryConfigDto ) {



            return ResultData.ok(contractConfigService.singleData( queryConfigDto));
    }


    /**
     * 通过id查询合同计算配置信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(contractConfigService.getById(id));
    }

    /**
     * 新增合同计算配置信息
     * @param contractConfig 合同计算配置信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增合同计算配置信息" )
    @ApiOperation(value = "新增合同计算配置信息")
    @PostMapping
    public ResultData save(@RequestBody ContractConfig contractConfig) {
      return ResultData.ok(contractConfigService.save(contractConfig));
    }

    /**
     * 修改合同计算配置信息
     * @param contractConfig 合同计算配置信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改合同计算配置信息" )
    @ApiOperation(value = "修改合同计算配置信息")
    @PutMapping
    public ResultData updateById(@RequestBody ContractConfig contractConfig) {
      return ResultData.ok(contractConfigService.updateById(contractConfig));
    }

    /**
     * 通过id删除合同计算配置信息
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除合同计算配置信息" )
    @ApiOperation(value = "通过ID删除合同计算配置信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(contractConfigService.removeById(id));
    }

    /**
     * 通过id批量删除合同计算配置信息
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除合同计算配置信息" )
    @ApiOperation(value = "通过ID批量删除合同计算配置信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        final String[] idArr  = ids.split(",");
        for (String id : idArr) {
            List<Long> existIds = contractConfigFieldService.list(Wrappers.<ContractConfigField>lambdaQuery()
                    .select(ContractConfigField::getId)
                    .eq(ContractConfigField::getContractConfigId, Long.valueOf(id)))
                    .stream().map(ContractConfigField::getId).collect(Collectors.toList());

            contractConfigFieldService.removeByIds(existIds);
        }


        return ResultData.ok(contractConfigService.removeByIds(Arrays.asList(ids.split(","))));
    }


    @GetMapping("/getDetailById" )
    @ApiOperation(value = "获取合同配置详细信息")
    public ResultData getDetailByContractConfigId(Long id){

        final ContractConfig byId = contractConfigService.getById(id);

        final List<ContractConfigField> list = contractConfigFieldService.list(Wrappers.<ContractConfigField>lambdaQuery()
                .eq(ContractConfigField::getContractConfigId, id));

        ContractConfigFieldBatchVo vo = new ContractConfigFieldBatchVo();
        vo.setContractConfig(byId);
        vo.setDatas(list);

        return  ResultData.ok(vo);
    }


}
