/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.ContractConfig;
import com.tofly.workflow.baseconf.entity.ContractConfigField;
import com.tofly.workflow.baseconf.entity.TfContractFieldP;
import com.tofly.workflow.baseconf.entity.TfTemplateFieldP;
import com.tofly.workflow.baseconf.entity.dto.ContractConfigFieldBatchDto;
import com.tofly.workflow.baseconf.service.*;
import com.tofly.workflow.buss.entity.vo.ContractConfigFieldVo;
import com.tofly.workflow.buss.entity.vo.ContractConfigTableVo;
import com.tofly.workflow.utils.compute.ScanFieldUtil;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 合同字段取值配置表
 *
 * @author WANGZHUN
 * @date Tue Apr 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/contractconfigfield")
@Api(tags="合同字段取值配置表接口")
public class ContractConfigFieldController {

    private final  ContractConfigFieldService contractConfigFieldService;

    private final ContractConfigService contractConfigService;

    private final TfContractFieldPService tfContractFieldPService;
    private final TfTemplateFieldPService templateFieldPService;
    private final TfContractTemplatePService tfContractTemplatePService;
    /**
    * 分页查询
    * @param page 分页对象
    * @param contractConfigField 合同字段取值配置表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getContractConfigFieldPage(Page page, @ModelAttribute ContractConfigField contractConfigField) {
      return ResultData.ok(contractConfigFieldService.page(page, Wrappers.query(contractConfigField)));
    }




    /**
     * 通过id查询合同字段取值配置表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(contractConfigFieldService.getById(id));
    }



    @ToFlyAppLog(title="批量新增或者修改合同字段取值配置表" )
    @ApiOperation(value = "批量新增或者修改合同字段取值配置表")
    @PostMapping("/batchSave")
    public ResultData batchSave(@RequestBody ContractConfigFieldBatchDto contractConfigField) {

        final ContractConfig contractConfig = contractConfigField.getContractConfig();

        contractConfigService.saveOrUpdate(contractConfig);
        //查询该配置 是否存在字段配置

         List<Long> existIds = contractConfigFieldService.list(Wrappers.<ContractConfigField>lambdaQuery()
                .select(ContractConfigField::getId)
                .eq(ContractConfigField::getContractConfigId, contractConfig.getId()))
                .stream().map(ContractConfigField::getId).collect(Collectors.toList());


        final List<ContractConfigField> datas = contractConfigField.getDatas();
        List<ContractConfigField> datasNew = new ArrayList<>();
        for (ContractConfigField data : datas) {
            data.setContractConfigId(contractConfig.getId());
            datasNew.add(data);
            if(data.getId()!=null && data.getId() !=0L){
                existIds.remove(data.getId());
            }
        }
        //删除不存在的数据
        contractConfigFieldService.removeByIds(existIds);

        return ResultData.ok(contractConfigFieldService.saveOrUpdateBatch(datasNew));
    }

    /**
     * 修改合同字段取值配置表
     * @param contractConfigField 合同字段取值配置表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改合同字段取值配置表" )
    @ApiOperation(value = "修改合同字段取值配置表")
    @PutMapping
    public ResultData updateById(@RequestBody ContractConfigField contractConfigField) {
      return ResultData.ok(contractConfigFieldService.updateById(contractConfigField));
    }

    /**
     * 通过id删除合同字段取值配置表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除合同字段取值配置表" )
    @ApiOperation(value = "通过ID删除合同字段取值配置表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(contractConfigFieldService.removeById(id));
    }

    /**
     * 通过id批量删除合同字段取值配置表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除合同字段取值配置表" )
    @ApiOperation(value = "通过ID批量删除合同字段取值配置表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(contractConfigFieldService.removeByIds(Arrays.asList(ids.split(","))));
    }
    @GetMapping("/getTableAndFields" )
    @ApiOperation(value = "获取所有的业务数据属性")
    public  ResultData getTableAndFields( Long templateId){
        Assert.notNull(templateId,"请传入使用的模板id");
        final Map<String, ContractConfigTableVo> stringContractConfigTableVoMap
                = ScanFieldUtil.scanTableAndField("");
        //查找合同模板配置的对应字段
        List<TfTemplateFieldP> templateFieldList = templateFieldPService.list(new QueryWrapper<TfTemplateFieldP>()
                .eq("TEMPLATE_ID",templateId));
        List<Long> idList = templateFieldList.stream().map(TfTemplateFieldP::getFieldId).collect(Collectors.toList());
        if(idList.size() >0){
            List<TfContractFieldP> contractFieldList = tfContractFieldPService.listByIds(idList);
            ContractConfigTableVo contractConfigTableVo = new ContractConfigTableVo();
            contractConfigTableVo.setTableName("合同配置模板");
            contractConfigTableVo.setNote("合同配置模板");
           final   List<ContractConfigFieldVo> fields = new ArrayList<>();
            contractFieldList.forEach(field->{
                ContractConfigFieldVo contractConfigFieldVo = new ContractConfigFieldVo();
                contractConfigFieldVo.setFieldName(field.getFieldName());
                contractConfigFieldVo.setFieldText(field.getName());
                fields.add(contractConfigFieldVo);
            });
            contractConfigTableVo.setFields(fields);
            stringContractConfigTableVoMap.put("合同配置模板",contractConfigTableVo);

        }


        return  ResultData.ok(stringContractConfigTableVoMap);

    }
}
