/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfBaseTypeP;
import com.tofly.workflow.baseconf.entity.dto.BaseTypeDto;
import com.tofly.workflow.baseconf.entity.dto.UpdateBaseTypeDto;
import com.tofly.workflow.baseconf.service.TfBaseTypePService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.wzyy.entity.YyOrgUnit;
import com.tofly.workflow.wzyy.service.YyOrgUnitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 基础类型表
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfbasetypep")
@Api(tags="基础类型表接口")
public class TfBaseTypePController extends BaseController {

    private final  TfBaseTypePService tfBaseTypePService;
    private final YyOrgUnitService yyOrgUnitService;
    /**
    * 分页查询
    * @param page 分页对象
    * @param baseType 基础类型表
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getBaseTypePage(Page page, @ModelAttribute TfBaseTypeP baseType) {
        //查询材料类型
        if(1570692286170427394l==baseType.getDataType()) {

            final Long tenantId = SecurityUtils.getTenantId();
            final YyOrgUnit one = yyOrgUnitService.getOne(Wrappers.<YyOrgUnit>lambdaQuery()
                    .eq(YyOrgUnit::getTenantId, tenantId));
            if(1569602913205559298l==SecurityUtils.getTenantId() ){
                return ResultData.ok(tfBaseTypePService.page(page, Wrappers.query(baseType).lambda()
                        .eq(TfBaseTypeP::getOrgCode,one.getOrgCode())
                        .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO).orderByAsc(TfBaseTypeP::getId)));

            }else{
                return ResultData.ok(tfBaseTypePService.page(page, Wrappers.query(baseType).lambda()
                        .and(i -> i.eq(TfBaseTypeP::getOrgCode,"global00").or(j -> j.eq(TfBaseTypeP::getOrgCode,one.getOrgCode())))
                        .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO).orderByAsc(TfBaseTypeP::getId)));

            }

        }
        return ResultData.ok(tfBaseTypePService.page(page, Wrappers.query(baseType).lambda()
                .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO).orderByAsc(TfBaseTypeP::getId)));
    }


    /**
     * 通过id查询基础类型表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
        return ResultData.ok(tfBaseTypePService.getById(id));
    }

    /**
     * 新增基础类型表
     * @param dto 基础类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增基础类型表" )
    @ApiOperation(value = "新增基础类型表")
    @PostMapping
    public ResultData save(@RequestBody @Valid BaseTypeDto dto) {
        if (dto.getForce()==null || !dto.getForce()){
            if (tfBaseTypePService.count(Wrappers.<TfBaseTypeP>lambdaQuery()
                    .eq(TfBaseTypeP::getTypeCode,dto.getTypeCode())
                    .eq(TfBaseTypeP::getDataType,dto.getDataType())
                    .eq(TfBaseTypeP::getIsDel,FlowConstant.SHORT_ZERO))!=0){
                return ResultData.fail("代码存在重复，无法保存");
            }
        }
        Object obj = tfBaseTypePService.saveBaseType(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultData.ok(FlowConstant.SUCCESS);
            }else {
                return ResultData.fail(FlowConstant.FAILED);
            }
        }else {
            return ResultData.fail(String.valueOf(obj));
        }
    }

    /**
     * 修改基础类型表
     * @param dto 基础类型表
     * @return ResultData
     */
    @ToFlyAppLog(title="修改基础类型表" )
    @ApiOperation(value = "修改基础类型表")
    @PutMapping
    public ResultData updateById(@RequestBody @Valid UpdateBaseTypeDto dto) {
        if (dto.getForce()==null || !dto.getForce()){
            List<TfBaseTypeP> list = tfBaseTypePService.list(Wrappers.<TfBaseTypeP>lambdaQuery()
                    .eq(TfBaseTypeP::getTypeCode, dto.getTypeCode()).eq(TfBaseTypeP::getIsDel,FlowConstant.SHORT_ZERO));
            if (!CollectionUtils.isEmpty(list)){
                for (TfBaseTypeP baseType : list) {
                    if (!baseType.getId().equals(dto.getId())){
                        return ResultData.ok("代码存在重复，是否保存");
                    }
                }
            }
        }
        Object obj = tfBaseTypePService.updateBaseType(dto);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultData.ok();
            }else {
                return ResultData.fail();
            }
        }else {
            return ResultData.fail(String.valueOf(obj));
        }
    }


    /**
     * 有效无效
     * @param id 主键id
     * @param valid 有效无效值 0无效，1有效
     * @return
     */
    @ToFlyAppLog(title="有效无效" )
    @ApiOperation(value = "有效无效")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "valid", value = "有效无效,0无效，1有效", required = true, dataType = "Long")
    })
    @GetMapping("/valid")
    public ResultData isValid(Long id,Short valid) {
        Object obj = tfBaseTypePService.updateValid(id,valid);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultData.ok();
            }else {
                return ResultData.fail();
            }
        }else {
            return ResultData.fail(String.valueOf(obj));
        }
    }


    /**
     * 通过id删除基础类型表,有关联数据验证
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除基础类型表,有关联数据验证" )
    @ApiOperation(value = "通过ID删除基础类型表,有关联数据验证")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
        if(tfBaseTypePService.removeUpdateById(id)){
            return ResultData.ok();
        }else {
            return ResultData.fail("该分类下有物资信息无法删除");
        }
    }

    /**
     * 通过id批量删除基础类型表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除基础类型表" )
    @ApiOperation(value = "通过ID批量删除基础类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        Object obj = tfBaseTypePService.removeUpdateByIds(ids);
        if(obj instanceof Boolean){
            if((boolean) obj){
                return ResultData.ok();
            }else {
                return ResultData.fail();
            }
        }else {
            return ResultData.fail(String.valueOf(obj));
        }
    }

    @ToFlyAppLog(title="获取类型树" )
    @ApiOperation(value = "获取类型树")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "datatypes", value = "区分类别，多个id以逗号隔开", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "0子级；1类别下类型；默认0",defaultValue = "0", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "id", value = "类型id",dataType = "Long")
    })
    @GetMapping("/typetree" )
    public ResultData getTypeTree(String datatypes,Long id,Integer type){
        List<String> orgCodes = new ArrayList<>();
        //贵州贞丰 和绵竹 不适用公共材料类型
        final Long tenantId = SecurityUtils.getTenantId();
        final String[] split = datatypes.split(",");
        if( Arrays.asList(split).contains(String.valueOf(1570692286170427394l))){
            if(1569602913205559298l ==tenantId  ){
                orgCodes.add(String.valueOf(tenantId));
            }else{

                orgCodes.add("global00");
            }

        }



        Object obj = tfBaseTypePService.getTypeTree(datatypes,id,type,orgCodes);

        if(obj instanceof String){
            return ResultData.fail(String.valueOf(obj),FlowConstant.FAILED);

        }else {
            return ResultData.ok((List)obj);
        }
    }

}
