/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfBuildUnitP;
import com.tofly.workflow.baseconf.entity.TfConstructionTeamP;
import com.tofly.workflow.baseconf.entity.TfTeamStaffP;
import com.tofly.workflow.baseconf.entity.dto.TfBuildUnitPDto;
import com.tofly.workflow.baseconf.entity.vo.TfBuildUnitPVo;
import com.tofly.workflow.baseconf.service.TfBuildUnitPService;
import com.tofly.workflow.baseconf.service.TfConstructionTeamPService;
import com.tofly.workflow.baseconf.service.TfTeamStaffPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 施工组织单位
 *
 * @author JF
 * @date Tue Aug 09 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfbuildunitp")
@Api(tags="施工组织单位接口")
public class TfBuildUnitPController {

    private final  TfBuildUnitPService tfBuildUnitPService;
    private final TfConstructionTeamPService constructionTeamPService;
    private final TfTeamStaffPService teamStaffPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param buildUnitPDto 施工组织单位Dto
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfBuildUnitPPage(Page page, @ModelAttribute TfBuildUnitPDto buildUnitPDto) {
      return ResultData.ok(tfBuildUnitPService.getTfBuildUnitPPage(page, buildUnitPDto));
    }


    /**
     * 通过id查询施工组织单位
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id查询施工组织单位" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过id查询施工组织单位")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
        TfBuildUnitP buildUnitP = tfBuildUnitPService.getById(id);
        TfBuildUnitPVo vo = new TfBuildUnitPVo();
        BeanUtils.copyProperties(buildUnitP, vo);
        List<TfConstructionTeamP> constructionTeamPList = constructionTeamPService.getConstructionTeamList(buildUnitP.getId());
        vo.setConstructionTeamPList(constructionTeamPList);
      return ResultData.ok(vo);
    }

    /**
     * 新增施工班组相关信息
     * @param buildUnitPDto 施工班组信息Dto
     * @return ResultData
     */
    @ToFlyAppLog(title="新增施工班组相关信息" )
    @ApiOperation(value = "新增施工班组相关信息")
    @PostMapping("saveTeamAll")
    public ResultData saveTeamAll(@RequestBody @Valid TfBuildUnitPDto buildUnitPDto) {
        tfBuildUnitPService.saveTeamAll(buildUnitPDto);
        return ResultData.ok();
    }

    /**
     * 新增施工组织单位
     * @param tfBuildUnitP 施工组织单位
     * @return ResultData
     */
    @ToFlyAppLog(title="新增施工组织单位" )
    @ApiOperation(value = "新增施工组织单位")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfBuildUnitP tfBuildUnitP) {
      return ResultData.ok(tfBuildUnitPService.save(tfBuildUnitP));
    }

    /**
     * 修改施工组织单位
     * @param tfBuildUnitP 施工组织单位
     * @return ResultData
     */
    @ToFlyAppLog(title="修改施工组织单位" )
    @ApiOperation(value = "修改施工组织单位")
    @PutMapping
    public ResultData updateById(@RequestBody TfBuildUnitP tfBuildUnitP) {
      return ResultData.ok(tfBuildUnitPService.updateById(tfBuildUnitP));
    }

    /**
     * 通过id删除施工组织单位
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除施工组织单位" )
    @ApiOperation(value = "通过ID删除施工组织单位")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfBuildUnitPService.removeById(id));
    }

    /**
     * 通过id批量删除施工组织单位
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除施工组织单位" )
    @ApiOperation(value = "通过ID批量删除施工组织单位")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids为空");
        }
        List<Long> idList = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        tfBuildUnitPService.removeByIds(idList);
        idList.forEach(info -> {
            List<TfConstructionTeamP> constructionTeamList = constructionTeamPService.list(new QueryWrapper<TfConstructionTeamP>().eq("BUILD_UNIT_ID",info));
            for(TfConstructionTeamP constructionTeamP : constructionTeamList){
                teamStaffPService.remove(new QueryWrapper<TfTeamStaffP>().eq("TEAM_ID",constructionTeamP.getId()));
            }
            constructionTeamPService.remove(new QueryWrapper<TfConstructionTeamP>().eq("BUILD_UNIT_ID",info));
        });
        return ResultData.ok();
    }
}
