/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfCalendarP;
import com.tofly.workflow.baseconf.service.TfCalendarPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 工作日应用日历（假期）
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfcalendarp")
@Api(tags="工作日应用日历（假期）接口")
public class TfCalendarPController extends BaseController {

    private final  TfCalendarPService tfCalendarPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfCalendarP 工作日应用日历（假期）
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfCalendarPPage(Page page, @ModelAttribute TfCalendarP tfCalendarP) {
      return ResultData.ok(tfCalendarPService.page(page, Wrappers.query(tfCalendarP)));
    }


    /**
     * 通过id查询工作日应用日历（假期）
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfCalendarPService.getById(id));
    }

    /**
     * 新增工作日应用日历（假期）
     * @param tfCalendarP 工作日应用日历（假期）
     * @return ResultData
     */
    @ToFlyAppLog(title="新增工作日应用日历（假期）" )
    @ApiOperation(value = "新增工作日应用日历（假期）")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfCalendarP tfCalendarP) {
      return ResultData.ok(tfCalendarPService.save(tfCalendarP));
    }

    /**
     * 修改工作日应用日历（假期）
     * @param tfCalendarP 工作日应用日历（假期）
     * @return ResultData
     */
    @ToFlyAppLog(title="修改工作日应用日历（假期）" )
    @ApiOperation(value = "修改工作日应用日历（假期）")
    @PutMapping
    public ResultData updateById(@RequestBody TfCalendarP tfCalendarP) {
      return ResultData.ok(tfCalendarPService.updateById(tfCalendarP));
    }

    /**
     * 通过id删除工作日应用日历（假期）
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除工作日应用日历（假期）" )
    @ApiOperation(value = "通过ID删除工作日应用日历（假期）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfCalendarPService.removeById(id));
    }

    /**
     * 通过id批量删除工作日应用日历（假期）
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除工作日应用日历（假期）" )
    @ApiOperation(value = "通过ID批量删除工作日应用日历（假期）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfCalendarPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
