/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfCalendarP;
import com.tofly.workflow.baseconf.entity.TfCalendarYearP;
import com.tofly.workflow.baseconf.entity.dto.CalendarYearPDto;
import com.tofly.workflow.baseconf.service.TfCalendarPService;
import com.tofly.workflow.baseconf.service.TfCalendarYearPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 日历年
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfcalendaryearp")
@Api(tags="日历年接口")
public class TfCalendarYearPController extends BaseController {

    private final  TfCalendarYearPService tfCalendarYearPService;
    private final TfCalendarPService tfCalendarPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfCalendarYearP 日历年
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfCalendarYearPPage(Page page, @ModelAttribute TfCalendarYearP tfCalendarYearP) {
        Page<TfCalendarYearP> contractTemplatePPage = tfCalendarYearPService.page(page, Wrappers.query(tfCalendarYearP));
        for(TfCalendarYearP one : contractTemplatePPage.getRecords()){
            one.setCalendarPList(tfCalendarPService.list(new QueryWrapper<TfCalendarP>().eq("YEAR_ID",one.getId())));
        }
      return ResultData.ok(contractTemplatePPage);
    }


    /**
     * 通过id查询日历年
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfCalendarYearPService.getById(id));
    }

    /**
     * 新增日历年日期相关信息
     * @param calendarYearPDto 日历年Dto
     * @return ResultData
     */
    @ToFlyAppLog(title="新增日历年日期相关信息" )
    @ApiOperation(value = "新增日历年日期相关信息")
    @PostMapping("saveCalendarAll")
    public ResultData saveCalendarAll(@RequestBody @Valid CalendarYearPDto calendarYearPDto) {
        Object obj = tfCalendarYearPService.saveCalendarAll(calendarYearPDto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 新增日历年
     * @param tfCalendarYearP 日历年
     * @return ResultData
     */
    @ToFlyAppLog(title="新增日历年" )
    @ApiOperation(value = "新增日历年")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfCalendarYearP tfCalendarYearP) {
        Object obj = tfCalendarYearPService.saveCalendarYear(tfCalendarYearP);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 修改日历年
     * @param tfCalendarYearP 日历年
     * @return ResultData
     */
    @ToFlyAppLog(title="修改日历年" )
    @ApiOperation(value = "修改日历年")
    @PutMapping
    public ResultData updateById(@RequestBody TfCalendarYearP tfCalendarYearP) {
        Object obj = tfCalendarYearPService.updateCalendarYear(tfCalendarYearP);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除日历年
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除日历年" )
    @ApiOperation(value = "通过ID删除日历年")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfCalendarYearPService.removeById(id));
    }

    /**
     * 通过id批量删除日历年
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除日历年" )
    @ApiOperation(value = "通过ID批量删除日历年")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids为空");
        }
        List<Long> idList = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        tfCalendarYearPService.removeByIds(idList);
        idList.forEach(info -> tfCalendarPService.remove(new QueryWrapper<TfCalendarP>().eq("YEAR_ID",info)));
        return ResultData.ok();
    }
}
