/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfConstructionTeamP;
import com.tofly.workflow.baseconf.service.TfConstructionTeamPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 施工班组
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfconstructionteamp")
@Api(tags="施工班组接口")
public class TfConstructionTeamPController extends BaseController {

    private final  TfConstructionTeamPService tfConstructionTeamPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfConstructionTeamP 施工班组
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfConstructionTeamPPage(Page page, @ModelAttribute TfConstructionTeamP tfConstructionTeamP) {
      return ResultData.ok(tfConstructionTeamPService.page(page, Wrappers.query(tfConstructionTeamP)));
    }


    /**
     * 通过id查询施工班组
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfConstructionTeamPService.getById(id));
    }

    /**
     * 新增施工班组
     * @param tfConstructionTeamP 施工班组
     * @return ResultData
     */
    @ToFlyAppLog(title="新增施工班组" )
    @ApiOperation(value = "新增施工班组")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfConstructionTeamP tfConstructionTeamP) {
      return ResultData.ok(tfConstructionTeamPService.save(tfConstructionTeamP));
    }

    /**
     * 修改施工班组
     * @param tfConstructionTeamP 施工班组
     * @return ResultData
     */
    @ToFlyAppLog(title="修改施工班组" )
    @ApiOperation(value = "修改施工班组")
    @PutMapping
    public ResultData updateById(@RequestBody TfConstructionTeamP tfConstructionTeamP) {
      return ResultData.ok(tfConstructionTeamPService.updateById(tfConstructionTeamP));
    }

    /**
     * 通过id删除施工班组
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除施工班组" )
    @ApiOperation(value = "通过ID删除施工班组")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfConstructionTeamPService.removeById(id));
    }

    /**
     * 通过id批量删除施工班组
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除施工班组" )
    @ApiOperation(value = "通过ID批量删除施工班组")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfConstructionTeamPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
