/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfContractFieldP;
import com.tofly.workflow.baseconf.entity.dto.ContractFieldDto;
import com.tofly.workflow.baseconf.service.TfContractFieldPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 合同模板公用字段
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfcontractfieldp")
@Api(tags="合同模板公用字段接口")
public class TfContractFieldPController extends BaseController {

    private final  TfContractFieldPService tfContractFieldPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 合同模板公用字段
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfContractFieldPPage(Page page, @ModelAttribute ContractFieldDto dto) {
        TfContractFieldP cf = new TfContractFieldP();
        BeanUtils.copyProperties(dto, cf);
        if(StringUtils.isNotEmpty(dto.getName())){
            cf.setName(null);
        }
      return ResultData.ok(tfContractFieldPService.page(page, Wrappers.query(cf).lambda()
              .and(StringUtils.isNotEmpty(dto.getName()),r->r.like(TfContractFieldP::getName,dto.getName()).or().like(TfContractFieldP::getFieldName,dto.getName()))));
    }


    /**
     * 通过id查询合同模板公用字段
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfContractFieldPService.getById(id));
    }

    /**
     * 新增合同模板公用字段
     * @param tfContractFieldP 合同模板公用字段
     * @return ResultData
     */
    @ToFlyAppLog(title="新增合同模板公用字段" )
    @ApiOperation(value = "新增合同模板公用字段")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfContractFieldP tfContractFieldP) {
        Object obj = tfContractFieldPService.saveContractField(tfContractFieldP);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 修改合同模板公用字段
     * @param tfContractFieldP 合同模板公用字段
     * @return ResultData
     */
    @ToFlyAppLog(title="修改合同模板公用字段" )
    @ApiOperation(value = "修改合同模板公用字段")
    @PutMapping
    public ResultData updateById(@RequestBody TfContractFieldP tfContractFieldP) {
        Object obj = tfContractFieldPService.updateContractField(tfContractFieldP);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除合同模板公用字段
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除合同模板公用字段" )
    @ApiOperation(value = "通过ID删除合同模板公用字段")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfContractFieldPService.removeById(id));
    }

    /**
     * 通过id批量删除合同模板公用字段
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除合同模板公用字段" )
    @ApiOperation(value = "通过ID批量删除合同模板公用字段")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfContractFieldPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
