/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.auth.ToflyUser;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.common.utils.POIUtil;
import com.tofly.workflow.baseconf.entity.TfContractFieldP;
import com.tofly.workflow.baseconf.entity.TfContractTemplateP;
import com.tofly.workflow.baseconf.entity.TfTemplateFieldP;
import com.tofly.workflow.baseconf.entity.dto.TfContractTemplatePDto;
import com.tofly.workflow.baseconf.service.TfContractFieldPService;
import com.tofly.workflow.baseconf.service.TfContractTemplatePService;
import com.tofly.workflow.baseconf.service.TfTemplateFieldPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 合同模板
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfcontracttemplatep")
@Api(tags="合同模板接口")
public class TfContractTemplatePController extends BaseController {

    private final  TfContractTemplatePService tfContractTemplatePService;
    private final TfTemplateFieldPService templateFieldPService;
    private final TfContractFieldPService tfContractFieldPService;
    /**
    * 分页查询
    * @param page 分页对象
    * @param tfContractTemplateP 合同模板
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfContractTemplatePPage(Page page, @ModelAttribute TfContractTemplateP tfContractTemplateP) {
        //添加默认租户id
        final Long tenantId = SecurityUtils.getTenantId();

        if(tenantId!=1){
            tfContractTemplateP.setTenantId(tenantId);
        }

        Page<TfContractTemplateP> contractTemplatePPage = tfContractTemplatePService.page(page, Wrappers.query(tfContractTemplateP));
        for(TfContractTemplateP contractTemplateP : contractTemplatePPage.getRecords()){
            List<TfTemplateFieldP> templateFieldList = templateFieldPService.list(new QueryWrapper<TfTemplateFieldP>()
                    .eq("TEMPLATE_ID",contractTemplateP.getId()));
            List<Long> idList = templateFieldList.stream().map(TfTemplateFieldP::getFieldId).collect(Collectors.toList());
            if(idList.size() == 0){
                continue;
            }
            List<TfContractFieldP> contractFieldList = tfContractFieldPService.listByIds(idList);
            for(TfContractFieldP contractField : contractFieldList){
                TfTemplateFieldP templateField = templateFieldPService.getOne(new QueryWrapper<TfTemplateFieldP>()
                        .eq("TEMPLATE_ID",contractTemplateP.getId())
                        .eq("FIELD_ID",contractField.getId()));
                contractField.setTemplateFieldId(templateField.getId());
                contractField.setSort(templateField.getSort());
            }
            contractTemplateP.setContractFieldList(contractFieldList);
        }
        return ResultData.ok(contractTemplatePPage);
    }

    /**
     * 通过id查询合同模板
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfContractTemplatePService.getById(id));
    }

    /**
     * 获取 Docx 中的 ${xxx} 中的内容xxx
     * @param
     * @return ResultData
     */
    @ToFlyAppLog(title="获取 Docx 中的 ${xxx} 中的内容xxx" )
    @PostMapping("/getDocFilterData" )
    @ApiOperation(value = "获取 Docx 中的 ${xxx} 中的内容xxx")
    public ResultData getDocFilterData(@RequestPart(value = "docx") MultipartFile docx) {
        try {
            return ResultData.ok(POIUtil.getDocFilterData(docx).stream().distinct());
        } catch (Exception e) {
            e.printStackTrace();
            return ResultData.fail(e.getMessage());
        }
    }

    /**
     * 新增合同模板
     * @param contractTemplatePDto 合同模板Dto
     * @return ResultData
     */
    @ToFlyAppLog(title="新增合同模板" )
    @ApiOperation(value = "新增合同模板")
    @PostMapping("/saveContractTemplateField")
    public ResultData saveContractTemplateField(@RequestPart @Valid TfContractTemplatePDto contractTemplatePDto,
                           @RequestPart(value = "contractAnnexList", required = false) List<MultipartFile> contractAnnexList,
                           @RequestPart(value = "annexList", required = false) List<MultipartFile> annexList) {
        tfContractTemplatePService.saveContractTemplateField(contractAnnexList, annexList, contractTemplatePDto);
        return ResultData.ok();
    }

        /**
     * 修改合同模板
     * @param contractTemplatePDto 修改合同模板
     * @return ResultRespone
     */
    @ApiOperation(value = "修改合同模板")
    @PutMapping("/updateContractTemplate")
    @ToFlyAppLog(title = "修改合同模板")
    public ResultData update(@RequestPart TfContractTemplatePDto contractTemplatePDto,
                                                             @RequestPart(value = "contractAnnexList", required = false) List<MultipartFile> contractAnnexList,
                                                             @RequestPart(value = "annexList", required = false) List<MultipartFile> annexList) {
        tfContractTemplatePService.updateContractTemplate(contractAnnexList, annexList, contractTemplatePDto);
        return ResultData.ok();

    }

    /**
     * 通过id批量删除合同模板
     * @param ids id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过id批量删除合同模板")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    @ToFlyAppLog(title = "通过id批量删除合同模板")
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids为空");
        }
        List<Long> removeIds = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        tfContractTemplatePService.deleteTfContractTemplateP(removeIds);
        return ResultData.ok();
    }

}
