/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfMatterAccessP;
import com.tofly.workflow.baseconf.entity.TfMatterNumberP;
import com.tofly.workflow.baseconf.service.TfMatterAccessPService;
import com.tofly.workflow.baseconf.service.TfMatterNumberPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 物资出入库
 *
 * @author JF
 * @date Wed Aug 10 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfmatteraccessp")
@Api(tags="物资出入库接口")
public class TfMatterAccessPController {

    private final  TfMatterAccessPService tfMatterAccessPService;
    private final TfMatterNumberPService tfMatterNumberPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfMatterAccessP 物资出入库
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfMatterAccessPPage(Page page, @ModelAttribute TfMatterAccessP tfMatterAccessP) {
        Page<TfMatterAccessP> matterAccessPPage = tfMatterAccessPService.page(page, Wrappers.query(tfMatterAccessP));
        for(TfMatterAccessP one : matterAccessPPage.getRecords()){
            one.setMatterNumberP(tfMatterNumberPService.getOne(new QueryWrapper<TfMatterNumberP>()
                    .eq("MATTER_ID",one.getId())
                    .eq("IS_VALID",1)));
        }
        return ResultData.ok(matterAccessPPage);
    }


    /**
     * 通过id查询物资出入库
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
        TfMatterAccessP matterAccessP = tfMatterAccessPService.getById(id);
        matterAccessP.setMatterNumberP(tfMatterNumberPService.getOne(new QueryWrapper<TfMatterNumberP>()
                .eq("MATTER_ID",id)
                .eq("IS_VALID",1)));
      return ResultData.ok(matterAccessP);
    }

    /**
     * 新增物资出入库
     * @param tfMatterAccessP 物资出入库
     * @return ResultData
     */
    @ToFlyAppLog(title="新增物资出入库" )
    @ApiOperation(value = "新增物资出入库")
    @PostMapping
    public ResultData save(@RequestPart @Valid TfMatterAccessP tfMatterAccessP, @RequestPart(value = "annexList", required = false) List<MultipartFile> annexList) {
        tfMatterAccessPService.saveTfMatterAccessP(tfMatterAccessP, annexList);
        return ResultData.ok();
    }

    /**
     * 修改物资出入库
     * @param tfMatterAccessP 物资出入库
     * @return ResultData
     */
    @ToFlyAppLog(title="修改物资出入库" )
    @ApiOperation(value = "修改物资出入库")
    @PutMapping
    public ResultData updateById(@RequestPart TfMatterAccessP tfMatterAccessP, @RequestPart(value = "annexList", required = false) List<MultipartFile> annexList) {
        tfMatterAccessPService.updateTfMatterAccessP(tfMatterAccessP, annexList);
        return ResultData.ok();
    }

    /**
     * 通过id删除物资出入库
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除物资出入库" )
    @ApiOperation(value = "通过ID删除物资出入库")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfMatterAccessPService.removeById(id));
    }

    /**
     * 通过id批量删除物资出入库
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除物资出入库" )
    @ApiOperation(value = "通过ID批量删除物资出入库")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        List<Long> removeId = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        tfMatterAccessPService.deleteTfMatterAccessP(removeId);
        return ResultData.ok();
//        return ResultData.ok(tfMatterAccessPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
