/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfBaseTypeP;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.entity.TfMatterPriceP;
import com.tofly.workflow.baseconf.entity.dto.MatterDto;
import com.tofly.workflow.baseconf.entity.dto.TfMatterPDto;
import com.tofly.workflow.baseconf.service.TfBaseTypePService;
import com.tofly.workflow.baseconf.service.TfMatterNumberPService;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.baseconf.service.TfMatterPricePService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.wzyy.entity.MatterOrg;
import com.tofly.workflow.wzyy.service.MatterOrgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 物资信息表
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfmatterp")
@Api(tags="物资信息表接口")
public class TfMatterPController extends BaseController {

    private final  TfMatterPService tfMatterPService;
    private final TfMatterPricePService tfMatterPricePService;
    private  final MatterOrgService matterOrgService;
    private final TfBaseTypePService tfBaseTypePService;
    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 物资信息表
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfMatterPPage(Page page, @ModelAttribute MatterDto dto) {
        TfMatterP tfMatterP = new TfMatterP();
        //查询当前的材料id
        //绵竹特殊处理   绵竹 贞丰 不看通用
        List<Long> collect=new ArrayList<>();
        if(1569602913205559298l==SecurityUtils.getTenantId() ){
            List<TfBaseTypeP> types = tfBaseTypePService.list(Wrappers.<TfBaseTypeP>lambdaQuery()
                    .eq(TfBaseTypeP::getOrgCode, String.valueOf(SecurityUtils.getTenantId()))
                    .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO));
              collect =
                    types.stream().map(TfBaseTypeP::getId).collect(Collectors.toList());

        }else{
            List<TfBaseTypeP> types = tfBaseTypePService.list(Wrappers.<TfBaseTypeP>lambdaQuery()
                    .and(i -> i.eq(TfBaseTypeP::getOrgCode, "global00").or(j -> j.eq(TfBaseTypeP::getOrgCode, String.valueOf(SecurityUtils.getTenantId()))))
                    .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO));
            collect =
                    types.stream().map(TfBaseTypeP::getId).collect(Collectors.toList());

        }


      //  String insql =    "select yy_id from TF_MATTER_ORG_P where org_code= '"+SecurityUtils.getTenantId()+"'";
        BeanUtils.copyProperties(dto,tfMatterP);
        Page<TfMatterP> matterPPage = tfMatterPService.page(page, Wrappers.query(tfMatterP).lambda().and(StringUtils.isNotEmpty(dto.getName())
                ,r->r.like(TfMatterP::getMatterName,dto.getName()).or().like(TfMatterP::getStandards,dto.getName())).in(TfMatterP::getMatterType,collect)
                );


        for(TfMatterP one : matterPPage.getRecords()){
            one.setMatterPrice(tfMatterPricePService.getOne(new QueryWrapper<TfMatterPriceP>()
                    .eq("MATTER_ID",one.getId())
                    .eq("TENANT_ID", SecurityUtils.getTenantId())
                    .eq("IS_VALID",1)));
        }
        return ResultData.ok(matterPPage);
    }

    /**
     * 列表查询
     * @param tfMatterP 列表查询
     * @return
     */
    @ToFlyAppLog(title="列表查询" )
    @GetMapping("/list" )
    @ApiOperation(value = "列表查询")
    public ResultData getTfMatterPList(@ModelAttribute TfMatterP tfMatterP) {
        return ResultData.ok(tfMatterPService.getTfMatterPList(tfMatterP));
    }


    /**
     * 通过id查询物资信息表
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
        TfMatterP matterP = tfMatterPService.getById(id);
        matterP.setMatterPrice(tfMatterPricePService.getOne(new QueryWrapper<TfMatterPriceP>()
              .eq("MATTER_ID",id).eq("TENANT_ID",SecurityUtils.getTenantId())
              .eq("IS_VALID",1)));
        return ResultData.ok();
    }


    /**
     * 通过ID集合查询物料
     * @param strs strs
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID集合查询物料" )
    @GetMapping("/getByids" )
    @ApiOperation(value = "通过ID集合查询物料")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "strs", value = "ID 多个用逗号分隔(,)", required = true, dataType = "String"),
            @ApiImplicitParam(name = "tenantId", value = "租户ID", required = true, dataType = "Long")
    })
    public ResultData getByIds(String strs,Long tenantId){
        Object obj = tfMatterPService.getByIds(strs,tenantId);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 新增物资信息及价格
     * @param matterPDto 物资信息表Dto
     * @return ResultData
     */
    @ToFlyAppLog(title="新增物资信息及价格" )
    @ApiOperation(value = "新增物资信息及价格")
    @PostMapping("saveMatterAndPrice")
    public ResultData saveMatterAndPrice(@RequestPart @Valid TfMatterPDto matterPDto, @RequestPart(value = "annexList", required = false) List<MultipartFile> annexList) {
        tfMatterPService.saveMatterAndPrice(matterPDto, annexList);
        return ResultData.ok();
    }

    /**
     * 新增物资信息表
     * @param tfMatterP 物资信息表
     * @return ResultData
     */
    @ToFlyAppLog(title="新增物资信息表" )
    @ApiOperation(value = "新增物资信息表")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfMatterP tfMatterP) {
      return ResultData.ok(tfMatterPService.save(tfMatterP));
    }

    /**
     * 修改物资信息表
     * @param matterPDto 物资信息表Dto
     * @return ResultData
     */
    @ToFlyAppLog(title="修改物资信息表" )
    @ApiOperation(value = "修改物资信息表")
    @PutMapping
    public ResultData updateById(@RequestPart TfMatterPDto matterPDto, @RequestPart(value = "annexList", required = false) List<MultipartFile> annexList) {
        tfMatterPService.updateTfMatterP(matterPDto, annexList);
        return ResultData.ok();
    }

    /**
     * 通过id删除物资信息表
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除物资信息表" )
    @ApiOperation(value = "通过ID删除物资信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfMatterPService.removeById(id));
    }

    /**
     * 通过id批量删除物资信息表
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除物资信息表" )
    @ApiOperation(value = "通过ID批量删除物资信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("id不能为空");
        }
        List<Long> removeId = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        tfMatterPService.deleteTfMatterP(removeId);
        return ResultData.ok();
    }
}
