/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfMatterPriceP;
import com.tofly.workflow.baseconf.service.TfMatterPricePService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 物资价格
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfmatterpricep")
@Api(tags="物资价格接口")
public class TfMatterPricePController extends BaseController {

    private final  TfMatterPricePService tfMatterPricePService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfMatterPriceP 物资价格
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfMatterPricePPage(Page page, @ModelAttribute TfMatterPriceP tfMatterPriceP) {
      return ResultData.ok(tfMatterPricePService.page(page, Wrappers.query(tfMatterPriceP)));
    }


    /**
     * 通过id查询物资价格
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfMatterPricePService.getById(id));
    }

    /**
     * 新增物资价格
     * @param tfMatterPriceP 物资价格
     * @return ResultData
     */
    @ToFlyAppLog(title="新增物资价格" )
    @ApiOperation(value = "新增物资价格")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfMatterPriceP tfMatterPriceP) {
      return ResultData.ok(tfMatterPricePService.save(tfMatterPriceP));
    }

    /**
     * 修改物资价格
     * @param tfMatterPriceP 物资价格
     * @return ResultData
     */
    @ToFlyAppLog(title="修改物资价格" )
    @ApiOperation(value = "修改物资价格")
    @PutMapping
    public ResultData updateById(@RequestBody TfMatterPriceP tfMatterPriceP) {
      return ResultData.ok(tfMatterPricePService.updateById(tfMatterPriceP));
    }

    /**
     * 通过id删除物资价格
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除物资价格" )
    @ApiOperation(value = "通过ID删除物资价格")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfMatterPricePService.removeById(id));
    }

    /**
     * 通过id批量删除物资价格
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除物资价格" )
    @ApiOperation(value = "通过ID批量删除物资价格")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfMatterPricePService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
