/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfPartnerP;
import com.tofly.workflow.baseconf.service.TfPartnerPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 合作伙伴/协作单位（设计院）
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfpartnerp")
@Api(tags="合作伙伴/协作单位（设计院）接口")
public class TfPartnerPController extends BaseController {

    private final  TfPartnerPService tfPartnerPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfPartnerP 合作伙伴/协作单位（设计院）
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfPartnerPPage(Page page, @ModelAttribute TfPartnerP tfPartnerP) {
      return ResultData.ok(tfPartnerPService.page(page, Wrappers.query(tfPartnerP)));
    }


    /**
     * 通过id查询合作伙伴/协作单位（设计院）
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfPartnerPService.getById(id));
    }

    /**
     * 新增合作伙伴/协作单位（设计院）
     * @param tfPartnerP 合作伙伴/协作单位（设计院）
     * @return ResultData
     */
    @ToFlyAppLog(title="新增合作伙伴/协作单位（设计院）" )
    @ApiOperation(value = "新增合作伙伴/协作单位（设计院）")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfPartnerP tfPartnerP) {
      return ResultData.ok(tfPartnerPService.save(tfPartnerP));
    }

    /**
     * 修改合作伙伴/协作单位（设计院）
     * @param tfPartnerP 合作伙伴/协作单位（设计院）
     * @return ResultData
     */
    @ToFlyAppLog(title="修改合作伙伴/协作单位（设计院）" )
    @ApiOperation(value = "修改合作伙伴/协作单位（设计院）")
    @PutMapping
    public ResultData updateById(@RequestBody TfPartnerP tfPartnerP) {
      return ResultData.ok(tfPartnerPService.updateById(tfPartnerP));
    }

    /**
     * 通过id删除合作伙伴/协作单位（设计院）
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除合作伙伴/协作单位（设计院）" )
    @ApiOperation(value = "通过ID删除合作伙伴/协作单位（设计院）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfPartnerPService.removeById(id));
    }

    /**
     * 通过id批量删除合作伙伴/协作单位（设计院）
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除合作伙伴/协作单位（设计院）" )
    @ApiOperation(value = "通过ID批量删除合作伙伴/协作单位（设计院）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfPartnerPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
