/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfProcessSqlP;
import com.tofly.workflow.baseconf.entity.dto.PSqlPageDto;
import com.tofly.workflow.baseconf.entity.dto.ProcessLDto;
import com.tofly.workflow.baseconf.service.TfProcessSqlPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 流程查询sql
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprocesssqlp")
@Api(tags="流程查询sql接口")
public class TfProcessSqlPController extends BaseController {

    private final  TfProcessSqlPService tfProcessSqlPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 流程查询sql
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProcessSqlPPage(Page page, @ModelAttribute PSqlPageDto dto) {
      return ResultData.ok(tfProcessSqlPService.pageProcessSql(page, dto));
    }


    /**
     * 流程工单列表
     * @param page 分页
     * @param dto 工单流程查询条件
     * @return
     */
    @PostMapping("/datas" )
    @ApiOperation(value = "流程工单列表")
    public ResultData getProcessList(Page page,@RequestBody ProcessLDto dto){
        Object obj = tfProcessSqlPService.getProcesssData(page,dto);
        if(obj instanceof Page){
            return ResultData.ok(obj);
        }else {
            return ResultData.fail(obj.toString());
        }

    }


    /**
     * 通过id查询流程查询sql
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProcessSqlPService.getById(id));
    }

    /**
     * 新增流程查询sql
     * @param tfProcessSqlP 流程查询sql
     * @return ResultData
     */
    @ToFlyAppLog(title="新增流程查询sql" )
    @ApiOperation(value = "新增流程查询sql")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfProcessSqlP tfProcessSqlP) {
      return ResultData.ok(tfProcessSqlPService.save(tfProcessSqlP));
    }

    /**
     * 修改流程查询sql
     * @param tfProcessSqlP 流程查询sql
     * @return ResultData
     */
    @ToFlyAppLog(title="修改流程查询sql" )
    @ApiOperation(value = "修改流程查询sql")
    @PutMapping
    public ResultData updateById(@RequestBody TfProcessSqlP tfProcessSqlP) {
      return ResultData.ok(tfProcessSqlPService.updateById(tfProcessSqlP));
    }


    /**
     * 通过id删除流程查询sql
     *
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title = "通过id删除流程查询sql")
    @ApiOperation(value = "通过ID删除流程查询sql")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultData removeById(@PathVariable Long id) {
        if(tfProcessSqlPService.removeUpdateById(id)){
            return ResultData.ok();
        }else {
            return ResultData.fail();
        }
    }

}
