/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfProjectProcessP;
import com.tofly.workflow.baseconf.entity.dto.TfProjectProcessDto;
import com.tofly.workflow.baseconf.service.TfProjectProcessPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 工程工序/工程管控
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprojectprocessp")
@Api(tags="工程工序/工程管控接口")
public class TfProjectProcessPController extends BaseController {

    private final  TfProjectProcessPService tfProjectProcessPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfProjectProcessP 工程工序/工程管控
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProjectProcessPPage(Page page, @ModelAttribute TfProjectProcessP tfProjectProcessP) {
      return ResultData.ok(tfProjectProcessPService.page(page, Wrappers.query(tfProjectProcessP)));
    }

    /**
     * 列表查询
     * @param tfProjectProcessP 工程工序/工程管控
     * @return
     */
    @ToFlyAppLog(title="列表查询" )
    @GetMapping("/list" )
    @ApiOperation(value = "列表查询")
    public ResultData getTfProjectProcessPPage(@ModelAttribute TfProjectProcessP tfProjectProcessP) {
        return ResultData.ok(tfProjectProcessPService.list(Wrappers.query(tfProjectProcessP)));
    }

    /**
     * 按工程性质分组查询
     * @param tfProjectProcessP 工程工序/工程管控
     * @return
     */
    @ToFlyAppLog(title="按工程性质分组查询" )
    @GetMapping("/getProjectNumByNature" )
    @ApiOperation(value = "按工程性质分组查询")
    public ResultData getProjectNumByNature(TfProjectProcessP tfProjectProcessP) {
        return ResultData.ok(tfProjectProcessPService.getProjectNumByNature(tfProjectProcessP));
    }


    /**
     * 通过id查询工程工序/工程管控
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProjectProcessPService.getById(id));
    }

    /**
     * 新增工程工序/工程管控
     * @param dto 工程工序/工程管控
     * @return ResultData
     */
    @ToFlyAppLog(title="新增工程工序/工程管控" )
    @ApiOperation(value = "新增工程工序/工程管控")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfProjectProcessDto dto) {
        tfProjectProcessPService.saveProjectProcess(dto);
      return ResultData.ok();
    }

    /**
     * 修改工程工序/工程管控
     * @param tfProjectProcessP 工程工序/工程管控
     * @return ResultData
     */
    @ToFlyAppLog(title="修改工程工序/工程管控" )
    @ApiOperation(value = "修改工程工序/工程管控")
    @PutMapping
    public ResultData updateById(@RequestBody TfProjectProcessP tfProjectProcessP) {
      return ResultData.ok(tfProjectProcessPService.updateById(tfProjectProcessP));
    }

//    /**
//     * 通过id删除工程工序/工程管控
//     * @param id id
//     * @return ResultData
//     */
//    @ToFlyAppLog(title="通过id删除工程工序/工程管控" )
//    @ApiOperation(value = "通过ID删除工程工序/工程管控")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultData removeById(@PathVariable Long id) {
//      return ResultData.ok(tfProjectProcessPService.removeById(id));
//    }
    /**
     * 通过tenantId和nature删除工程工序/工程管控
     * @return ResultData
     */
    @ToFlyAppLog(title="通过tenantId和nature删除工程工序/工程管控" )
    @ApiOperation(value = "通过tenantId和nature删除工程工序/工程管控")
    @PostMapping("/removeByNatureAndTenantId" )
    public ResultData removeByNatureAndTenantId(@RequestParam(value = "nature") Long nature, @RequestParam(value = "tenantId") Long tenantId) {
        TfProjectProcessP projectProcess = new TfProjectProcessP();
        projectProcess.setIsDel((short)1);
        return ResultData.ok(tfProjectProcessPService.update(projectProcess,new QueryWrapper<TfProjectProcessP>()
                .eq("NATURE",nature)
                .eq("TENANT_ID",tenantId)
                .eq("IS_DEL",(short)0)));
    }

    /**
     * 通过id批量删除工程工序/工程管控
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除工程工序/工程管控" )
    @ApiOperation(value = "通过ID批量删除工程工序/工程管控")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfProjectProcessPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
