/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfReportEntryP;
import com.tofly.workflow.baseconf.service.TfReportEntryPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 开工报告条目
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfreportentryp")
@Api(tags="开工报告条目接口")
public class TfReportEntryPController extends BaseController {

    private final  TfReportEntryPService tfReportEntryPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfReportEntryP 开工报告条目
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfReportEntryPPage(Page page, @ModelAttribute TfReportEntryP tfReportEntryP) {
      return ResultData.ok(tfReportEntryPService.page(page, Wrappers.query(tfReportEntryP)));
    }


    /**
     * 通过id查询开工报告条目
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfReportEntryPService.getById(id));
    }

    /**
     * 新增开工报告条目
     * @param tfReportEntryP 开工报告条目
     * @return ResultData
     */
    @ToFlyAppLog(title="新增开工报告条目" )
    @ApiOperation(value = "新增开工报告条目")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfReportEntryP tfReportEntryP) {
      return ResultData.ok(tfReportEntryPService.save(tfReportEntryP));
    }

    /**
     * 修改开工报告条目
     * @param tfReportEntryP 开工报告条目
     * @return ResultData
     */
    @ToFlyAppLog(title="修改开工报告条目" )
    @ApiOperation(value = "修改开工报告条目")
    @PutMapping
    public ResultData updateById(@RequestBody TfReportEntryP tfReportEntryP) {
      return ResultData.ok(tfReportEntryPService.updateById(tfReportEntryP));
    }

    /**
     * 通过id删除开工报告条目
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除开工报告条目" )
    @ApiOperation(value = "通过ID删除开工报告条目")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfReportEntryPService.removeById(id));
    }

    /**
     * 通过id批量删除开工报告条目
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除开工报告条目" )
    @ApiOperation(value = "通过ID批量删除开工报告条目")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfReportEntryPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
