/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfSequence;
import com.tofly.workflow.baseconf.entity.dto.SequenceDto;
import com.tofly.workflow.baseconf.service.TfSequenceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
/**
 * 自增序列
 *
 * @author xqf
 * @date Tue Dec 13 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfsequence")
@Api(tags="自增序列接口")
public class TfSequenceController {

    private final  TfSequenceService tfSequenceService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfSequence 自增序列
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfSequencePage(Page page, @ModelAttribute TfSequence tfSequence) {
      return ResultData.ok(tfSequenceService.page(page, Wrappers.query(tfSequence)));
    }

    /**
     * 获取自增序列数值
     * @param dto 自增序列
     * @return ResultRespone
     */
    @ToFlyAppLog(title="获取自增序列数值" )
    @ApiOperation(value = "获取自增序列数值")
    @PostMapping("sequenceValue")
    public ResultData getSequenceValue(@RequestBody SequenceDto dto){
        Object obj = tfSequenceService.getSequenceValue(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 新增自增序列
     * @param tfSequence 自增序列
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增自增序列" )
    @ApiOperation(value = "新增自增序列")
    @PostMapping
    public ResultData save(@RequestBody TfSequence tfSequence) {
      return ResultData.ok(tfSequenceService.save(tfSequence));
    }

    /**
     * 修改自增序列
     * @param tfSequence 自增序列
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改自增序列" )
    @ApiOperation(value = "修改自增序列")
    @PutMapping
    public ResultData updateById(@RequestBody TfSequence tfSequence) {
      return ResultData.ok(tfSequenceService.updateById(tfSequence));
    }

    /**
     * 通过id删除自增序列
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除自增序列" )
    @ApiOperation(value = "通过ID删除自增序列")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfSequenceService.removeById(id));
    }

}
