/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfTaskP;
import com.tofly.workflow.baseconf.entity.dto.ProjectTaskDto;
import com.tofly.workflow.baseconf.service.TfTaskPService;
import com.tofly.workflow.process.entity.dto.ProcessDto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

/**
 * 待办信息
 *
 * @author xqf
 * @date Mon Sep 19 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tftaskp")
@Api(tags="待办信息接口")
public class TfTaskPController {

    private final  TfTaskPService tfTaskPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 待办信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfTaskPPage(Page page, @ModelAttribute ProjectTaskDto dto) {
      return ResultData.ok(tfTaskPService.pageTask(page,dto));
    }

    /**
     * 类型统计
     * @param dto 待办信息
     * @return
     */
    @GetMapping("/count" )
    @ApiOperation(value = "类型统计")
    public ResultData getTfTaskCount( @ModelAttribute ProjectTaskDto dto) {
        return ResultData.ok(tfTaskPService.taskCount(dto));
    }


    /**
     * 通过id查询待办信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfTaskPService.getById(id));
    }

    /**
     * 新增待办信息
     * @param tfTaskP 待办信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增待办信息" )
    @ApiOperation(value = "新增待办信息")
    @PostMapping
    public ResultData save(@RequestBody TfTaskP tfTaskP) {
      return ResultData.ok(tfTaskPService.save(tfTaskP));
    }

    /**
     * 修改待办信息
     * @param tfTaskP 待办信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改待办信息" )
    @ApiOperation(value = "修改待办信息")
    @PutMapping
    public ResultData updateById(@RequestBody TfTaskP tfTaskP) {
      return ResultData.ok(tfTaskPService.updateById(tfTaskP));
    }

    /**
     * 测试文件保存
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="测试文件保存" )
//    @ApiOperation(value = "测试文件保存")
//    @PostMapping("testFile")
    public ResultData testfileSave(@RequestParam(name = "file")MultipartFile file) {
        return ResultData.ok(tfTaskPService.testfileSave(file));
    }

}
