/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfTeamStaffP;
import com.tofly.workflow.baseconf.service.TfTeamStaffPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
/**
 * 施工班组人员
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfteamstaffp")
@Api(tags="施工班组人员接口")
public class TfTeamStaffPController extends BaseController {

    private final  TfTeamStaffPService tfTeamStaffPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfTeamStaffP 施工班组人员
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfTeamStaffPPage(Page page, @ModelAttribute TfTeamStaffP tfTeamStaffP) {
//      return ResultData.ok(tfTeamStaffPService.getTfTeamStaffPPage(page, teamStaffPDto));
       return ResultData.ok(tfTeamStaffPService.page(page, Wrappers.query(tfTeamStaffP)));
    }


    /**
     * 通过id查询施工班组人员
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfTeamStaffPService.getById(id));
    }

    /**
     * 新增施工班组人员
     * @param tfTeamStaffP 施工班组人员
     * @return ResultData
     */
    @ToFlyAppLog(title="新增施工班组人员" )
    @ApiOperation(value = "新增施工班组人员")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfTeamStaffP tfTeamStaffP) {
      return ResultData.ok(tfTeamStaffPService.save(tfTeamStaffP));
    }

    /**
     * 修改施工班组人员
     * @param tfTeamStaffP 施工班组人员
     * @return ResultData
     */
    @ToFlyAppLog(title="修改施工班组人员" )
    @ApiOperation(value = "修改施工班组人员")
    @PutMapping
    public ResultData updateById(@RequestBody TfTeamStaffP tfTeamStaffP) {
      return ResultData.ok(tfTeamStaffPService.updateById(tfTeamStaffP));
    }

    /**
     * 通过id删除施工班组人员
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除施工班组人员" )
    @ApiOperation(value = "通过ID删除施工班组人员")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfTeamStaffPService.removeById(id));
    }

    /**
     * 通过id批量删除施工班组人员
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除施工班组人员" )
    @ApiOperation(value = "通过ID批量删除施工班组人员")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfTeamStaffPService.removeByIds(Arrays.asList(ids.split(","))));
    }
    /**
     * 通过施工班组id查询施工班组所有人员
     * @param teamId id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过施工班组id查询施工班组所有人员" )
    @ApiOperation(value = "通过施工班组id查询施工班组所有人员")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "teamId", value = "teamId", required = true, dataType = "Long")
    })
    @GetMapping("/getByTeamId" )
    public ResultData getByTeamId(Long teamId){
        return ResultData.ok(tfTeamStaffPService.getByTeamId(teamId));
    }
}
