/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfTemplateFieldP;
import com.tofly.workflow.baseconf.service.TfTemplateFieldPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
/**
 * 合同模板字段关联信息
 *
 * @author JF
 * @date Mon Sep 19 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tftemplatefieldp")
@Api(tags="合同模板字段关联信息接口")
public class TfTemplateFieldPController {

    private final  TfTemplateFieldPService tfTemplateFieldPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfTemplateFieldP 合同模板字段关联信息
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfTemplateFieldPPage(Page page, @ModelAttribute TfTemplateFieldP tfTemplateFieldP) {
      return ResultData.ok(tfTemplateFieldPService.page(page, Wrappers.query(tfTemplateFieldP)));
    }


    /**
     * 通过id查询合同模板字段关联信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfTemplateFieldPService.getById(id));
    }

    /**
     * 新增合同模板字段关联信息
     * @param tfTemplateFieldP 合同模板字段关联信息
     * @return ResultData
     */
    @ToFlyAppLog(title="新增合同模板字段关联信息" )
    @ApiOperation(value = "新增合同模板字段关联信息")
    @PostMapping
    public ResultData save(@RequestBody TfTemplateFieldP tfTemplateFieldP) {
      return ResultData.ok(tfTemplateFieldPService.save(tfTemplateFieldP));
    }

    /**
     * 修改合同模板字段关联信息
     * @param tfTemplateFieldP 合同模板字段关联信息
     * @return ResultData
     */
    @ToFlyAppLog(title="修改合同模板字段关联信息" )
    @ApiOperation(value = "修改合同模板字段关联信息")
    @PutMapping
    public ResultData updateById(@RequestBody TfTemplateFieldP tfTemplateFieldP) {
      return ResultData.ok(tfTemplateFieldPService.updateById(tfTemplateFieldP));
    }

    /**
     * 通过id删除合同模板字段关联信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除合同模板字段关联信息" )
    @ApiOperation(value = "通过ID删除合同模板字段关联信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfTemplateFieldPService.removeById(id));
    }

    /**
     * 通过id批量删除合同模板字段关联信息
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除合同模板字段关联信息" )
    @ApiOperation(value = "通过ID批量删除合同模板字段关联信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfTemplateFieldPService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
