/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.baseconf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.entity.TfProjectProcessP;
import com.tofly.workflow.baseconf.entity.TfWorkProcessP;
import com.tofly.workflow.baseconf.service.TfProjectProcessPService;
import com.tofly.workflow.baseconf.service.TfWorkProcessPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 工序管理（工序名称）/工程管控管理
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfworkprocessp")
@Api(tags="工序管理（工序名称）/工程管控管理接口")
public class TfWorkProcessPController extends BaseController {

    private final  TfWorkProcessPService tfWorkProcessPService;
    private final TfProjectProcessPService tfProjectProcessPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfWorkProcessP 工序管理（工序名称）/工程管控管理
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfWorkProcessPPage(Page page, @ModelAttribute TfWorkProcessP tfWorkProcessP) {
        Page<TfWorkProcessP> workProcessPPage = tfWorkProcessPService.page(page, Wrappers.query(tfWorkProcessP));
        return ResultData.ok(workProcessPPage);
    }


    /**
     * 通过id查询工序管理（工序名称）/工程管控管理
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfWorkProcessPService.getById(id));
    }

    /**
     * 新增工序管理（工序名称）/工程管控管理
     * @param tfWorkProcessP 工序管理（工序名称）/工程管控管理
     * @return ResultData
     */
    @ToFlyAppLog(title="新增工序管理（工序名称）/工程管控管理" )
    @ApiOperation(value = "新增工序管理（工序名称）/工程管控管理")
    @PostMapping
    public ResultData save(@RequestBody @Valid TfWorkProcessP tfWorkProcessP) {
        Object obj = tfWorkProcessPService.saveWorkProcess(tfWorkProcessP);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

//    /**
//     * 新增工程管控管理相关信息
//     * @param dto 工序管理（工序名称）/工程管控管理Dto
//     * @return ResultData
//     */
//    @ToFlyAppLog(title="新增工程管控管理相关信息" )
//    @ApiOperation(value = "新增工程管控管理相关信息")
//    @PostMapping("saveWorkProjectProcess")
//    public ResultData saveWorkProjectProcess(@RequestBody @Valid TfWorkProcessDto dto) {
//        tfWorkProcessPService.saveWorkProjectProcess(dto);
//        return ResultData.ok();
//    }

    /**
     * 修改工序管理（工序名称）/工程管控管理
     * @param tfWorkProcessP 工序管理（工序名称）/工程管控管理
     * @return ResultData
     */
    @ToFlyAppLog(title="修改工序管理（工序名称）/工程管控管理" )
    @ApiOperation(value = "修改工序管理（工序名称）/工程管控管理")
    @PutMapping
    public ResultData updateById(@RequestBody TfWorkProcessP tfWorkProcessP) {
        Object obj = tfWorkProcessPService.updateWorkProcess(tfWorkProcessP);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除工序管理（工序名称）/工程管控管理
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除工序管理（工序名称）/工程管控管理" )
    @ApiOperation(value = "通过ID删除工序管理（工序名称）/工程管控管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfWorkProcessPService.removeById(id));
    }

    /**
     * 通过id批量删除工序管理（工序名称）/工程管控管理
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除工序管理（工序名称）/工程管控管理" )
    @ApiOperation(value = "通过ID批量删除工序管理（工序名称）/工程管控管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids为空");
        }
        List<Long> idList = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        tfWorkProcessPService.removeByIds(idList);
        idList.forEach(info -> tfProjectProcessPService.remove(new QueryWrapper<TfProjectProcessP>().eq("PROCESS_ID",info)));
        return ResultData.ok();
    }
}
