/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfBaseTypeP;
import com.tofly.workflow.baseconf.entity.dto.BaseTypeDto;
import com.tofly.workflow.baseconf.entity.dto.UpdateBaseTypeDto;
import com.tofly.workflow.baseconf.entity.vo.BaseTypeDataTypeVo;
import com.tofly.workflow.baseconf.entity.vo.BaseTypeVo;
import com.tofly.workflow.baseconf.mapper.TfBaseTypePMapper;
import com.tofly.workflow.baseconf.service.TfBaseTypePService;
import com.tofly.workflow.common.FlowConstant;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 基础类型表
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfBaseTypePService")
public class TfBaseTypePServiceImpl extends ServiceImpl<TfBaseTypePMapper, TfBaseTypeP> implements TfBaseTypePService {

    /**
     * 保存基础类型
     *
     * @param dto
     * @return
     */
    @Override
    public Object saveBaseType(BaseTypeDto dto) {
        TfBaseTypeP dt = null;
        if(dto.getDataType()!=null&&dto.getDataType()!= FlowConstant.LONG_ZERO) {
            dt = this.getById(dto.getDataType());
            if (dt == null && (dt.getIsValid() == FlowConstant.SHORT_ZERO || dt.getIsDel() == FlowConstant.SHORT_ONE)) {
                return FlowConstant.DATATYPE_ERROR;
            }
        }
        if (dto.getParentId() == null) {
            dto.setParentId(FlowConstant.LONG_ZERO);
        } else if (dto.getParentId() > 0) {
            int count = this.count(new QueryWrapper<TfBaseTypeP>().lambda().eq(TfBaseTypeP::getId, dto.getParentId())
                    .eq(TfBaseTypeP::getDataType, dto.getDataType())
                    .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO));
            if (count == 0) {
                return FlowConstant.DATA_NOTHINGNESS;
            }
        }
        int count = this.count(new QueryWrapper<TfBaseTypeP>().lambda()
                .eq(TfBaseTypeP::getDataType, dto.getDataType()).eq(TfBaseTypeP::getParentId, dto.getParentId())
                .eq(TfBaseTypeP::getTypeName, dto.getTypeName()).eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO));
        if (count > 0) {
            return FlowConstant.NAME_EXISTS;
        }

        TfBaseTypeP baseType = new TfBaseTypeP();
        BeanUtils.copyProperties(dto, baseType);
        if (baseType.getIsValid() == null) {
            baseType.setIsValid(FlowConstant.SHORT_ONE);
        }
        baseType.setIsDel(FlowConstant.SHORT_ZERO);
        return save(baseType);
    }

    /**
     * 修改基础类型信息
     *
     * @param dto
     * @return
     */
    @Override
    public Object updateBaseType(UpdateBaseTypeDto dto) {
        TfBaseTypeP bt = this.getById(dto.getId());
        if (bt == null || bt.getIsDel() == FlowConstant.SHORT_ONE) {
            return FlowConstant.DATA_NOTHINGNESS;
        }
        int count = this.count(new QueryWrapper<TfBaseTypeP>().lambda()
                .eq(TfBaseTypeP::getDataType, bt.getDataType()).eq(TfBaseTypeP::getParentId, bt.getParentId())
                .eq(TfBaseTypeP::getTypeName, dto.getTypeName()).eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO)
                .ne(TfBaseTypeP::getId, dto.getId()));
        if (count > 0) {
            return FlowConstant.NAME_EXISTS;
        }
        TfBaseTypeP baseType = new TfBaseTypeP();
        if (dto.getIsValid() != null) {
            Object obj = updateValid(dto.getId(), dto.getIsValid());
            if (obj instanceof String) {
                return obj;
            }
        }
        BeanUtils.copyProperties(dto, baseType);
        return updateById(baseType);
    }


    /**
     * 修改类型是否有效
     *
     * @param id    id主键
     * @param valid 有效无效 0 无效，1有效
     * @return
     */
    @Override
    public Object updateValid(Long id, Short valid) {
        if (id == null) {
            return FlowConstant.ISNOTNUL;
        }
        if (FlowConstant.SHORT_ZERO.equals(valid)) {
            List<Long> ids = this.baseMapper.getTopToBottomId(id);
            return update(new UpdateWrapper<TfBaseTypeP>().lambda().set(TfBaseTypeP::getIsValid, FlowConstant.SHORT_ZERO)
                    .set(TfBaseTypeP::getUpdateTime, new Date()).set(TfBaseTypeP::getUpdateUser, SecurityUtils.getUserId())
                    .eq(TfBaseTypeP::getIsValid, FlowConstant.SHORT_ONE)
                    .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO)
                    .in(TfBaseTypeP::getId, ids));
        } else if (FlowConstant.SHORT_ONE.equals(valid)) {
            List<Long> ids = this.baseMapper.getBottomToTopId(id);
            return update(new UpdateWrapper<TfBaseTypeP>().lambda().set(TfBaseTypeP::getIsValid, FlowConstant.SHORT_ONE)
                    .set(TfBaseTypeP::getUpdateTime, new Date()).set(TfBaseTypeP::getUpdateUser, SecurityUtils.getUserId())
                    .eq(TfBaseTypeP::getIsValid, FlowConstant.SHORT_ZERO)
                    .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO)
                    .in(TfBaseTypeP::getId, ids));
        } else {
            return FlowConstant.PARAMETER_ERROR;
        }
    }

    @Override
    public List<Long> selectTypeChildById(Long id) {
        return this.baseMapper.selectTypeChildById(id);
    }

    @Override
    public List<TfBaseTypeP> selectTypeParentById(Long id) {
        return this.baseMapper.selectTypeParentById(id);
    }


    /**
     * 逻辑删除急匆匆类型
     *
     * @param id 主键id
     * @return
     */
    @Override
    public boolean removeUpdateById(Long id) {
        TfBaseTypeP bt = this.getById(id);
        if(bt==null){
            return false;
        }
        int count = 0;
        List<Long> ids = this.baseMapper.getTopToBottomId(id);
        if (CollUtil.isNotEmpty(ids)) {
            return update(new UpdateWrapper<TfBaseTypeP>().lambda().set(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ONE)
                    .set(TfBaseTypeP::getUpdateTime, new Date()).set(TfBaseTypeP::getUpdateUser, SecurityUtils.getUserId())
                    .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO)
                    .in(TfBaseTypeP::getId, ids));
        }
        return false;
    }

    @Override
    public Object removeUpdateByIds(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return FlowConstant.ISNOTNUL;
        }
        try {
            List<Long> lids = Arrays.asList(ids.split(",")).stream().map(st ->new Long(st)).collect(Collectors.toList());
            lids = this.baseMapper.getTopToBottomIds(lids);
            if (CollUtil.isNotEmpty(lids)) {
                return this.baseMapper.update(new TfBaseTypeP(), new UpdateWrapper<TfBaseTypeP>().lambda()
                        .set(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ONE)
                        .eq(TfBaseTypeP::getIsDel, FlowConstant.SHORT_ZERO)
                        .in(TfBaseTypeP::getId, lids));
            }
            return false;
        } catch (Exception e) {
            log.error("异常：", e);
            return e.getMessage();
        }
    }

    @Override
    public Object getTypeTree(String datatypes, Long id, Integer type,List<String> orgCodes) {
        if (type == FlowConstant.INTEGER_ONE && (StringUtils.isEmpty(datatypes) || !datatypes.contains(FlowConstant.FLAG) || datatypes.trim().length() > 2)) {
            return FlowConstant.DATATYPE_ERROR;
        }
        if (StringUtils.isEmpty(datatypes) && id == null) {
            return FlowConstant.ISNOTNUL;
        }
        try {
            List<Long> lids = null;
            if (StringUtils.isNotEmpty(datatypes)) {
                lids = Arrays.asList(datatypes.split(",")).stream().map(st -> new Long(st)).collect(Collectors.toList());
            }
            List<BaseTypeVo> lists = null;
            if (type == FlowConstant.INTEGER_ONE) {
                lists = this.baseMapper.getDataTypeNode(FlowConstant.LONG_ZERO, id,orgCodes);

            } else {
                if (id != null) {
                    lists = this.baseMapper.getBaseTypeId(lids, id,orgCodes);
                } else {
                    lists = this.baseMapper.getBaseTypeDataType(lids,orgCodes);
                }
            }
            Map<Long, BaseTypeDataTypeVo> mapdt = new HashMap<>();
            Map<Long, BaseTypeVo> map = new HashMap<>();
            if (CollUtil.isNotEmpty(lists)) {
                for (BaseTypeVo vo : lists) {
                    vo.setVos(new ArrayList<>());
                    if (map.containsKey(vo.getParentId())) {
                        BaseTypeVo tovo = map.get(vo.getParentId());
                        tovo.getVos().add(vo);
                    } else {
                        if (mapdt.containsKey(vo.getDataType())) {
                            BaseTypeDataTypeVo dtvo = mapdt.get(vo.getDataType());
                            dtvo.getVos().add(vo);
                        } else {
                            BaseTypeDataTypeVo dtvo = new BaseTypeDataTypeVo();
                            dtvo.setDataType(vo.getDataType());
                            dtvo.setDataTypeName(vo.getDataTypeName());
                            dtvo.setVos(new ArrayList<>());
                            dtvo.getVos().add(vo);
                            mapdt.put(dtvo.getDataType(), dtvo);
                        }
                    }
                    map.put(vo.getId(), vo);
                }
            }
            return new ArrayList<>(mapdt.values());
        } catch (Exception e) {
            log.error("异常：", e);
            return e.getMessage();
        }
    }


}
