/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.workflow.baseconf.entity.TfBuildUnitP;
import com.tofly.workflow.baseconf.entity.TfConstructionTeamP;
import com.tofly.workflow.baseconf.entity.TfTeamStaffP;
import com.tofly.workflow.baseconf.entity.dto.TeamStaffMoreDto;
import com.tofly.workflow.baseconf.entity.dto.TfBuildUnitPDto;
import com.tofly.workflow.baseconf.entity.vo.TfBuildUnitPVo;
import com.tofly.workflow.baseconf.mapper.TfBuildUnitPMapper;
import com.tofly.workflow.baseconf.service.TfBuildUnitPService;
import com.tofly.workflow.baseconf.service.TfConstructionTeamPService;
import com.tofly.workflow.baseconf.service.TfTeamStaffPService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 施工组织单位
 *
 * @author JF
 * @date Tue Aug 09 00:00:00 CST 2022
 */
@Service("tfBuildUnitPService")
public class TfBuildUnitPServiceImpl extends ServiceImpl<TfBuildUnitPMapper, TfBuildUnitP> implements TfBuildUnitPService {

    @Autowired
    private TfConstructionTeamPService constructionTeamPService;
    @Autowired
    private TfTeamStaffPService teamStaffPService;

    @Override
    public Page<TfBuildUnitPVo> getTfBuildUnitPPage(Page page, TfBuildUnitPDto buildUnitPDto) {
        Page<TfBuildUnitPVo> buildUnitPVoPage = this.baseMapper.getTfBuildUnitPPage(page, buildUnitPDto);
        List<TfBuildUnitPVo> buildUnitPVos = buildUnitPVoPage.getRecords();
        for (TfBuildUnitPVo buildUnitPVo : buildUnitPVos) {
            List<TfConstructionTeamP> constructionTeamPList = constructionTeamPService.getConstructionTeamList(buildUnitPVo.getId());
            buildUnitPVo.setConstructionTeamPList(constructionTeamPList);
        }
        buildUnitPVoPage.setRecords(buildUnitPVos);
        return buildUnitPVoPage;
    }

    /**
     * 施工班组配置
     *
     * @param buildUnitPDto
     */
    @Override
    public void saveTeamAll(TfBuildUnitPDto buildUnitPDto) {
        if (buildUnitPDto == null) {
            return;
        }
        if (buildUnitPDto.getTfBuildUnitP() == null) {
            throw new ToflyDeniedException("施工组织单位为空，请重新输入");
        }
        TfBuildUnitP tfBuildUnitP = buildUnitPDto.getTfBuildUnitP();
        Long id;
        if (tfBuildUnitP.getId() != null) {
            id = tfBuildUnitP.getId();
            //查询当前单位的所有施工班组
            List<TfConstructionTeamP> constructionTeamList = constructionTeamPService
                    .list(new QueryWrapper<TfConstructionTeamP>().eq("BUILD_UNIT_ID", tfBuildUnitP.getId()));
            for (TfConstructionTeamP constructionTeamP : constructionTeamList) {
                // 删除每一个班组的人员
                teamStaffPService.remove(new QueryWrapper<TfTeamStaffP>().eq("TEAM_ID", constructionTeamP.getId()));
            }

            // 删除所有的施工班组
            constructionTeamPService.remove(new QueryWrapper<TfConstructionTeamP>().eq("BUILD_UNIT_ID",tfBuildUnitP.getId()));
            this.updateById(tfBuildUnitP);
        } else {
            this.save(tfBuildUnitP);
            TfBuildUnitP buildUnitPQuery = this.getOne(new QueryWrapper<TfBuildUnitP>()
                    .eq("TENANT_ID", tfBuildUnitP.getTenantId())
                    .eq("TEAM_TYPE", tfBuildUnitP.getTeamType())
                    .eq("ORDER_NUMBER", tfBuildUnitP.getOrderNumber())
                    .eq("DEPT_ID", tfBuildUnitP.getDeptId()));
            id = buildUnitPQuery.getId();
        }

        if (buildUnitPDto.getTeamStaffMoreDtoList() == null || buildUnitPDto.getTeamStaffMoreDtoList().size() == 0) {
            return;
        }
        //循环前端传递过来的施工班组
        for (TeamStaffMoreDto one : buildUnitPDto.getTeamStaffMoreDtoList()) {
            TfConstructionTeamP tfConstructionTeamP = one.getTfConstructionTeamP();
            TfConstructionTeamP constructionTeamPQuery = null;
            if (tfConstructionTeamP != null) {
                tfConstructionTeamP.setBuildUnitId(id);
                constructionTeamPService.save(tfConstructionTeamP);
                constructionTeamPQuery = constructionTeamPService.getOne(new QueryWrapper<TfConstructionTeamP>()
                        .eq("NAME", tfConstructionTeamP.getName())
                        .eq("BUILD_UNIT_ID", id));
            }
            if (constructionTeamPQuery == null) {
                continue;
            }
            //保存施工班组负责人
            TfTeamStaffP tfTeamStaffP = one.getTeamStaffP();
            if (tfTeamStaffP != null) {
                tfTeamStaffP.setTeamId(constructionTeamPQuery.getId());
                teamStaffPService.save(tfTeamStaffP);
            }
            //保存施工班组成员
            if (one.getTeamStaffPList() != null && one.getTeamStaffPList().size() != 0) {
                for (TfTeamStaffP teamStaffP : one.getTeamStaffPList()) {
                    teamStaffP.setTeamId(constructionTeamPQuery.getId());
                    teamStaffPService.save(teamStaffP);
                }
            }
        }
    }

}
