/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.workflow.baseconf.entity.TfCalendarP;
import com.tofly.workflow.baseconf.entity.TfCalendarYearP;
import com.tofly.workflow.baseconf.entity.dto.CalendarYearPDto;
import com.tofly.workflow.baseconf.mapper.TfCalendarYearPMapper;
import com.tofly.workflow.baseconf.service.TfCalendarPService;
import com.tofly.workflow.baseconf.service.TfCalendarYearPService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 日历年
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfCalendarYearPService")
public class TfCalendarYearPServiceImpl extends ServiceImpl<TfCalendarYearPMapper, TfCalendarYearP> implements TfCalendarYearPService {

    @Autowired
    private TfCalendarPService calendarPService;

    @Override
    public Object saveCalendarAll(CalendarYearPDto calendarYearPDto) {
        if(calendarYearPDto == null){
            return "保存数据为空！";
        }
        if(calendarYearPDto.getCalendarYearP() == null){
            return "日历年为空，请重新输入";
        }
        if(calendarYearPDto.getCalendarPList() == null || calendarYearPDto.getCalendarPList().size() == 0){
            return "工作日应用日历列表为空，请重新输入";
        }
        TfCalendarYearP calendarYearP = calendarYearPDto.getCalendarYearP();
        int count = this.count(new QueryWrapper<TfCalendarYearP>().lambda().eq(TfCalendarYearP::getYearName,calendarYearP.getYearName())
                .ne(calendarYearP.getId()!=null,TfCalendarYearP::getId,calendarYearP.getId()));
        if(count>0){
            return "年份重复！";
        }

        Long id;
        if(calendarYearP.getId() != null) {
            id = calendarYearP.getId();
            calendarPService.remove(new QueryWrapper<TfCalendarP>().eq("YEAR_ID",calendarYearP.getId()));
            this.updateById(calendarYearP);
        }else {
            this.save(calendarYearP);
//            TfCalendarYearP calendarYearPQuery = this.getOne(new QueryWrapper<TfCalendarYearP>()
//                    .eq("YEAR_NAME",calendarYearP.getYearName()));
//            id = calendarYearPQuery.getId();
            id = calendarYearP.getId();
        }
        for(TfCalendarP calendarP : calendarYearPDto.getCalendarPList()){
            calendarP.setYearId(id);
            calendarPService.save(calendarP);
        }
        return true;
    }

    /**
     * 保存日历年
     * @param tfCalendarYearP 日历年信息
     * @return
     */
    @Override
    public Object saveCalendarYear(TfCalendarYearP tfCalendarYearP) {
        String check = checkCalendar(tfCalendarYearP);
        if (check != null) return check;
        return this.save(tfCalendarYearP);
    }

    private String checkCalendar(TfCalendarYearP tfCalendarYearP) {
        if(StringUtils.isNotEmpty(tfCalendarYearP.getYearName())){
            return "年份为空！";
        }
        int count = this.count(new QueryWrapper<TfCalendarYearP>().lambda().eq(TfCalendarYearP::getYearName, tfCalendarYearP.getYearName())
                .ne(tfCalendarYearP.getId()!=null,TfCalendarYearP::getId, tfCalendarYearP.getId()));
        if(count>0){
            return "名称重复！";
        }
        return null;
    }

    /**
     * 修改日历年
     * @param tfCalendarYearP 日历年信息
     * @return
     */
    @Override
    public Object updateCalendarYear(TfCalendarYearP tfCalendarYearP) {
        if (tfCalendarYearP.getId()==null){
            return "修改ID为空！";
        }
        String check = checkCalendar(tfCalendarYearP);
        if (check != null) return check;
        return this.updateById(tfCalendarYearP);
    }
}
