/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.baseconf.entity.TfContractTemplateP;
import com.tofly.workflow.baseconf.entity.TfTemplateFieldP;
import com.tofly.workflow.baseconf.entity.dto.TfContractTemplatePDto;
import com.tofly.workflow.baseconf.mapper.TfContractTemplatePMapper;
import com.tofly.workflow.baseconf.service.TfContractTemplatePService;
import com.tofly.workflow.baseconf.service.TfTemplateFieldPService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 合同模板
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfContractTemplatePService")
public class TfContractTemplatePServiceImpl extends ServiceImpl<TfContractTemplatePMapper, TfContractTemplateP> implements TfContractTemplatePService {

    @Autowired
    private TfTemplateFieldPService templateFieldPService;
    @Autowired
    private IFileInfoService iFileInfoService;
    @Value("${zmrq.ftp.contractTemplate}")
    private String contractTemplatePath;

    @Override
    public void saveContractTemplateField(List<MultipartFile> contractAnnexList, List<MultipartFile> annexList, TfContractTemplatePDto contractTemplatePDto) {
        if(contractTemplatePDto == null){
            return;
        }
        if(contractTemplatePDto.getContractTemplateP() == null){
            throw new ToflyDeniedException("合同模板不能为空，请重新输入");
        }
        if(contractTemplatePDto.getTemplateFieldList() == null || contractTemplatePDto.getTemplateFieldList().size() == 0){
            throw new ToflyDeniedException("合同模板字段关联信息列表不能为空，请重新输入");
        }
        TfContractTemplateP contractTemplateP = contractTemplatePDto.getContractTemplateP();
        if(contractAnnexList != null && contractAnnexList.size() != 0){
            List<FileInfo> contractAnnexFileInfo = iFileInfoService.saveFileList(contractAnnexList, contractTemplatePath, FileTableEnum.TF_CONTRACT_TEMPLATE_P.getTableName());
            contractTemplateP.setContractAnnex(JSON.toJSONString(contractAnnexFileInfo));
        }
        if(annexList != null && annexList.size() != 0){
            List<FileInfo> annexFileInfo = iFileInfoService.saveFileList(annexList, contractTemplatePath, FileTableEnum.TF_CONTRACT_TEMPLATE_P.getTableName());
            contractTemplateP.setAnnex(JSON.toJSONString(annexFileInfo));
        }
        // 对象入库
        this.save(contractTemplateP);

        TfContractTemplateP contractTemplatePQuery = this.getOne(new QueryWrapper<TfContractTemplateP>()
                .eq("CONTRACT_TYPE",contractTemplateP.getContractType())
                .eq("TENANT_ID",contractTemplateP.getTenantId())
                .eq("NAME",contractTemplateP.getName()));
        for(TfTemplateFieldP templateField : contractTemplatePDto.getTemplateFieldList()){
            templateField.setTemplateId(contractTemplatePQuery.getId());
            templateFieldPService.save(templateField);
        }
    }

    @Override
    public void updateContractTemplate(List<MultipartFile> contractAnnexList, List<MultipartFile> annexList, TfContractTemplatePDto contractTemplatePDto) {
        if(contractTemplatePDto == null){
            return;
        }
        if(contractTemplatePDto.getContractTemplateP() == null){
            throw new ToflyDeniedException("合同模板不能为空，请重新输入");
        }
        if(contractTemplatePDto.getTemplateFieldList() == null || contractTemplatePDto.getTemplateFieldList().size() == 0){
            throw new ToflyDeniedException("合同模板字段关联信息列表不能为空，请重新输入");
        }
        TfContractTemplateP contractTemplateP = contractTemplatePDto.getContractTemplateP();
        if(contractTemplateP.getId() == null){
            throw new ToflyDeniedException("ID不能为空");
        }
        List<FileInfo> nowContractAnnexFileInfos = null;
        List<FileInfo> nowAnnexFileInfos = null;
        List<FileInfo> agoContractAnnexFileInfos = null;
        List<FileInfo> agoAnnexFileInfos = null;
        List<FileInfo> upContractAnnexFileInfos = new ArrayList<>();
        List<FileInfo> upAnnexFileInfos = new ArrayList<>();
        if(StringUtils.isNotBlank(contractTemplateP.getContractAnnex())){
            nowContractAnnexFileInfos = JSON.parseArray(contractTemplateP.getContractAnnex(),FileInfo.class);
        }
        if(StringUtils.isNotBlank(contractTemplateP.getAnnex())){
            nowAnnexFileInfos = JSON.parseArray(contractTemplateP.getAnnex(),FileInfo.class);
        }
        if (null != contractAnnexList && contractAnnexList.size() > 0) {
            upContractAnnexFileInfos = iFileInfoService.saveFileList(contractAnnexList, contractTemplatePath, FileTableEnum.TF_CONTRACT_TEMPLATE_P.getTableName());
        }else if (null != annexList && annexList.size() > 0) {
            upAnnexFileInfos = iFileInfoService.saveFileList(annexList, contractTemplatePath, FileTableEnum.TF_CONTRACT_TEMPLATE_P.getTableName());
        }
        TfContractTemplateP contractTemplateQuery = this.getById(contractTemplateP.getId());
        String agoContractAnnexFileInfo = contractTemplateQuery.getContractAnnex();
        String agoAnnexFileInfo = contractTemplateQuery.getAnnex();

        if(StringUtils.isNotBlank(agoContractAnnexFileInfo)){
            agoContractAnnexFileInfos = JSON.parseArray(agoContractAnnexFileInfo,FileInfo.class);
        }
        if(StringUtils.isNotBlank(agoAnnexFileInfo)){
            agoAnnexFileInfos = JSON.parseArray(agoAnnexFileInfo,FileInfo.class);
        }
        if(agoContractAnnexFileInfos != null && nowContractAnnexFileInfos != null){
//            List<FileInfo> ords = Stream.of(agoContractAnnexFileInfos, nowContractAnnexFileInfos)
//                    .flatMap(Collection::stream)
//                    .distinct()
//                    .collect(Collectors.toList());
            List<FileInfo> finalNowContractAnnexFileInfos = nowContractAnnexFileInfos;
            List<FileInfo> reduce = agoContractAnnexFileInfos.stream().filter(item -> !finalNowContractAnnexFileInfos.contains(item)).collect(Collectors.toList());
            iFileInfoService.removeFileList(reduce);
            upContractAnnexFileInfos.addAll(nowContractAnnexFileInfos);
        }
        if(agoAnnexFileInfos != null && nowAnnexFileInfos != null){
//            List<FileInfo> ords = Stream.of(agoAnnexFileInfos, nowAnnexFileInfos)
//                    .flatMap(Collection::stream)
//                    .distinct()
//                    .collect(Collectors.toList());
            List<FileInfo> finalNowAnnexFileInfos = nowAnnexFileInfos;
            List<FileInfo> reduce = agoAnnexFileInfos.stream().filter(item -> !finalNowAnnexFileInfos.contains(item)).collect(Collectors.toList());
            iFileInfoService.removeFileList(reduce);
            upAnnexFileInfos.addAll(nowAnnexFileInfos);
        }
        if(upContractAnnexFileInfos.size() != 0){
            contractTemplateP.setContractAnnex(JSON.toJSONString(upContractAnnexFileInfos));
        }
        if(upAnnexFileInfos.size() != 0){
            contractTemplateP.setAnnex(JSON.toJSONString(upAnnexFileInfos));
        }
        this.updateById(contractTemplateP);
        templateFieldPService.remove(new QueryWrapper<TfTemplateFieldP>().eq("TEMPLATE_ID",contractTemplateP.getId()));
        for(TfTemplateFieldP templateField : contractTemplatePDto.getTemplateFieldList()){
            templateField.setTemplateId(contractTemplateP.getId());
            templateFieldPService.save(templateField);
        }
    }

    @Override
    public void deleteTfContractTemplateP(List<Long> idList) {
        idList.forEach(info ->{
            TfContractTemplateP contractTemplateP = this.getById(info);
            if(contractTemplateP == null){
                contractTemplateP = new TfContractTemplateP();
            }
            String contractAnnex = contractTemplateP.getContractAnnex();
            String annex = contractTemplateP.getAnnex();
            if(StringUtils.isNotBlank(contractAnnex)){
                List<FileInfo> fileInfoList = JSON.parseArray(contractAnnex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
            templateFieldPService.remove(new QueryWrapper<TfTemplateFieldP>().eq("TEMPLATE_ID",info));
        });
        this.removeByIds(idList);
    }

}
