/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.baseconf.entity.TfMatterAccessP;
import com.tofly.workflow.baseconf.mapper.TfMatterAccessPMapper;
import com.tofly.workflow.baseconf.service.TfMatterAccessPService;
import com.tofly.workflow.common.FlowConstant;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 物资出入库
 *
 * @author JF
 * @date Wed Aug 10 00:00:00 CST 2022
 */
@Service("tfMatterAccessPService")
public class TfMatterAccessPServiceImpl extends ServiceImpl<TfMatterAccessPMapper, TfMatterAccessP> implements TfMatterAccessPService {

    @Autowired
    private IFileInfoService iFileInfoService;
    @Value("${zmrq.ftp.matterAccess}")
    private String matterAccessPath;

    @Override
    public void saveTfMatterAccessP(TfMatterAccessP mp, List<MultipartFile> annexList) {
        int count = this.count(new QueryWrapper<TfMatterAccessP>().lambda().eq(TfMatterAccessP::getTenantId,mp.getTenantId())
                .eq(TfMatterAccessP::getWarehouseName,mp.getWarehouseName()).eq(TfMatterAccessP::getAccessType,mp.getAccessType())
                .eq(TfMatterAccessP::getIsDel, FlowConstant.SHORT_ZERO).eq(TfMatterAccessP::getIsValid,FlowConstant.SHORT_ONE));
        if(count > FlowConstant.INTEGER_ZERO){
            throw new RuntimeException("名称已经存在");
        }
        if(mp == null){
            return;
        }
        if(annexList != null && annexList.size() != 0){
            List<FileInfo> annexFileInfo = iFileInfoService.saveFileList(annexList, matterAccessPath, FileTableEnum.TF_CONTRACT_TEMPLATE_P.getTableName());
            mp.setAnnex(JSON.toJSONString(annexFileInfo));
        }
        if(mp.getDateTime()==null) mp.setDateTime(new Date());
        this.save(mp);
    }

    @Override
    public void updateTfMatterAccessP(TfMatterAccessP mp, List<MultipartFile> annexList) {
        if(mp == null||mp.getId() == null){
            throw new ToflyDeniedException("修改的对象不能为空或ID为空，请重新输入");
        }
        int count = this.count(new QueryWrapper<TfMatterAccessP>().lambda().eq(TfMatterAccessP::getTenantId,mp.getTenantId())
                .eq(TfMatterAccessP::getWarehouseName,mp.getWarehouseName()).eq(TfMatterAccessP::getAccessType,mp.getAccessType())
                .eq(TfMatterAccessP::getIsDel, FlowConstant.SHORT_ZERO).eq(TfMatterAccessP::getIsValid,FlowConstant.SHORT_ONE)
                .ne(TfMatterAccessP::getId,mp.getId()));
        if(count > FlowConstant.INTEGER_ZERO){
            throw new RuntimeException("名称已经存在");
        }

        List<FileInfo> nowAnnexFileInfos = null;
        List<FileInfo> agoAnnexFileInfos = null;
        List<FileInfo> upAnnexFileInfos = new ArrayList<>();
        if(StringUtils.isNotBlank(mp.getAnnex())){
            nowAnnexFileInfos = JSON.parseArray(mp.getAnnex(),FileInfo.class);
        }
        if (null != annexList && annexList.size() > 0) {
            upAnnexFileInfos = iFileInfoService.saveFileList(annexList, matterAccessPath, FileTableEnum.TF_MATTER_P.getTableName());
        }
        TfMatterAccessP matterAccessQuery = this.getById(mp.getId());
        String agoAnnexFileInfo = matterAccessQuery.getAnnex();

        if(StringUtils.isNotBlank(agoAnnexFileInfo)){
            agoAnnexFileInfos = JSON.parseArray(agoAnnexFileInfo,FileInfo.class);
        }
        if(agoAnnexFileInfos != null && nowAnnexFileInfos != null){
//            List<FileInfo> ords = Stream.of(agoAnnexFileInfos, nowAnnexFileInfos)
//                    .flatMap(Collection::stream)
//                    .distinct()
//                    .collect(Collectors.toList());
            List<FileInfo> finalNowAnnexFileInfos = nowAnnexFileInfos;
            List<FileInfo> reduce = agoAnnexFileInfos.stream().filter(item -> !finalNowAnnexFileInfos.contains(item)).collect(Collectors.toList());
            iFileInfoService.removeFileList(reduce);
            upAnnexFileInfos.addAll(nowAnnexFileInfos);
        }
        if(upAnnexFileInfos.size() != 0){
            mp.setAnnex(JSON.toJSONString(upAnnexFileInfos));
        }
        this.updateById(mp);
    }

    @Override
    public void deleteTfMatterAccessP(List<Long> idList) {
        idList.forEach(info ->{
            TfMatterAccessP matterAccessP = this.getById(info);
            String annex = matterAccessP.getAnnex();
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
        });
        this.removeByIds(idList);
    }

}
