/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.baseconf.entity.TfMatterNumberP;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.entity.vo.MatterNumberVo;
import com.tofly.workflow.baseconf.mapper.TfMatterNumberPMapper;
import com.tofly.workflow.baseconf.service.TfMatterNumberPService;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.buss.service.TfProjectMaterialWService;
import com.tofly.workflow.common.FlowConstant;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 物资出入库数量
 *
 * @author JF
 * @date Wed Aug 10 00:00:00 CST 2022
 */
@Slf4j
@Service("tfMatterNumberPService")
public class TfMatterNumberPServiceImpl extends ServiceImpl<TfMatterNumberPMapper, TfMatterNumberP> implements TfMatterNumberPService {

    @Autowired
    private TfProjectMaterialWService pMaterialService;

    @Autowired
    private TfMatterPService matterPService;
    @Override
    public List<TfMatterNumberP> getTfMatterNumberPList(TfMatterNumberP tfMatterNumberP) {
        List<TfMatterNumberP> matterNumberPList = this.list(Wrappers.query(tfMatterNumberP));
        TfMatterP matterPQuery = new TfMatterP();
        for (TfMatterNumberP one : matterNumberPList){
            matterPQuery.setId(one.getMatterId());
            one.setMatterP(matterPService.getTfMatterPList(matterPQuery).get(0));
        }
        return matterNumberPList;
    }

    /**
     * 流程完后执行出入库操作
     * @param id 流程id
     * @param type 出入库类型 0 入库 1出库/核销
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void inOrOutMaterial(Long id, Short type) {
        List<MatterNumberVo> vos = pMaterialService.getInOrOutMaterial(id,type);
        if(CollUtil.isEmpty(vos)){
            log.error("流程ID：："+id +" 为获取到相应数据无法执行 出入库 type::"+type);
            return;
        }
        List<TfMatterNumberP> saves = new ArrayList<>();
        List<TfMatterNumberP> updates = new ArrayList<>();
        QueryWrapper<TfMatterNumberP> qw = new QueryWrapper<>();
        qw.lambda().eq(TfMatterNumberP::getIsDel,FlowConstant.SHORT_ZERO).eq(TfMatterNumberP::getIsValid,FlowConstant.SHORT_ONE)
                .and(r->{
                    for (MatterNumberVo vo:vos) {
                        r.or(l->l.eq(TfMatterNumberP::getMatterId,vo.getMatterId()).eq(TfMatterNumberP::getAccessId,vo.getAccessId()));
                    }
                });
        List<TfMatterNumberP> list = this.list(qw);
        Map<String,TfMatterNumberP> map=null;
        if(CollUtil.isNotEmpty(list)){
            map = new HashMap<>();
            for (TfMatterNumberP mn:list) {
                String key = mn.getMatterId()+"_"+mn.getAccessId();
                map.put(key,mn);
            }
        }
        if(FlowConstant.SHORT_EIGHT.equals(type)){
            for (MatterNumberVo vo:vos ) {
                String key = vo.getMatterId()+"_"+vo.getAccessId();
                if(map!=null&&map.containsKey(key)){
                    TfMatterNumberP mn= map.get(key);
                    mn.setQuantity(mn.getQuantity()+vo.getQuantity());
                    mn.setInQuantity(mn.getInQuantity()+vo.getQuantity());
                    updates.add(mn);
                }else {
                    TfMatterNumberP mn= new TfMatterNumberP();
                    mn.setMatterId(vo.getMatterId());
                    mn.setAccessId(vo.getAccessId());
                    mn.setQuantity(vo.getQuantity());
                    mn.setInQuantity(vo.getQuantity());
                    mn.setOutQuantity(FlowConstant.LONG_ZERO);
                    mn.setIsDel(FlowConstant.SHORT_ZERO);
                    mn.setIsValid(FlowConstant.SHORT_ONE);
                    saves.add(mn);
                }
            }
        }else if(FlowConstant.SHORT_NINE.equals(type)){
            for (MatterNumberVo vo:vos ) {
                String key = vo.getMatterId() + "_" + vo.getAccessId();
                if (map != null && map.containsKey(key)) {
                    TfMatterNumberP mn= map.get(key);
                    mn.setQuantity(mn.getQuantity()-vo.getQuantity());
                    mn.setOutQuantity(mn.getOutQuantity()+vo.getQuantity());
                    updates.add(mn);
                }
            }

        }
        if(CollUtil.isNotEmpty(saves)){
            this.saveBatch(saves);
        }
        if(CollUtil.isNotEmpty(updates)){
            this.updateBatchById(updates);
        }
    }
}
