/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.baseconf.entity.TfProcessSqlP;
import com.tofly.workflow.baseconf.entity.dto.*;
import com.tofly.workflow.baseconf.mapper.TfProcessSqlPMapper;
import com.tofly.workflow.baseconf.service.TfProcessSqlPService;
import com.tofly.workflow.common.FlowConstant;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 流程查询sql
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Slf4j
@Service("tfProcessSqlPService")
public class TfProcessSqlPServiceImpl extends ServiceImpl<TfProcessSqlPMapper, TfProcessSqlP> implements TfProcessSqlPService {

    /**
     * 逻辑删除sql查询
     * @param id 主键ID
     * @return 修改结果
     */
    @Override
    public boolean removeUpdateById(Long id) {
        return update(new UpdateWrapper<TfProcessSqlP>().lambda().set(TfProcessSqlP::getIsDel, FlowConstant.SHORT_ONE)
                .eq(TfProcessSqlP::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(TfProcessSqlP::getId, id));
    }

    /**
     * 根据id查询数据列表
     * @param page 分页
     * @param dto sql的ID及查询条件
     * @return
     */
    @Override
    public Object getProcesssData(Page page, ProcessLDto dto) {
        /**获取执行的sql语句*/
        TfProcessSqlP psql = this.getById(dto.getId());
        if(psql==null || psql.getIsDel()==FlowConstant.SHORT_ONE){
            return null;
        }
        if(StringUtils.isEmpty(psql.getContent())){
            return null;
        }
        /**查询语句实体*/
        SqlSelectDto sqldto = JSONObject.parseObject(psql.getContent(),SqlSelectDto.class);
        Map<String,String> map = new HashMap<>();
        Map<String,Object> mapv = dto.getMap()==null?new HashMap<>():dto.getMap();
        /**拼接每个次序中的查询条件*/
        if(CollUtil.isNotEmpty(sqldto.getWhere())) {
            for (SqlWhereDto wdto : sqldto.getWhere()) {
                String key = "#{" + wdto.getNumber() + "}";
                String sql = null;
                if (CollUtil.isNotEmpty(mapv)) {
                    StringBuilder sb = new StringBuilder();
                    if (CollUtil.isNotEmpty(wdto.getCondition())) {
                        for (SqlConditionDto cdto : wdto.getCondition()) {
                            if (CollUtil.isNotEmpty(cdto.getNames())) {
                                Map<String,StringBuilder> mapvalue = new HashMap<>();
                                boolean isif = true;
                                for (String name : cdto.getNames()) {
                                    if (!mapv.containsKey(name)) {
                                        isif = false;
                                        break;
                                    }
                                    if(mapv.get(name) instanceof ArrayList){
                                        String mk = "#{"+name+"}";
                                        StringBuilder msb = new StringBuilder();
                                        for (Object o:(List)mapv.get(name)) {
                                            msb.append(o).append(",");
                                        }
                                        msb.deleteCharAt(msb.length()-1);
                                        mapvalue.put(mk,msb);
                                    }
                                }
                                if (isif && StringUtils.isNotEmpty(cdto.getSql())) {
                                    if(CollUtil.isNotEmpty(mapvalue)){
                                        String sqlw = cdto.getSql();
                                        for (Map.Entry<String,StringBuilder> e:mapvalue.entrySet()
                                             ) {
                                            sqlw = sqlw.replace(e.getKey(),e.getValue().toString());
                                        }
                                        sb.append(sqlw);
                                    }else {
                                        sb.append(cdto.getSql());
                                    }
                                } else if (!isif && StringUtils.isNotEmpty(cdto.getElsesql())) {
                                    sb.append(cdto.getElsesql());
                                }
                            }
                        }
                    }
                    sql = sb.toString();
//                    if (wdto.getNumber() == FlowConstant.INTEGER_ONE) {
//                        sql = sql.replaceFirst("AND", "");
//                    }
                }
                map.put(key, sql);
            }
        }
        String sql = sqldto.getSelect();
        /**拼接整体执行的查询sql*/
        for (Map.Entry<String,String> entry:map.entrySet()) {
            String value = entry.getValue();
            if(StringUtils.isNotEmpty(value)){
                int kindex = sql.indexOf(entry.getKey());
                int windex = sql.lastIndexOf("WHERE",kindex);
                if(sql.substring(windex,kindex).trim().length()==0){
                    if(value.trim().indexOf("AND") == 0) {
                        value = value.replaceFirst("AND", "");
                    }
                }
            }
            sql = sql.replace(entry.getKey(),value==null?"":value);
        }
        /**执行查询总数*/
        String countsql = "SELECT COUNT(1) CT FROM ("+sql+") COUNT_TABLE";
        mapv.put("sql",countsql);
        int count = this.baseMapper.getCount(mapv);
        /**分页查询*/
        if (page.getSize() * (page.getCurrent() - 1) <= count) {
            // 当前页第一条数据在List中的位置
            int start = (int) ((page.getCurrent() - 1) * page.getSize());
            // 当前页最后一条数据在List中的位置
            int end = (int) ((start + page.getSize()) > count ? count : (page.getSize() * page.getCurrent()));
            page.setRecords(new ArrayList<>());
            String pagesql = "SELECT * FROM ( SELECT ROWNUM r, COUNT_TABLE.* FROM ("+sql+") COUNT_TABLE) FY_TABLE WHERE r BETWEEN "+(start==0?start:start+1)+ " AND "+end;
            mapv.put("sql",pagesql);
            log.info(pagesql);
            List<Map<String, Object>> datas = this.baseMapper.getProcesss(mapv);
            // 分隔列表 当前页存在数据时 设置
            page.setRecords(datas);
        }
        page.setTotal(count);

        return page;
    }

    /**
     * 分页查询sql 语句
     * @param page 分页
     * @param dto 查询条件
     * @return
     */
    @Override
    public Page<TfProcessSqlP> pageProcessSql(Page page, PSqlPageDto dto) {
        return this.page(page,new QueryWrapper<TfProcessSqlP>().lambda().eq(dto.getId()!=null,TfProcessSqlP::getId,dto.getId())
                .eq(dto.getEdition()!=null,TfProcessSqlP::getEdition,dto.getEdition())
                .eq(StringUtils.isNotEmpty(dto.getModel()),TfProcessSqlP::getModel,dto.getModel())
                .and(StringUtils.isNotEmpty(dto.getContent()),r->r.like(TfProcessSqlP::getModel,dto.getContent()).or().like(TfProcessSqlP::getRemark,dto.getContent())));
    }

}
