/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.mapper.SysCommonMapper;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.workflow.baseconf.entity.TfProjectProcessP;
import com.tofly.workflow.baseconf.entity.dto.TfProjectProcessDto;
import com.tofly.workflow.baseconf.mapper.TfProjectProcessPMapper;
import com.tofly.workflow.baseconf.service.TfProjectProcessPService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 工程工序/工程管控
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfProjectProcessPService")
public class TfProjectProcessPServiceImpl extends ServiceImpl<TfProjectProcessPMapper, TfProjectProcessP> implements TfProjectProcessPService {

    @Autowired
    private SysCommonMapper commonMapper;

    @Override
    public List<Map<String, Object>> getProjectNumByNature(TfProjectProcessP tfProjectProcessP) {
        List<Map<String, Object>> list = this.baseMapper.getProjectNumByNature(tfProjectProcessP);
        for(Map<String, Object> one : list){
            if(one.get("nature") == null || one.get("tenantId") == null){
                continue;
            }
            Long nature = Long.parseLong(String.valueOf(one.get("nature")));
            Long tenantId = Long.parseLong(String.valueOf(one.get("tenantId")));
            final List<TfProjectProcessP> list1 = this.list(new QueryWrapper<TfProjectProcessP>().eq("NATURE", nature)
                    .eq("TENANT_ID", tenantId)
                    .eq(tfProjectProcessP.getIsForce() != null, "IS_FORCE", tfProjectProcessP.getIsForce())
                    .eq("IS_DEL", (short) 0));
            one.put("list",list1);
            one.put("tenantName",commonMapper.getTenantName(tenantId));
            one.put("natureName",this.baseMapper.getNatureName(nature));
            one.put("total",list1.size());
        }
        return list;
    }

    @Override
    public void saveProjectProcess(TfProjectProcessDto dto) {
        if(dto == null){
            return;
        }
        if(dto.getTenantId() == null){
            throw new ToflyDeniedException("租户ID为空");
        }
        if(dto.getNature() == null){
            throw new ToflyDeniedException("工程性质为空");
        }
        if(dto.getProjectProcessList() == null || dto.getProjectProcessList().size() == 0){
            throw  new ToflyDeniedException("工程工序列表为空");
        }
        TfProjectProcessP projectProcessUp = new TfProjectProcessP();
        projectProcessUp.setIsDel((short)1);
        this.update(projectProcessUp, new QueryWrapper<TfProjectProcessP>()
                .eq("NATURE",dto.getNature())
                .eq("TENANT_ID",dto.getTenantId())
                .eq("IS_DEL",(short)0));
        List<TfProjectProcessP> projectProcessList = dto.getProjectProcessList();
        for (TfProjectProcessP projectProcess : projectProcessList) {
            projectProcess.setTenantId(dto.getTenantId());
            projectProcess.setNature(dto.getNature());
        }
        this.saveBatch(projectProcessList);
    }

}
