/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.baseconf.service.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfSequence;
import com.tofly.workflow.baseconf.entity.dto.SequenceDto;
import com.tofly.workflow.baseconf.service.TfSequenceService;
import com.tofly.workflow.common.FlowConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import com.tofly.workflow.baseconf.mapper.TfSequenceMapper;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 自增序列
 *
 * @author xqf
 * @date Tue Dec 13 00:00:00 CST 2022
 */
@Service("tfSequenceService")
public class TfSequenceServiceImpl extends ServiceImpl<TfSequenceMapper, TfSequence> implements TfSequenceService {

    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;

    private final String REGEX = "^[0-9a-zA-Z]+$";

    private final String YEAR = "yyyy";
    private final String MOTH = "yyyyMM";
    private final String DAY = "yyyyMMdd";
    private final String HOUR = "yyyyMMddHH";

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized Object getSequenceValue(SequenceDto dto) {
        if(!dto.getSName().matches(REGEX)){
            return "sName不符合要求！";
        }
        Date date = new Date();
        String dt = null;
        Date vTime = new Date();
        TimeUnit tu = null;
        long timeOut = 0l;
        if(dto.getSType().equals(FlowConstant.SHORT_ZERO)){
            dt = "0";
            vTime = null;
            tu = TimeUnit.DAYS;
            timeOut = 30l;
        }else if(dto.getSType().equals(FlowConstant.SHORT_ONE)){
            dt = DateUtil.format(date, HOUR);
            tu = TimeUnit.MINUTES;
            vTime = DateUtil.offsetHour(DateUtil.beginOfHour(date), 1);
            timeOut = DateUtil.between(date, vTime, DateUnit.MINUTE);
        }else if(dto.getSType().equals(FlowConstant.SHORT_TWO)){
            dt = DateUtil.format(date, DAY);
            tu = TimeUnit.HOURS;
            vTime = DateUtil.offsetDay(DateUtil.beginOfDay(date), 1);
            timeOut = DateUtil.between(date, vTime, DateUnit.HOUR);
        }else if(dto.getSType().equals(FlowConstant.SHORT_THREE)){
            tu = TimeUnit.DAYS;
            dt = DateUtil.format(date, MOTH);
            vTime = DateUtil.offsetMonth(DateUtil.beginOfMonth(date), 1);
            timeOut = DateUtil.between(date, vTime, DateUnit.DAY);
        }else if(dto.getSType().equals(FlowConstant.SHORT_FOUR)){
            dt = DateUtil.format(date, YEAR);
            tu = TimeUnit.DAYS;
            timeOut = 30l;
            vTime = DateUtil.offsetMonth(DateUtil.beginOfYear(date), 12);
        }

        TfSequence sequence =this.getOne(new QueryWrapper<TfSequence>().lambda().eq(TfSequence::getSName,dto.getSName()).eq(TfSequence::getSType,dto.getSType())
                .eq(TfSequence::getTenantId, SecurityUtils.getTenantId()).eq(TfSequence::getIsDel,FlowConstant.SHORT_ZERO));
        if(sequence!=null&&sequence.getValidTime().before(date)){
            this.removeById(sequence.getId());
            sequence = null;
        }

        String key = FlowConstant.REDIS_SEQUENCE_KEY+SecurityUtils.getTenantId()+":"+dto.getSName()+"-"+dt;
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
        Long value = counter.incrementAndGet();
        if(value == 1) {
            if(sequence!=null){
                counter.set(sequence.getNumberValue());
                counter.expire(timeOut, tu);
                value = counter.incrementAndGet();
            }
        }
        if(sequence!=null){
            sequence.setNumberValue(value);
            this.updateById(sequence);
        }else {
            sequence = new TfSequence();
            sequence.setNumberValue(value);
            sequence.setTenantId(SecurityUtils.getTenantId());
            sequence.setSName(dto.getSName());
            sequence.setSType(dto.getSType());
            sequence.setDateTime(date);
            sequence.setValidTime(vTime);
            sequence.setIsDel(FlowConstant.SHORT_ZERO);
            this.save(sequence);
        }
        return value;
    }

}
