/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.buss.entity.TfAcceptanceApplicationW;
import com.tofly.workflow.buss.service.TfAcceptanceApplicationWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
/**
 * 验收申请
 *
 * @author JF
 * @date Tue Sep 20 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfacceptanceapplicationw")
@Api(tags="验收申请接口")
public class TfAcceptanceApplicationWController {

    private final  TfAcceptanceApplicationWService tfAcceptanceApplicationWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfAcceptanceApplicationW 验收申请
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfAcceptanceApplicationWPage(Page page, @ModelAttribute TfAcceptanceApplicationW tfAcceptanceApplicationW) {
      return ResultData.ok(tfAcceptanceApplicationWService.page(page, Wrappers.query(tfAcceptanceApplicationW)));
    }


    /**
     * 通过id查询验收申请
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfAcceptanceApplicationWService.getById(id));
    }

    /**
     * 新增验收申请
     * @param tfAcceptanceApplicationW 验收申请
     * @return ResultData
     */
    @ToFlyAppLog(title="新增验收申请" )
    @ApiOperation(value = "新增验收申请")
    @PostMapping
    public ResultData save(@RequestBody TfAcceptanceApplicationW tfAcceptanceApplicationW) {
      return ResultData.ok(tfAcceptanceApplicationWService.save(tfAcceptanceApplicationW));
    }

    /**
     * 修改验收申请
     * @param tfAcceptanceApplicationW 验收申请
     * @return ResultData
     */
    @ToFlyAppLog(title="修改验收申请" )
    @ApiOperation(value = "修改验收申请")
    @PutMapping
    public ResultData updateById(@RequestBody TfAcceptanceApplicationW tfAcceptanceApplicationW) {
      return ResultData.ok(tfAcceptanceApplicationWService.updateById(tfAcceptanceApplicationW));
    }

    /**
     * 通过id删除验收申请
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除验收申请" )
    @ApiOperation(value = "通过ID删除验收申请")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfAcceptanceApplicationWService.removeById(id));
    }

    /**
     * 通过id批量删除验收申请
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除验收申请" )
    @ApiOperation(value = "通过ID批量删除验收申请")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfAcceptanceApplicationWService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
