/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.buss.entity.TfContractInfoW;
import com.tofly.workflow.buss.service.TfContractInfoWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
/**
 * 合同详情信息
 *
 * @author JF
 * @date Wed Aug 24 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfcontractinfow")
@Api(tags="合同详情信息接口")
public class TfContractInfoWController {

    private final  TfContractInfoWService tfContractInfoWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfContractInfoW 合同详情信息
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfContractInfoWPage(Page page, @ModelAttribute TfContractInfoW tfContractInfoW) {
      return ResultData.ok(tfContractInfoWService.page(page, Wrappers.query(tfContractInfoW)));
    }


    /**
     * 通过id查询合同详情信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfContractInfoWService.getById(id));
    }

    /**
     * 新增合同详情信息
     * @param tfContractInfoW 合同详情信息
     * @return ResultData
     */
    @ToFlyAppLog(title="新增合同详情信息" )
    @ApiOperation(value = "新增合同详情信息")
    @PostMapping
    public ResultData save(@RequestBody TfContractInfoW tfContractInfoW) {
      return ResultData.ok(tfContractInfoWService.save(tfContractInfoW));
    }

    /**
     * 修改合同详情信息
     * @param tfContractInfoW 合同详情信息
     * @return ResultData
     */
    @ToFlyAppLog(title="修改合同详情信息" )
    @ApiOperation(value = "修改合同详情信息")
    @PutMapping
    public ResultData updateById(@RequestBody TfContractInfoW tfContractInfoW) {
      return ResultData.ok(tfContractInfoWService.updateById(tfContractInfoW));
    }

    /**
     * 通过id删除合同详情信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除合同详情信息" )
    @ApiOperation(value = "通过ID删除合同详情信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfContractInfoWService.removeById(id));
    }

    /**
     * 通过id批量删除合同详情信息
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除合同详情信息" )
    @ApiOperation(value = "通过ID批量删除合同详情信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfContractInfoWService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
