/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.buss.entity.TfCostSumsW;
import com.tofly.workflow.buss.entity.dto.CostSumsDto;
import com.tofly.workflow.buss.service.TfCostSumsWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
/**
 * 费用汇总
 *
 * @author xqf
 * @date Wed Feb 15 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfcostsumsw")
@Api(tags="费用汇总接口")
public class TfCostSumsWController {

    private final  TfCostSumsWService tfCostSumsWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfCostSumsW 费用汇总
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfCostSumsWPage(Page page, @ModelAttribute TfCostSumsW tfCostSumsW) {
      return ResultData.ok(tfCostSumsWService.page(page, Wrappers.query(tfCostSumsW)));
    }


    /**
     * 通过id查询费用汇总
     * @param id 主流程ID
     * @return ResultData
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主流程ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfCostSumsWService.getByProcessId(id));
    }

    /**
     * 新增费用汇总
     * @param dto 费用汇总
     * @return ResultData
     */
    @ToFlyAppLog(title="新增费用汇总" )
    @ApiOperation(value = "新增费用汇总")
    @PostMapping
    public ResultData save(@Valid @RequestBody CostSumsDto dto) {
        Object obj = tfCostSumsWService.saveCostSums(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除费用汇总
     * @param processId id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除费用汇总" )
    @ApiOperation(value = "通过ID删除费用汇总")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "processId", value = "主流程ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long processId) {
      return ResultData.ok(tfCostSumsWService.removeByprocessId(processId));
    }

}
