/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfDesignBudgetW;
import com.tofly.workflow.buss.service.TfDesignBudgetWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 设计预算
 *
 * @author JF
 * @date Mon Aug 22 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfdesignbudgetw")
@Api(tags="设计预算接口")
public class TfDesignBudgetWController {

    private final  TfDesignBudgetWService tfDesignBudgetWService;
    private final IFileInfoService iFileInfoService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfDesignBudgetW 设计预算
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfDesignBudgetWPage(Page page, @ModelAttribute TfDesignBudgetW tfDesignBudgetW) {
      return ResultData.ok(tfDesignBudgetWService.page(page, Wrappers.query(tfDesignBudgetW)));
    }


    /**
     * 通过id查询设计预算
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfDesignBudgetWService.getById(id));
    }

    /**
     * 新增设计预算
     * @param tfDesignBudgetW 设计预算
     * @return ResultData
     */
    @ToFlyAppLog(title="新增设计预算" )
    @ApiOperation(value = "新增设计预算")
    @PostMapping
    public ResultData save(@RequestBody TfDesignBudgetW tfDesignBudgetW) {
      return ResultData.ok(tfDesignBudgetWService.save(tfDesignBudgetW));
    }

    /**
     * 修改设计预算
     * @param tfDesignBudgetW 设计预算
     * @return ResultData
     */
    @ToFlyAppLog(title="修改设计预算" )
    @ApiOperation(value = "修改设计预算")
    @PutMapping
    public ResultData updateById(@RequestBody TfDesignBudgetW tfDesignBudgetW) {
      return ResultData.ok(tfDesignBudgetWService.updateById(tfDesignBudgetW));
    }

    /**
     * 通过id删除设计预算
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除设计预算" )
    @ApiOperation(value = "通过ID删除设计预算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfDesignBudgetWService.removeById(id));
    }

    /**
     * 通过id批量删除设计预算
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除设计预算" )
    @ApiOperation(value = "通过ID批量删除设计预算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids为空");
        }
        List<Long> idList = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        idList.forEach(info ->{
            TfDesignBudgetW entity = tfDesignBudgetWService.getById(info);
            if(entity == null){
                entity = new TfDesignBudgetW();
            }
            String annex = entity.getAnnex();
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
            String costAnnex = entity.getCostAnnex();
            if(StringUtils.isNotBlank(costAnnex)){
                List<FileInfo> fileInfoList = JSON.parseArray(costAnnex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }String drawingAnnex = entity.getDrawingAnnex();
            if(StringUtils.isNotBlank(drawingAnnex)){
                List<FileInfo> fileInfoList = JSON.parseArray(drawingAnnex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
            tfDesignBudgetWService.remove(new QueryWrapper<TfDesignBudgetW>().eq("ID",info));
        });
        tfDesignBudgetWService.removeByIds(idList);
        return ResultData.ok();
    }
}
