/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfMonitoringReportW;
import com.tofly.workflow.buss.service.TfMonitoringReportWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 监检提交上报登记(监检上报、监检登记）
 *
 * @author JF
 * @date Tue Aug 30 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfmonitoringreportw")
@Api(tags="监检提交上报登记(监检上报、监检登记）接口")
public class TfMonitoringReportWController {

    private final  TfMonitoringReportWService tfMonitoringReportWService;
    private final IFileInfoService iFileInfoService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfMonitoringReportW 监检提交上报登记(监检上报、监检登记）
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfMonitoringReportWPage(Page page, @ModelAttribute TfMonitoringReportW tfMonitoringReportW) {
      return ResultData.ok(tfMonitoringReportWService.page(page, Wrappers.query(tfMonitoringReportW)));
    }


    /**
     * 通过id查询监检提交上报登记(监检上报、监检登记）
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfMonitoringReportWService.getById(id));
    }

    /**
     * 新增监检提交上报登记(监检上报、监检登记）
     * @param tfMonitoringReportW 监检提交上报登记(监检上报、监检登记）
     * @return ResultData
     */
    @ToFlyAppLog(title="新增监检提交上报登记(监检上报、监检登记）" )
    @ApiOperation(value = "新增监检提交上报登记(监检上报、监检登记）")
    @PostMapping
    public ResultData save(@RequestBody TfMonitoringReportW tfMonitoringReportW) {
      return ResultData.ok(tfMonitoringReportWService.save(tfMonitoringReportW));
    }

    /**
     * 修改监检提交上报登记(监检上报、监检登记）
     * @param tfMonitoringReportW 监检提交上报登记(监检上报、监检登记）
     * @return ResultData
     */
    @ToFlyAppLog(title="修改监检提交上报登记(监检上报、监检登记）" )
    @ApiOperation(value = "修改监检提交上报登记(监检上报、监检登记）")
    @PutMapping
    public ResultData updateById(@RequestBody TfMonitoringReportW tfMonitoringReportW) {
      return ResultData.ok(tfMonitoringReportWService.updateById(tfMonitoringReportW));
    }

    /**
     * 通过id删除监检提交上报登记(监检上报、监检登记）
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除监检提交上报登记(监检上报、监检登记）" )
    @ApiOperation(value = "通过ID删除监检提交上报登记(监检上报、监检登记）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfMonitoringReportWService.removeById(id));
    }

    /**
     * 通过id批量删除监检提交上报登记(监检上报、监检登记）
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除监检提交上报登记(监检上报、监检登记）" )
    @ApiOperation(value = "通过ID批量删除监检提交上报登记(监检上报、监检登记）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids为空");
        }
        List<Long> idList = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        idList.forEach(info ->{
            TfMonitoringReportW entity = tfMonitoringReportWService.getById(info);
            if(entity == null){
                entity = new TfMonitoringReportW();
            }
            String annex = entity.getAnnex();
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
            String checkAnnex = entity.getCheckAnnex();
            if(StringUtils.isNotBlank(checkAnnex)){
                List<FileInfo> fileInfoList = JSON.parseArray(checkAnnex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }String reportAnnex = entity.getReportAnnex();
            if(StringUtils.isNotBlank(reportAnnex)){
                List<FileInfo> fileInfoList = JSON.parseArray(reportAnnex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
            tfMonitoringReportWService.remove(new QueryWrapper<TfMonitoringReportW>().eq("ID",info));
        });
        tfMonitoringReportWService.removeByIds(idList);
        return ResultData.ok();
    }
}
