/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.core.result.ResultData;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.dto.BuilderBaseDto;
import com.tofly.workflow.buss.entity.dto.ProjectBaseDto;
import com.tofly.workflow.buss.entity.dto.TallyDateDto;
import com.tofly.workflow.buss.entity.vo.ProjectBaseVo;
import com.tofly.workflow.buss.entity.vo.ThirdBaseInfo;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.process.entity.vo.IndoorRenovationVo;
import com.tofly.workflow.report.vo.SignApplyVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 工程基础表
 *
 * @author xqf
 * @date Wed Sep 14 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprojectbasew")
@Api(tags = "工程基础表接口")
public class TfProjectBaseWController {

    private final TfProjectBaseWService tfProjectBaseWService;

    /**
     * 分页查询
     *
     * @param page           分页对象
     * @param dto 工程基础表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultData getTfProjectBaseWPage(Page page, @ModelAttribute ProjectBaseDto dto) {
        return ResultData.ok(tfProjectBaseWService.pageProjectBase(page, dto));
    }


    /**
     * 通过id查询工程基础表 ,id 或 processId 必传其一
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/getbase")
    @ApiOperation(value = "通过ID查询,id 或 processId 必传其一")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "processId", value = "processId", required = false, dataType = "Long")
    })
    public ResultData getById(Long id,Long processId) {
        return ResultData.ok(tfProjectBaseWService.getBaseById(id,processId));
    }

    /**
     * 户内改造工程台账
     *
     * @param
     * @return ResultRespone
     */
    @GetMapping("/indoorRenovationReport")
    @ApiOperation(value = "户内改造工程台账")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tenantId", value = "公司（多个用,分隔）", dataType = "String"),
            @ApiImplicitParam(name = "projectNature", value = "工程性质（多个用,分隔）", dataType = "String"),
            @ApiImplicitParam(name = "processType", value = "工程类型（多个用,分隔）", dataType = "String"),
            @ApiImplicitParam(name = "likeStr", value = "关键字(工程名称、申请安装地址)", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "发起开始时间(yyyy-MM-dd HH:mm:ss)", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "发起结束时间(yyyy-MM-dd HH:mm:ss)", dataType = "String"),
            @ApiImplicitParam(name = "isExport", value = "是否导出(0否1是)", required = true, dataType = "String")
    })
    public ResultData IndoorRenovationReport(Page page,String tenantId,String projectNature,String processType,String likeStr,String startTime,String endTime,String isExport) {
        Page<IndoorRenovationVo> vos = tfProjectBaseWService.IndoorRenovationReport(page,tenantId,projectNature,processType,likeStr,startTime,endTime);
        if (isExport.equals("1")) {
            ExcelUtils.getListToExcel(IndoorRenovationVo.class,vos.getRecords(),"户内改造工程台账");
        }
        return ResultData.ok(vos);
    }

    /**
     * 通过id查询工程基础表 ,id 或 processId 必传其一
     *
     * @return ResultRespone
     */
    @GetMapping("/thirdGetbase")
    @ApiOperation(value = "通过ID查询,id 或 processId 必传其一")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = false, dataType = "Long")
    })
    public ResultData thirdGetbase(Long id) {


        final ProjectBaseVo baseById = tfProjectBaseWService.getBaseById(null, id);
        ThirdBaseInfo thirdBaseInfo = new ThirdBaseInfo();
        BeanUtils.copyProperties(baseById,thirdBaseInfo);

        return ResultData.ok(thirdBaseInfo);
    }

    /**
     * 施工人员查询自己当前关联的施工项目
     *
     * @param dto 施工人员Id
     * @return ResultRespone
     */
    @PostMapping("/processByBuilder")
    @ApiOperation(value = "施工人员查询自己当前关联的施工项目")
    public ResultData processByBuilder(Page page, BuilderBaseDto dto){
        return ResultData.ok(tfProjectBaseWService.getProjectBaseByBuilder(page,dto));
    }



    /**
     * 计算日期
     *
     * @param dto
     * @return ResultRespone
     */
    @GetMapping("/tallyDate")
    @ApiOperation(value = "计算日期")
    public ResultData getTallyDate(TallyDateDto dto){
        Object obj = tfProjectBaseWService.getTallyDate(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }



}
