/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfProjectDispatchW;
import com.tofly.workflow.buss.service.TfProjectDispatchWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 工程派工
 *
 * @author JF
 * @date Mon Aug 29 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprojectdispatchw")
@Api(tags="工程派工接口")
public class TfProjectDispatchWController {

    private final  TfProjectDispatchWService tfProjectDispatchWService;
    private final IFileInfoService iFileInfoService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfProjectDispatchW 工程派工
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProjectDispatchWPage(Page page, @ModelAttribute TfProjectDispatchW tfProjectDispatchW) {
      return ResultData.ok(tfProjectDispatchWService.page(page, Wrappers.query(tfProjectDispatchW)));
    }


    /**
     * 通过id查询工程派工
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProjectDispatchWService.getById(id));
    }

    /**
     * 新增工程派工
     * @param tfProjectDispatchW 工程派工
     * @return ResultData
     */
    @ToFlyAppLog(title="新增工程派工" )
    @ApiOperation(value = "新增工程派工")
    @PostMapping
    public ResultData save(@RequestBody TfProjectDispatchW tfProjectDispatchW) {
      return ResultData.ok(tfProjectDispatchWService.save(tfProjectDispatchW));
    }

    /**
     * 修改工程派工
     * @param tfProjectDispatchW 工程派工
     * @return ResultData
     */
    @ToFlyAppLog(title="修改工程派工" )
    @ApiOperation(value = "修改工程派工")
    @PutMapping
    public ResultData updateById(@RequestBody TfProjectDispatchW tfProjectDispatchW) {
      return ResultData.ok(tfProjectDispatchWService.updateById(tfProjectDispatchW));
    }

    /**
     * 通过id删除工程派工
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除工程派工" )
    @ApiOperation(value = "通过ID删除工程派工")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfProjectDispatchWService.removeById(id));
    }

    /**
     * 通过id批量删除工程派工
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除工程派工" )
    @ApiOperation(value = "通过ID批量删除工程派工")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids为空");
        }
        List<Long> idList = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        idList.forEach(info ->{
            TfProjectDispatchW entity = tfProjectDispatchWService.getById(info);
            if(entity == null){
                entity = new TfProjectDispatchW();
            }
            String annex = entity.getAnnex();
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
            tfProjectDispatchWService.remove(new QueryWrapper<TfProjectDispatchW>().eq("ID",info));
        });
        tfProjectDispatchWService.removeByIds(idList);
        return ResultData.ok();
    }
}
