/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.buss.entity.TfProjectMaterialW;
import com.tofly.workflow.buss.entity.dto.DispatchRecordDto;
import com.tofly.workflow.buss.entity.dto.TfProjectMaterialDto;
import com.tofly.workflow.buss.entity.vo.TfProjectMaterialWVo;
import com.tofly.workflow.buss.service.TfProjectMaterialWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 工程材料(设计预算、领料出库)
 *
 * @author JF
 * @date Tue Aug 30 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprojectmaterialw")
@Api(tags="工程材料(设计预算、领料出库)接口")
public class TfProjectMaterialWController {

    private final  TfProjectMaterialWService tfProjectMaterialWService;
    private final TfMatterPService tfMatterPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 工程材料(设计预算、领料出库)DTO
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProjectMaterialPage(Page page, @ModelAttribute TfProjectMaterialDto dto) {
        if(StringUtils.isNotBlank(dto.getMaterialTypeName())){
            List<Long> idList = Arrays.stream(dto.getMaterialTypeName().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                    .collect(Collectors.toList());
            dto.setMaterialTypeList(idList);
        }
        Page<TfProjectMaterialW> projectMaterialWPage = tfProjectMaterialWService.getTfProjectMaterialPage(page, dto);
        for(TfProjectMaterialW one : projectMaterialWPage.getRecords()){
            one.setMatterP(tfMatterPService.getById(one.getMatterId()));
        }
        return ResultData.ok(projectMaterialWPage);
    }

    /**
     * 领料记录
     * @param dto
     * @return
     */
    @ToFlyAppLog(title="领料记录" )
    @GetMapping("/getDispatchRecord" )
    @ApiOperation(value = "领料记录")
    public ResultData getDispatchRecord(@Valid DispatchRecordDto dto) {
        return ResultData.ok(tfProjectMaterialWService.getDispatchRecord(dto));
    }

    /**
     * 按材料分组查询
     * @param dto 工程材料(设计预算、领料出库)Dto
     * @return
     */
    @ToFlyAppLog(title="按材料分组查询" )
    @GetMapping("/getListByMatter" )
    @ApiOperation(value = "按材料分组查询")
    public ResultData getListByMatter(TfProjectMaterialDto dto) {
        List<TfProjectMaterialWVo> list = tfProjectMaterialWService.getListByMatter(dto);
        if(list==null){
            return null;
        }
        return ResultData.ok(list);
    }


    /**
     * 通过id查询工程材料(设计预算、领料出库)
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProjectMaterialWService.getById(id));
    }

    /**
     * 新增工程材料(设计预算、领料出库)
     * @param tfProjectMaterialW 工程材料(设计预算、领料出库)
     * @return ResultData
     */
    @ToFlyAppLog(title="新增工程材料(设计预算、领料出库)" )
    @ApiOperation(value = "新增工程材料(设计预算、领料出库)")
    @PostMapping
    public ResultData save(@RequestBody TfProjectMaterialW tfProjectMaterialW) {
      return ResultData.ok(tfProjectMaterialWService.save(tfProjectMaterialW));
    }

    /**
     * 修改工程材料(设计预算、领料出库)
     * @param tfProjectMaterialW 工程材料(设计预算、领料出库)
     * @return ResultData
     */
    @ToFlyAppLog(title="修改工程材料(设计预算、领料出库)" )
    @ApiOperation(value = "修改工程材料(设计预算、领料出库)")
    @PutMapping
    public ResultData updateById(@RequestBody TfProjectMaterialW tfProjectMaterialW) {
      return ResultData.ok(tfProjectMaterialWService.updateById(tfProjectMaterialW));
    }

    /**
     * 通过id删除工程材料(设计预算、领料出库)
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除工程材料(设计预算、领料出库)" )
    @ApiOperation(value = "通过ID删除工程材料(设计预算、领料出库)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfProjectMaterialWService.removeById(id));
    }

    /**
     * 通过id批量删除工程材料(设计预算、领料出库)
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除工程材料(设计预算、领料出库)" )
    @ApiOperation(value = "通过ID批量删除工程材料(设计预算、领料出库)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfProjectMaterialWService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
