package com.tofly.workflow.buss.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.workflow.buss.entity.TfProjectTransferW;
import com.tofly.workflow.buss.service.TfProjectTransferWService;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 项目移交 (主要潼南应用项目移交登记--单独填报)(TfProjectTransferW)表控制层
 *
 * @author HaiQiu
 * @since 2022-09-19 17:52:12
 */
@Api(tags = "项目移交 (主要潼南应用项目移交登记--单独填报)")
@RestController
@RequestMapping("tfProjectTransferW")
public class TfProjectTransferWController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private TfProjectTransferWService tfProjectTransferWService;

    /**
     * 分页查询数据
     *
     * @param page 分页对象
     * @param tfProjectTransferW 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultData pageList(Page<TfProjectTransferW> page, TfProjectTransferW tfProjectTransferW) {
        return decide(tfProjectTransferWService.pageList(page, tfProjectTransferW));
    }
    
     /**
     * 查询所有数据
     *
     * @param tfProjectTransferW 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData all(TfProjectTransferW tfProjectTransferW) {
        return decide(tfProjectTransferWService.all(tfProjectTransferW));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData get(@PathVariable Serializable id) {
        return decide(tfProjectTransferWService.get(id));
    }

    /**
     * 新增数据
     *
     * @param tfProjectTransferW 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@RequestPart @Validated TfProjectTransferW tfProjectTransferW,
                             @ApiParam(value = "附件列表") @RequestPart(required = false) @RequestParam(name = "fileList",required = false)
                             List<MultipartFile> fileList) {
        tfProjectTransferWService.insert(tfProjectTransferW,fileList);
        return decide("新增成功");
    }

    /**
     * 修改数据
     *
     * @param tfProjectTransferW 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData edit(@RequestPart @Validated TfProjectTransferW tfProjectTransferW,
                           @ApiParam(value = "附件列表") @RequestPart(required = false) @RequestParam(name = "fileList",required = false)
                           List<MultipartFile> fileList) {
        tfProjectTransferWService.edit(tfProjectTransferW,fileList);
        return decide("修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        tfProjectTransferWService.delete(idList);
        return decide("删除成功");
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        tfProjectTransferWService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") MultipartFile file) {
        tfProjectTransferWService.importData(file);
        return decide("导入成功");
    }


     /**
     * 数据导出
     * @param page 分页
     * @param tfProjectTransferW 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<TfProjectTransferW> page, TfProjectTransferW tfProjectTransferW,Boolean isAll) {
        tfProjectTransferWService.export(page,tfProjectTransferW,isAll);
    }
}

