/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfRegisterItemW;
import com.tofly.workflow.buss.service.TfRegisterItemWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 工程登记/立项
 *
 * @author JF
 * @date Tue Aug 16 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfregisteritemw")
@Api(tags="工程登记/立项接口")
public class TfRegisterItemWController {

    private final IFileInfoService iFileInfoService;

    private final TfRegisterItemWService tfRegisterItemWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfRegisterItemW 工程登记/立项
     */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfRegisterItemWPage(Page page, @ModelAttribute TfRegisterItemW tfRegisterItemW,@RequestParam(value = "likeStr",required = false) String likeStr) {
      return ResultData.ok(tfRegisterItemWService.page(page, Wrappers.query(tfRegisterItemW)
              .like(StringUtils.isNotBlank(likeStr),"PROJECT_NAME",likeStr)
              .or()
              .like(StringUtils.isNotBlank(likeStr),"ADDRE",likeStr)));
    }


    /**
     * 通过id查询工程登记/立项
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfRegisterItemWService.getById(id));
    }

    /**
     * 新增工程登记/立项
     * @param tfRegisterItemW 工程登记/立项
     * @return ResultData
     */
    @ToFlyAppLog(title="新增工程登记/立项" )
    @ApiOperation(value = "新增工程登记/立项")
    @PostMapping
    public ResultData save(@RequestBody TfRegisterItemW tfRegisterItemW) {
      return ResultData.ok(tfRegisterItemWService.save(tfRegisterItemW));
    }

    /**
     * 修改工程登记/立项
     * @param tfRegisterItemW 工程登记/立项
     * @return ResultData
     */
    @ToFlyAppLog(title="修改工程登记/立项" )
    @ApiOperation(value = "修改工程登记/立项")
    @PutMapping
    public ResultData updateById(@RequestBody TfRegisterItemW tfRegisterItemW) {
      return ResultData.ok(tfRegisterItemWService.updateById(tfRegisterItemW));
    }

    /**
     * 通过id删除工程登记/立项
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除工程登记/立项" )
    @ApiOperation(value = "通过ID删除工程登记/立项")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfRegisterItemWService.removeById(id));
    }

    /**
     * 通过id批量删除工程登记/立项
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除工程登记/立项" )
    @ApiOperation(value = "通过ID批量删除工程登记/立项")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids不能为空");
        }
        List<Long> removeIds = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        removeIds.forEach(info ->{
            TfRegisterItemW registerItem = tfRegisterItemWService.getById(info);
            String annex = registerItem.getAnnex();
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
        });
        tfRegisterItemWService.removeByIds(removeIds);
        return ResultData.ok(tfRegisterItemWService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
