package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfSafetyCheckW;
import com.tofly.workflow.buss.service.TfSafetyCheckWService;
import com.tofly.workflow.common.FlowConstant;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * @author HaiQiu
 * {@code @date} 2022/9/5
 * 安全检查(单独填报)
 */
@RestController
@RequestMapping("/tfsafetycheckw")
@Api(tags="安全检查(单独填报)")
public class TfSafetyCheckWController extends BaseController {

    @Autowired
    private TfSafetyCheckWService tfSafetyCheckWService;

    @Autowired
    private IFileInfoService fileInfoService;

    @Value("${zmrq.ftp.tfsafetycheckw}")
    private String path;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param tfSafetyCheckW 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page<TfSafetyCheckW> page, TfSafetyCheckW tfSafetyCheckW) {
        return decide(tfSafetyCheckWService.page(page, new QueryWrapper<>(tfSafetyCheckW)));
    }


    /**
     * 查询所有数据
     *
     * @param tfSafetyCheckW 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/allList")
    public ResultData list(TfSafetyCheckW tfSafetyCheckW) {
        return decide(tfSafetyCheckWService.list(new QueryWrapper<>(tfSafetyCheckW)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Serializable id) {
        return decide(tfSafetyCheckWService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param tfSafetyCheckW 实体对象
     * @return 新增结果
     */
//    @ApiOperation(value = "新增数据")
//    @PostMapping
    public ResultData insert(@RequestPart @Validated TfSafetyCheckW tfSafetyCheckW,
                             @RequestPart(required = false) @RequestParam(name = "fileList",required = false) List<MultipartFile> fileList) {
        String files = fileInfoService.uploadFiles(fileList, tfSafetyCheckW.getAnnex(), null, path, "TF_SAFETY_CHECK_W");
        tfSafetyCheckW.setAnnex(files);
        tfSafetyCheckW.setIsDel(FlowConstant.INTEGER_ZERO);
        return ResultData.ok(tfSafetyCheckWService.save(tfSafetyCheckW));
    }

    /**
     * 修改数据
     *
     * @param tfSafetyCheckW 实体对象
     * @return 修改结果
     */
//    @ApiOperation(value = "修改数据")
//    @PutMapping
    public ResultData update(@RequestPart @Validated(value = TfSafetyCheckW.Update.class) TfSafetyCheckW tfSafetyCheckW,
                             @RequestPart(required = false) @RequestParam(name = "fileList",required = false) List<MultipartFile> fileList) {
        TfSafetyCheckW byId = tfSafetyCheckWService.getById(tfSafetyCheckW.getId());
        String files = fileInfoService.uploadFiles(fileList, tfSafetyCheckW.getAnnex(), byId.getAnnex(), path, "TF_SAFETY_CHECK_W");
        tfSafetyCheckW.setAnnex(files);
        return ResultData.ok(tfSafetyCheckWService.updateById(tfSafetyCheckW));
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
//    @ApiOperation(value = "删除数据")
//    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
//    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        tfSafetyCheckWService.update(new UpdateWrapper<TfSafetyCheckW>().lambda()
                .set(TfSafetyCheckW::getIsDel,FlowConstant.INTEGER_ONE).in(TfSafetyCheckW::getId,idList)
                .eq(TfSafetyCheckW::getIsDel,FlowConstant.SHORT_ZERO));
        return decide(ResultEnum.DELETE_SUCCESS);
    }
}
