/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfSurveyW;
import com.tofly.workflow.buss.service.TfSurveyWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 工程勘察
 *
 * @author JF
 * @date Mon Aug 22 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfsurveyw")
@Api(tags="工程勘察接口")
public class TfSurveyWController {

    private final IFileInfoService iFileInfoService;

    private final  TfSurveyWService tfSurveyWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfSurveyW 工程勘察
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfSurveyWPage(Page page, @ModelAttribute TfSurveyW tfSurveyW) {
      return ResultData.ok(tfSurveyWService.page(page, Wrappers.query(tfSurveyW)));
    }


    /**
     * 通过id查询工程勘察
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfSurveyWService.getById(id));
    }

    /**
     * 新增工程勘察
     * @param tfSurveyW 工程勘察
     * @return ResultData
     */
    @ToFlyAppLog(title="新增工程勘察" )
    @ApiOperation(value = "新增工程勘察")
    @PostMapping
    public ResultData save(@RequestBody TfSurveyW tfSurveyW) {
      return ResultData.ok(tfSurveyWService.save(tfSurveyW));
    }

    /**
     * 修改工程勘察
     * @param tfSurveyW 工程勘察
     * @return ResultData
     */
    @ToFlyAppLog(title="修改工程勘察" )
    @ApiOperation(value = "修改工程勘察")
    @PutMapping
    public ResultData updateById(@RequestBody TfSurveyW tfSurveyW) {
      return ResultData.ok(tfSurveyWService.updateById(tfSurveyW));
    }

    /**
     * 通过id删除工程勘察
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除工程勘察" )
    @ApiOperation(value = "通过ID删除工程勘察")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfSurveyWService.removeById(id));
    }

    /**
     * 通过id批量删除工程勘察
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除工程勘察" )
    @ApiOperation(value = "通过ID批量删除工程勘察")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        if(StringUtils.isBlank(ids)){
            return ResultData.fail("ids不能为空");
        }
        List<Long> removeIds = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        removeIds.forEach(info ->{
            TfSurveyW surveyW = tfSurveyWService.getById(info);
            String annex = surveyW.getAnnex();
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
        });
        tfSurveyWService.removeByIds(removeIds);
        return ResultData.ok(tfSurveyWService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
