/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.buss.entity.TfTaskAllocationW;
import com.tofly.workflow.buss.service.TfTaskAllocationWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
/**
 * 任务分配
 *
 * @author JF
 * @date Tue Sep 13 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tftaskallocationw")
@Api(tags="任务分配接口")
public class TfTaskAllocationWController {

    private final  TfTaskAllocationWService tfTaskAllocationWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfTaskAllocationW 任务分配
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfTaskAllocationWPage(Page page, @ModelAttribute TfTaskAllocationW tfTaskAllocationW) {
      return ResultData.ok(tfTaskAllocationWService.page(page, Wrappers.query(tfTaskAllocationW)));
    }


    /**
     * 通过id查询任务分配
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfTaskAllocationWService.getById(id));
    }

    /**
     * 新增任务分配
     * @param tfTaskAllocationW 任务分配
     * @return ResultData
     */
    @ToFlyAppLog(title="新增任务分配" )
    @ApiOperation(value = "新增任务分配")
    @PostMapping
    public ResultData save(@RequestBody TfTaskAllocationW tfTaskAllocationW) {
      return ResultData.ok(tfTaskAllocationWService.save(tfTaskAllocationW));
    }

    /**
     * 修改任务分配
     * @param tfTaskAllocationW 任务分配
     * @return ResultData
     */
    @ToFlyAppLog(title="修改任务分配" )
    @ApiOperation(value = "修改任务分配")
    @PutMapping
    public ResultData updateById(@RequestBody TfTaskAllocationW tfTaskAllocationW) {
      return ResultData.ok(tfTaskAllocationWService.updateById(tfTaskAllocationW));
    }

    /**
     * 通过id删除任务分配
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除任务分配" )
    @ApiOperation(value = "通过ID删除任务分配")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfTaskAllocationWService.removeById(id));
    }

    /**
     * 通过id批量删除任务分配
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除任务分配" )
    @ApiOperation(value = "通过ID批量删除任务分配")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        return ResultData.ok(tfTaskAllocationWService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
