/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.buss.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.buss.entity.TfTransferFixedAssetsW;
import com.tofly.workflow.buss.entity.dto.TransferFixedAssetsDto;
import com.tofly.workflow.buss.service.TfTransferFixedAssetsWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 转入固定资产 （单独填报)
 *
 * @author JF
 * @date Thu Sep 08 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tftransferfixedassetsw")
@Api(tags="转入固定资产 （单独填报)接口")
public class TfTransferFixedAssetsWController {

    private final  TfTransferFixedAssetsWService tfTransferFixedAssetsWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 转入固定资产 （单独填报)
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfTransferFixedAssetsWPage(Page page, @ModelAttribute TransferFixedAssetsDto dto) {
      return ResultData.ok(tfTransferFixedAssetsWService.pageTransferFixedAssets(page, dto));
    }


    /**
     * 通过id查询转入固定资产 （单独填报)
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfTransferFixedAssetsWService.getById(id));
    }

    /**
     * 新增转入固定资产 （单独填报)
     * @param tfTransferFixedAssetsW 转入固定资产 （单独填报)
     * @return ResultData
     */
    @ToFlyAppLog(title="新增转入固定资产 （单独填报)" )
    @ApiOperation(value = "新增转入固定资产 （单独填报)")
    @PostMapping
    public ResultData save(@Valid TfTransferFixedAssetsW tfTransferFixedAssetsW, @RequestParam(value = "annexList", required = false) List<MultipartFile> annexList) {
        tfTransferFixedAssetsWService.saveTransferFixedAssets(tfTransferFixedAssetsW,annexList);
        return ResultData.ok();
    }

    /**
     * 修改转入固定资产 （单独填报)
     * @param tfTransferFixedAssetsW 转入固定资产 （单独填报)
     * @return ResultData
     */
    @ToFlyAppLog(title="修改转入固定资产 （单独填报)" )
    @ApiOperation(value = "修改转入固定资产 （单独填报)")
    @PutMapping
    public ResultData updateById(@Valid TfTransferFixedAssetsW tfTransferFixedAssetsW, @RequestParam(value = "annexList", required = false) List<MultipartFile> annexList) {
      tfTransferFixedAssetsWService.updateTransferFixedAssets(tfTransferFixedAssetsW,annexList);
      return ResultData.ok();
    }

//    /**
//     * 通过id删除转入固定资产 （单独填报)
//     * @param id id
//     * @return ResultData
//     */
//    @ToFlyAppLog(title="通过id删除转入固定资产 （单独填报)" )
//    @ApiOperation(value = "通过ID删除转入固定资产 （单独填报)")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultData removeById(@PathVariable Long id) {
//      return ResultData.ok(tfTransferFixedAssetsWService.removeById(id));
//    }

    /**
     * 通过id批量删除转入固定资产 （单独填报)
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除转入固定资产 （单独填报)" )
    @ApiOperation(value = "通过ID批量删除转入固定资产 （单独填报)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultData removeById(String ids) {
        List<Long> removeId = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        tfTransferFixedAssetsWService.deleteTransferFixedAssets(removeId);
        return ResultData.ok();
    }
}
