/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.dto.FileInfoDto;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.entity.TfMatterPriceP;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.baseconf.service.TfMatterPricePService;
import com.tofly.workflow.buss.entity.TfAcceptanceApplicationW;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.TfProjectMaterialW;
import com.tofly.workflow.buss.entity.dto.TfAcceptanceApplicationDto;
import com.tofly.workflow.buss.mapper.TfAcceptanceApplicationWMapper;
import com.tofly.workflow.buss.service.TfAcceptanceApplicationWService;
import com.tofly.workflow.buss.service.TfMonitoringReportWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfProjectMaterialWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 验收申请
 *
 * @author JF
 * @date Tue Sep 20 00:00:00 CST 2022
 */
@Service("tfAcceptanceApplicationWService")
public class TfAcceptanceApplicationWServiceImpl extends ServiceImpl<TfAcceptanceApplicationWMapper, TfAcceptanceApplicationW> implements TfAcceptanceApplicationWService, BaseService {

    /**
     * 监检提交上报登记(监检上报、监检登记）
     */
    @Autowired
    private TfMonitoringReportWService mReportService;
    @Autowired
    private TfMatterPricePService matterPricePService;
    /**
     * 工程基础
     */
    @Autowired
    private TfProjectBaseWService pBaseWService;
    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;

    @Autowired
    private TfMatterPService tfMatterPService;

    @Value("${zmrq.ftp.acceptanceApplication}")
    private String acceptanceApplicationPath;

    @Autowired
    private IFileInfoService fileInfoService;
    @Autowired
    private TfProjectMaterialWService projectMaterialWService;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        TfAcceptanceApplicationDto entityDto = JSON.parseObject(String.valueOf(t),TfAcceptanceApplicationDto.class);
        if(entityDto.getAcceptanceApplicationW() == null){
            return "新增/修改对象不能为空";
        }
//        if(entityDto.getProjectMaterialList() == null || entityDto.getProjectMaterialList().size() == 0){
//            return "工程材料列表为空";
//        }
        TfAcceptanceApplicationW entity = entityDto.getAcceptanceApplicationW();
        List<String> annexNameList = entity.getAnnexList();
        List<String> checkAnnexNameList = entity.getTestAnnexList();
        if(StringUtils.isNotBlank(FileNameCheckUtils.checkNodeFile(fileList,annexNameList))){
            return FileNameCheckUtils.checkNodeFile(fileList,annexNameList);
        }else {
            return FileNameCheckUtils.checkNodeFile(fileList,checkAnnexNameList);
        }
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }
        if(pb.getSaveState()==FlowConstant.INTEGER_ONE) {
            Integer typeValue = pBaseWService.getProjectTypeValue(pb.getId());
            if (typeValue != null && typeValue == FlowConstant.INTEGER_THREE) {
                int count = mReportService.getMReportCount(pb.getId(), 2, null);
                if (count == FlowConstant.INTEGER_ZERO) {
                    throw new RuntimeException("流程未监检上报，请先上报！");
                }
//                count = mReportService.getMReportCount(pb.getId(), null, Arrays.asList(1, 2));
//                if (count > FlowConstant.INTEGER_ZERO) {
//                    throw new RuntimeException("流程监检未完成，请先完成监检！");
//                }
            }
        }

        TfAcceptanceApplicationDto entityDto = JSON.parseObject(String.valueOf(t), TfAcceptanceApplicationDto.class);
        TfAcceptanceApplicationW entity = entityDto.getAcceptanceApplicationW();
        List<TfProjectMaterialW> projectMaterialList = entityDto.getProjectMaterialList();
        entity.setProcessId(pb.getId());
        entity.setNodeId(pb.getNodeId());

        TfAcceptanceApplicationW entityQ = this.getOne(new QueryWrapper<TfAcceptanceApplicationW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId()));
        if(fileList != null && fileList.size() != 0){
            FileInfoDto fileInfoDto = new FileInfoDto();
            fileInfoDto.setFileList(fileList);
            fileInfoDto.setPath(acceptanceApplicationPath);
            fileInfoDto.setTableName(FileTableEnum.TF_ACCEPTANCE_APPLICATION_W.getTableName());
            fileInfoDto.setFileNameList(entity.getAnnexList());
            if(entityQ == null){
                entity.setAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(entity.getTestAnnexList());
                entity.setTestAnnex(fileInfoService.getFileInfo(fileInfoDto));
            }else {
                fileInfoDto.setAgoInfos(entityQ.getAnnex());
                fileInfoDto.setNowInfos(entity.getAnnex());
                entity.setAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(entity.getTestAnnexList());
                fileInfoDto.setAgoInfos(entityQ.getTestAnnex());
                fileInfoDto.setNowInfos(entity.getTestAnnex());
                entity.setTestAnnex(fileInfoService.getFileInfo(fileInfoDto));
            }
        }
        if(entityQ != null){
            this.removeById(entityQ.getId());
            projectMaterialWService.remove(new QueryWrapper<TfProjectMaterialW>().lambda()
                    .eq(TfProjectMaterialW::getProcessId,pb.getId())
                    .eq(TfProjectMaterialW::getNodeId,pb.getNodeId()));
        }
        // 对象入库
        this.save(entity);
        if(projectMaterialList == null || projectMaterialList.size() == 0){
            return null;
        }
        for(TfProjectMaterialW projectMaterial : projectMaterialList){
            projectMaterial.setProcessId(pb.getId());
            projectMaterial.setNodeId(pb.getNodeId());
            projectMaterial.setLikeProcessId(pb.getRelationId());
            projectMaterialWService.save(projectMaterial);
        }
        if(pb.getSaveState()== FlowConstant.INTEGER_ONE){
            Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getId()).select(TfProjectBaseW::getId)
                    , r -> Long.parseLong(String.valueOf(r)));
            TfProjectBaseW npbase = new TfProjectBaseW();
            npbase.setApplyDate(entity.getApplyDate());
            npbase.setApplyUser(entity.getApplyUser());
            npbase.setApplyUserName(entity.getApplyUserName());

            if (pbId == null) {
                npbase.setConfigId(pb.getConfigId());
                npbase.setEdition(pb.getEdition());
                npbase.setProcessNumber(pb.getProcessNumber());
                npbase.setTenantId(pb.getTenantId());
                npbase.setRelationId(pb.getRelationId());
                npbase.setProcessId(pb.getId());
                npbase.setCategory(pb.getCategory());
                npbase.setProcessType(pb.getProcessType());
                npbase.setRegisterDept(pb.getRegisterDept());
                npbase.setRegisterStaff(pb.getRegisterStaff());
                npbase.setRegisterPost(pb.getRegisterPost());
                npbase.setRegisterTime(pb.getRegisterTime());
            } else {
                npbase.setId(pbId);
            }
            pBaseService.saveOrUpdate(npbase);
        }
        return null;
    }
    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        List<TfAcceptanceApplicationW> acceptanceApplicationList = list(new QueryWrapper<TfAcceptanceApplicationW>().lambda()
                .eq(processId != null, TfAcceptanceApplicationW::getProcessId, processId)
                .eq(nodeId != null, TfAcceptanceApplicationW::getNodeId, nodeId));
        for(TfAcceptanceApplicationW one : acceptanceApplicationList){
            if(one.getNodeId() == null || one.getProcessId() == null){
                continue;
            }
            List<TfProjectMaterialW> materialList = projectMaterialWService.list(new QueryWrapper<TfProjectMaterialW>().lambda()
                    .eq(processId != null, TfProjectMaterialW::getProcessId, processId)
                    .eq(nodeId != null, TfProjectMaterialW::getNodeId, nodeId));
            if(CollUtil.isNotEmpty(materialList)){
                materialList.forEach(e -> {
                    TfMatterP mp = tfMatterPService.getById(e.getMatterId());
                    if(mp!=null){
                        mp.setMatterPrice(matterPricePService.getOne(new QueryWrapper<TfMatterPriceP>()
                                .eq("MATTER_ID",mp.getId())
                                .eq("TENANT_ID",tenantId)
                                .eq("IS_VALID",1)));
                        e.setMatterP(mp);
                    }

                });
            }
            one.setMaterialList(materialList);
        }
        return acceptanceApplicationList;
    }

}
