package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.baseconf.entity.TfConstructionTeamP;
import com.tofly.workflow.baseconf.entity.TfTeamStaffP;
import com.tofly.workflow.baseconf.service.TfConstructionTeamPService;
import com.tofly.workflow.baseconf.service.TfTeamStaffPService;
import com.tofly.workflow.buss.entity.TfAlterBuildUnit;
import com.tofly.workflow.buss.entity.TfConstructionTeamW;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.mapper.TfAlterBuildUnitMapper;
import com.tofly.workflow.buss.service.TfAlterBuildUnitService;
import com.tofly.workflow.buss.service.TfConstructionTeamWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import com.tofly.workflow.process.service.TfProcessLService;
import com.tofly.workflow.processconfig.entity.TfProcessConfigStepL;
import com.tofly.workflow.processconfig.service.TfProcessConfigStepLService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * 施工班组变更实现类
 *
 * @author sheldon
 * @Description
 * @create 2024-02-01 17:01
 */
@Slf4j
@Service("tfAlterBuildUnitService")
@Transactional(rollbackFor = Exception.class)
public class TfAlterBuildUnitServiceImpl extends ServiceImpl<TfAlterBuildUnitMapper, TfAlterBuildUnit> implements TfAlterBuildUnitService, BaseService {
    @Resource
    TfConstructionTeamWService tfConstructionTeamWService;
    @Resource
    TfConstructionTeamPService tfConstructionTeamPService;
    @Resource
    TfProcessConfigStepLService tfProcessConfigStepLService;
    @Resource
    TfProcessLService tfProcessLService;
    @Resource
    TfTeamStaffPService tfTeamStaffPService;
    @Resource
    private TfProjectBaseWService pBaseService;


    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseObject(json, TfAlterBuildUnit.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
//        if(t==null){
//            return FlowConstant.ISNOTNUL;
//        }
//        PNodeUpdateDto pb = null;
//        if(t instanceof String){
//            pb = JSON.parseObject(t.toString(), PNodeUpdateDto.class);
//        }else {
//            pb = (PNodeUpdateDto)t;
//        }
//        if(pb.getDutyUser()==null){
//            return "dutyUser"+FlowConstant.ISNOTNUL;
//        }
//        if(pb.getDutyPost()==null){
//            return "dutyPost"+FlowConstant.ISNOTNUL;
//        }
//        if(StringUtils.isEmpty(pb.getTelephone())){
//            return "telephone"+FlowConstant.ISNOTNUL;
//        }
        Assert.notNull(t, "施工班组变更的参数不能为空");
        Assert.isTrue(t instanceof TfAlterBuildUnit, "施工班组变更的参数类型错误");
        assert t instanceof TfAlterBuildUnit;
        TfAlterBuildUnit entity = (TfAlterBuildUnit) t;
        Assert.notNull(entity.getProcessId(), "流程id不能为空");
        Assert.notNull(entity.getTeamId(), "施工班组id不能为空");
        Assert.notBlank(entity.getConUserIds(), "施工班组人员id不能为空");
        Assert.notBlank(entity.getConUserNames(), "施工班组人员姓名不能为空");
        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        Assert.isTrue(t instanceof TfAlterBuildUnit, "施工班组变更的参数类型错误");
        assert t instanceof TfAlterBuildUnit;
        TfAlterBuildUnit entity = (TfAlterBuildUnit) t;
        TfConstructionTeamW teamW = tfConstructionTeamWService.getOne(Wrappers.lambdaQuery(TfConstructionTeamW.class)
                .eq(TfConstructionTeamW::getProcessId, pb.getRelationId()));
        Assert.notNull(teamW, "当前流程未安排施工班组");
        //todo 验证 流程在综合验收之后，就不能修改施工班组  TF_PROCESS_CONFIG_STEP_L
        TfProcessL processL = tfProcessLService.getById(pb.getRelationId());
        TfProcessConfigStepL stepL = tfProcessConfigStepLService.getOne(Wrappers.lambdaQuery(TfProcessConfigStepL.class)
                .eq(TfProcessConfigStepL::getConfigId, pb.getConfigId())
                .eq(TfProcessConfigStepL::getStepName, "综合验收"));
        if (Objects.nonNull(stepL)) {
            Integer stepOrder = stepL.getStepOrder();
            String passingNode = processL.getPassingNode();
            Assert.isFalse(passingNode.contains(stepOrder + "_1"), "当前流程已通过综合验收，不能修改施工班组");
        }
        entity.setProcessId(pb.getId());
        entity.setOldTeamId(teamW.getTeamId());
        entity.setOldConUserIds(teamW.getConUserIds());
        entity.setOldConUserNames(teamW.getConUserNames());
        entity.setIsDel(FlowConstant.SHORT_ZERO);
        entity.setIsValid(FlowConstant.SHORT_ONE);
        this.remove(Wrappers.lambdaQuery(TfAlterBuildUnit.class)
                .eq(TfAlterBuildUnit::getProcessId, pb.getId()));
        this.save(entity);
        //如果是提交
        if (FlowConstant.INTEGER_ONE.equals(pb.getSaveState())) {
            //修改施工班组
            teamW.setTeamId(entity.getTeamId());
            teamW.setConUserIds(entity.getConUserIds());
            teamW.setConUserNames(entity.getConUserNames());
            tfConstructionTeamWService.updateById(teamW);
            //修改项目基础信息里面的施工负责人
            Long leaderId = entity.getLeaderId();
            log.info(entity.getProcessId().toString());
            log.info(pb.getRelationId().toString());
            TfProjectBaseW one = pBaseService.getOne(Wrappers.lambdaQuery(TfProjectBaseW.class)
                    .eq(TfProjectBaseW::getProcessId, pb.getRelationId()));
            one.setBuildUserId(leaderId);
            pBaseService.updateById(one);
        }
        return true;
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId, Long tenantId) {
        List<TfAlterBuildUnit> list = this.list(new QueryWrapper<TfAlterBuildUnit>().lambda()
                .eq(TfAlterBuildUnit::getProcessId, processId)
                .eq(TfAlterBuildUnit::getIsValid, FlowConstant.INTEGER_ONE)
                .eq(TfAlterBuildUnit::getIsDel, FlowConstant.INTEGER_ZERO));
        if (CollectionUtil.isNotEmpty(list)) {
            for (TfAlterBuildUnit unit : list) {
                Long oldTeamId = unit.getOldTeamId();
                unit.setOldTss(tfTeamStaffPService.list(Wrappers.lambdaQuery(TfTeamStaffP.class).eq(TfTeamStaffP::getTeamId, oldTeamId)
                        .eq(TfTeamStaffP::getIsDel, FlowConstant.SHORT_ZERO)));
                TfConstructionTeamP oTeamP = tfConstructionTeamPService.getById(oldTeamId);
                unit.setOldTeamName(oTeamP.getName());
                Long teamId = unit.getTeamId();
                unit.setTss(tfTeamStaffPService.list(Wrappers.lambdaQuery(TfTeamStaffP.class).eq(TfTeamStaffP::getTeamId, teamId)
                        .eq(TfTeamStaffP::getIsDel, FlowConstant.SHORT_ZERO)));
                TfConstructionTeamP teamP = tfConstructionTeamPService.getById(teamId);
                unit.setTeamName(teamP.getName());
            }
        }
        log.info(list.toString());
        return list;
    }

}
