/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.buss.entity.TfAlterDutyW;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.TfTaskAllocationW;
import com.tofly.workflow.buss.mapper.TfAlterDutyWMapper;
import com.tofly.workflow.buss.service.TfAlterDutyWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfTaskAllocationWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.dto.PNodeUpdateDto;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import com.tofly.workflow.process.service.TfProcessNodeLService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 负责人变更(同时需要变更流程中的负责人)
 *
 * @author xqf
 * @date Thu Sep 15 00:00:00 CST 2022
 */
@Slf4j
@Service("tfAlterDutyWService")
public class TfAlterDutyWServiceImpl extends ServiceImpl<TfAlterDutyWMapper, TfAlterDutyW> implements TfAlterDutyWService, BaseService {


    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;
    /**
     * 项目负责人
     */
    @Autowired
    private TfTaskAllocationWService tAllocationService;
    /**
     * 流程节点处理信息
     */
    @Autowired
    private TfProcessNodeLService pNodeService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public Object saveAlterDutyUser(PNodeUpdateDto dto) {

        Object obj = pNodeService.processNodeUpdate(dto);
        if(obj instanceof String){
            return obj;
        }
        TfTaskAllocationW ta = tAllocationService.getOne(new QueryWrapper<TfTaskAllocationW>().lambda()
                .eq(TfTaskAllocationW::getProcessId,dto.getProcessId()).eq(TfTaskAllocationW::getNodeId,dto.getNodeId()));
        TfAlterDutyW td = new TfAlterDutyW();
        BeanUtils.copyProperties(dto,td);
        td.setProcessId(dto.getProcessId());
        td.setNodeId(dto.getNodeId());
        td.setBillDutyUser(ta.getDutyUser());
        td.setBillDutyDept(ta.getDutyDept());
        td.setBillDutyPost(ta.getDutyPost());
        td.setBillTelephone(ta.getTelephone());
        this.save(td);

        /**填充工程基础信息*/
        Long pbId= pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId,dto.getProcessId()).select(TfProjectBaseW::getId)
                ,r->Long.parseLong(String.valueOf(r)));
        TfProjectBaseW npbase = new TfProjectBaseW();
        npbase.setDutyDept(dto.getDutyDept());
        npbase.setDutyPost(dto.getDutyPost());
        npbase.setDutyUser(dto.getDutyUser());
        npbase.setId(pbId);
        pBaseService.updateById(npbase);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void mackingAlterDuty(long processId) {
        TfAlterDutyW ad = this.baseMapper.getTfAlterDutyW(processId);

        Long relationId = this.baseMapper.getAlterDutyRelationIdByADId(processId);
        if(relationId==null||ad==null){
            log.error("负责人变更未查找到数据！！ processId：："+processId);
        }
        TfTaskAllocationW ta = tAllocationService.getValidTaskAllocation(relationId);
        if(ta==null){
            log.warn("主流程负责人未找到！processId：："+processId);
        }
        TfProcessNodeL pn = pNodeService.getNextProcessNode(ta.getProcessId(),ta.getNodeId());
        if(pn==null){
            log.warn("流程更改负责人无效！processId：："+processId);
        }
        PNodeUpdateDto dto = new PNodeUpdateDto();
        BeanUtils.copyProperties(ad,dto);
        dto.setProcessId(relationId);
        dto.setNodeId(pn.getId());
        dto.setProcessStep(pn.getProcessStep());
        dto.setStepNode(pn.getStepNode());
        Object obj = pNodeService.processNodeUpdate(dto);
        if(obj instanceof String){
            log.error("负责人员变更修改流程数据失败");
            return ;
        }
        /**填充工程基础信息*/
        ta.setDutyDept(ad.getDutyDept());
        ta.setDutyPost(ad.getDutyPost());
        ta.setDutyUser(ad.getDutyUser());
        ta.setTelephone(ad.getTelephone());
        ta.setAssignDate(ad.getDataTime());
        tAllocationService.updateById(ta);
        Long pbId= pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId,dto.getProcessId()).select(TfProjectBaseW::getId)
                ,r->Long.parseLong(String.valueOf(r)));
        TfProjectBaseW npbase = new TfProjectBaseW();
        npbase.setDutyDept(dto.getDutyDept());
        npbase.setDutyPost(dto.getDutyPost());
        npbase.setDutyUser(dto.getDutyUser());
        npbase.setId(pbId);
        pBaseService.updateById(npbase);
    }

    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseObject(json, PNodeUpdateDto.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t==null){
            return FlowConstant.ISNOTNUL;
        }
        PNodeUpdateDto pb = null;
        if(t instanceof String){
            pb = JSON.parseObject(t.toString(), PNodeUpdateDto.class);
        }else {
            pb = (PNodeUpdateDto)t;
        }
        if(pb.getDutyUser()==null){
            return "dutyUser"+FlowConstant.ISNOTNUL;
        }
        if(pb.getDutyPost()==null){
            return "dutyPost"+FlowConstant.ISNOTNUL;
        }
        if(StringUtils.isEmpty(pb.getTelephone())){
            return "telephone"+FlowConstant.ISNOTNUL;
        }
        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(t==null){
            return true;
        }
        if(pb.getRelationId()==null){
            return "关联主流程不能为空！";
        }
        PNodeUpdateDto dto = null;
        if(t instanceof String){
            dto = JSON.parseObject(t.toString(), PNodeUpdateDto.class);
        }else {
            dto = (PNodeUpdateDto)t;
        }
        if(dto==null){
            return true;
        }
        TfAlterDutyW oldad = this.getOne(new QueryWrapper<TfAlterDutyW>().lambda().eq(TfAlterDutyW::getProcessId,pb.getId()).eq(TfAlterDutyW::getNodeId,pb.getNodeId())
                .eq(TfAlterDutyW::getIsDel,FlowConstant.SHORT_ZERO).eq(TfAlterDutyW::getIsValid,FlowConstant.SHORT_ONE));
        if(oldad!=null){
            this.removeById(oldad.getId());
        }
        TfTaskAllocationW ta = tAllocationService.getValidTaskAllocation(pb.getRelationId());
        TfAlterDutyW ad = new TfAlterDutyW();
        BeanUtils.copyProperties(dto, ad);
        ad.setProcessId(pb.getId());
        if(ad.getBillDutyDept()==null)ad.setBillDutyDept(ta.getDutyDept());
        if(ad.getBillDutyUser()==null)ad.setBillDutyUser(ta.getDutyUser());
        if(ad.getBillDutyPost()==null)ad.setBillDutyPost(ta.getDutyPost());
        if(StringUtils.isEmpty(ad.getBillTelephone()))ad.setBillTelephone(ta.getTelephone());
        ad.setNodeId(pb.getNodeId());
        ad.setIsDel(FlowConstant.SHORT_ZERO);
        ad.setIsValid(FlowConstant.SHORT_ONE);
        return this.save(ad);
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return this.list(new QueryWrapper<TfAlterDutyW>().lambda().eq(TfAlterDutyW::getProcessId,processId).eq(TfAlterDutyW::getNodeId,nodeId)
                .eq(TfAlterDutyW::getIsValid,FlowConstant.SHORT_ONE).eq(TfAlterDutyW::getIsDel,FlowConstant.SHORT_ZERO));
    }
}
