package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.buss.entity.TfBillW;
import com.tofly.workflow.buss.mapper.TfBillWMapper;
import com.tofly.workflow.buss.service.TfBillWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author haiqiu
 * @description 针对表【TF_BILL_W(工程用户账单/管道--费用支持账单)】的数据库操作Service实现
 * @createDate 2022-08-29 14:29:39
 */
@Service("tfBillWService")
public class TfBillWServiceImpl extends ServiceImpl<TfBillWMapper, TfBillW>
        implements TfBillWService, BaseService {

    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseArray(json, TfBillW.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        return null;

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if (t == null) {
            throw new RuntimeException("数据不完整");
        }
        String s = JSONObject.toJSONString(t);
        List<TfBillW> list = JSONObject.parseArray(s, TfBillW.class);
        //保存
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        list.forEach(data -> {
            data.setProcessId(pb.getId());
            data.setNodeId(pb.getNodeId());
        });
        //移除旧数据
        remove(Wrappers.<TfBillW>lambdaQuery().
                eq(pb.getId() != null, TfBillW::getProcessId, pb.getId())
                .eq(pb.getNodeId() != null, TfBillW::getNodeId, pb.getNodeId()));
        //添加新数据
        return saveOrUpdateBatch(list);
    }

    /**
     * 获取数据
     *
     * @param processId 流程ID
     * @param nodeId    步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfBillW>().lambda()
                .eq(processId != null, TfBillW::getProcessId, processId)
                .eq(nodeId != null, TfBillW::getNodeId, nodeId));
    }
}




