/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.dto.FileInfoDto;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfChargeRefundW;
import com.tofly.workflow.buss.entity.dto.TfChargeRefundDto;
import com.tofly.workflow.buss.mapper.TfChargeRefundWMapper;
import com.tofly.workflow.buss.service.TfChargeRefundWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 收费/退费（与收款对接表另行设计）
 *
 * @author JF
 * @date Mon Aug 29 00:00:00 CST 2022
 */
@Service("tfChargeRefundWService")
public class TfChargeRefundWServiceImpl extends ServiceImpl<TfChargeRefundWMapper, TfChargeRefundW> implements TfChargeRefundWService, BaseService {

    @Value("${zmrq.ftp.chargeRefund}")
    private String chargeRefundPath;
    @Autowired
    private IFileInfoService fileInfoService;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        TfChargeRefundW entity = JSON.parseObject(String.valueOf(t),TfChargeRefundW.class);
        return FileNameCheckUtils.checkNodeFile(fileList,entity.getAnnexNameList());
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }

        TfChargeRefundW entity = JSON.parseObject(String.valueOf(t),TfChargeRefundW.class);
        entity.setProcessId(pb.getId());
        entity.setNodeId(pb.getNodeId());
        if(pb.getTenantId() != null){
            entity.setTenantId(pb.getTenantId());
        }

        TfChargeRefundW entityQ = this.getOne(new QueryWrapper<TfChargeRefundW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId()));
        if(fileList != null && fileList.size() != 0){
            FileInfoDto fileInfoDto = new FileInfoDto();
            fileInfoDto.setFileList(fileList);
            fileInfoDto.setPath(chargeRefundPath);
            fileInfoDto.setTableName(FileTableEnum.TF_CHARGE_REFUND_W.getTableName());
            fileInfoDto.setFileNameList(entity.getAnnexNameList());
            if (entityQ != null) {
                fileInfoDto.setAgoInfos(entityQ.getAnnex());
                fileInfoDto.setNowInfos(entity.getAnnex());
            }
            entity.setAnnex(fileInfoService.getFileInfo(fileInfoDto));
        }
        if(entityQ != null){
            this.removeById(entityQ.getId());
        }
        // 对象入库
        this.save(entity);
        return null;
    }

    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfChargeRefundW>().lambda()
                .eq(processId != null, TfChargeRefundW::getProcessId, processId)
                .eq(nodeId != null, TfChargeRefundW::getNodeId, nodeId));
    }

    @Override
    public Page<TfChargeRefundW> getTfChargeRefundPage(Page page, TfChargeRefundDto dto) {
        return baseMapper.getTfChargeRefundPage(page,dto);
    }
}
