/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.baseconf.entity.TfConstructionTeamP;
import com.tofly.workflow.baseconf.entity.TfTeamStaffP;
import com.tofly.workflow.baseconf.service.TfConstructionTeamPService;
import com.tofly.workflow.baseconf.service.TfTeamStaffPService;
import com.tofly.workflow.buss.entity.TfConstructionTeamW;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.dto.ConstructionTeamDto;
import com.tofly.workflow.buss.entity.vo.ConstructionTeamVo;
import com.tofly.workflow.buss.mapper.TfConstructionTeamWMapper;
import com.tofly.workflow.buss.service.TfConstructionTeamWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 施工班组信息
 *
 * @author xqf
 * @date Thu Sep 15 00:00:00 CST 2022
 */
@Service("tfConstructionTeamWService")
public class TfConstructionTeamWServiceImpl extends ServiceImpl<TfConstructionTeamWMapper, TfConstructionTeamW> implements TfConstructionTeamWService , BaseService {
    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;
    /**
     * 施工班组人员
     */
    @Autowired
    private TfTeamStaffPService tStaffService;

    /**
     * 保存文件
     */
    @Autowired
    private IFileInfoService fInfoService;

    /**
     * 施工班组
     */
    @Autowired
    private TfConstructionTeamPService cTeamService;

    @Value("${zmrq.ftp.constructionTeam}")
    private String constructionTeamPath;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t==null){
            return FlowConstant.ISNOTNUL;
        }
        ConstructionTeamDto dto =  JSON.parseObject(String.valueOf(t),ConstructionTeamDto.class);
        if(dto.getTeamId()==null){
            return "team"+FlowConstant.ISNOTNUL;
        }
        int count = cTeamService.count(new QueryWrapper<TfConstructionTeamP>().lambda().eq(TfConstructionTeamP::getId,dto.getTeamId()));
        if(count==0){
            return "TeamId"+FlowConstant.DATA_NOTHINGNESS;
        }
        if(CollUtil.isNotEmpty(dto.getFileName())){
            String check = FileNameCheckUtils.checkNodeFile(fileList,dto.getFileName());
            if(StringUtils.isNotEmpty(check)){
                return check;
            }
        }

        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> files, ProcessBaseDto pb) {
        if(t==null){
            return null;
        }
        ConstructionTeamDto dto =  JSON.parseObject(String.valueOf(t),ConstructionTeamDto.class);
        TfConstructionTeamW oldTct = this.getOne(new QueryWrapper<TfConstructionTeamW>().lambda().eq(TfConstructionTeamW::getProcessId,pb.getId())
                .eq(TfConstructionTeamW::getNodeId,pb.getNodeId()));
        TfConstructionTeamW tct = new TfConstructionTeamW();
        BeanUtils.copyProperties(dto,tct);
        tct.setProcessId(pb.getId());
        tct.setNodeId(pb.getNodeId());
        if(CollUtil.isNotEmpty(dto.getFileName())) {
            List<MultipartFile> fileannex = files.stream().filter(r->dto.getFileName().contains(r.getOriginalFilename())).collect(Collectors.toList());
            String path = fInfoService.uploadFiles(fileannex, dto.getAnnex(), oldTct == null ? null : oldTct.getAnnex(), constructionTeamPath, FileTableEnum.TF_CONSTRUCTION_TEAM_W.getTableName());
            tct.setAnnex(path);
        }
        if(oldTct!=null){
            this.remove(new QueryWrapper<TfConstructionTeamW>().lambda().eq(TfConstructionTeamW::getProcessId,pb.getId())
                    .eq(TfConstructionTeamW::getNodeId,pb.getNodeId()));
        }
        this.save(tct);
        /***/
        if(tct.getTeamId()!=null) {
            List<TfTeamStaffP> list = tStaffService.list(new QueryWrapper<TfTeamStaffP>().lambda().eq(TfTeamStaffP::getTeamId, tct.getTeamId())
                    .eq(TfTeamStaffP::getType, FlowConstant.SHORT_ONE).eq(TfTeamStaffP::getIsDel, FlowConstant.SHORT_ZERO));


            Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getId()).select(TfProjectBaseW::getId)
                    , r -> Long.parseLong(String.valueOf(r)));
            TfProjectBaseW npbase = new TfProjectBaseW();
            if (CollUtil.isNotEmpty(list)) {
                npbase.setBuildUserId(list.get(0).getUserId());
            }
            npbase.setTeamId(tct.getTeamId());
            if (pbId != null) {
                npbase.setId(pbId);
            } else {
                npbase.setConfigId(pb.getConfigId());
                npbase.setEdition(pb.getEdition());
                npbase.setProcessNumber(pb.getProcessNumber());
                npbase.setRelationId(pb.getRelationId());
                npbase.setTenantId(pb.getTenantId());
                npbase.setProcessId(pb.getId());
                npbase.setCategory(pb.getCategory());
                npbase.setProcessType(pb.getProcessType());
                npbase.setRegisterDept(pb.getRegisterDept());
                npbase.setRegisterStaff(pb.getRegisterStaff());
                npbase.setRegisterPost(pb.getRegisterPost());
                npbase.setRegisterTime(pb.getRegisterTime());
            }
            pBaseService.saveOrUpdate(npbase);
        }
        return null;
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        TfConstructionTeamW ct = this.getOne(new QueryWrapper<TfConstructionTeamW>().lambda()
                .eq(TfConstructionTeamW::getProcessId,processId).eq(TfConstructionTeamW::getNodeId,nodeId));
        if(ct==null){
            return null;
        }
        ConstructionTeamVo vo = new ConstructionTeamVo();
        BeanUtils.copyProperties(ct,vo);
        if(vo.getTeamId()!=null){
            vo.setTss(tStaffService.list(new QueryWrapper<TfTeamStaffP>().lambda().eq(TfTeamStaffP::getTeamId,vo.getTeamId())
                    .eq(TfTeamStaffP::getIsDel,FlowConstant.SHORT_ZERO)));
        }
        return vo;
    }
}
