/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.aspose.words.Document;
import com.aspose.words.SaveFormat;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.Constant;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.dto.FileInfoDto;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.ftp.config.FtpUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.common.utils.POIUtil;
import com.tofly.workflow.baseconf.entity.TfContractTemplateP;
import com.tofly.workflow.baseconf.service.TfContractTemplatePService;
import com.tofly.workflow.buss.entity.TfContractInfoW;
import com.tofly.workflow.buss.entity.TfContractW;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.dto.GenerateContractDto;
import com.tofly.workflow.buss.mapper.TfContractWMapper;
import com.tofly.workflow.buss.service.TfContractInfoWService;
import com.tofly.workflow.buss.service.TfContractWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 工程的合同
 *
 * @author JF
 * @date Wed Aug 24 00:00:00 CST 2022
 */
@Service("tfContractWService")
public class TfContractWServiceImpl extends ServiceImpl<TfContractWMapper, TfContractW> implements TfContractWService, BaseService {

    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;

    @Value("${zmrq.ftp.contract}")
    private String contractPath;
    @Value("${zmrq.flow.tempFile}")
    private String tempFilePath;
    @Autowired
    private TfContractTemplatePService contractTemplatePService;
    @Autowired
    private TfContractInfoWService contractInfoWService;
    @Autowired
    private IFileInfoService fileInfoService;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        TfContractW entity = JSON.parseObject(String.valueOf(t),TfContractW.class);
//        if(entity.getContractType() == null){
//            return "合同类型为空";
//        }
//        if(entity.getTemplateId() == null){
//            return "模板ID为空";
//        }
//        if(entity.getContractInfoList() == null || entity.getContractInfoList().size() == 0){
//            return "字段列表为空";
//        }
        return FileNameCheckUtils.checkNodeFile(fileList,entity.getAnnexNameList());
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }
        TfContractW contract = JSON.parseObject(String.valueOf(t),TfContractW.class);
        List<TfContractInfoW> contractInfoList = contract.getContractInfoList();
        contract.setProcessId(pb.getId());
        contract.setNodeId(pb.getNodeId());
        TfContractW entityQ = this.getOne(new QueryWrapper<TfContractW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId()));
        if(contract.getContractType() != 0 && contract.getSignMode() == 1){
            if(StringUtils.isBlank(contract.getContractFile())){
                return "合同模板为空";
            }
            if(entityQ == null){
                // 对象入库
                this.save(contract);
                TfContractW contractQ = this.getOne(new QueryWrapper<TfContractW>()
                        .eq("NODE_ID",pb.getNodeId())
                        .eq("PROCESS_ID",pb.getId()));
                if(contractQ == null){
                    return "工程的合同信息新增失败";
                }
                //新增合同字段列表
                for (TfContractInfoW contractInfo : contractInfoList){
                    contractInfo.setContractId(contractQ.getId());
                    contractInfo.setId(null);
                    contractInfoWService.save(contractInfo);
                }
            }else {
                contract.setId(entityQ.getId());
                this.updateById(contract);
                contractInfoWService.remove(new QueryWrapper<TfContractInfoW>()
                        .eq("CONTRACT_ID",entityQ.getId()));
                //新增合同字段列表
                for (TfContractInfoW contractInfo : contractInfoList){
                    contractInfo.setContractId(entityQ.getId());
                    contractInfoWService.save(contractInfo);
                }
            }
        }else {
            FileInfoDto fileInfoDto = null;
            if(CollUtil.isNotEmpty(contract.getAnnexNameList())&&CollUtil.isNotEmpty(fileList)) {
                fileInfoDto = new FileInfoDto();
                fileInfoDto.setFileList(fileList);
                fileInfoDto.setFileNameList(contract.getAnnexNameList());
            }
            //工程自定义保存、提交都必须上传文件
            if(entityQ == null && fileInfoDto == null){
                return "上传文件为空";
            }
            if(fileInfoDto == null){
                fileInfoDto = new FileInfoDto();
            }
            fileInfoDto.setPath(contractPath);
            fileInfoDto.setTableName(FileTableEnum.TF_CONTRACT_W.getTableName());
            if(entityQ == null){
                contract.setAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(contract.getContractFileNameList());
                contract.setContractFile(fileInfoService.getFileInfo(fileInfoDto));
                // 对象入库
                this.save(contract);
            }else{
                contractInfoWService.remove(new QueryWrapper<TfContractInfoW>().lambda().eq(TfContractInfoW::getContractId,entityQ.getId()));
                fileInfoDto.setAgoInfos(entityQ.getAnnex());
                fileInfoDto.setNowInfos(contract.getAnnex());
                contract.setAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(contract.getContractFileNameList());
                fileInfoDto.setAgoInfos(entityQ.getContractFile());
                fileInfoDto.setNowInfos(contract.getContractFile());
                contract.setContractFile(fileInfoService.getFileInfo(fileInfoDto));

                contract.setId(entityQ.getId());
                this.updateById(contract);
            }
        }

        /**保存工程公共信息*/
        if(contract.getTotalContract()!=null||contract.getSettlementMode()!=null||contract.getAdvancePayment()!=null||contract.getSignMode()!=null) {
            Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getRelationId()!=null?pb.getRelationId():pb.getId())
                            .select(TfProjectBaseW::getId)
                    , r -> Long.parseLong(String.valueOf(r)));
            TfProjectBaseW npbase = new TfProjectBaseW();
            //npbase.setProjectNature(contract.getProjectNature());
            npbase.setTotalContract(contract.getTotalContract());
            npbase.setSettlementMode(contract.getSettlementMode());
            npbase.setAdvancePayment(contract.getAdvancePayment());
            npbase.setSignMode(contract.getSignMode());
            if (pbId == null) {
                npbase.setConfigId(pb.getConfigId());
                npbase.setEdition(pb.getEdition());
                npbase.setProcessNumber(pb.getProcessNumber());
                npbase.setTenantId(pb.getTenantId());
                npbase.setRelationId(pb.getRelationId());
                npbase.setProcessId(pb.getId());
                npbase.setCategory(pb.getCategory());
                npbase.setProcessType(pb.getProcessType());
                npbase.setRegisterDept(pb.getRegisterDept());
                npbase.setRegisterStaff(pb.getRegisterStaff());
                npbase.setRegisterPost(pb.getRegisterPost());
                npbase.setRegisterTime(pb.getRegisterTime());
            } else {
                npbase.setId(pbId);
            }
            pBaseService.saveOrUpdate(npbase);
        }
        return null;
    }

    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        List<TfContractW> contractList = list(new QueryWrapper<TfContractW>().lambda()
                .eq(processId != null, TfContractW::getProcessId, processId)
                .eq(nodeId != null, TfContractW::getNodeId, nodeId));
        for(TfContractW contractW : contractList){
            List<TfContractInfoW> contractInfoList = contractInfoWService.list(
                    new QueryWrapper<TfContractInfoW>().lambda()
                    .eq(contractW.getId() != null, TfContractInfoW::getContractId, contractW.getId()));
            contractW.setContractInfoList(contractInfoList);
        }
        return contractList;
    }

    @Override
    public Map<String,List<FileInfo>> generateContract(GenerateContractDto dto) {
        Map<String,List<FileInfo>> mapRes = new HashMap<>();
        if(dto == null){
            throw new ToflyDeniedException("筛选对象参数为空");
        }
        String deleteFilepath = dto.getDeleteFilepath();
        Long templateId = dto.getTemplateId();
        TfContractW contractW = dto.getContractW();
        List<TfContractInfoW> contractInfoList = dto.getContractInfoList();
        if(StringUtils.isNotBlank(deleteFilepath)){
            FtpUtil.deleteFile(deleteFilepath);
        }
        if(contractW == null){
            return null;
        }
        if (contractInfoList == null || contractInfoList.size() == 0){
            throw new ToflyDeniedException("字段列表为空");
        }
        TfContractTemplateP contractTemplateP = contractTemplatePService.getOne(Wrappers.<TfContractTemplateP>lambdaQuery().eq(TfContractTemplateP::getId,templateId));
        if(contractTemplateP == null){
            throw new ToflyDeniedException("查询不到此模板ID的数据");
        }
        String contractAnnex = contractTemplateP.getContractAnnex();
        if (StringUtils.isBlank(contractAnnex)){
            throw new ToflyDeniedException("合同模板附件为空");
        }
        Map<String,Object> map = null;
        try {
            map = beanToMap(contractW);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        if(map == null){
            throw new ToflyDeniedException("未获取到工程的合同相关信息Map集合");
        }
        for (TfContractInfoW contractInfo : contractInfoList){
            map.put(contractInfo.getFieldName(),contractInfo.getDataValue());
        }
        List<FileInfo> fileInfoList = JSON.parseArray(contractAnnex,FileInfo.class);
        List<FileInfo> fileInfos = new ArrayList<>();
        List<FileInfo> fileInfoPDFs = new ArrayList<>();
        for(FileInfo fileInfo : fileInfoList){
            if(fileInfo == null){
                continue;
            }
            String filePath = fileInfo.getFilePath();
            String originalName = fileInfo.getFileName();
            try {
                FileInfo info = POIUtil.templateWrite(originalName,filePath,map);
                fileInfos.add(info);
                String filePathBean = filePath.substring(0,filePath.lastIndexOf("/") + 1);
                String otherName = IdWorker.getIdStr() + ".pdf";
                String remotePath = filePathBean + otherName;
                String originPath = info.getFilePath();
                doc2pdf(originPath,tempFilePath,remotePath);
                FileInfo fileInfoPDF = new FileInfo();
                fileInfoPDF.setFileName(originalName.substring(0,originalName.lastIndexOf(Constant.POINT)) + ".pdf");
                fileInfoPDF.setOtherName(otherName);
                fileInfoPDF.setFilePath(remotePath);
                fileInfoPDF.setFileFormat(".pdf");
                fileInfoPDFs.add(fileInfoPDF);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if(fileInfos.size() != 0 ){
            mapRes.put("generateContract",fileInfos);
            if(fileInfoPDFs.size() != 0){
                mapRes.put("generateContractPdf",fileInfoPDFs);
            }else {
                throw new ToflyDeniedException("生成的合同附件转PDF失败");
            }
            return mapRes;
        }else {
            throw new ToflyDeniedException("合同附件生成失败");
        }
    }

    /**
     * word转pdf
     * @param inPath 输入路径
     * @param outPath 输出路径
     * @param remotePath FTP远端上传路径
     */
    public static void doc2pdf(String inPath, String outPath, String remotePath) {
        byte[] buf = FtpUtil.getByteDataforFile(inPath);
        if(buf == null || buf.length == 0){
            throw new ToflyDeniedException("未找到指定文件");
        }
        InputStream is = new ByteArrayInputStream(buf);
        try {
            Document doc = new Document(is); // Address是将要被转化的word文档
            String tempPath = outPath + IdWorker.getIdStr() + ".pdf";
            File file = new File(tempPath); // 新建一个空白pdf文档
            FileOutputStream os = new FileOutputStream(file);
            doc.save(os, SaveFormat.PDF);// 全面支持DOC, DOCX, OOXML, RTF HTML, OpenDocument, PDF,
            os.close();
            InputStream pdfIn = new FileInputStream(tempPath);
            FtpUtil.uploadFile2Ftp(pdfIn,remotePath);
            pdfIn.close();
            if(file.exists()){
                file.delete();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 对象转Map
     * @param object
     * @return
     * @throws IllegalAccessException
     */
    public static Map<String, Object> beanToMap(Object object) throws IllegalAccessException {
        Map<String, Object> map = new HashMap<>();
        Field[] fields = object.getClass().getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true);
            map.put(field.getName(), field.get(object));
        }
        return map;
    }

}
