/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.buss.entity.TfCostSumsW;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.TfSumsOtherItemW;
import com.tofly.workflow.buss.entity.dto.CostSumsDto;
import com.tofly.workflow.buss.mapper.TfCostSumsWMapper;
import com.tofly.workflow.buss.service.TfCostSumsWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfSumsOtherItemWService;
import com.tofly.workflow.common.FlowConstant;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 费用汇总
 *
 * @author xqf
 * @date Wed Feb 15 00:00:00 CST 2023
 */
@Service("tfCostSumsWService")
public class TfCostSumsWServiceImpl extends ServiceImpl<TfCostSumsWMapper, TfCostSumsW> implements TfCostSumsWService {
    /**
     * 工程基础
     */
    @Autowired
    private TfProjectBaseWService projectBaseService;
    /**
     * 费用汇总其它目
     */
    @Autowired
    private TfSumsOtherItemWService sumsOtherItemService;

    /**
     * 修改保存费用汇总
     * @param dto 费用汇总信息
     * @return 保存结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveCostSums(CostSumsDto dto) {
        int count = projectBaseService.count(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId,dto.getProcessId())
        //        .isNotNull(TfProjectBaseW::getApplyDate)
        );
        if(count==0){
            return "工程不存在，不能汇总费用";
        }
        count = this.count(new QueryWrapper<TfCostSumsW>().lambda().eq(TfCostSumsW::getProcessId,dto.getProcessId()).eq(TfCostSumsW::getIsComplete,FlowConstant.SHORT_ONE)
                .eq(TfCostSumsW::getIsDel,FlowConstant.SHORT_ZERO));
        if(count>0){
            return "费用已经汇总完成，无法在汇总！";
        }


        this.remove(new QueryWrapper<TfCostSumsW>().lambda().eq(TfCostSumsW::getProcessId,dto.getProcessId())
                .eq(TfCostSumsW::getIsValid, FlowConstant.SHORT_ONE).eq(TfCostSumsW::getIsDel,FlowConstant.SHORT_ZERO));
        TfCostSumsW costSums = new TfCostSumsW();
        BeanUtils.copyProperties(dto, costSums);
        costSums.setIsDel(FlowConstant.SHORT_ZERO);
        costSums.setIsValid(FlowConstant.SHORT_ONE);
        costSums.setCreateDept(CollUtil.isNotEmpty(SecurityUtils.getUserDeptIds())?SecurityUtils.getUserDeptIds().get(0):null);
        costSums.setCreatePost(CollUtil.isNotEmpty(SecurityUtils.getUserPostIds())?SecurityUtils.getUserPostIds().get(0):null);
        this.save(costSums);
        if(CollUtil.isNotEmpty(dto.getOtherItem())){
            sumsOtherItemService.saveOtherItems(dto.getOtherItem(),dto.getProcessId());
        }
        return true;
    }

    /**
     * 根据流程ID删除费用汇总信息
     * @param processId 主流程id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean removeByprocessId(Long processId) {
        this.update(new UpdateWrapper<TfCostSumsW>().lambda().eq(TfCostSumsW::getProcessId,processId).eq(TfCostSumsW::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(TfCostSumsW::getIsValid,FlowConstant.SHORT_ONE).set(TfCostSumsW::getIsValid,FlowConstant.SHORT_ZERO).set(TfCostSumsW::getIsDel,FlowConstant.SHORT_ONE));
        sumsOtherItemService.update(new UpdateWrapper<TfSumsOtherItemW>().lambda().eq(TfSumsOtherItemW::getProcessId,processId)
                .eq(TfSumsOtherItemW::getIsValid,FlowConstant.SHORT_ONE).eq(TfSumsOtherItemW::getIsDel,FlowConstant.SHORT_ZERO)
                .set(TfSumsOtherItemW::getIsDel,FlowConstant.SHORT_ONE).set(TfSumsOtherItemW::getIsValid,FlowConstant.SHORT_ZERO));
        return true;
    }

    /**
     * 根据主流程ID查询费用汇总信息
     * @param processId 主流程ID
     * @return 费用汇总信息
     */
    @Override
    public CostSumsDto getByProcessId(Long processId) {
        return this.baseMapper.getCostSumsByProcessId(processId);
    }
}
