package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfDisclosureW;
import com.tofly.workflow.buss.mapper.TfDisclosureWMapper;
import com.tofly.workflow.buss.service.TfDisclosureWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author haiqiu
 * @description 针对表【TF_DISCLOSURE_W(技术交底)】的数据库操作Service实现
 * @createDate 2022-08-29 10:17:38
 */
@Service("tfDisclosureWService")
public class TfDisclosureWServiceImpl extends ServiceImpl<TfDisclosureWMapper, TfDisclosureW> implements TfDisclosureWService, BaseService {

    @Value("${zmrq.ftp.disclosure}")
    private String disclosure;


    @Autowired
    private IFileInfoService iFileInfoService;

    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseObject(json, TfDisclosureW.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if (t == null) {
            throw new RuntimeException("数据不完整");
        }
        String s = JSONObject.toJSONString(t);
        TfDisclosureW data = JSONObject.parseObject(s, TfDisclosureW.class);
        //数据校验
        if(pb.getSaveState()!=null&&pb.getSaveState()== FlowConstant.INTEGER_ONE) {
            checkData(data);
        }
        //附件
        List<MultipartFile> nodeFile = FileNameCheckUtils.getNodeFile(fileList, data.getAnnexNameList());
        if (!CollectionUtils.isEmpty(nodeFile)) {
            data.setAnnex(iFileInfoService.uploadFiles(nodeFile, data.getAnnex(), null, disclosure, "TF_DISCLOSURE_W"));
        }
        //保存
        data.setProcessId(pb.getId());
        data.setNodeId(pb.getNodeId());
        //移除旧数据
        remove(Wrappers.<TfDisclosureW>lambdaQuery().
                eq(pb.getId() != null, TfDisclosureW::getProcessId, pb.getId())
                .eq(pb.getNodeId() != null, TfDisclosureW::getNodeId, pb.getNodeId()));
        //添加新数据
        return saveOrUpdate(data);
    }

    /**
     * 获取数据
     *
     * @param processId 流程ID
     * @param nodeId    步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfDisclosureW>().lambda()
                .eq(processId != null, TfDisclosureW::getProcessId, processId)
                .eq(nodeId != null, TfDisclosureW::getNodeId, nodeId));
    }

    protected void checkData(TfDisclosureW tfDisclosureW) {
        if (ObjectUtils.isEmpty(tfDisclosureW)) {
            throw new RuntimeException("业务数据不能为空");
        }
        if (tfDisclosureW.getDisclosureDate() == null) {
            throw new RuntimeException("交底日期不能为空");
        }
        if (!StringUtils.hasText(tfDisclosureW.getAdder())) {
            throw new RuntimeException("交底地址不能为空");
        }
        if (!StringUtils.hasText(tfDisclosureW.getProjectUsers())) {
            throw new RuntimeException("建设单位不能为空");
        }
        if (!StringUtils.hasText(tfDisclosureW.getBuildersUsers())) {
            throw new RuntimeException("施工单位不能为空");
        }
        if (!StringUtils.hasText(tfDisclosureW.getSupervisor())) {
            throw new RuntimeException("监理单位不能为空");
        }
    }

    @Override
    public Page<Object> pageList(Page page, TfDisclosureW tfDisclosureW) {
        return page(page, Wrappers.query(tfDisclosureW));
    }
}




