/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.dto.FileInfoDto;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfMonitoringReportW;
import com.tofly.workflow.buss.mapper.TfMonitoringReportWMapper;
import com.tofly.workflow.buss.service.TfMonitoringReportWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 监检提交上报登记(监检上报、监检登记）
 *
 * @author JF
 * @date Tue Aug 30 00:00:00 CST 2022
 */
@Service("tfMonitoringReportWService")
public class TfMonitoringReportWServiceImpl extends ServiceImpl<TfMonitoringReportWMapper, TfMonitoringReportW> implements TfMonitoringReportWService, BaseService {

    @Value("${zmrq.ftp.monitoringReport}")
    private String monitoringReportPath;

    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;
    @Autowired
    private IFileInfoService fileInfoService;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        TfMonitoringReportW entity = JSON.parseObject(String.valueOf(t),TfMonitoringReportW.class);
        List<String> annexNameList = entity.getAnnexNameList();
        List<String> checkAnnexNameList = entity.getCheckAnnexList();
        List<String> reportAnnexNameList = entity.getReportAnnexList();
        if(StringUtils.isNotBlank(FileNameCheckUtils.checkNodeFile(fileList,annexNameList))){
            return FileNameCheckUtils.checkNodeFile(fileList,annexNameList);
        }else if(StringUtils.isNotBlank(FileNameCheckUtils.checkNodeFile(fileList,checkAnnexNameList))){
            return FileNameCheckUtils.checkNodeFile(fileList,checkAnnexNameList);
        }
        return FileNameCheckUtils.checkNodeFile(fileList,reportAnnexNameList);
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }

        TfMonitoringReportW entity = JSON.parseObject(String.valueOf(t),TfMonitoringReportW.class);
        entity.setProcessId(pb.getId());
        entity.setNodeId(pb.getNodeId());

        TfMonitoringReportW entityQ = this.getOne(new QueryWrapper<TfMonitoringReportW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId()));
        if(fileList != null && fileList.size() != 0){
            FileInfoDto fileInfoDto = new FileInfoDto();
            fileInfoDto.setFileList(fileList);
            fileInfoDto.setPath(monitoringReportPath);
            fileInfoDto.setTableName(FileTableEnum.TF_DESIGN_BUDGET_W.getTableName());
            fileInfoDto.setFileNameList(entity.getAnnexNameList());
            if(entityQ == null){
                entity.setAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(entity.getCheckAnnexList());
                entity.setCheckAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(entity.getReportAnnexList());
            }else {
                fileInfoDto.setAgoInfos(entityQ.getAnnex());
                fileInfoDto.setNowInfos(entity.getAnnex());
                entity.setAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(entity.getCheckAnnexList());
                fileInfoDto.setAgoInfos(entityQ.getCheckAnnex());
                fileInfoDto.setNowInfos(entity.getCheckAnnex());
                entity.setCheckAnnex(fileInfoService.getFileInfo(fileInfoDto));

                fileInfoDto.setFileNameList(entity.getReportAnnexList());
                fileInfoDto.setAgoInfos(entityQ.getReportAnnex());
                fileInfoDto.setNowInfos(entity.getReportAnnex());
            }
            entity.setReportAnnex(fileInfoService.getFileInfo(fileInfoDto));
        }
        if(entityQ != null){
            this.removeById(entityQ.getId());
        }
        // 对象入库
        this.save(entity);

        /**保存工程公共信息*/
//        if(StringUtils.isNotEmpty(entity.getMonitoringUnit())) {
//            Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getId()).select(TfProjectBaseW::getId)
//                    , r -> Long.parseLong(String.valueOf(r)));
//            TfProjectBaseW npbase = new TfProjectBaseW();
//            npbase.setMonitoringUnit(entity.getMonitoringUnit());
//            if (pbId == null) {
//                npbase.setConfigId(pb.getConfigId());
//                npbase.setEdition(pb.getEdition());
//                npbase.setProcessNumber(pb.getProcessNumber());
//                npbase.setTenantId(pb.getTenantId());
//                npbase.setRelationId(pb.getRelationId());
//                npbase.setProcessId(pb.getId());
//                npbase.setCategory(pb.getCategory());
//                npbase.setProcessType(pb.getProcessType());
//                npbase.setRegisterDept(pb.getRegisterDept());
//                npbase.setRegisterStaff(pb.getRegisterStaff());
//                npbase.setRegisterPost(pb.getRegisterPost());
//                npbase.setRegisterTime(pb.getRegisterTime());
//            } else {
//                npbase.setId(pbId);
//            }
//            pBaseService.saveOrUpdate(npbase);
//        }
        return null;
    }
    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfMonitoringReportW>().lambda()
                .eq(processId != null, TfMonitoringReportW::getProcessId, processId)
                .eq(nodeId != null, TfMonitoringReportW::getNodeId, nodeId));
    }

    /**
     * 获取监检上报登记数
     * @param processId 流程id
     * @param nodestate 节点状态
     * @param processSstates 流程状态
     * @return
     */
    @Override
    public int getMReportCount(Long processId, Integer nodestate, List<Integer> processSstates) {
        return this.baseMapper.getMReportCount(processId,nodestate,processSstates);
    }
}
