/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfPayBillW;
import com.tofly.workflow.buss.entity.dto.PayBillDto;
import com.tofly.workflow.buss.mapper.TfPayBillWMapper;
import com.tofly.workflow.buss.service.TfPayBillWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 工程支出账单(支付给第三方公司)
 *
 * @author xqf
 * @date Tue Sep 06 00:00:00 CST 2022
 */
@Service("tfPayBillWService")
public class TfPayBillWServiceImpl extends ServiceImpl<TfPayBillWMapper, TfPayBillW> implements TfPayBillWService , BaseService {

    /**
     * 文件
     */
    @Autowired
    private IFileInfoService fileService;

    @Value("${zmrq.ftp.payBillPath}")
    private String payBillPath;

    @Override
    public Object getJsonToObject(String json) {
        if(StringUtils.isEmpty(json)){
            return null;
        }
        List<PayBillDto> pb = JSON.parseArray(json, PayBillDto.class);
        return pb;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t==null){
            return FlowConstant.ISNOTNUL;
        }
        List<PayBillDto> pbs = null;
        if(t instanceof String){
            pbs = JSON.parseArray(t.toString(), PayBillDto.class);
        }else {
            pbs = (List<PayBillDto>)t;
        }
        for (PayBillDto pb:pbs) {
            if(StringUtils.isEmpty(pb.getSettlementUnit())){
                return "SettlementUnit"+FlowConstant.ISNOTNUL;
            }
            if(pb.getAmountMoney()==null){
                return "AmountMoney"+FlowConstant.ISNOTNUL;
            }
            if(CollUtil.isNotEmpty(pb.getFileName())) {
                String check = FileNameCheckUtils.checkNodeFile(fileList, pb.getFileName());
                if(StringUtils.isNotEmpty(check)){
                    return check;
                }
            }
        }

        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(t==null){
            return true;
        }
        List<PayBillDto> pbs = null;
        if(t instanceof String){
            pbs = JSON.parseArray(t.toString(), PayBillDto.class);
        }else {
            pbs = (List<PayBillDto>)t;
        }
        if(CollUtil.isEmpty(pbs)){
            return true;
        }
        Map<Long,TfPayBillW> map = this.list(new QueryWrapper<TfPayBillW>().lambda().eq(TfPayBillW::getProcessId,pb.getId())
                .eq(TfPayBillW::getNodeId,pb.getNodeId())).stream().collect(Collectors.toMap(TfPayBillW::getId,r->r));
        this.remove(new QueryWrapper<TfPayBillW>().lambda().eq(TfPayBillW::getProcessId,pb.getId())
                .eq(TfPayBillW::getNodeId,pb.getNodeId()));
        List<TfPayBillW> saves = new ArrayList<>();
        for (PayBillDto pbdto:pbs) {
            TfPayBillW npb = new TfPayBillW();
            BeanUtils.copyProperties(pbdto, npb);
            npb.setProcessId(pb.getId());
            npb.setNodeId(pb.getNodeId());
            TfPayBillW oldTb = null;
            if(pbdto.getId()!=null) {
                oldTb = map.get(pbdto.getId());
            }
            if(CollUtil.isNotEmpty(pbdto.getFileName())||(oldTb!=null&& StringUtils.isNotEmpty(oldTb.getAnnex())&& !oldTb.equals(pbdto.getAnnex()))) {
                String str = fileService.uploadFiles(fileList,pbdto.getAnnex(),oldTb!=null?oldTb.getAnnex():null,payBillPath,FileTableEnum.TF_PAY_BILL_W.getTableName());
                npb.setAnnex(str);
            }
            saves.add(npb);
        }
        return this.saveBatch(saves);
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return this.list(new QueryWrapper<TfPayBillW>().lambda().eq(TfPayBillW::getProcessId,processId).eq(TfPayBillW::getNodeId,nodeId));
    }
}
