/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.Constant;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.vo.CalendarVo;
import com.tofly.workflow.baseconf.service.TfCalendarPService;
import com.tofly.workflow.buss.entity.TfChargeRefundW;
import com.tofly.workflow.buss.entity.TfConstructionTeamW;
import com.tofly.workflow.buss.entity.TfContractW;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.dto.BuilderBaseDto;
import com.tofly.workflow.buss.entity.dto.ProjectBaseDto;
import com.tofly.workflow.buss.entity.dto.TallyDateDto;
import com.tofly.workflow.buss.entity.vo.ConstructionTeamVo;
import com.tofly.workflow.buss.entity.vo.ProjectBaseVo;
import com.tofly.workflow.buss.mapper.TfProjectBaseWMapper;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.vo.IndoorRenovationVo;
import com.tofly.workflow.process.entity.vo.ProcessNodeVo;
import com.tofly.workflow.process.entity.vo.ProcessVo;
import com.tofly.workflow.process.service.BaseService;
import com.tofly.workflow.process.service.TfProcessLService;
import com.tofly.workflow.utils.WorkDayCalculateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 工程基础表
 *
 * @author xqf
 * @date Wed Sep 14 00:00:00 CST 2022
 */
@Service("tfProjectBaseWService")
public class TfProjectBaseWServiceImpl extends ServiceImpl<TfProjectBaseWMapper, TfProjectBaseW> implements TfProjectBaseWService {

    /**
     * 工作日应用日历
     */
    @Autowired
    private TfCalendarPService calendarService;
    @Autowired
    private TfProcessLService processLService;

    /**
     * 通过id、主流程id 查询工程基础表
     * @param id 工程基础信息ID
     * @param processId 主流程id
     * @return 查询结果
     */
    @Override
    public ProjectBaseVo getBaseById(Long id, Long processId) {
        if(id==null&& processId==null){
            return null;
        }
        return this.baseMapper.getBaseById(id,processId);
    }

    @Override
    public Page<IndoorRenovationVo> IndoorRenovationReport(Page page,String tenantId,String projectNature,String processType,String likeStr,String startTime,String endTime) {
        List<Long> tenantIds = null;
        if (StringUtils.isNotBlank(tenantId)) {
            tenantIds = Arrays.stream(tenantId.split(Constant.CSV)).map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
        }
        List<Long> projectNatures = null;
        if (StringUtils.isNotBlank(projectNature)) {
            projectNatures = Arrays.stream(projectNature.split(Constant.CSV)).map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
        }
        List<Long> processTypes = null;
        if (StringUtils.isNotBlank(processType)) {
            processTypes = Arrays.stream(processType.split(Constant.CSV)).map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
        }
        Page<IndoorRenovationVo> projectBaseVoPage = this.baseMapper.getBaseList(page,tenantIds,projectNatures,processTypes,likeStr,startTime,endTime);
        if (CollUtil.isEmpty(projectBaseVoPage.getRecords())) {
            return projectBaseVoPage;
        }
        Map<Long, List<ProcessNodeVo>> processVoList = processLService.getProcessNodeList(projectBaseVoPage.getRecords().stream().map(IndoorRenovationVo::getProcessId).collect(Collectors.toList()));
        projectBaseVoPage.getRecords().forEach(vo -> {
            List<ProcessNodeVo> nodes = processVoList.get(vo.getProcessId());
            if (CollUtil.isEmpty(nodes)) {
                return;
            }
            nodes.forEach(node -> {
                if (node.getPageCode().equals("register_addProject")) {
                    vo.setRegisterUser(node.getHandleUserName());
                    vo.setRegisterTime(node.getHandleTime());
                    return;
                }
                if (node.getPageCode().equals("register_taskAllocation")) {
                    vo.setTaskShareUser(node.getHandleUserName());
                    vo.setTaskShareTime(node.getHandleTime());
                    return;
                }
                if (node.getPageCode().equals("designBudget_addBudget")) {
                    vo.setSurveyBudgetUser(node.getHandleUserName());
                    vo.setSurveyBudgetTime(node.getHandleTime());
                    return;
                }
                if (node.getPageCode().equals("userPayment_addUserSettle")) {
                    vo.setChargeUser(node.getHandleUserName());
                    vo.setChargeTime(node.getHandleTime());
                    Map<String,Object> dtmap = new HashMap<>();
                    BaseService bs = SpringUtil.getBean("tfChargeRefundWService");
                    dtmap.put("tfChargeRefundWService",bs.getBaseDatas(node.getProcessId(),node.getId(),node.getTenantId()));
                    if (CollUtil.isNotEmpty((List<TfChargeRefundW>) dtmap.get("tfChargeRefundWService"))) {
                        vo.setContractTotalPrice(((List<TfChargeRefundW>) dtmap.get("tfChargeRefundWService")).get(0).getAmount());
                    }
                    return;
                }
                if (node.getPageCode().equals("proDispatch_addDispatch")) {
                    vo.setDispatchUser(node.getHandleUserName());
                    vo.setDispatchTime(node.getHandleTime());
                    return;
                }
                if (node.getPageCode().equals("proDispatch_proTeamPerson")) {
                    vo.setTeamGroupUser(node.getHandleUserName());
                    vo.setTeamGroupTime(node.getHandleTime());
                    Map<String,Object> dtmap = new HashMap<>();
                    BaseService bs = SpringUtil.getBean("tfConstructionTeamWService");
                    dtmap.put("tfConstructionTeamWService",bs.getBaseDatas(node.getProcessId(),node.getId(),node.getTenantId()));
                    if (dtmap.get("tfConstructionTeamWService") != null) {
                        vo.setTeamGroupWorker(((ConstructionTeamVo) dtmap.get("tfConstructionTeamWService")).getConUserNames());
                    }
                    return;
                }
                if (node.getPageCode().equals("proCheck_addCheck")) {
                    vo.setAcceptUser(node.getHandleUserName());
                    vo.setAcceptTime(node.getHandleTime());
                    return;
                }
                if (node.getPageCode().equals("proSettlement_addSettle")) {
                    vo.setSettleUser(node.getHandleUserName());
                    vo.setSettleTime(node.getHandleTime());
                    return;
                }
                if (node.getPageCode().equals("proSettlement_userSettle")) {
                    vo.setHandleUser(node.getHandleUserName());
                    vo.setHandleTime(node.getHandleTime());
                    Map<String,Object> dtmap = new HashMap<>();
                    BaseService bs = SpringUtil.getBean("tfChargeRefundWService");
                    dtmap.put("tfChargeRefundWService",bs.getBaseDatas(node.getProcessId(),node.getId(),node.getTenantId()));
                    if (CollUtil.isNotEmpty((List<TfChargeRefundW>) dtmap.get("tfChargeRefundWService"))) {
                        vo.setHandlePrice(((List<TfChargeRefundW>) dtmap.get("tfChargeRefundWService")).get(0).getAmountPayable());
                    }
                }
            });
        });
        return projectBaseVoPage;
    }

    /**
     * 工程基础表分页查询
     * @param page 分页
     * @param dto 基础信息查询条件
     * @return 查询结果
     */
    @Override
    public Page<TfProjectBaseW> pageProjectBase(Page page, ProjectBaseDto dto) {
        return this.baseMapper.pageProjectBase( page,  dto);
    }

    /**
     * 工程基础表
     * @param page 分页
     * @param dto 施工人员
     * @return 工程基础表
     */
    @Override
    public Page<TfProjectBaseW> getProjectBaseByBuilder(Page page, BuilderBaseDto dto) {
        return this.baseMapper.getProjectBaseByBuilder(page,dto);
    }

    /**
     * 获取流程工程性质
     * @param processId 流程id
     * @return
     */
    @Override
    public Integer getProjectTypeValue(Long processId) {
        return this.baseMapper.getProjectTypeValue(processId);
    }

    /**
     * 计算日期
     * @param dto 参数
     * @return
     */
    @Override
    public Object getTallyDate(TallyDateDto dto) {
        Date date = null;
        Date dateTime = dto.getTime()==null?new Date():dto.getTime();
        if (dto.getIsWork() == FlowConstant.SHORT_ONE) {
            Date endDate = DateUtil.offset(dateTime, DateField.DAY_OF_MONTH, (dto.getNumber() / 7 + 1) * 2 + (dto.getNumber() / 30 + 1) * 7);
            List<CalendarVo> calendars = calendarService.getCalendar(dateTime, endDate, SecurityUtils.getTenantId());
            if (CollUtil.isEmpty(calendars)) {
                calendars = calendarService.getCalendar(dateTime, endDate, null);
            }
            List<String> workDays = new ArrayList<>();
            List<String> restDays = new ArrayList<>();
            if (CollUtil.isNotEmpty(calendars)) {
                workDays = calendars.stream().map(r -> {
                    if (r.getType() == FlowConstant.SHORT_ONE) return DateUtil.formatDate(r.getDataTime());
                    return null;
                }).filter(r -> r != null).collect(Collectors.toList());
                restDays = calendars.stream().map(r -> {
                    if (r.getType() == FlowConstant.SHORT_ZERO) return DateUtil.formatDate(r.getDataTime());
                    return null;
                }).filter(r -> r != null).collect(Collectors.toList());
            }
            date = WorkDayCalculateUtils.getDate(dateTime, dto.getNumber(), workDays, restDays);
        } else {
            date = DateUtil.endOfDay(DateUtil.offset(dateTime, DateField.DAY_OF_MONTH, dto.getNumber()));
        }
        return date;
    }

}
