/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.dto.FileInfoDto;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.entity.TfMatterPriceP;
import com.tofly.workflow.baseconf.service.TfMatterPricePService;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.TfProjectDispatchW;
import com.tofly.workflow.buss.entity.TfProjectMaterialW;
import com.tofly.workflow.buss.entity.dto.TfProjectDispatchWDto;
import com.tofly.workflow.buss.entity.vo.PMaterialVo;
import com.tofly.workflow.buss.mapper.TfProjectDispatchWMapper;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfProjectDispatchWService;
import com.tofly.workflow.buss.service.TfProjectMaterialWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 工程派工
 *
 * @author JF
 * @date Mon Aug 29 00:00:00 CST 2022
 */
@Service("tfProjectDispatchWService")
public class TfProjectDispatchWServiceImpl extends ServiceImpl<TfProjectDispatchWMapper, TfProjectDispatchW> implements TfProjectDispatchWService, BaseService {

    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;

    @Value("${zmrq.ftp.projectDispatch}")
    private String projectDispatchPath;
    @Autowired
    private TfMatterPricePService matterPricePService;
    @Autowired
    private TfProjectMaterialWService projectMaterialWService;
    @Autowired
    private IFileInfoService fileInfoService;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        TfProjectDispatchWDto entityDto = JSON.parseObject(String.valueOf(t), TfProjectDispatchWDto.class);
        if(entityDto.getProjectDispatchW() == null){
            return "工程派工对象为空";
        }
//        if(entityDto.getProjectMaterialList() == null || entityDto.getProjectMaterialList().size() == 0){
//            return "工程材料列表为空";
//        }
//        if(projectMaterialW.getMatterId() == null){
//            return "物料ID为空";
//        }
//        if(projectMaterialW.getMaterialType() == null){
//            return "物资类型为空";
//        }
        TfProjectDispatchW entity = entityDto.getProjectDispatchW();
        return FileNameCheckUtils.checkNodeFile(fileList,entity.getAnnexNameList());
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }

        TfProjectDispatchWDto entityDto = JSON.parseObject(String.valueOf(t), TfProjectDispatchWDto.class);
        List<TfProjectMaterialW> projectMaterialList = entityDto.getProjectMaterialList();
        TfProjectDispatchW entity = entityDto.getProjectDispatchW();
        entity.setProcessId(pb.getId());
        entity.setNodeId(pb.getNodeId());
        if(entity.getIsSupply()==null) {
            entity.setIsSupply(FlowConstant.SHORT_ONE);
        }

        TfProjectDispatchW entityQ = this.getOne(new QueryWrapper<TfProjectDispatchW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId()));
        if(fileList != null && fileList.size() != 0){
            FileInfoDto fileInfoDto = new FileInfoDto();
            fileInfoDto.setFileList(fileList);
            fileInfoDto.setPath(projectDispatchPath);
            fileInfoDto.setTableName(FileTableEnum.TF_PROJECT_DISPATCH_W.getTableName());
            fileInfoDto.setFileNameList(entity.getAnnexNameList());
            if (entityQ != null) {
                fileInfoDto.setAgoInfos(entityQ.getAnnex());
                fileInfoDto.setNowInfos(entity.getAnnex());
            }
            entity.setAnnex(fileInfoService.getFileInfo(fileInfoDto));
        }
        if(entityQ != null){
            this.removeById(entityQ.getId());
        }
        projectMaterialWService.remove(new QueryWrapper<TfProjectMaterialW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId())
                .eq(pb.getRelationId() !=null,"LIKE_PROCESS_ID",pb.getRelationId()));
        // 对象入库
        this.save(entity);
        if(projectMaterialList != null && projectMaterialList.size() != 0){
            for(TfProjectMaterialW projectMaterial : projectMaterialList){
                projectMaterial.setId(null);
                projectMaterial.setProcessId(pb.getId());
                projectMaterial.setNodeId(pb.getNodeId());
                projectMaterial.setLikeProcessId(pb.getRelationId());
                //projectMaterialWService.save(projectMaterial);
            }
            projectMaterialWService.saveBatch(projectMaterialList);
        }

        /**填充工程基础信息*/
        if(FlowConstant.INTEGER_ONE.equals(pb.getSaveState())) {
            Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getId()).select(TfProjectBaseW::getId)
                    , r -> Long.parseLong(String.valueOf(r)));
            TfProjectBaseW npbase = new TfProjectBaseW();
            if (pbId == null) {
                npbase.setConfigId(pb.getConfigId());
                npbase.setEdition(pb.getEdition());
                npbase.setProcessNumber(pb.getProcessNumber());
                npbase.setTenantId(pb.getTenantId());
                npbase.setRelationId(pb.getRelationId());
                npbase.setProcessId(pb.getId());
                npbase.setCategory(pb.getCategory());
                npbase.setProcessType(pb.getProcessType());
                npbase.setRegisterDept(pb.getRegisterDept());
                npbase.setRegisterStaff(pb.getRegisterStaff());
                npbase.setRegisterPost(pb.getRegisterPost());
                npbase.setRegisterTime(pb.getRegisterTime());
            } else {
                npbase.setId(pbId);
            }
            npbase.setIsSupply(entity.getIsSupply());
            npbase.setProjectNature(entity.getProjectType());
            npbase.setFinishDate(entity.getFinishDate());
            npbase.setBuildUnit(entity.getBuildUnit());
            npbase.setSupervisor(entity.getSupervisor());
            npbase.setDataTime(entity.getDataTime());
            pBaseService.saveOrUpdate(npbase);
        }
        return null;
    }

    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        List<TfProjectDispatchW> list = list(new QueryWrapper<TfProjectDispatchW>().lambda()
                .eq(processId != null, TfProjectDispatchW::getProcessId, processId)
                .eq(nodeId != null, TfProjectDispatchW::getNodeId, nodeId));
        for(TfProjectDispatchW one : list){
            if(one.getNodeId() == null || one.getProcessId() == null){
                continue;
            }
            //List<TfProjectMaterialW> materialList = projectMaterialWService.getMaterialByProcessId(processId,nodeId);
            List<PMaterialVo> materialList = projectMaterialWService.getMaterialByProcessId(processId,nodeId);
            if(CollUtil.isNotEmpty(materialList)){
                for (PMaterialVo tpm:materialList) {
                    TfMatterP mp = tpm.getMatterP();
                    if(mp!=null) {
                       mp.setMatterPrice(matterPricePService.getOne(new QueryWrapper<TfMatterPriceP>()
                               .eq("MATTER_ID",mp.getId())
                               .eq("TENANT_ID",tenantId)
                               .eq("IS_VALID",1)));
                    }
                }
            }
            one.setMaterialList(materialList);
        }
        return list;
    }

}
