/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.entity.TfMatterPriceP;
import com.tofly.workflow.baseconf.entity.vo.MatterNumberVo;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.baseconf.service.TfMatterPricePService;
import com.tofly.workflow.buss.entity.TfProjectMaterialW;
import com.tofly.workflow.buss.entity.dto.DispatchRecordDto;
import com.tofly.workflow.buss.entity.dto.TfProjectMaterialDto;
import com.tofly.workflow.buss.entity.vo.MaterialItemVo;
import com.tofly.workflow.buss.entity.vo.PMaterialVo;
import com.tofly.workflow.buss.entity.vo.TfProjectMaterialWVo;
import com.tofly.workflow.buss.mapper.TfProjectMaterialWMapper;
import com.tofly.workflow.buss.service.TfProjectMaterialWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import com.tofly.workflow.wzyy.entity.MaterialItemOutIn;
import com.tofly.workflow.wzyy.entity.MaterialOutIn;
import com.tofly.workflow.wzyy.service.MaterialOutInService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 工程材料(设计预算、领料出库)
 *
 * @author JF
 * @date Tue Aug 30 00:00:00 CST 2022
 */
@Service("tfProjectMaterialWService")
@Slf4j
public class TfProjectMaterialWServiceImpl extends ServiceImpl<TfProjectMaterialWMapper, TfProjectMaterialW> implements TfProjectMaterialWService, BaseService {

    @Autowired
    private TfMatterPService matterPService;
    @Autowired
    private TfMatterPricePService matterPricePService;
    @Autowired
    private TfMatterPService tfMatterPService;
    @Autowired
    private TfMatterPricePService tfMatterPricePService;
    @Autowired
    private MaterialOutInService  materialOutInService;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        List<TfProjectMaterialW> entityList = JSON.parseArray(String.valueOf(t),TfProjectMaterialW.class);
        for(TfProjectMaterialW entity : entityList){
            if(entity.getMatterId() == null){
                return "物料ID为空";
            }
            if(entity.getMaterialType() == null){
                return "物资类型为空";
            }
        }
        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }
        this.remove(new QueryWrapper<TfProjectMaterialW>().eq("PROCESS_ID",pb.getId()).eq("NODE_ID",pb.getNodeId()));
        List<TfProjectMaterialW> entityList = JSON.parseArray(String.valueOf(t),TfProjectMaterialW.class);
        List<Long> mid = new ArrayList<>();
        for(TfProjectMaterialW entity : entityList){
            if(entity.getMatterId()==null){
                return "物资ID为空！";
            }
            if(mid.contains(entity.getMatterId())){
                return "物资id重复！";
            }
            log.info("物资类型{}，processId:{}",entity.getMaterialType(),entity.getProcessId());
            mid.add(entity.getMatterId());
            entity.setId(null);
            entity.setProcessId(pb.getId());
            entity.setNodeId(pb.getNodeId());
            entity.setLikeProcessId(pb.getRelationId());
            // 对象入库
            //this.save(entity);
        }
        if(CollUtil.isNotEmpty(entityList)){
            this.saveBatch(entityList);
        }
        return null;
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        List<TfProjectMaterialW> list = this.list(new QueryWrapper<TfProjectMaterialW>().lambda()
                .eq(processId != null, TfProjectMaterialW::getProcessId, processId)
                .eq(nodeId != null, TfProjectMaterialW::getNodeId, nodeId));



        if (CollUtil.isEmpty(list)) {
            return null;
        }
        Long zpid = list.get(0).getLikeProcessId();
        list.forEach(e -> {e.setBudgetQuantity(0d);e.setMatterP(tfMatterPService.getById(e.getMatterId()));});
        List<TfProjectMaterialWVo> budgetNumList = this.baseMapper.getTotalBudgetNumList(zpid);

        if(CollUtil.isNotEmpty(list)) {
            Map<Long, TfProjectMaterialW> pmmap = list.stream().collect(Collectors.toMap(TfProjectMaterialW::getMatterId, r -> r));
            if (CollUtil.isNotEmpty(budgetNumList)) {
                for (TfProjectMaterialWVo budget : budgetNumList) {
                    if (pmmap.containsKey(budget.getMatterId())) {
                        TfProjectMaterialW projectMaterial = pmmap.get(budget.getMatterId());
                        if(projectMaterial!=null&&projectMaterial.getMatterP()!=null) {
                            TfMatterP mp = projectMaterial.getMatterP();
                            mp.setMatterPrice(tfMatterPricePService.getOne(new QueryWrapper<TfMatterPriceP>()
                                    .eq("MATTER_ID",mp.getId())
                                    .eq("TENANT_ID", tenantId)
                                    .eq("IS_VALID",1)));

                            //通过该材料id 查询
                            final Long matterId = projectMaterial.getMatterId();
                            //通过流程id查询
                            final List<MaterialOutIn> list1 = materialOutInService.list(Wrappers.<MaterialOutIn>lambdaQuery()
                                    .eq(MaterialOutIn::getMType, FlowConstant.SHORT_ONE).
                                            eq(MaterialOutIn::getProcessId, processId));
                            for (MaterialOutIn materialOutIn : list1) {
                                final List<MaterialItemOutIn> item = materialOutIn.getItem();
                                for (MaterialItemOutIn materialItemOutIn : item) {
                                    if(mp.getYyId().equals(materialItemOutIn.getProduct())){
                                        if(StringUtils.isNotBlank(materialOutIn.getCode())){
                                            projectMaterial.setCode(materialOutIn.getCode());
                                        }else{// 1790578952795848719 1790578952795848719
                                            //调用接口查询 并更新
                                            final Long yyId = materialOutIn.getYyId();

                                            final String yyCode = materialOutInService.getYyCode(String.valueOf(yyId));
                                            materialOutIn.setCode(yyCode);
                                            materialOutInService.saveOrUpdate(materialOutIn);
                                            projectMaterial.setCode(yyCode);
                                        }


                                    }else{
                                       log.error("未找到关联的出库信息");
                                    }
                                }
                            }
                        }
                        projectMaterial.setBudgetQuantity(budget.getQuantityAll());

                        // 从出入库记录查询


                    }
                }
            }
        }
        return list;
    }

    @Override
    public Page<TfProjectMaterialW> getTfProjectMaterialPage(Page page, TfProjectMaterialDto dto) {
        return this.baseMapper.getTfProjectMaterialPage(page,dto);
    }
    @Override
    public List<TfProjectMaterialWVo> getListByMatter(TfProjectMaterialDto dto) {
        List<TfProjectMaterialWVo> list = this.baseMapper.getAllListByMatter(dto, dto.getTenantId());
        for(TfProjectMaterialWVo vo : list){
            if(vo == null || vo.getMatterId() == null){
                continue;
            }
            vo.setQuantityAll(vo.getQuantity());
            //获取已领量
            vo.setReceivedNum(this.baseMapper.getPickingNum(dto.getProcessId(),vo.getMatterId()));
            //获取补料量
            Double supplyNums = this.baseMapper.getSupplyNum(null,dto.getProcessId(),vo.getMatterId(),FlowConstant.SHORT_FOUR);
            //获取退料量
            Double returnNums = this.baseMapper.getReturnNum(dto.getProcessId(),vo.getMatterId());
            //获取待确认材料数量
            vo.setApplyNum(this.baseMapper.getApplyNum(dto.getProcessId(),vo.getMatterId()));


            vo.setReturnNums(returnNums);

            if(supplyNums != null){
                vo.setSupplyNum(supplyNums);
                //获取总可领量
                vo.setQuantity(vo.getQuantity()-supplyNums);
            }
        }
        if(FlowConstant.SHORT_ONE==dto.getIsExport()){
            ExcelUtils.getListToExcel(TfProjectMaterialWVo.class,list,"材料计划");
            return null;
        }
        return list;

    }

    @Override
    public List<Map<String, Object>> getDispatchRecord(DispatchRecordDto dto) {
        return this.baseMapper.getDispatchRecord(dto);
    }

    @Override
    public List<PMaterialVo> getMaterialByProcessId(Long processId, Long nodeId) {
        return this.baseMapper.getMaterialByProcessId(processId,nodeId);
    }

    @Override
    public List<MatterNumberVo> getInOrOutMaterial(Long id, Short type) {
        return this.baseMapper.getInOrOutMaterial(id,type);
    }

    /**
     * 获取物料出库信息
     * @param processId 流程id
     * @param nodeId 步骤id
     * @param type 物资类型 1预算;2开工;3领料;4补料;5退料;6其它;7验收申请;8入库;9核销
     * @return
     */
    @Override
    public List<MaterialItemVo> getMaterialOutIn(Long processId, Long nodeId, Short type) {
        return this.baseMapper.getMaterialOutIn(processId,nodeId,type);
    }

}


