package com.tofly.workflow.buss.service.impl;


import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfProjectTransferW;
import com.tofly.workflow.buss.entity.dto.ProjectTransferDto;
import com.tofly.workflow.buss.entity.dto.TransferProjectDto;
import com.tofly.workflow.buss.entity.vo.ProjectTransferVo;
import com.tofly.workflow.buss.mapper.TfProjectTransferWMapper;
import com.tofly.workflow.buss.service.TfProjectTransferWService;
import com.tofly.workflow.buss.service.TfTransferProjectWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 项目移交 (主要潼南应用项目移交登记--单独填报)(TfProjectTransferW)表服务实现类
 *
 * @author HaiQiu
 * @since 2022-09-19 17:52:12
 */
@Service("tfProjectTransferWService")
public class TfProjectTransferWServiceImpl extends ServiceImpl<TfProjectTransferWMapper, TfProjectTransferW> implements TfProjectTransferWService, BaseService {

    @Autowired
    private IFileInfoService fileInfoService;

    @Value("${zmrq.ftp.tfsafetycheckw}")
    private String path;

    /**
     * 移交的项目列表
     */
    @Autowired
    private TfTransferProjectWService tProjectService;

    /**
     * 根据ID查询
     * @param id id
     * @return 数据
     */
    @Override
    public TfProjectTransferW get(Serializable id){
        return getById(id);
    }

    /**
     * 新增
     * @param tfProjectTransferW 实体
     */
    @Override
    public void insert(TfProjectTransferW tfProjectTransferW,List<MultipartFile> fileList){
        String files = fileInfoService.uploadFiles(fileList, tfProjectTransferW.getAnnex(), null, path, "TF_PROJECT_TRANSFER_W");
        tfProjectTransferW.setAnnex(files);
        if(!save(tfProjectTransferW)){
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     * @param tfProjectTransferW 实体
     */
    @Override
    public void edit(TfProjectTransferW tfProjectTransferW,List<MultipartFile> fileList){
        TfProjectTransferW transferW = getById(tfProjectTransferW.getId());
        String files = fileInfoService.uploadFiles(fileList, tfProjectTransferW.getAnnex(), transferW.getAnnex(), path, "TF_PROJECT_TRANSFER_W");
        tfProjectTransferW.setAnnex(files);
        if(!updateById(tfProjectTransferW)){
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 批量删除
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList){
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template(){
        TfProjectTransferW tfProjectTransferW = new TfProjectTransferW();
//        HqExcelUtils.createTemplate("TfProjectTransferW项目移交 (主要潼南应用项目移交登记--单独填报)", "TfProjectTransferW项目移交 (主要潼南应用项目移交登记--单独填报)",
//                TfProjectTransferW.class, Collections.singletonList(tfProjectTransferW));
        ExcelUtils.createTemplate("TfProjectTransferW项目移交 (主要潼南应用项目移交登记--单独填报)", "TfProjectTransferW项目移交 (主要潼南应用项目移交登记--单独填报)",
                null,TfProjectTransferW.class, Collections.singletonList(tfProjectTransferW));
    }

    /**
     * 数据导入
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file){
        List<TfProjectTransferW> datas = ExcelUtils.getExcelToList("TfProjectTransferW项目移交 (主要潼南应用项目移交登记--单独填报)", file, TfProjectTransferW.class);
        if (!CollectionUtils.isEmpty(datas)) {
            saveBatch(datas);
        }
    }

    /**
     * 分页查询
     * @param page 分页
     * @param tfProjectTransferW 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<TfProjectTransferW> pageList(Page<TfProjectTransferW> page, TfProjectTransferW tfProjectTransferW){
        return page(page, new QueryWrapper<>(tfProjectTransferW));
    }

    /**
     * 数据导出
     * @param page 分页
     * @param tfProjectTransferW 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @Override
    public void export(Page<TfProjectTransferW> page, TfProjectTransferW tfProjectTransferW, Boolean isAll){
        Page<TfProjectTransferW> pageData = new Page<>();
        if (isAll==null || !isAll){
            pageData = page(page,new QueryWrapper<>(tfProjectTransferW));
        }else {
            pageData.setRecords(list(new QueryWrapper<>(tfProjectTransferW)));
        }
        //导出数据
        ExcelUtils.getListToExcel(TfProjectTransferW.class,pageData.getRecords(),"TfProjectTransferW项目移交 (主要潼南应用项目移交登记--单独填报)");

    }

    /**
     * 查询所有数据
     * @param tfProjectTransferW 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<TfProjectTransferW> all(TfProjectTransferW tfProjectTransferW){
        return list(new QueryWrapper<>(tfProjectTransferW));
    }

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        ProjectTransferDto pt = JSON.parseObject(String.valueOf(t),ProjectTransferDto.class);
        String check = FileNameCheckUtils.checkNodeFile(fileList,pt.getFileName());
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        if(CollUtil.isNotEmpty(pt.getProjects())){
            return tProjectService.checkProjects(pt.getProjects(),fileList);
        }
        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }
        ProjectTransferDto dto = JSON.parseObject(String.valueOf(t),ProjectTransferDto.class);
        if(CollUtil.isNotEmpty(dto.getProjects())){
            List<Long> pid = new ArrayList<>();
            for (TransferProjectDto pd:dto.getProjects()) {
                if(pd.getProcessId()==null||pid.contains(pd.getProcessId())){
                    return "工程项目ID不存在或重复！";
                }
            }
        }
        TfProjectTransferW pt = new TfProjectTransferW();
        BeanUtils.copyProperties(dto,pt);
        pt.setIsDel(FlowConstant.SHORT_ZERO);
        pt.setProcessId(pb.getId());
        pt.setNodeId(pb.getNodeId());

        TfProjectTransferW oldPt = this.getOne(new QueryWrapper<TfProjectTransferW>().lambda().eq(TfProjectTransferW::getProcessId,pb.getId())
                .eq(TfProjectTransferW::getNodeId,pb.getNodeId()).eq(TfProjectTransferW::getIsDel,FlowConstant.SHORT_ZERO));
        List<MultipartFile> nodeFile1 = null;
        if(CollUtil.isNotEmpty(dto.getFileName())) {
            //附件
            nodeFile1 = FileNameCheckUtils.getNodeFile(fileList, dto.getFileName());
        }
        pt.setAnnex(fileInfoService.uploadFiles(nodeFile1, pt.getAnnex(),oldPt==null?null: oldPt.getAnnex(), path, "TF_PROJECT_TRANSFER_W"));
        if(oldPt!=null) {
            this.removeById(oldPt.getId());
        }
        this.save(pt);
        tProjectService.saveProject(dto.getProjects(),fileList,pt,oldPt!=null?oldPt.getId():null);

        return null;
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        List<ProjectTransferVo> list= this.baseMapper.listProjectTransfer(processId,nodeId);
        return list;
    }
}

