/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.TfRegisterItemW;
import com.tofly.workflow.buss.mapper.TfRegisterItemWMapper;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfRegisterItemWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 工程登记/立项
 *
 * @author JF
 * @date Tue Aug 16 00:00:00 CST 2022
 */
@Service("tfRegisterItemWService")
public class TfRegisterItemWServiceImpl extends ServiceImpl<TfRegisterItemWMapper, TfRegisterItemW> implements TfRegisterItemWService, BaseService {

    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;

    @Autowired
    private IFileInfoService iFileInfoService;
    @Value("${zmrq.ftp.registerItem}")
    private String registerItemPath;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        TfRegisterItemW registerItem = JSON.parseObject(String.valueOf(t),TfRegisterItemW.class);
        return FileNameCheckUtils.checkNodeFile(fileList,registerItem.getAnnexNameList());
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }

        TfRegisterItemW registerItem = JSON.parseObject(String.valueOf(t),TfRegisterItemW.class);
        registerItem.setProcessId(pb.getId());
        registerItem.setNodeId(pb.getNodeId());

        TfRegisterItemW registerItemQ = this.getOne(new QueryWrapper<TfRegisterItemW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId()));
        if(fileList != null && fileList.size() != 0){
            if(registerItemQ == null){
                registerItem.setAnnex(JSON.toJSONString(iFileInfoService.saveFileList(fileList, registerItemPath, FileTableEnum.TF_REGISTER_ITEM_W.getTableName())));
            }else {
                List<FileInfo> nowAnnexFileInfos = null;
                List<FileInfo> agoAnnexFileInfos = null;
                if(StringUtils.isNotBlank(registerItem.getAnnex())){
                    nowAnnexFileInfos = JSON.parseArray(registerItem.getAnnex(),FileInfo.class);
                }
                List<FileInfo> upAnnexFileInfos = iFileInfoService.saveFileList(fileList, registerItemPath, FileTableEnum.TF_REGISTER_ITEM_W.getTableName());

                if(StringUtils.isNotBlank(registerItemQ.getAnnex())){
                    agoAnnexFileInfos = JSON.parseArray(registerItemQ.getAnnex(),FileInfo.class);
                }
                if(agoAnnexFileInfos != null && nowAnnexFileInfos != null){
//                    List<FileInfo> ords = Stream.of(agoAnnexFileInfos, nowAnnexFileInfos)
//                            .flatMap(Collection::stream)
//                            .distinct()
//                            .collect(Collectors.toList());
                    List<FileInfo> finalNowAnnexFileInfos = nowAnnexFileInfos;
                    List<FileInfo> reduce = agoAnnexFileInfos.stream().filter(item -> !finalNowAnnexFileInfos.contains(item)).collect(Collectors.toList());
                    iFileInfoService.removeFileList(reduce);
                    upAnnexFileInfos.addAll(nowAnnexFileInfos);
                }
                if(upAnnexFileInfos.size() != 0){
                    registerItem.setAnnex(JSON.toJSONString(upAnnexFileInfos));
                }
            }
        }
        if(registerItemQ != null){
            this.removeById(registerItemQ.getId());
        }
        // 对象入库
        this.save(registerItem);
        /**保存工程公共信息*/

        Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getId()).select(TfProjectBaseW::getId)
                , r -> Long.parseLong(String.valueOf(r)));
        TfProjectBaseW npbase = new TfProjectBaseW();
        BeanUtils.copyProperties(registerItem, npbase);
        npbase.setUserPhone(registerItem.getTelephone());
        npbase.setStartTime(registerItem.getStartTime());
        npbase.setEndTime(registerItem.getEndTime());
        npbase.setUserName(registerItem.getUserName());
        npbase.setHouseholdNumber(registerItem.getHouseholdNumber());
        npbase.setId(null);
        npbase.setRegion(registerItem.getRegion());
        npbase.setRegionName(registerItem.getRegionName());
        npbase.setAddre(registerItem.getAddre());
        npbase.setInstallContent(registerItem.getContent());
        if (pbId != null) {
            npbase.setId(pbId);
        } else {
            npbase.setConfigId(pb.getConfigId());
            npbase.setEdition(pb.getEdition());
            npbase.setProcessNumber(pb.getProcessNumber());
            npbase.setRelationId(pb.getRelationId());
            npbase.setTenantId(pb.getTenantId());
            npbase.setCategory(pb.getCategory());
            npbase.setProcessType(pb.getProcessType());
            npbase.setRegisterDept(pb.getRegisterDept());
            npbase.setRegisterStaff(pb.getRegisterStaff());
            npbase.setRegisterPost(pb.getRegisterPost());
            npbase.setRegisterTime(pb.getRegisterTime());
        }
        pBaseService.saveOrUpdate(npbase);
        return null;
    }

    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfRegisterItemW>().lambda()
                .eq(processId != null, TfRegisterItemW::getProcessId, processId)
                .eq(nodeId != null, TfRegisterItemW::getNodeId, nodeId));
    }

}
