package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.buss.entity.TfReportEntryW;
import com.tofly.workflow.buss.mapper.TfReportEntryWMapper;
import com.tofly.workflow.buss.service.TfMonitoringReportWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfReportEntryWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author haiqiu
 * @description 针对表【TF_REPORT_ENTRY_W(开工报告条目(多条))】的数据库操作Service实现
 * @createDate 2022-08-29 11:55:04
 */
@Service("tfReportEntryWService")
public class TfReportEntryWServiceImpl extends ServiceImpl<TfReportEntryWMapper, TfReportEntryW>
        implements TfReportEntryWService, BaseService {
    /**
     * 监检提交上报登记(监检上报、监检登记）
     */
    @Autowired
    private TfMonitoringReportWService mReportService;

    /**
     * 工程基础
     */
    @Autowired
    private TfProjectBaseWService pBaseWService;
    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseArray(json, TfReportEntryW.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if (t == null) {
            throw new RuntimeException("数据不完整");
        }
//        if(pb.getSaveState()==FlowConstant.INTEGER_ONE) {
//            Integer typeValue = pBaseWService.getProjectTypeValue(pb.getId());
//            if (typeValue != null && typeValue == FlowConstant.INTEGER_THREE) {
//                int count = mReportService.getMReportCount(pb.getId(), 2, null);
//                if (count == FlowConstant.INTEGER_ZERO) {
//                    throw new RuntimeException("流程未监检上报，请先上报！");
//                }
//            }
//        }
        String s = JSONObject.toJSONString(t);
        List<TfReportEntryW> reportEntryWList = JSONObject.parseArray(s, TfReportEntryW.class);
        //保存
        if (CollectionUtils.isEmpty(reportEntryWList)){
            return null;
        }
        reportEntryWList.forEach(tfReportEntryW -> {
            tfReportEntryW.setProcessId(pb.getId());
            tfReportEntryW.setNodeId(pb.getNodeId());
        });
        //移除旧数据
        remove(Wrappers.<TfReportEntryW>lambdaQuery().
                eq(pb.getId() != null, TfReportEntryW::getProcessId, pb.getId())
                .eq(pb.getNodeId() != null, TfReportEntryW::getNodeId, pb.getNodeId()));
        //添加新数据
        return saveOrUpdateBatch(reportEntryWList);
    }

    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfReportEntryW>().lambda()
                .eq(processId != null, TfReportEntryW::getProcessId, processId)
                .eq(nodeId != null, TfReportEntryW::getNodeId, nodeId));
    }
}




