package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfSafetyCheckW;
import com.tofly.workflow.buss.entity.dto.SafetyCheckDto;
import com.tofly.workflow.buss.mapper.TfSafetyCheckWMapper;
import com.tofly.workflow.buss.service.TfSafetyCheckWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
* @author HaiQiu
* @description 针对表【TF_SAFETY_CHECK_W(安全检查(单独填报))】的数据库操作Service实现
* @createDate 2022-09-05 10:07:07
*/
@Service("tfSafetyCheckWService")
public class TfSafetyCheckWServiceImpl extends ServiceImpl<TfSafetyCheckWMapper, TfSafetyCheckW> implements TfSafetyCheckWService, BaseService {

    @Value("${zmrq.ftp.tfsafetycheckw}")
    private String path;
    /**
     * 文件
     */
    @Autowired
    private IFileInfoService fileService;
    @Override
    public Object getJsonToObject(String json) {
        if(StringUtils.isEmpty(json)){
            return null;
        }
        SafetyCheckDto pb = JSON.parseObject(json, SafetyCheckDto.class);
        return pb;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t==null){
            return FlowConstant.ISNOTNUL;
        }
        SafetyCheckDto pb = null;
        if(t instanceof String){
            pb = JSON.parseObject(t.toString(), SafetyCheckDto.class);
        }else {
            pb = (SafetyCheckDto)t;
        }
        if(StringUtils.isEmpty(pb.getAdder())){
            return "annex"+FlowConstant.ISNOTNUL;
        }
        if(pb.getAdder().length()>300){
            return "地址长度大于300";
        }
        if(pb.getInspectTime()!=null){
            return "inspectTime"+FlowConstant.ISNOTNUL;
        }
        if(CollUtil.isNotEmpty(pb.getFileName())) {
            String check = FileNameCheckUtils.checkNodeFile(fileList, pb.getFileName());
            if(StringUtils.isNotEmpty(check)){
                return check;
            }
        }
        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(t==null){
            return true;
        }
        SafetyCheckDto dto = null;
        if(t instanceof String){
            dto = JSON.parseObject(t.toString(), SafetyCheckDto.class);
        }else {
            dto = (SafetyCheckDto)t;
        }
        if(dto==null){
            return true;
        }
        TfSafetyCheckW oldsc = this.getOne(new QueryWrapper<TfSafetyCheckW>().lambda().eq(TfSafetyCheckW::getProcessId,pb.getId())
                .eq(TfSafetyCheckW::getNodeId,pb.getNodeId()).eq(TfSafetyCheckW::getIsDel,FlowConstant.SHORT_ZERO));
        this.remove(new QueryWrapper<TfSafetyCheckW>().lambda().eq(TfSafetyCheckW::getProcessId,pb.getId())
                .eq(TfSafetyCheckW::getNodeId,pb.getNodeId()));
        TfSafetyCheckW sc = new TfSafetyCheckW();
        BeanUtils.copyProperties(dto, sc);
        sc.setProcessId(pb.getId());
        sc.setNodeId(pb.getNodeId());
        sc.setIsDel(FlowConstant.INTEGER_ZERO);
        if(CollUtil.isNotEmpty(dto.getFileName())||(oldsc!=null&& StringUtils.isNotEmpty(oldsc.getAnnex())&& !oldsc.getAnnex().equals(dto.getAnnex()))) {
            String str = fileService.uploadFiles(fileList,dto.getAnnex(),oldsc!=null?oldsc.getAnnex():null,path, FileTableEnum.TF_PAY_BILL_W.getTableName());
            sc.setAnnex(str);
        }
        return this.save(sc);
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return this.list(new QueryWrapper<TfSafetyCheckW>().lambda().eq(TfSafetyCheckW::getProcessId,processId).eq(TfSafetyCheckW::getNodeId,nodeId)
                .eq(TfSafetyCheckW::getIsDel,FlowConstant.SHORT_ZERO));
    }
}




