package com.tofly.workflow.buss.service.impl;


import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfSafetyMonitorW;
import com.tofly.workflow.buss.entity.dto.SafetyMonitorDto;
import com.tofly.workflow.buss.mapper.TfSafetyMonitorWMapper;
import com.tofly.workflow.buss.service.TfSafetyMonitorWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;

/**
 * 安全监护 (单独填报)(TfSafetyMonitorW)表服务实现类
 *
 * @author HaiQiu
 * @since 2022-09-20 10:10:23
 */
@Service("tfSafetyMonitorWService")
public class TfSafetyMonitorWServiceImpl extends ServiceImpl<TfSafetyMonitorWMapper, TfSafetyMonitorW> implements TfSafetyMonitorWService,BaseService {

    @Autowired
    private IFileInfoService fileInfoService;

    /**
     * 文件
     */
    @Autowired
    private IFileInfoService fileService;

    @Value("${zmrq.ftp.tfsafetycheckw}")
    private String path;

    /**
     *
     * 根据ID查询
     * @param id id
     * @return 数据
     */
    @Override
    public TfSafetyMonitorW get(Serializable id){
        return getById(id);
    }

    /**
     * 新增
     * @param tfSafetyMonitorW 实体
     */
    @Override
    public void insert(TfSafetyMonitorW tfSafetyMonitorW,List<MultipartFile> fileList){
        String files = fileInfoService.uploadFiles(fileList, tfSafetyMonitorW.getAnnex(), null, path, "TF_PROJECT_TRANSFER_W");
        tfSafetyMonitorW.setAnnex(files);
        if(!save(tfSafetyMonitorW)){
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     * @param tfSafetyMonitorW 实体
     */
    @Override
    public void edit(TfSafetyMonitorW tfSafetyMonitorW,List<MultipartFile> fileList){
        TfSafetyMonitorW safetyMonitorW = getById(tfSafetyMonitorW.getId());
        String files = fileInfoService.uploadFiles(fileList, tfSafetyMonitorW.getAnnex(), safetyMonitorW.getAnnex(), path, "TF_PROJECT_TRANSFER_W");
        tfSafetyMonitorW.setAnnex(files);
        if(!updateById(tfSafetyMonitorW)){
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 批量删除
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList){
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template(){
        TfSafetyMonitorW tfSafetyMonitorW = new TfSafetyMonitorW();
        ExcelUtils.createTemplate("TfSafetyMonitorW安全监护 (单独填报)", "TfSafetyMonitorW安全监护 (单独填报)",
                null,TfSafetyMonitorW.class, Collections.singletonList(tfSafetyMonitorW));
    }

    /**
     * 数据导入
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file){
        List<TfSafetyMonitorW> datas = ExcelUtils.getExcelToList("TfSafetyMonitorW安全监护 (单独填报)", file, TfSafetyMonitorW.class);
        if (!CollectionUtils.isEmpty(datas)) {
            saveBatch(datas);
        }
    }

    /**
     * 分页查询
     * @param page 分页
     * @param tfSafetyMonitorW 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<TfSafetyMonitorW> pageList(Page<TfSafetyMonitorW> page, TfSafetyMonitorW tfSafetyMonitorW){
        return page(page, new QueryWrapper<>(tfSafetyMonitorW));
    }

    /**
     * 数据导出
     * @param page 分页
     * @param tfSafetyMonitorW 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @Override
    public void export(Page<TfSafetyMonitorW> page, TfSafetyMonitorW tfSafetyMonitorW, Boolean isAll){
        Page<TfSafetyMonitorW> pageData = new Page<>();
        if (isAll==null || !isAll){
            pageData = page(page,new QueryWrapper<>(tfSafetyMonitorW));
        }else {
            pageData.setRecords(list(new QueryWrapper<>(tfSafetyMonitorW)));
        }
        //导出数据
        ExcelUtils.getListToExcel(TfSafetyMonitorW.class,pageData.getRecords(),"TfSafetyMonitorW安全监护 (单独填报)");
    }

    /**
     * 查询所有数据
     * @param tfSafetyMonitorW 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<TfSafetyMonitorW> allList(TfSafetyMonitorW tfSafetyMonitorW){
        return list(new QueryWrapper<>(tfSafetyMonitorW));
    }

    @Override
    public Object getJsonToObject(String json) {
        if(StringUtils.isEmpty(json)){
            return null;
        }
        SafetyMonitorDto pb = JSON.parseObject(json, SafetyMonitorDto.class);
        return pb;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t==null){
            return FlowConstant.ISNOTNUL;
        }
        SafetyMonitorDto pb = null;
        if(t instanceof String){
            pb = JSON.parseObject(t.toString(), SafetyMonitorDto.class);
        }else {
            pb = (SafetyMonitorDto)t;
        }
        if(pb.getPostId()==null){
            return "postId"+FlowConstant.ISNOTNUL;
        }
        if(pb.getDeptId()==null){
            return "deptId"+FlowConstant.ISNOTNUL;
        }
        if(pb.getUserId()==null){
            return "userId"+FlowConstant.ISNOTNUL;
        }
        return null;
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(t==null){
            return true;
        }
        SafetyMonitorDto dto = null;
        if(t instanceof String){
            dto = JSON.parseObject(t.toString(), SafetyMonitorDto.class);
        }else {
            dto = (SafetyMonitorDto)t;
        }
        if(dto==null){
            return true;
        }
        TfSafetyMonitorW oldsm = this.getOne(new QueryWrapper<TfSafetyMonitorW>().lambda().eq(TfSafetyMonitorW::getProcessId,pb.getId()).eq(TfSafetyMonitorW::getNodeId,pb.getNodeId())
                .eq(TfSafetyMonitorW::getIsDel,FlowConstant.SHORT_ZERO));
        if(oldsm!=null){
            this.removeById(oldsm.getId());
        }
        TfSafetyMonitorW sm = new TfSafetyMonitorW();
        BeanUtils.copyProperties(dto, sm);
        sm.setProcessId(pb.getId());
        sm.setNodeId(pb.getNodeId());
        sm.setIsDel(FlowConstant.SHORT_ZERO);
        if(CollUtil.isNotEmpty(dto.getFileName())||(oldsm!=null&& StringUtils.isNotEmpty(oldsm.getAnnex())&& !oldsm.getAnnex().equals(dto.getAnnex()))) {
            String str = fileService.uploadFiles(fileList,dto.getAnnex(),oldsm!=null?oldsm.getAnnex():null,path, FileTableEnum.TF_PAY_BILL_W.getTableName());
            sm.setAnnex(str);
        }
        return this.save(sm);
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return this.list(new QueryWrapper<TfSafetyMonitorW>().lambda().eq(TfSafetyMonitorW::getProcessId,processId).eq(TfSafetyMonitorW::getNodeId,nodeId)
                .eq(TfSafetyMonitorW::getIsDel,FlowConstant.SHORT_ZERO));
    }
}

