package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.buss.entity.TfSelfCheckW;
import com.tofly.workflow.buss.mapper.TfSelfCheckWMapper;
import com.tofly.workflow.buss.service.TfSelfCheckWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author HaiQiu
 * @description 针对表【TF_SELF_CHECK_W(自检(多条))】的数据库操作Service实现
 * @createDate 2022-09-05 09:52:21
 */
@Service("tfSelfCheckWService")
public class TfSelfCheckWServiceImpl extends ServiceImpl<TfSelfCheckWMapper, TfSelfCheckW>
        implements TfSelfCheckWService, BaseService {

    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseArray(json, TfSelfCheckW.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if (t == null) {
            throw new RuntimeException("数据不完整");
        }
        String s = JSONObject.toJSONString(t);
        List<TfSelfCheckW> data = JSONObject.parseArray(s, TfSelfCheckW.class);
        //数据校验
        checkData(data);
        //保存
        for (TfSelfCheckW datum : data) {
            datum.setProcessId(pb.getId());
            datum.setNodeId(pb.getNodeId());
        }
        //移除旧数据
        remove(Wrappers.<TfSelfCheckW>lambdaQuery().
                eq(pb.getId() != null, TfSelfCheckW::getProcessId, pb.getId())
                .eq(pb.getNodeId() != null, TfSelfCheckW::getNodeId, pb.getNodeId()));
        //添加新数据
        return saveOrUpdateBatch(data);
    }


    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfSelfCheckW>().lambda()
                .eq(TfSelfCheckW::getProcessId, processId)
                .eq(TfSelfCheckW::getNodeId, nodeId));
    }

    private void checkData(List<TfSelfCheckW> data) {
        if (CollectionUtils.isEmpty(data)) {
            throw new RuntimeException("数据不能为空");
        }
        for (TfSelfCheckW datum : data) {
            if (datum.getCheckReturn() == null) {
                throw new RuntimeException("自检结果不能为空");
            }
        }

    }
}




