/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.TfSurveyW;
import com.tofly.workflow.buss.mapper.TfSurveyWMapper;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfSurveyWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 工程勘察
 *
 * @author JF
 * @date Mon Aug 22 00:00:00 CST 2022
 */
@Service("tfSurveyWService")
public class TfSurveyWServiceImpl extends ServiceImpl<TfSurveyWMapper, TfSurveyW> implements TfSurveyWService, BaseService {

    @Autowired
    private IFileInfoService iFileInfoService;
    @Value("${zmrq.ftp.survey}")
    private String surveyPath;

    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;

    @Override
    public Object getJsonToObject(String json) {
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        if(t == null){
            return "新增/修改对象不能为空";
        }
        TfSurveyW surveyW = JSON.parseObject(String.valueOf(t),TfSurveyW.class);
        return FileNameCheckUtils.checkNodeFile(fileList,surveyW.getAnnexNameList());
    }

    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if(pb == null){
            return "流程信息基础对象不能为空";
        }else if(pb.getId() == null){
            return "流程ID不能为空";
        }else if(pb.getNodeId() == null){
            return "节点ID不能为空";
        }

        TfSurveyW surveyW = JSON.parseObject(String.valueOf(t),TfSurveyW.class);
        surveyW.setProcessId(pb.getId());
        surveyW.setNodeId(pb.getNodeId());

        TfSurveyW surveyWQ = this.getOne(new QueryWrapper<TfSurveyW>()
                .eq("PROCESS_ID",pb.getId())
                .eq("NODE_ID",pb.getNodeId()));
        if(fileList != null && fileList.size() != 0){
            if(surveyWQ == null){
                surveyW.setAnnex(JSON.toJSONString(iFileInfoService.saveFileList(fileList, surveyPath, FileTableEnum.TF_SURVEY_W.getTableName())));
            }else {
                List<FileInfo> nowAnnexFileInfos = null;
                List<FileInfo> agoAnnexFileInfos = null;
                if(StringUtils.isNotBlank(surveyW.getAnnex())){
                    nowAnnexFileInfos = JSON.parseArray(surveyW.getAnnex(),FileInfo.class);
                }
                List<FileInfo> upAnnexFileInfos = iFileInfoService.saveFileList(fileList, surveyPath, FileTableEnum.TF_SURVEY_W.getTableName());

                if(StringUtils.isNotBlank(surveyWQ.getAnnex())){
                    agoAnnexFileInfos = JSON.parseArray(surveyWQ.getAnnex(),FileInfo.class);
                }
                if(agoAnnexFileInfos != null && nowAnnexFileInfos != null){
//                    List<FileInfo> ords = Stream.of(agoAnnexFileInfos, nowAnnexFileInfos)
//                            .flatMap(Collection::stream)
//                            .distinct()
//                            .collect(Collectors.toList());
                    List<FileInfo> finalNowAnnexFileInfos = nowAnnexFileInfos;
                    List<FileInfo> reduce = agoAnnexFileInfos.stream().filter(item -> !finalNowAnnexFileInfos.contains(item)).collect(Collectors.toList());
                    iFileInfoService.removeFileList(reduce);
                    upAnnexFileInfos.addAll(nowAnnexFileInfos);
                }
                if(upAnnexFileInfos.size() != 0){
                    surveyW.setAnnex(JSON.toJSONString(upAnnexFileInfos));
                }
                this.removeById(surveyWQ.getId());
            }
        }
        if(surveyWQ != null){
            this.removeById(surveyWQ.getId());
        }
        // 对象入库
        this.save(surveyW);
        if(surveyW.getSurveyDate()!=null||surveyW.getIsTally()!=null) {
            /**保存工程公共信息*/
            Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getId()).select(TfProjectBaseW::getId)
                    , r -> Long.parseLong(String.valueOf(r)));
            TfProjectBaseW npbase = new TfProjectBaseW();
            npbase.setSurveyDate(surveyW.getSurveyDate());
            npbase.setIsTally(surveyW.getIsTally());
            if (pbId != null) {
                npbase.setId(pbId);
            } else {
                npbase.setConfigId(pb.getConfigId());
                npbase.setEdition(pb.getEdition());
                npbase.setProcessNumber(pb.getProcessNumber());
                npbase.setRelationId(pb.getRelationId());
                npbase.setTenantId(pb.getTenantId());
                npbase.setCategory(pb.getCategory());
                npbase.setProcessType(pb.getProcessType());
                npbase.setRegisterDept(pb.getRegisterDept());
                npbase.setRegisterStaff(pb.getRegisterStaff());
                npbase.setRegisterPost(pb.getRegisterPost());
                npbase.setRegisterTime(pb.getRegisterTime());
            }
            pBaseService.saveOrUpdate(npbase);
        }
        return null;
    }

    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfSurveyW>().lambda()
                .eq(processId != null, TfSurveyW::getProcessId, processId)
                .eq(nodeId != null, TfSurveyW::getNodeId, nodeId));
    }

}
