/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.service.IFileInfoService;
import com.tofly.workflow.buss.entity.TfTransferFixedAssetsW;
import com.tofly.workflow.buss.entity.dto.TransferFixedAssetsDto;
import com.tofly.workflow.buss.entity.vo.TransferFixedAssetsVo;
import com.tofly.workflow.buss.mapper.TfTransferFixedAssetsWMapper;
import com.tofly.workflow.buss.service.TfTransferFixedAssetsWService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 转入固定资产 （单独填报)
 *
 * @author JF
 * @date Thu Sep 08 00:00:00 CST 2022
 */
@Service("tfTransferFixedAssetsWService")
public class TfTransferFixedAssetsWServiceImpl extends ServiceImpl<TfTransferFixedAssetsWMapper, TfTransferFixedAssetsW> implements TfTransferFixedAssetsWService {

    @Autowired
    private IFileInfoService iFileInfoService;
    @Value("${zmrq.ftp.transferFixedAssets}")
    private String transferFixedAssetsPath;

    @Override
    public void saveTransferFixedAssets(TfTransferFixedAssetsW entity, List<MultipartFile> annexList) {
        if(entity == null){
            return;
        }
        if(annexList != null && annexList.size() != 0){
            List<FileInfo> annexFileInfo = iFileInfoService.saveFileList(annexList, transferFixedAssetsPath, FileTableEnum.TF_TRANSFER_FIXED_ASSETS_W.getTableName());;
            entity.setAnnex(JSON.toJSONString(annexFileInfo));
        }
        this.save(entity);
    }

    @Override
    public void updateTransferFixedAssets(TfTransferFixedAssetsW entity, List<MultipartFile> annexList) {
        if(entity == null){
            throw new ToflyDeniedException("修改的对象不能为空，请重新输入");
        }
        List<FileInfo> nowAnnexFileInfos = null;
        List<FileInfo> agoAnnexFileInfos = null;
        List<FileInfo> upAnnexFileInfos = new ArrayList<>();
        if(StringUtils.isNotBlank(entity.getAnnex())){
            nowAnnexFileInfos = JSON.parseArray(entity.getAnnex(),FileInfo.class);
        }
        if (null != annexList && annexList.size() > 0) {
            upAnnexFileInfos = iFileInfoService.saveFileList(annexList, transferFixedAssetsPath, FileTableEnum.TF_TRANSFER_FIXED_ASSETS_W.getTableName());
        }
        TfTransferFixedAssetsW entityQuery = this.getById(entity.getId());
        String agoAnnexFileInfo = entityQuery.getAnnex();

        if(StringUtils.isNotBlank(agoAnnexFileInfo)){
            agoAnnexFileInfos = JSON.parseArray(agoAnnexFileInfo,FileInfo.class);
        }
        if(agoAnnexFileInfos != null && nowAnnexFileInfos != null){
//            List<FileInfo> ords = Stream.of(agoAnnexFileInfos, nowAnnexFileInfos)
//                    .flatMap(Collection::stream)
//                    .distinct()
//                    .collect(Collectors.toList());
            List<FileInfo> finalNowAnnexFileInfos = nowAnnexFileInfos;
            List<FileInfo> reduce = agoAnnexFileInfos.stream().filter(item -> !finalNowAnnexFileInfos.contains(item)).collect(Collectors.toList());
            iFileInfoService.removeFileList(reduce);
            upAnnexFileInfos.addAll(nowAnnexFileInfos);
        }
        if(upAnnexFileInfos.size() != 0){
            entity.setAnnex(JSON.toJSONString(upAnnexFileInfos));
        }
        this.updateById(entity);
    }

    @Override
    public void deleteTransferFixedAssets(List<Long> idList) {
        idList.forEach(info ->{
            TfTransferFixedAssetsW entity = this.getById(info);
            String annex = entity.getAnnex();
            if(StringUtils.isNotBlank(annex)){
                List<FileInfo> fileInfoList = JSON.parseArray(annex,FileInfo.class);
                iFileInfoService.removeFileList(fileInfoList);
            }
        });
        this.removeByIds(idList);
    }

    @Override
    public Page<TransferFixedAssetsVo> pageTransferFixedAssets(Page page, TransferFixedAssetsDto dto) {
        return this.baseMapper.pageTransferFixedAssets(page,dto);
    }

}
