package com.tofly.workflow.buss.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfProjectTransferW;
import com.tofly.workflow.buss.entity.TfTransferProjectW;
import com.tofly.workflow.buss.entity.dto.TransferProjectDto;
import com.tofly.workflow.buss.mapper.TfTransferProjectWMapper;
import com.tofly.workflow.buss.service.TfTransferProjectWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* @author HaiQiu
* @description 针对表【TF_TRANSFER_PROJECT_W(移交的项目列表)】的数据库操作Service实现
* @createDate 2022-09-20 09:16:14
*/
@Service("tfTransferProjectWService")
public class TfTransferProjectWServiceImpl extends ServiceImpl<TfTransferProjectWMapper, TfTransferProjectW>
    implements TfTransferProjectWService, BaseService {

    @Autowired
    private IFileInfoService iFileInfoService;

    @Value("${zmrq.ftp.tfsafetycheckw}")
    private String path;

    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseArray(json, TfTransferProjectW.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if (t == null) {
            throw new RuntimeException("数据不完整");
        }
        String s = JSONObject.toJSONString(t);
        List<TfTransferProjectW> array = JSONObject.parseArray(s, TfTransferProjectW.class);
        //文件列表
        if (!CollectionUtils.isEmpty(array)) {
            for (TfTransferProjectW data : array) {
                //附件
                List<MultipartFile> nodeFile1 = FileNameCheckUtils.getNodeFile(fileList, data.getAnnexNameList());
                if (!CollectionUtils.isEmpty(nodeFile1)) {
                    data.setAnnex(iFileInfoService.uploadFiles(nodeFile1, data.getAnnex(), null, path, "TF_TRANSFER_PROJECT_W"));
                }
            }
            //移除旧数据
            remove(Wrappers.<TfTransferProjectW>lambdaQuery().
                    eq(pb.getId() != null, TfTransferProjectW::getProcessId, pb.getId())
                    .eq(pb.getNodeId() != null, TfTransferProjectW::getNodeId, pb.getNodeId()));
            //添加新数据
            return saveOrUpdateBatch(array);
        }
        return null;
    }

    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(Wrappers.<TfTransferProjectW>lambdaQuery().eq(TfTransferProjectW::getProcessId,processId));
    }

    @Override
    public String checkProjects(List<TransferProjectDto> projects, List<MultipartFile> fileList) {
        for (TransferProjectDto dto:projects ) {
            String check = FileNameCheckUtils.checkNodeFile(fileList,dto.getAnnexNameList());
            if(org.apache.commons.lang3.StringUtils.isNotEmpty(check)){
                return check;
            }
        }
        return null;
    }

    @Override
    public void saveProject(List<TransferProjectDto> projects, List<MultipartFile> fileList, TfProjectTransferW pt, Long oldId) {
        Map<Long, TfTransferProjectW> map = null;
        if(oldId!=null) {
            map = this.list(new QueryWrapper<TfTransferProjectW>().lambda().eq(TfTransferProjectW::getTransferId, oldId))
                    .stream().collect(Collectors.toMap(TfTransferProjectW::getProcessId, r -> r));
            this.remove(new QueryWrapper<TfTransferProjectW>().lambda().eq(TfTransferProjectW::getTransferId, oldId));
        }
        if(CollUtil.isNotEmpty(projects)) {
            List<TfTransferProjectW> list = new ArrayList<>();
            for (TransferProjectDto dto:projects){
                TfTransferProjectW tp = new TfTransferProjectW();
                BeanUtils.copyProperties(dto,tp);
                tp.setTransferId(pt.getId());
                List<MultipartFile> nodeFile1 = null;
                if(CollUtil.isNotEmpty(dto.getAnnexNameList())) {
                    //附件
                    nodeFile1 = FileNameCheckUtils.getNodeFile(fileList, dto.getAnnexNameList());
                }
                TfTransferProjectW oldtp = map!=null?map.get(tp.getProcessId()):null;
                tp.setAnnex(iFileInfoService.uploadFiles(nodeFile1, tp.getAnnex(),oldtp==null?null: oldtp.getAnnex(), path, "TF_TRANSFER_PROJECT_W"));
                tp.setIsDel(FlowConstant.INTEGER_ZERO);
                list.add(tp);
            }
            this.saveBatch(list);
        }
        
    }
}




