package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfTransferFileW;
import com.tofly.workflow.buss.entity.TfTransferW;
import com.tofly.workflow.buss.mapper.TfTransferWMapper;
import com.tofly.workflow.buss.service.TfTransferFileWService;
import com.tofly.workflow.buss.service.TfTransferWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author haiqiu
 * @description 针对表【TF_TRANSFER_W(资料移交)】的数据库操作Service实现
 * @createDate 2022-08-29 15:18:44
 */
@Service("tfTransferWService")
public class TfTransferWServiceImpl extends ServiceImpl<TfTransferWMapper, TfTransferW>
        implements TfTransferWService, BaseService {


    @Value("${zmrq.ftp.transfer}")
    private String transfer;

    @Autowired
    private IFileInfoService iFileInfoService;

    @Autowired
    private TfTransferFileWService tfTransferFileWService;

    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)) {
            return JSONObject.parseObject(json, TfTransferW.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if (t == null) {
            throw new RuntimeException("数据不完整");
        }
        String s = JSONObject.toJSONString(t);
        TfTransferW data = JSONObject.parseObject(s, TfTransferW.class);
        //数据校验
        checkData(data);
        //附件
        List<MultipartFile> nodeFile = FileNameCheckUtils.getNodeFile(fileList, data.getAnnexNameList());
        if (!CollectionUtils.isEmpty(nodeFile)) {
            data.setAnnex(iFileInfoService.uploadFiles(nodeFile, data.getAnnex(), null, transfer, "TF_TRANSFER_W"));
        }
        //保存
        data.setProcessId(pb.getId());
        data.setNodeId(pb.getNodeId());
        //移除旧数据
        remove(Wrappers.<TfTransferW>lambdaQuery().
                eq(pb.getId() != null, TfTransferW::getProcessId, pb.getId())
                .eq(pb.getNodeId() != null, TfTransferW::getNodeId, pb.getNodeId()));
        //添加新数据
        boolean save = saveOrUpdate(data);
        //移交文件列表
        List<TfTransferFileW> tfTransferFileWS = data.getTfTransferFileWS();
        if (!CollectionUtils.isEmpty(tfTransferFileWS)) {
            for (TfTransferFileW tfTransferFileW : tfTransferFileWS) {
                tfTransferFileW.setTransferId(data.getId());
                //附件
                List<MultipartFile> nodeFile1 = FileNameCheckUtils.getNodeFile(fileList, tfTransferFileW.getAnnexNameList());
                if (!CollectionUtils.isEmpty(nodeFile1)) {
                    tfTransferFileW.setAnnex(iFileInfoService.uploadFiles(nodeFile1, tfTransferFileW.getAnnex(), null, transfer, "TF_TRANSFER_FILE_W"));
                }
            }
            tfTransferFileWService.saveOrUpdateBatch(tfTransferFileWS);
        }
        return save;
    }

    /**
     * 获取数据
     *
     * @param processId 流程ID
     * @param nodeId    步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        List<TfTransferW> list = list(new QueryWrapper<TfTransferW>().lambda()
                .eq(processId != null, TfTransferW::getProcessId, processId)
                .eq(nodeId != null, TfTransferW::getNodeId, nodeId));
        if (!CollectionUtils.isEmpty(list)) {
            for (TfTransferW tfTransferW : list) {
                List<TfTransferFileW> transferFileWS = tfTransferFileWService.list(new QueryWrapper<TfTransferFileW>().lambda().eq(TfTransferFileW::getTransferId, tfTransferW.getId()));
                tfTransferW.setTfTransferFileWS(transferFileWS);
            }
        }
        return list;
    }

    protected void checkData(TfTransferW data) {
        if (ObjectUtils.isEmpty(data)) {
            throw new RuntimeException("业务数据不能为空");
        }
        if (data.getDataTime() == null) {
            throw new RuntimeException("移交日期不能为空");
        }
//        if (data.getHandOverDept() == null) {
//            throw new RuntimeException("移交部门不能为空");
//        }
        if (data.getHandOverUser() == null) {
            throw new RuntimeException("移交人不能为空");
        }
    }
}




