package com.tofly.workflow.buss.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.buss.entity.TfProjectBaseW;
import com.tofly.workflow.buss.entity.TfUserSettlementW;
import com.tofly.workflow.buss.mapper.TfUserSettlementWMapper;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.buss.service.TfUserSettlementWService;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
* @author haiqiu
* @description 针对表【TF_USER_SETTLEMENT_W(工程用户结算)】的数据库操作Service实现
* @createDate 2022-08-29 15:28:31
*/
@Service("tfUserSettlementWService")
public class TfUserSettlementWServiceImpl extends ServiceImpl<TfUserSettlementWMapper, TfUserSettlementW>
    implements TfUserSettlementWService, BaseService {


    @Value("${zmrq.ftp.usersettlementw}")
    private String usersettlementw;

    @Autowired
    private IFileInfoService iFileInfoService;

    /**
     * 业务基础数据
     */
    @Autowired
    private TfProjectBaseWService pBaseService;

    @Override
    public Object getJsonToObject(String json) {
        if (StringUtils.hasText(json)){
            return JSONObject.parseObject(json, TfUserSettlementW.class);
        }
        return null;
    }

    @Override
    public String saveDataCheck(Object t, List<MultipartFile> fileList) {
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object saveData(Object t, List<MultipartFile> fileList, ProcessBaseDto pb) {
        if (t == null) {
            throw new RuntimeException("数据不完整");
        }
        String s = JSONObject.toJSONString(t);
        TfUserSettlementW data = JSONObject.parseObject(s, TfUserSettlementW.class);
        //数据校验
        checkData(data);
        //附件
        List<MultipartFile> nodeFile = FileNameCheckUtils.getNodeFile(fileList, data.getAnnexNameList());
        if (!CollectionUtils.isEmpty(nodeFile)){
            data.setAnnex(iFileInfoService.uploadFiles(nodeFile,data.getAnnex(),null,usersettlementw,"TF_USER_SETTLEMENT_W"));
        }
        //保存
        data.setProcessId(pb.getId());
        data.setNodeId(pb.getNodeId());
        //移除旧数据
        remove(Wrappers.<TfUserSettlementW>lambdaQuery().
                eq(pb.getId() != null, TfUserSettlementW::getProcessId, pb.getId())
                .eq(pb.getNodeId() != null, TfUserSettlementW::getNodeId, pb.getNodeId()));
        //添加新数据
        saveOrUpdate(data);

        /**保存工程公共信息*/
        if(data.getTotalPrice()!=null) {
            Long pbId = pBaseService.getObj(new QueryWrapper<TfProjectBaseW>().lambda().eq(TfProjectBaseW::getProcessId, pb.getId()).select(TfProjectBaseW::getId)
                    , r -> Long.parseLong(String.valueOf(r)));
            TfProjectBaseW npbase = new TfProjectBaseW();
            npbase.setTotalPrice(data.getTotalPrice());
            if (pbId != null) {
                npbase.setId(pbId);
            } else {
                npbase.setConfigId(pb.getConfigId());
                npbase.setEdition(pb.getEdition());
                npbase.setProcessNumber(pb.getProcessNumber());
                npbase.setRelationId(pb.getRelationId());
                npbase.setTenantId(pb.getTenantId());
                npbase.setCategory(pb.getCategory());
                npbase.setProcessType(pb.getProcessType());
                npbase.setRegisterDept(pb.getRegisterDept());
                npbase.setRegisterStaff(pb.getRegisterStaff());
                npbase.setRegisterPost(pb.getRegisterPost());
                npbase.setRegisterTime(pb.getRegisterTime());
            }
            pBaseService.saveOrUpdate(npbase);
        }
        return null;
    }
    /**
     * 获取数据
     * @param processId 流程ID
     * @param nodeId 步骤节点ID
     * @return 业务数据
     */
    @Override
    public Object getBaseDatas(Long processId, Long nodeId,Long tenantId) {
        return list(new QueryWrapper<TfUserSettlementW>().lambda()
                .eq(processId != null, TfUserSettlementW::getProcessId, processId)
                .eq(nodeId != null, TfUserSettlementW::getNodeId, nodeId));
    }

    protected void checkData(TfUserSettlementW data){
        if (ObjectUtils.isEmpty(data)){
            throw new RuntimeException("业务数据不能为空");
        }
        if (data.getAmountReceived()==null){
            throw new RuntimeException("实收金额不能为空");
        }
        if (data.getPayableAmount()==null){
            throw new RuntimeException("应缴金额不能为空");
        }
    }
}




