/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.pay.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.pay.entity.dto.BankInfoDto;
import com.tofly.workflow.pay.entity.dto.BankPageDto;
import com.tofly.workflow.pay.service.BankInfoPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
/**
 * 银行（金融）配置信息
 *
 * @author xqf
 * @date Wed Nov 16 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/bankinfop")
@Api(tags="银行（金融）配置信息接口")
public class BankInfoPController {

    private final  BankInfoPService bankInfoPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 银行（金融）配置信息
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getBankInfoPPage(Page page, @ModelAttribute BankPageDto dto) {
      return ResultData.ok(bankInfoPService.pageBank(page,dto));
    }


    /**
     * 通过id查询银行（金融）配置信息
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(bankInfoPService.getById(id));
    }

    /**
     * 新增银行（金融）配置信息
     * @param dto 银行（金融）配置信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增银行（金融）配置信息" )
    @ApiOperation(value = "新增银行（金融）配置信息")
    @PostMapping
    public ResultData save(@RequestBody BankInfoDto dto) {
        Object obj = bankInfoPService.saveBankInfo(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 测试银行配置信息
     * @param dto 测试银行配置信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title="测试银行配置信息" )
    @ApiOperation(value = "测试银行配置信息")
    @PostMapping("bankTest")
    public ResultData bankTest(@RequestBody BankInfoDto dto) {
        Object obj = bankInfoPService.bankTest(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除银行（金融）配置信息
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除银行（金融）配置信息" )
    @ApiOperation(value = "通过ID删除银行（金融）配置信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(bankInfoPService.removeBankById(id));
    }

}
