/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.pay.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.pay.entity.PayImplInfoP;
import com.tofly.workflow.pay.entity.dto.PayImplInfoDto;
import com.tofly.workflow.pay.service.PayImplInfoPService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
/**
 * 银行支付相关接口配置
 *
 * @author xqf
 * @date Wed Nov 16 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/payimplinfop")
@Api(tags="银行支付相关接口配置接口")
public class PayImplInfoPController {

    private final  PayImplInfoPService payImplInfoPService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param payImplInfoP 银行支付相关接口配置
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getPayImplInfoPPage(Page page, @ModelAttribute PayImplInfoP payImplInfoP) {
      return ResultData.ok(payImplInfoPService.page(page, Wrappers.query(payImplInfoP)));
    }


    /**
     * 通过id查询银行支付相关接口配置
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(payImplInfoPService.getById(id));
    }

    /**
     * 新增银行支付相关接口配置
     * @param dto 银行支付相关接口配置
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增银行支付相关接口配置" )
    @ApiOperation(value = "新增银行支付相关接口配置")
    @PostMapping
    public ResultData save(@RequestBody PayImplInfoDto dto) {
        Object obj = payImplInfoPService.savePayImpl(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除银行支付相关接口配置
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除银行支付相关接口配置" )
    @ApiOperation(value = "通过ID删除银行支付相关接口配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(payImplInfoPService.removePayImplById(id));
    }

}
