/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.pay.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.pay.entity.PaySelectW;
import com.tofly.workflow.pay.service.PaySelectWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
/**
 * 银行（支付）交易查询相应报文
 *
 * @author xqf
 * @date Wed Nov 16 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/payselectw")
@Api(tags="银行（支付）交易查询相应报文接口")
public class PaySelectWController {

    private final  PaySelectWService paySelectWService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param paySelectW 银行（支付）交易查询相应报文
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getPaySelectWPage(Page page, @ModelAttribute PaySelectW paySelectW) {
      return ResultData.ok(paySelectWService.page(page, Wrappers.query(paySelectW)));
    }


    /**
     * 通过id查询银行（支付）交易查询相应报文
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(paySelectWService.getById(id));
    }


    /**
     * 通过id删除银行（支付）交易查询相应报文
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除银行（支付）交易查询相应报文" )
    @ApiOperation(value = "通过ID删除银行（支付）交易查询相应报文")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(paySelectWService.removePaySelectById(id));
    }

}
