package com.tofly.workflow.pay.controller;

import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.pay.entity.dto.ResultQueryDto;
import com.tofly.workflow.pay.entity.dto.UserMainScanDto;
import com.tofly.workflow.pay.entity.vo.QueryResultVo;
import com.tofly.workflow.pay.entity.vo.RefundVo;
import com.tofly.workflow.pay.entity.vo.UserMainScanVo;
import com.tofly.workflow.pay.service.QrcodepayImplService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 银行（金融）扫码支付请求接口
 *
 * @author xqf
 * @date Wed Nov 16 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/qrcodepay")
@Api(tags="银行（金融）扫码支付请求接口")
public class QrcodepayImplController {

    private final QrcodepayImplService qrcodepayImplService;


    /**
     * 银行（金融）扫码支付-预下单（用户主扫）
     * @param dto 请求
     * @return ResultRespone
     */
    @ToFlyAppLog(title="银行（金融）扫码支付-预下单（用户主扫）" )
    @ApiOperation(value = "银行（金融）扫码支付-预下单（用户主扫）")
    @PostMapping("primaryScan")
    public ResultData primaryScan(@RequestBody UserMainScanDto dto) {
        UserMainScanVo vo = qrcodepayImplService.userMainScan(dto);
        if(!FlowConstant.INTEGER_ONE.equals(vo.getCode())){
            return ResultData.fail(vo.getDescribe());
        }
        return ResultData.ok(vo);
    }

    /**
     * 银行（金融）扫码支付-预下单（用户主扫）-支付结果查询
     * @param dto 请求
     * @return ResultRespone
     */
    @ToFlyAppLog(title="银行（金融）扫码支付-预下单（用户主扫）-支付结果查询" )
    @ApiOperation(value = "银行（金融）扫码支付-预下单（用户主扫）-支付结果查询")
    @PostMapping("resultQuery")
    public ResultData resultQuery(@RequestBody ResultQueryDto dto){
        QueryResultVo vo = qrcodepayImplService.resultQuery(dto);
        if(!FlowConstant.INTEGER_ONE.equals(vo.getCode())){
            return ResultData.fail(vo.getDescribe());
        }
        return ResultData.ok(vo);
    }



    /**
     * 银行（金融）退款请求
     * @param dto 请求
     * @return ResultRespone
     */
    @ToFlyAppLog(title="银行（金融）退款请求" )
    @ApiOperation(value = "银行（金融）退款请求")
    @PostMapping("refund")
    public ResultData refund(@RequestBody UserMainScanDto dto){
        RefundVo vo = qrcodepayImplService.refund(dto);
        if(!FlowConstant.INTEGER_ONE.equals(vo.getCode())){
            return ResultData.fail(vo.getDescribe());
        }
        return ResultData.ok(vo);
    }



}
