package com.tofly.workflow.pay.entity.dto;

import com.baomidou.mybatisplus.annotation.TableField;
import com.tofly.workflow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * 银行（金融）配置信息 保存、修改
 */
@Data
@EqualsAndHashCode
public class BankInfoDto implements Serializable {

    /**
     * ID
     */
    @ApiModelProperty(value = "ID")
    private Long id;

    /**
     * 银行(金融机构)名称
     */
    @NotEmpty(message = "bankName 不能为空")
    @ApiModelProperty(value = "银行(金融机构)名称")
    private String bankName;

    /**
     * 是否有效1有效；0无效
     */
    @Max(value = 1,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 0,message = FlowConstant.DATA_LOWER_LIMIT)
    @NotNull(message = "isValid"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "是否有效1有效；0无效  测试后标记有效")
    private Short isValid;
    /**
     * 联系人
     */
    @ApiModelProperty(value = "联系人")
    private String contacts;
    /**
     * 电话
     */
    @ApiModelProperty(value = "电话")
    private String telephone;
    /**
     * APPID应用编号
     */
    @ApiModelProperty(value = "APPID应用编号")
    private String appid;
    /**
     * 合作方密钥
     */
    @ApiModelProperty(value = "合作方密钥")
    private String appsecret;
    /**
     * 平台加密公钥
     */
    @ApiModelProperty(value = "平台加密公钥")
    private String openepubk;
    /**
     * 合作方加密私钥
     */
    @ApiModelProperty(value = "合作方加密私钥")
    private String appdprik;
    /**
     * 平台加签公钥
     */
    @ApiModelProperty(value = "平台加签公钥")
    private String openvpubk;
    /**
     * 合作方加签私钥
     */
    @ApiModelProperty(value = "合作方加签私钥")
    private String appsprik;
    /**
     * 网络地址--域名
     */
    @NotEmpty(message = "http 不能为空")
    @ApiModelProperty(value = "网络地址--域名")
    private String http;
    /**
     * 字符集
     */
    @ApiModelProperty(value = "字符集")
    private String charset;
    /**
     * 交易行政代码
     */
    @ApiModelProperty(value = "交易行政代码")
    private String txnAreaInfo;
    /**
     * 接口版本号
     */
    @ApiModelProperty(value = "接口版本号")
    private String version;
    /**
     * 访问令牌
     */
    @ApiModelProperty(value = "访问令牌")
    private String accessToken;
    /**
     * 银行用户编号
     */
    @ApiModelProperty(value = "银行用户编号")
    @TableField(value = "USER_ID")
    private String userId;

    /**
     * 商户号
     */
    @ApiModelProperty(value = "商户号")
    private String mchtNo;
    /**
     * service名称
     */
    @NotEmpty(message = "service 不能为空")
    @ApiModelProperty(value = "service名称")
    private String service;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;



}
