package com.tofly.workflow.pay.entity.dto;

import com.tofly.workflow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * 银行支付相关接口配置
 */
@Data
@EqualsAndHashCode
public class PayImplInfoDto implements Serializable {

    /**
     * ID
     */
    @ApiModelProperty(value = "ID")
    private Long id;
    /**
     * 银行ID
     */
    @NotNull(message = "bankId"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "银行ID")
    private Long bankId;
    /**
     * 支付类型1用户主扫;2被扫;3交易查询;4退款;5对账
     */
    @Max(value = 5,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 1,message = FlowConstant.DATA_LOWER_LIMIT)
    @NotNull(message = "payType"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "支付类型1用户主扫;2被扫;3交易查询;4退款;5对账")
    private Short payType;
    /**
     * 支付名称
     */
    @ApiModelProperty(value = "支付名称")
    private String payName;
    /**
     * 请求路径
     */
    @NotEmpty(message = "path 不能为空")
    @ApiModelProperty(value = "请求路径")
    private String path;
    /**
     * 回调地址
     */
    @ApiModelProperty(value = "回调地址")
    private String notifyUrl;
    /**
     * 有效时间，单位为秒
     */
    @ApiModelProperty(value = "有效时间，单位为秒")
    private Long validTime;
    /**
     * 商户的终端设备编号
     */
    @ApiModelProperty(value = "商户的终端设备编号")
    private String mchtTermId;
    /**
     * 密钥
     */
    @ApiModelProperty(value = "密钥")
    private String secretKey;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;
}
