/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.pay.service.impl;

import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.pay.entity.BankInfoP;
import com.tofly.workflow.pay.entity.dto.BankInfoDto;
import com.tofly.workflow.pay.entity.dto.BankPageDto;
import com.tofly.workflow.pay.mapper.BankInfoPMapper;
import com.tofly.workflow.pay.service.BankInfoPService;
import com.tofly.workflow.pay.service.QrcodepayBaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * 银行（金融）配置信息
 *
 * @author xqf
 * @date Wed Nov 16 00:00:00 CST 2022
 */
@Service("bankInfoPService")
public class BankInfoPServiceImpl extends ServiceImpl<BankInfoPMapper, BankInfoP> implements BankInfoPService {

    /**
     * 保存银行（金融）配置信息
     * @param dto 配置信息
     * @return
     */
    @Override
    public Object saveBankInfo(BankInfoDto dto) {
        String check = checkBankInfo(dto);
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        BankInfoP info = new BankInfoP();
        BeanUtils.copyProperties(dto,info);
        if(FlowConstant.SHORT_ONE==dto.getIsValid()&& StringUtils.isNotEmpty(dto.getService())) {
            QrcodepayBaseService base = SpringUtil.getBean(dto.getService());
            Object obj = base.getSdk(info);
            if (obj == null) {
                return "调用三方校验失败！";
            }
        }
        info.setTenantId(SecurityUtils.getTenantId());
        info.setIsDel(FlowConstant.SHORT_ZERO);
        return this.saveOrUpdate(info);
    }

    @Override
    public Boolean removeBankById(Long id) {
        return this.update(new UpdateWrapper<BankInfoP>().lambda().eq(BankInfoP::getId,id).eq(BankInfoP::getIsDel,FlowConstant.SHORT_ZERO)
                .set(BankInfoP::getIsDel,FlowConstant.SHORT_ONE));
    }

    @Override
    public Page<BankInfoP> pageBank(Page page, BankPageDto dto) {
        return this.page(page,new QueryWrapper<BankInfoP>().lambda().eq(dto.getTenantId()!=null,BankInfoP::getTenantId,dto.getTenantId())
                .like(StringUtils.isNotEmpty(dto.getTelephone()),BankInfoP::getTelephone,dto.getTelephone())
                .like(StringUtils.isNotEmpty(dto.getContent()),BankInfoP::getBankName,dto.getContent()));
    }

    /**
     * 测试银行配置信息
     * @param dto 配置信息
     * @return
     */
    @Override
    public Object bankTest(BankInfoDto dto) {
        BankInfoP info = new BankInfoP();
        BeanUtils.copyProperties(dto,info);
        QrcodepayBaseService base = SpringUtil.getBean(dto.getService());
        Object obj = base.getSdk(info);
        if(obj==null){
            return "测试失败！";
        }
        return true;
    }

    /**
     * 数据校验
     * @param dto 配置信息
     * @return
     */
    private String checkBankInfo(BankInfoDto dto) {
        if(dto.getId()!=null){
            BankInfoP info= this.getById(dto.getId());
            if(info==null|| FlowConstant.SHORT_ONE==info.getIsDel()|| !SecurityUtils.getTenantId().equals(info.getTenantId())){
                return "公司银行配置信息不存在！";
            }
        }else {
            int count = this.count(new QueryWrapper<BankInfoP>().lambda().eq(BankInfoP::getTenantId,SecurityUtils.getTenantId())
                    .eq(BankInfoP::getIsDel,FlowConstant.SHORT_ZERO));
            if(count>FlowConstant.INTEGER_ONE){
                return "公司银行配置已存在！";
            }
        }
        return null;
    }


}
