/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.pay.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.pay.entity.BankInfoP;
import com.tofly.workflow.pay.entity.PayImplInfoP;
import com.tofly.workflow.pay.entity.dto.PayImplInfoDto;
import com.tofly.workflow.pay.mapper.PayImplInfoPMapper;
import com.tofly.workflow.pay.service.BankInfoPService;
import com.tofly.workflow.pay.service.PayImplInfoPService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 银行支付相关接口配置
 *
 * @author xqf
 * @date Wed Nov 16 00:00:00 CST 2022
 */
@Service("payImplInfoPService")
public class PayImplInfoPServiceImpl extends ServiceImpl<PayImplInfoPMapper, PayImplInfoP> implements PayImplInfoPService {

    /**
     * 银行（金融）配置信息
     */
    @Autowired
    private BankInfoPService bankInfoService;

    /**
     * 保持银行支付接口配置
     * @param dto 配置信息
     * @return
     */
    @Override
    public Object savePayImpl(PayImplInfoDto dto) {
        String check = saveCheck(dto);
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        PayImplInfoP info = new PayImplInfoP();
        BeanUtils.copyProperties(dto,info);
        info.setIsDel(FlowConstant.SHORT_ZERO);
        return this.saveOrUpdate(info);
    }

    /**
     * 删除银行支付相关接口配置
     * @param id id
     * @return
     */
    @Override
    public Boolean removePayImplById(Long id) {
        return this.update(new UpdateWrapper<PayImplInfoP>().lambda().eq(PayImplInfoP::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(PayImplInfoP::getId,id).set(PayImplInfoP::getIsDel,FlowConstant.SHORT_ONE));
    }

    /**
     * 数据交易
     * @param dto 配置信息
     * @return
     */
    private String saveCheck(PayImplInfoDto dto) {
        int bc = bankInfoService.count(new QueryWrapper<BankInfoP>().lambda().eq(BankInfoP::getId,dto.getBankId())
                .eq(BankInfoP::getIsDel,FlowConstant.SHORT_ZERO).eq(BankInfoP::getIsValid,FlowConstant.SHORT_ONE));
        if(bc==FlowConstant.SHORT_ZERO){
            return "银行（金融）配置信息不存在！";
        }
        if(dto.getId()!=null){
            PayImplInfoP info = this.getById(dto.getId());
            if(info==null){
                return "支付配置不存在！";
            }
            if(!dto.getBankId().equals(info.getBankId())){
                return "银行信息不能更改";
            }
            int count = this.count(new QueryWrapper<PayImplInfoP>().lambda().eq(PayImplInfoP::getBankId,dto.getBankId())
                    .eq(PayImplInfoP::getPayType,dto.getPayType()).ne(PayImplInfoP::getId,dto.getId()).eq(PayImplInfoP::getIsDel,FlowConstant.SHORT_ZERO));
            if(count> FlowConstant.INTEGER_ONE){
                return "支付类型已存在";
            }
        }else {
            int count = this.count(new QueryWrapper<PayImplInfoP>().lambda().eq(PayImplInfoP::getBankId,dto.getBankId()).eq(PayImplInfoP::getPayType,dto.getPayType())
                    .eq(PayImplInfoP::getIsDel,FlowConstant.SHORT_ZERO));
            if(count> FlowConstant.INTEGER_ONE){
                return "支付类型已存在";
            }
        }

        return null;
    }


}
