package com.tofly.workflow.pay.service.impl;

import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.pay.entity.BankInfoP;
import com.tofly.workflow.pay.entity.dto.ResultQueryDto;
import com.tofly.workflow.pay.entity.dto.UserMainScanDto;
import com.tofly.workflow.pay.entity.vo.CodeVo;
import com.tofly.workflow.pay.entity.vo.QueryResultVo;
import com.tofly.workflow.pay.entity.vo.RefundVo;
import com.tofly.workflow.pay.entity.vo.UserMainScanVo;
import com.tofly.workflow.pay.service.BankInfoPService;
import com.tofly.workflow.pay.service.QrcodepayBaseService;
import com.tofly.workflow.pay.service.QrcodepayImplService;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.service.TfProcessLService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 银行（金融）请求接口
 *
 * @author xqf
 * @date Wed Nov 16 00:00:00 CST 2022
 */
@Service("bankImplService")
public class QrcodepayImplServiceImpl implements QrcodepayImplService {

    /**
     * 工单流程
     */
    @Autowired
    private TfProcessLService processService;

    /**
     * 银行（金融）配置信息
     */
    @Autowired
    private BankInfoPService bankInfoPService;

    @Override
    public UserMainScanVo userMainScan(UserMainScanDto dto) {
        TfProcessL pl = processService.getById(dto.getProcessId());
        if(pl==null){
            UserMainScanVo vo = new UserMainScanVo(FlowConstant.INTEGER_ZERO,"流程不存在！");
            return vo;
        }
        BankInfoP infoP = new BankInfoP();
        CodeVo codeVo = fillBankInfo(infoP);
        if(codeVo!=null){
            UserMainScanVo vo = new UserMainScanVo();
            BeanUtils.copyProperties(codeVo,vo);

            return vo;
        }
        QrcodepayBaseService baseService = SpringUtil.getBean(infoP.getService());
        UserMainScanVo vo = baseService.userMainScan(infoP,dto);
        return vo;
    }

    /**
     * 查询支付（退款）结果
     * @param dto 查询请求
     * @return 支付结果
     */
    @Override
    public QueryResultVo resultQuery(ResultQueryDto dto) {
        BankInfoP infoP = new BankInfoP();
        CodeVo codeVo = fillBankInfo(infoP);
        if(codeVo!=null){
            QueryResultVo vo = new QueryResultVo();
            BeanUtils.copyProperties(codeVo,vo);
            return vo;
        }
        QrcodepayBaseService baseService = SpringUtil.getBean(infoP.getService());
        QueryResultVo vo = baseService.resultQuery(infoP,dto);
        return vo;
    }

    /**
     * 请求退款
     * @param dto 请求信息
     * @return 退款结果
     */
    @Override
    public RefundVo refund(UserMainScanDto dto) {
        BankInfoP infoP = new BankInfoP();
        CodeVo codeVo = fillBankInfo(infoP);
        if(codeVo!=null){
            RefundVo vo = new RefundVo();
            BeanUtils.copyProperties(codeVo,vo);
            return vo;
        }
        QrcodepayBaseService baseService = SpringUtil.getBean(infoP.getService());
        RefundVo vo = baseService.refund(infoP,dto);
        return vo;
    }

    private CodeVo fillBankInfo(BankInfoP infoP){
        BankInfoP infoPl = bankInfoPService.getOne(new QueryWrapper<BankInfoP>().lambda().eq(BankInfoP::getTenantId, SecurityUtils.getTenantId())
                .eq(BankInfoP::getIsValid,FlowConstant.SHORT_ONE)
                .eq(BankInfoP::getIsDel, FlowConstant.SHORT_ZERO));
        CodeVo vo = null;
        if(infoPl==null|| StringUtils.isEmpty(infoPl.getService())){
            vo = new CodeVo(FlowConstant.INTEGER_ZERO,"银行(金融)配置不存在或无效");
            return vo;
        }
        BeanUtils.copyProperties(infoPl,infoP);
        return null;
    }
}
