package com.tofly.workflow.pay.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.scrcu.opensdk.ScrcuRequest;
import com.scrcu.opensdk.ScrcuResponse;
import com.scrcu.opensdk.ScrcuSdk;
import com.scrcu.opensdk.exception.ScrcuApiException;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.pay.entity.BankInfoP;
import com.tofly.workflow.pay.entity.PayImplInfoP;
import com.tofly.workflow.pay.entity.PaySelectW;
import com.tofly.workflow.pay.entity.PayTradeW;
import com.tofly.workflow.pay.entity.dto.ResultQueryDto;
import com.tofly.workflow.pay.entity.dto.UserMainScanDto;
import com.tofly.workflow.pay.entity.vo.QueryResultVo;
import com.tofly.workflow.pay.entity.vo.RefundVo;
import com.tofly.workflow.pay.entity.vo.UserMainScanVo;
import com.tofly.workflow.pay.service.*;
import com.tofly.workflow.utils.MD5Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Service("scanPayService")
public class ScanPayServiceImpl implements ScanPayService, QrcodepayBaseService {

    @Autowired
    private BankInfoPService bankInfoPService;

    @Autowired
    private PayImplInfoPService payImplInfoPService;

    @Autowired
    private PayTradeWService payTradeWService;

    @Autowired
    private PaySelectWService paySelectWService;

    /*
     *单例
     */
    private static MD5Util instance;
    public static MD5Util getInstance(){
        if(instance==null){
            instance=new MD5Util();
        }
        return instance;
    }

    @Override
    public Object getSdk(BankInfoP infoP) {
        ScrcuSdk sdk = null;
        try {
            sdk = ScrcuSdk.getInstance(infoP.getAppid(), infoP.getAppsecret(),infoP.getOpenepubk(),infoP.getAppdprik(),infoP.getOpenvpubk(),infoP.getAppsprik());
        } catch (ScrcuApiException e) {
            e.printStackTrace();
        }
        return sdk;
    }

    @Override
    public UserMainScanVo userMainScan(BankInfoP infoP, UserMainScanDto dto) {
        PayTradeW payTrade = new PayTradeW();
        payTrade.setPayType((short) 1);
        payTrade.setTxnType("01");
        payTrade.setDevFpId("00000000");
        payTrade.setProcessId(dto.getProcessId());
        payTrade.setProcessStep(dto.getProcessStep());
        payTrade.setStepNode(dto.getStepNode());
        payTrade.setTenantId(SecurityUtils.getTenantId());

        UserMainScanVo userMainScanVo = new UserMainScanVo();
        userMainScanVo.setDescribe(dto.getOrderDesc());
        ScrcuSdk sdk = (ScrcuSdk) getSdk(infoP);
        //获取银行支付相关接口配置信息
        PayImplInfoP payImplInfoP = payImplInfoPService.getOne(new QueryWrapper<PayImplInfoP>().lambda()
                .eq(PayImplInfoP::getBankId,infoP.getId())
                .eq(PayImplInfoP::getPayType,"1")
                .eq(PayImplInfoP::getIsDel,"0"));
        if(payImplInfoP == null){
            throw new ToflyDeniedException("银行支付相关接口配置信息不存在");
        }
        payTrade.setImplId(payImplInfoP.getId());
        payTrade.setPath(infoP.getHttp() + payImplInfoP.getPath());
        //设置自定义域名
        sdk.setDomainName(infoP.getHttp());
        //设置请求body参数--以map为例,可以使用实体类或者json格式字符串
        HashMap<String, Object> body = new HashMap<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdfSSS = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        String requestId = sdfSSS.format(new Date()) + infoP.getAppid() + new Random().nextInt(900)+100;
        payTrade.setRequestId(requestId);
        userMainScanVo.setRequestId(requestId);
        String reqTime = sdf.format(new Date());
        payTrade.setReqTime(reqTime);
        userMainScanVo.setReqTime(reqTime);
        String reqSsn = getInstance().getMD5(requestId);
        payTrade.setReqSsn(reqSsn);
        userMainScanVo.setReqSsn(reqSsn);
        body.put("mchtNo", infoP.getMchtNo());
        body.put("notifyUrl", payImplInfoP.getNotifyUrl());
        payTrade.setNotifyUrl(payImplInfoP.getNotifyUrl());
        body.put("validTime", payImplInfoP.getValidTime());
        payTrade.setValidTime(payImplInfoP.getValidTime());
        body.put("reqTime", reqTime);
        body.put("orderAmt", dto.getOrderAmt());
        payTrade.setOrderAmt(dto.getOrderAmt());
        body.put("reqSsn", reqSsn);
        body.put("mchtTermId", "00000000");
        payTrade.setMchtTermId("00000000");
        body.put("txnAreaInfo", "511381");
        payTrade.setTxnAreaInfo("511381");
        body.put("orderDesc", dto.getOrderDesc());
        payTrade.setOrderDesc(dto.getOrderDesc());
        //初始化ScrcuRequest对象，放入body
        //如果需要传递request中的参数，请初始化时放入
        ScrcuRequest<HashMap<String, Object>> request= new ScrcuRequest<>(body);
        String requestTime = sdf.format(new Date());
        payTrade.setRequestTime(requestTime);
        try {
            request.setRequestParam(request, "appId", infoP.getAppid());
            request.setRequestParam(request, "appSecret", infoP.getAppsecret());
            request.setRequestParam(request, "requestId", requestId);
            request.setRequestParam(request, "requestTime", requestTime);
            request.setRequestParam(request, "charset", infoP.getCharset());
            request.setRequestParam(request, "version", infoP.getVersion());
            request.setRequestParam(request, "devFpId", "00000000");
            //发起调用--传入url，ScrcuRequest对象，版本号
            log.info("payImplInfoP::"+ payImplInfoP.toString());
            ScrcuResponse response = sdk.call(payImplInfoP.getPath(), request, infoP.getVersion());
            log.info("Header::"+response.getHeaderString());
            log.info("ErrorMsg::"+response.getErrorMsg());
            log.info("ErrorCode::"+response.getErrorCode());
            log.info("Body::"+response.getBodyString());
            String errorCode = response.getErrorCode();
            if(errorCode.equals("0000000000")){
                payTrade.setRespCode("1");
                payTrade.setRespState(FlowConstant.SHORT_ONE);
                userMainScanVo.setCode(1);
            }else {
                payTrade.setRespCode("0");
                payTrade.setRespState(FlowConstant.SHORT_ZERO);
                userMainScanVo.setCode(0);
            }
            //获取response里面的数据---不同形式
            Map<String, Object> bodyData = response.getBody();
            payTrade.setRespBody(response.getBodyString());
            payTrade.setResponseId(response.getRequestId());
            payTrade.setResponseTime(String.valueOf(bodyData.get("responseTime")));
            payTrade.setRespCode(errorCode);
            payTrade.setRespMsg(String.valueOf(bodyData.get("respMsg")));
            payTrade.setRespSsn(String.valueOf(bodyData.get("respSsn")));
            payTrade.setRespTime(String.valueOf(bodyData.get("respTime")));
            payTrade.setToken(String.valueOf(bodyData.get("token")));
            userMainScanVo.setToken(String.valueOf(bodyData.get("token")));
        } catch (Exception e) {
            e.printStackTrace();
            payTradeWService.save(payTrade);
            return userMainScanVo;
        }
        payTradeWService.save(payTrade);
        return userMainScanVo;
    }

    @Override
    public QueryResultVo resultQuery(BankInfoP infoP, ResultQueryDto dto) {
        PaySelectW paySelect = new PaySelectW();
        paySelect.setOrigReqSsn(dto.getReqSsn());
        paySelect.setOrigReqTime(dto.getReqTime());
        paySelect.setMchtNo(infoP.getMchtNo());
        paySelect.setReqType((short) 1);

        QueryResultVo queryResultVo = new QueryResultVo();
        queryResultVo.setPayType(dto.getPayType());
        ScrcuSdk sdk = (ScrcuSdk) getSdk(infoP);
        //获取银行支付相关接口配置信息
        PayImplInfoP payImplInfoP = payImplInfoPService.getOne(new QueryWrapper<PayImplInfoP>().lambda()
                .eq(PayImplInfoP::getBankId,infoP.getId())
                .eq(PayImplInfoP::getPayType,"3")
                .eq(PayImplInfoP::getIsDel,"0"));
        if(payImplInfoP == null){
            throw new ToflyDeniedException("银行支付相关接口配置信息不存在");
        }
        paySelect.setPath(infoP.getHttp() + payImplInfoP.getPath());
        //设置自定义域名--网关为例
        sdk.setDomainName(infoP.getHttp());
        //设置请求body参数--以map为例,可以使用实体类或者json格式字符串
        HashMap<String, Object> body = new HashMap<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdfSSS = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        String requestId = sdfSSS.format(new Date()) + infoP.getAppid() + new Random().nextInt(900);
        queryResultVo.setRequestId(requestId);
        String reqTime = sdf.format(new Date());
        paySelect.setReqTime(reqTime);
        String reqSsn = getInstance().getMD5(requestId);
        paySelect.setReqSsn(reqSsn);
        queryResultVo.setReqSsn(reqSsn);
        body.put("origReqSsn", dto.getReqSsn());
        body.put("mchtNo", infoP.getMchtNo());
        body.put("reqTime", reqTime);
        body.put("origReqTime", dto.getReqTime());
        body.put("reqSsn", reqSsn);
        //初始化ScrcuRequest对象，放入body
        //如果需要传递request中的参数，请初始化时放入
        ScrcuRequest<HashMap<String, Object>> request= new ScrcuRequest<>(body);
        try {
            request.setRequestParam(request, "appId", infoP.getAppid());
            request.setRequestParam(request, "appSecret", infoP.getAppsecret());
            request.setRequestParam(request, "requestId", requestId);
            request.setRequestParam(request, "requestTime", sdf.format(new Date()));
            request.setRequestParam(request, "charset", infoP.getCharset());
            request.setRequestParam(request, "version", infoP.getVersion());
            //发起调用--传入url，ScrcuRequest对象，版本号
            ScrcuResponse response = sdk.call(payImplInfoP.getPath(), request, infoP.getVersion());
            String errorCode = response.getErrorCode();
            if(errorCode.equals("0000000000")){
                paySelect.setReqStatus((short) 1);
                queryResultVo.setCode(1);
            }else {
                paySelect.setReqStatus((short) 0);
                queryResultVo.setCode(0);
            }
            //获取response里面的数据---不同形式
            Map<String, Object> bodyData = response.getBody();
            paySelect.setRespBody(response.getBodyString());
            paySelect.setRespHeader(response.getHeaderString());
            PayTradeW payTradeUpdate = new PayTradeW();
            if(StringUtils.isNotBlank(errorCode)&&bodyData.get("respMsg")!=null){
                queryResultVo.setDescribe(String.valueOf(bodyData.get("respMsg")));
            }

            if (bodyData.get("txnAmt") != null) {
                String str = String.valueOf(bodyData.get("txnAmt"));
                Double txnamt = StringUtils.isNotBlank(str)?Double.parseDouble(str):0;
                queryResultVo.setTxnAmt(txnamt);
                payTradeUpdate.setTxnAmt(txnamt);
            }
            if (bodyData.get("payAmt") != null) {
                String str = String.valueOf(bodyData.get("payAmt"));
                Double payamt = StringUtils.isNotBlank(str)?Double.parseDouble(str):0;
                queryResultVo.setPayAmt(payamt);
                payTradeUpdate.setPayAmt(payamt);
            }
            if (bodyData.get("activeAmt") != null) {
                String str = String.valueOf(bodyData.get("activeAmt"));
                Double activeamt = StringUtils.isNotBlank(str)?Double.parseDouble(str):0;
                queryResultVo.setActiveAmt(activeamt);
                payTradeUpdate.setActiveAmt(activeamt);
            }
            if(bodyData.get("respCode")!=null){
                payTradeUpdate.setRespCode(String.valueOf(bodyData.get("respCode")));
            }
            if (bodyData.get("orderState") != null) {
                String str = String.valueOf(bodyData.get("orderState"));
                payTradeUpdate.setOrderState(str);
            }
            payTradeWService.update(payTradeUpdate,new QueryWrapper<PayTradeW>().lambda().eq(PayTradeW::getReqSsn,dto.getReqSsn()));

        } catch (ScrcuApiException e) {
            e.printStackTrace();
            PaySelectW paySelectQuery = paySelectWService.getOne(new QueryWrapper<PaySelectW>().lambda()
                    .eq(PaySelectW::getReqSsn,reqSsn)
                    .eq(PaySelectW::getReqType,"1"));
            if(paySelectQuery == null){
                paySelectWService.save(paySelect);
            }
            return queryResultVo;
        }
        PaySelectW paySelectQuery = paySelectWService.getOne(new QueryWrapper<PaySelectW>().lambda()
                .eq(PaySelectW::getReqSsn,reqSsn)
                .eq(PaySelectW::getReqType,"1"));
        if(paySelectQuery == null){
            paySelectWService.save(paySelect);
        }
        return queryResultVo;
    }

    @Override
    public RefundVo refund(BankInfoP infoP, UserMainScanDto dto) {
        PayTradeW payTrade = new PayTradeW();
        payTrade.setPayType((short) 4);
        payTrade.setTxnType("02");
        payTrade.setProcessId(dto.getProcessId());
        payTrade.setProcessStep(dto.getProcessStep());
        payTrade.setStepNode(dto.getStepNode());
        payTrade.setOrigReqSsn(dto.getReqSsn());
        payTrade.setOrigReqTime(dto.getReqTime());

        RefundVo refundVo = new RefundVo();
        ScrcuSdk sdk = (ScrcuSdk) getSdk(infoP);
        //获取银行支付相关接口配置信息
        PayImplInfoP payImplInfoP = payImplInfoPService.getOne(new QueryWrapper<PayImplInfoP>().lambda()
                .eq(PayImplInfoP::getBankId,infoP.getId())
                .eq(PayImplInfoP::getPayType,"4")
                .eq(PayImplInfoP::getIsDel,"0"));
        if(payImplInfoP == null){
            throw new ToflyDeniedException("银行支付相关接口配置信息不存在");
        }
        payTrade.setImplId(payImplInfoP.getId());
        payTrade.setPath(infoP.getHttp() + payImplInfoP.getPath());
        //设置自定义域名--网关为例
        sdk.setDomainName(infoP.getHttp());
        //设置请求body参数--以map为例,可以使用实体类或者json格式字符串
        HashMap<String, Object> body = new HashMap<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdfSSS = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        String requestId = sdfSSS.format(new Date()) + infoP.getAppid() + new Random().nextInt(900)+100;
        payTrade.setRequestId(requestId);
        refundVo.setRequestId(requestId);
        String reqTime = sdf.format(new Date());
        payTrade.setReqTime(reqTime);
        refundVo.setReqTime(reqTime);
        String reqSsn = getInstance().getMD5(requestId);
        payTrade.setReqSsn(reqSsn);
        refundVo.setReqSsn(reqSsn);
        body.put("origReqSsn", dto.getReqSsn());
        body.put("origReqTime", dto.getReqTime());
        body.put("mchtNo", infoP.getMchtNo());
        body.put("reqTime", reqTime);
        body.put("reqSsn", reqSsn);
        body.put("orderAmt", dto.getOrderAmt());
        payTrade.setOrderAmt(dto.getOrderAmt());
        body.put("orderDesc", dto.getOrderDesc());
        payTrade.setOrderDesc(dto.getOrderDesc());
        refundVo.setDescribe(dto.getOrderDesc());
        //初始化ScrcuRequest对象，放入body
        //如果需要传递request中的参数，请初始化时放入
        ScrcuRequest<HashMap<String, Object>> request= new ScrcuRequest<>(body);
        String requestTime = sdf.format(new Date());
        payTrade.setRequestTime(requestTime);
        try {
            request.setRequestParam(request, "appId", infoP.getAppid());
            request.setRequestParam(request, "appSecret", infoP.getAppsecret());
            request.setRequestParam(request, "requestId", requestId);
            request.setRequestParam(request, "requestTime", requestTime);
            request.setRequestParam(request, "charset", infoP.getCharset());
            request.setRequestParam(request, "version", infoP.getVersion());
            //发起调用--传入url，ScrcuRequest对象，版本号
            ScrcuResponse response = sdk.call(payImplInfoP.getPath(), request, infoP.getVersion());
            String errorCode = response.getErrorCode();
            if(errorCode.equals("0000000000")){
                refundVo.setCode(1);
            }else {
                refundVo.setCode(0);
            }
            //获取response里面的数据---不同形式
            Map<String, Object> bodyData = response.getBody();
            payTrade.setRespBody(response.getBodyString());
            payTrade.setResponseId(response.getRequestId());
            payTrade.setResponseTime(String.valueOf(bodyData.get("responseTime")));
            payTrade.setRespCode(errorCode);
            payTrade.setRespMsg(String.valueOf(bodyData.get("respMsg")));
            payTrade.setRespSsn(String.valueOf(bodyData.get("respSsn")));
            payTrade.setRespTime(String.valueOf(bodyData.get("respTime")));
            if(bodyData.get("refundAmt") != null){
                payTrade.setRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("refundAmt"))));
                refundVo.setRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("refundAmt"))));
            }
            if(bodyData.get("custRefundAmt") != null){
                payTrade.setCRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("custRefundAmt"))));
                refundVo.setCRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("custRefundAmt"))));
            }
            if(bodyData.get("mchtRefundAmt") != null){
                payTrade.setMRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("mchtRefundAmt"))));
                refundVo.setMRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("mchtRefundAmt"))));
            }
            if(bodyData.get("platRefundAmt") != null){
                payTrade.setPRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("platRefundAmt"))));
                refundVo.setPRefundAmt(Double.parseDouble(String.valueOf(bodyData.get("platRefundAmt"))));
            }
        } catch (ScrcuApiException e) {
            e.printStackTrace();
            payTradeWService.save(payTrade);
            return refundVo;
        }
        payTradeWService.save(payTrade);
        return refundVo;
    }

    @Override
    public String payResultCallback(String callBackData) {
        BankInfoP bankInfo = bankInfoPService.getOne(new QueryWrapper<BankInfoP>().lambda()
                .eq(BankInfoP::getBankName,"四川农信")
                .eq(BankInfoP::getIsDel,"0"));
        if(bankInfo == null){
            throw new ToflyDeniedException("银行配置数据为空");
        }
        String callBackJson = null;
        try {
            //初始化sdk
            ScrcuSdk sdk = ScrcuSdk.getInstance(bankInfo.getAppid(), bankInfo.getAppsecret(),bankInfo.getOpenepubk(),bankInfo.getAppdprik(),bankInfo.getOpenvpubk(),bankInfo.getAppsprik());
            //获取验签解密的回调报文
            callBackJson = sdk.getCallBackJson(callBackData);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Map<String,String> data = JSON.parseObject(callBackJson, Map.class);
        if(data == null){
            throw new ToflyDeniedException("解析数据为空");
        }
        PaySelectW paySelect = new PaySelectW();
        paySelect.setOrigReqSsn(data.get("origReqSsn"));
        paySelect.setOrigReqTime(data.get("origReqTime"));
        paySelect.setReqType((short) 2);
        paySelect.setRespBody(callBackJson);
        paySelect.setMchtNo(bankInfo.getMchtNo());
        String str = String.valueOf(data.get("orderState"));
        if(StringUtils.isNotBlank(str)){
            PayTradeW payTradeUpdate = new PayTradeW();
            payTradeUpdate.setOrderState("0" + str);
            String str1;
            if (data.get("txnAmt") != null) {
                str1 = String.valueOf(data.get("txnAmt"));
                payTradeUpdate.setTxnAmt(StringUtils.isNotBlank(str1)?Double.parseDouble(str1):0);
            }
            if (data.get("payAmt") != null) {
                str1 = String.valueOf(data.get("payAmt"));
                payTradeUpdate.setPayAmt(StringUtils.isNotBlank(str1)?Double.parseDouble(str1):0);
            }
            if (data.get("activeAmt") != null) {
                str1 = String.valueOf(data.get("activeAmt"));
                payTradeUpdate.setActiveAmt(StringUtils.isNotBlank(str1)?Double.parseDouble(str1):0);
            }
            payTradeWService.update(payTradeUpdate,new QueryWrapper<PayTradeW>().lambda().eq(PayTradeW::getReqSsn,data.get("origReqSsn")));
        }
        paySelectWService.save(paySelect);
        return "SUCCESS";
    }

    /**
     * 定时任务，每 1 分钟执行一次
     * 获得前一小时中未取得交易结果的成功状态流水
     */
    //@Scheduled(cron = "0 */1 * * * ?")
    @Scheduled(cron = "*/30 * * * * ?")
    public void getTodayOrderResult(){
        SimpleDateFormat sdfymd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //当前时间前一小时

        Date date = new Date();
        Date beg_date = DateUtil.offsetHour(date, -1).toJdkDate();

        List<PayTradeW> payTradeList = payTradeWService.list(new QueryWrapper<PayTradeW>().lambda()
                .eq(PayTradeW::getRespState,FlowConstant.SHORT_ONE).and(r->r.isNull(PayTradeW::getOrderState).or()
                        .eq(PayTradeW::getOrderState,"11").or().eq(PayTradeW::getOrderState,"12").or().eq(PayTradeW::getOrderState,"13"))
                .and(r->r.isNull(PayTradeW::getRespCode).or().eq(PayTradeW::getRespCode,"0000000000").or().eq(PayTradeW::getRespCode,"EBAP001026"))
                .between(PayTradeW::getCreateTime,beg_date,date));
        if(payTradeList == null){
            return;
        }

        Map<Long,BankInfoP> map = new HashMap<>();
        for (PayTradeW payTradeW : payTradeList) {
            if(payTradeW.getReqSsn() == null){
                continue;
            }
            BankInfoP bankInfo = null;
            if(map.containsKey(payTradeW.getTenantId())){
                bankInfo = map.get(payTradeW.getTenantId());
            }else {
                bankInfo = bankInfoPService.getOne(new QueryWrapper<BankInfoP>().lambda().eq(BankInfoP::getTenantId,payTradeW.getTenantId())
                        .eq(BankInfoP::getIsValid,FlowConstant.SHORT_ONE).eq(BankInfoP::getIsDel,FlowConstant.SHORT_ZERO));
                map.put(payTradeW.getTenantId(),bankInfo);
            }
            if(bankInfo==null){
                log.error("银行（金融）配置信息 不存在 tenantId::"+payTradeW.getTenantId());
                return;
            }

            ResultQueryDto dto = new ResultQueryDto();
            dto.setPayType(payTradeW.getPayType());
            dto.setRequestId(payTradeW.getRequestId());
            dto.setReqTime(payTradeW.getReqTime());
            dto.setReqSsn(payTradeW.getReqSsn());
            dto.setProcessId(payTradeW.getProcessId());
//            List<PaySelectW> paySelectList = paySelectWService.list(new QueryWrapper<PaySelectW>().lambda()
//                    .eq(PaySelectW::getReqSsn,payTradeW.getReqSsn()));
            //如果没有查询到此流水号的交易结果，则调用主动交易结果查询
//            if(CollUtil.isNotEmpty(paySelectList)){
//                resultQuery(bankInfo, dto);
//            }
            resultQuery(bankInfo, dto);
        }
    }

}
