/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.process.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.process.entity.TfNowHandleL;
import com.tofly.workflow.process.service.TfNowHandleLService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Arrays;
/**
 * 流程当前处理信息
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfnowhandlel")
@Api(tags="流程当前处理信息接口")
public class TfNowHandleLController extends BaseController {

    private final  TfNowHandleLService tfNowHandleLService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfNowHandleL 流程当前处理信息
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfNowHandleLPage(Page page, @ModelAttribute TfNowHandleL tfNowHandleL) {
      return ResultData.ok(tfNowHandleLService.page(page, Wrappers.query(tfNowHandleL)));
    }


    /**
     * 通过id查询流程当前处理信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfNowHandleLService.getById(id));
    }


    /**
     * 根据流程id获取当前处理步骤及人员
     * @param processId 流程id
     * @return ResultData
     */
    @ToFlyAppLog(title="根据流程id获取当前处理步骤及人员" )
    @GetMapping("/handleUser" )
    @ApiOperation(value = "根据流程id获取当前处理步骤及人员")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "processId", value = "processId", required = true, dataType = "Long")
    })
    public ResultData getHandleUserById(Long processId) {
        return ResultData.ok(tfNowHandleLService.getNowHandleByProcessId(processId));
    }


}
