/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.process.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.Constant;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.utils.POIUtil;
import com.tofly.workflow.buss.entity.TfContractW;
import com.tofly.workflow.buss.service.TfContractWService;
import com.tofly.workflow.buss.service.impl.TfContractWServiceImpl;
import com.tofly.workflow.process.entity.dto.PageProcessDto;
import com.tofly.workflow.process.entity.dto.ProcessDto;
import com.tofly.workflow.process.entity.dto.FsProcessDto;
import com.tofly.workflow.process.service.TfProcessLService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 流程信息
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprocessl")
@Api(tags="流程信息接口")
public class TfProcessLController extends BaseController {
   // @Value("${zmrq.flow.tempFile}")
    private static  String tempStr="D:/zmrq";

    private final  TfProcessLService tfProcessLService;
    private final TfContractWService tfContractWService;
    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 流程信息
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProcessLPage(Page page, @ModelAttribute PageProcessDto dto) {
      return ResultData.ok(tfProcessLService.pageProcess(page, dto));
    }


    /**
     * 获取附属流程
     * @param dto 查询条件
     * @return
     */
    @ToFlyAppLog(title="获取附属流程" )
    @GetMapping("/fsProcess" )
    @ApiOperation(value = "获取附属流程")
    public ResultData getSubsidiaryProcess(Page page,@ModelAttribute FsProcessDto dto){
        return ResultData.ok(tfProcessLService.getSubsidiaryProcess(page,dto));
    }


    /**
     * 通过id查询流程信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProcessLService.getTfProcessLById(id));
    }

    /**
     * 新增流程信息
     * @param dto 流程工单上报
     * @param files 文件集合
     * @return
     */
    @ToFlyAppLog(title="新增流程信息" )
    @ApiOperation(value = "新增流程信息")
    @PostMapping
    public ResultData save(@RequestPart @Valid ProcessDto dto, @RequestPart List<MultipartFile> files) {
        Object obj = tfProcessLService.processSave(dto,files);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除流程信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除流程信息" )
    @ApiOperation(value = "通过ID删除流程信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfProcessLService.removeProcessById(id));
    }
    @ApiOperation(value = "通过ID删除流程信息")
    @GetMapping("testPngAdd")
    public ResultData testFileAdd() {
        final List<TfContractW> list = tfContractWService.list(Wrappers.<TfContractW>lambdaQuery().eq(TfContractW::getProcessId, 1759095414276395009l));
        for (TfContractW tfContractW : list) {
            final String annex = tfContractW.getContractFile();
            final JSONArray objects = JSON.parseArray(annex);
            List<FileInfo> fileInfos = new ArrayList<>();
            List<FileInfo> pdfFileInfos = new ArrayList<>();
            for (int i = 0; i < objects.size(); i++) {
                final JSONObject jsonObject = objects.getJSONObject(i);
                final FileInfo oldFileInfo = jsonObject.toJavaObject(FileInfo.class);

                final FileInfo fileInfo = POIUtil.addSign(oldFileInfo.getFilePath(),oldFileInfo);

                fileInfos.add(fileInfo);


                String filePathBean = oldFileInfo.getFilePath().substring(0,oldFileInfo.getFilePath().lastIndexOf("/") + 1);
                String otherName = IdWorker.getIdStr() + ".pdf";
                String remotePath = filePathBean + otherName;
                String originPath = fileInfo.getFilePath();
                TfContractWServiceImpl.doc2pdf(originPath,tempStr,remotePath);

                FileInfo fileInfoPDF = new FileInfo();
                fileInfoPDF.setFileName(fileInfo.getFileName().substring(0,fileInfo.getFileName().lastIndexOf(Constant.POINT)) + ".pdf");
                fileInfoPDF.setOtherName(otherName);
                fileInfoPDF.setFilePath(remotePath);
                fileInfoPDF.setFileFormat(".pdf");
                pdfFileInfos.add(fileInfoPDF);

            }
            tfContractW.setContractFile(JSON.toJSONString(fileInfos));
            tfContractW.setContractPdfFile(JSON.toJSONString(pdfFileInfos));
            tfContractWService.updateById(tfContractW);
        }
        return  ResultData.ok();
    }
}
