/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.process.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.dto.PNodeSaveDto;
import com.tofly.workflow.process.service.TfProcessNodeLService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.spring.web.json.Json;

import javax.validation.Valid;
import java.util.List;

/**
 * 流程节点处理信息（操作步骤都有记录）
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprocessnodel")
@Api(tags="流程节点处理信息（操作步骤都有记录）接口")
@Slf4j
public class TfProcessNodeLController extends BaseController {

    private final  TfProcessNodeLService tfProcessNodeLService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfProcessNodeL 流程节点处理信息（操作步骤都有记录）
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProcessNodeLPage(Page page, @ModelAttribute TfProcessNodeL tfProcessNodeL) {
      return ResultData.ok(tfProcessNodeLService.page(page, Wrappers.query(tfProcessNodeL)));
    }

    @ToFlyAppLog(title="流程节点列表" )
    @ApiOperation(value = "流程节点列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "label", value = "标签 label", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "processId", value = "流程ID", required = true, dataType = "Long")
    })
    @GetMapping("/nodeList" )
    public ResultData getProcessNode(Long label,Long processId){
        return ResultData.ok(tfProcessNodeLService.getNodeList(label,processId));
    }

    @ToFlyAppLog(title="流程节点列表" )
    @ApiOperation(value = "流程节点列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "processId", value = "流程ID", required = true, dataType = "Long")
    })
    @GetMapping("/nodeListReject" )
    public ResultData getProcessNodeForReject(Long processId){
        return ResultData.ok(tfProcessNodeLService.getProcessNodes(processId));
    }


    /**
     * 通过id查询流程节点处理信息（操作步骤都有记录）
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProcessNodeLService.getById(id));
    }

    @ToFlyAppLog(title="通过流程Id步骤节点获取数据" )
    @ApiOperation(value = "通过流程Id步骤节点获取数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "流程ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "processStep", value = "流程步骤", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "stepNode", value = "步骤节点", required = true, dataType = "Integer")
    })
    @GetMapping("/getNode" )
    public ResultData getNodeByProcessIdAndNode(Long id,Integer processStep,Integer stepNode){
        return ResultData.ok(tfProcessNodeLService.getNodeByProcessIdAndNode(id,processStep,stepNode));
    }

    /**
     * 新增流程节点处理信息（操作步骤都有记录）
     * @param dto 流程节点处理信息（操作步骤都有记录）
     * @return ResultData
     */
    @ToFlyAppLog(title="新增流程节点处理信息（操作步骤都有记录）" )
    @ApiOperation(value = "新增流程节点处理信息（操作步骤都有记录）")
    @PostMapping
    public ResultData save(@RequestPart @Valid PNodeSaveDto dto, @RequestPart List<MultipartFile> files) {

        log.info("新增流程节点:{}", JSON.toJSONString(dto));

        for (MultipartFile multipartFile : files) {
            log.info("multipartFile.getOriginalFilename()"+multipartFile.getOriginalFilename());
        }
        Object obj = tfProcessNodeLService.processNodeSave(dto,files);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 修改流程节点处理信息（操作步骤都有记录）
     * @param dto 流程节点处理信息（操作步骤都有记录）
     * @return ResultData
     */
//    @ToFlyAppLog(title="修改流程节点处理信息（操作步骤都有记录）" )
//    @ApiOperation(value = "修改流程节点处理信息（操作步骤都有记录）")
//    @PutMapping
//    public ResultData updateById(@RequestBody PNodeUpdateDto dto) {
//        Object obj = tfProcessNodeLService.processNodeUpdate(dto);
//        if(obj instanceof String){
//            return ResultData.fail(obj.toString());
//        }
//        return ResultData.ok(obj);
//    }

}
