/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.process.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.process.entity.TfProcessSuperviseL;
import com.tofly.workflow.process.entity.dto.PSuperviseDto;
import com.tofly.workflow.process.service.TfProcessSuperviseLService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Arrays;
/**
 * 流程督办记录信息
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprocesssupervisel")
@Api(tags="流程督办记录信息接口")
public class TfProcessSuperviseLController extends BaseController {

    private final  TfProcessSuperviseLService tfProcessSuperviseLService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfProcessSuperviseL 流程督办记录信息
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProcessSuperviseLPage(Page page, @ModelAttribute TfProcessSuperviseL tfProcessSuperviseL) {
      return ResultData.ok(tfProcessSuperviseLService.page(page, Wrappers.query(tfProcessSuperviseL)));
    }


    /**
     * 通过id查询流程督办记录信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProcessSuperviseLService.getById(id));
    }

    /**
     * 新增流程督办记录信息
     * @param dto 流程督办记录信息
     * @return ResultData
     */
    @ToFlyAppLog(title="新增流程督办记录信息" )
    @ApiOperation(value = "新增流程督办记录信息")
    @PostMapping
    public ResultData save(@RequestBody PSuperviseDto dto) {
        Object obj = tfProcessSuperviseLService.pSuperviseSave(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 通过id删除流程督办记录信息
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除流程督办记录信息" )
    @ApiOperation(value = "通过ID删除流程督办记录信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfProcessSuperviseLService.removeById(id));
    }


}
