package com.tofly.workflow.process.entity.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.workflow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 流程节点信息
 */
@Data
@EqualsAndHashCode
public class PNodeDto implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * 流程步骤
     */
    @NotNull(message = "processStep" + FlowConstant.ISNOTNUL)
    @Min(value = 1, message = "processStep" + FlowConstant.NOT_LESS_THAN + 1)
    @ApiModelProperty(value = "流程步骤")
    private Integer processStep;
    /**
     * 步骤节点
     */
    @NotNull(message = "stepNode" + FlowConstant.ISNOTNUL)
    @Min(value = 1, message = "stepNode" + FlowConstant.NOT_LESS_THAN + 1)
    @ApiModelProperty(value = "步骤节点")
    private Integer stepNode;
    /**
     * 开始时间
     */
    @ApiModelProperty(value = "开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;
    /**
     * 结束时间
     */
    @ApiModelProperty(value = "结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;
    /**
     * 条件（判断选择条件多个用逗号分隔）
     */
    @ApiModelProperty(value = "条件（判断选择条件多个用逗号分隔）")
    private String condition;
    /**
     * 处理意见
     */
    @ApiModelProperty(value = "处理意见")
    private String opinion;
    /**
     * 处理结果(页面自行控制)
     */
    @ApiModelProperty(value = "处理结果(页面自行控制)")
    private Short handleResults;

    /**
     * 经度
     */
    @ApiModelProperty(value = "经度")
    private Double longitude;
    /**
     * 纬度
     */
    @ApiModelProperty(value = "纬度")
    private Double latitude;
    /**
     * 数值
     */
    @ApiModelProperty(value = "数值")
    private Double numberValue;
    /**
     * 数值1
     */
    @ApiModelProperty(value = "数值1")
    private Double numberValue1;
    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;

    /**
     * 签字附件
     */
    @ApiModelProperty(value = "签字附件")
    private String signAnnex;

    /**
     * 人员id
     */
    @ApiModelProperty(value = "人员id")
    private Long userId;
    /**
     * 其它人员ID
     */
    @ApiModelProperty(value = "其它人员ID")
    private Long staffId;
    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String handleRemark;

    /**
     * 日期时间
     */
    @ApiModelProperty(value = "日期时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date dataTime;

    /**
     * 地址
     */
    @ApiModelProperty(value = "地址")
    private String adder;

    /**
     * 结果(字典值)
     */
    @ApiModelProperty(value = "结果(字典值)")
    private Long outcome;
    /**
     * 关联id1
     */
    @ApiModelProperty(value = "关联id1")
    private Long oneId;
    /**
     * 创建岗位
     */
    @ApiModelProperty(value = "创建岗位")
    private Long createPost;

    /**
     * 创建部门
     */
    @ApiModelProperty(value = "创建部门")
    private Long createDept;

    /**
     * 处理状态 1通过;2:驳回;3:转派;4:终止;
     */
    @NotNull(message = "headleState"+FlowConstant.ISNOTNUL)
    @Min(value = 1,message = "headleState" + FlowConstant.NOT_LESS_THAN+1 )
    @Max(value = 4,message = "headleState" + FlowConstant.NOT_GREATER_THAN+4)
    @ApiModelProperty(value = "处理状态：1通过;2:驳回;3:转派;4:终止;")
    private Integer handleState;

    /**
     * 保存状态 0保存，1提交
     */
    @NotNull(message = "saveState"+ FlowConstant.ISNOTNUL)
    @Min(value = 0,message = "saveState" + FlowConstant.NOT_LESS_THAN+0)
    @Max(value = 1,message = "saveState" + FlowConstant.NOT_GREATER_THAN+1)
    @ApiModelProperty(value = "0保存，1提交")
    private Integer saveState;

    /**
     * 下一步处理人员
     */
    @Valid
    @ApiModelProperty(value = "下一步处理人员")
    private List<NextStepDto> nextSteps;

    /**
     * 业务数据
     */
    @ApiModelProperty(value = "业务数据")
    private Map<String,String> map;

    /**
     * 文件名称集合
     */
    @ApiModelProperty(value = "文件名称集合")
    private List<String> fileName;




    /**
     * 签字文件名称集合
     */
    @ApiModelProperty(value = "签字文件名称集合")
    private List<String> signFileName;
}
