/*
 * Author: wangjian
 * 生成mapper
 */
package com.tofly.workflow.process.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.vo.NodeListVo;
import com.tofly.workflow.process.entity.vo.ProcessNodeVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Set;

/**
 * 流程节点处理信息（操作步骤都有记录）
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
public interface TfProcessNodeLMapper extends BaseMapper<TfProcessNodeL> {
    /**
     * 查询被驳回的流程节点信息
     * @param ids 被驳回节点ID
     * @return
     */
    List<String> getRejectStepNodes(@Param("ids") Set<Long> ids);

    /**
     * 修改关联流程中的负责人
     * @param post 负责岗位
     * @param dept 负责部门
     * @param user 负责人
     * @param processId 流程ID
     * @param oldUser 历史负责人
     * @return
     */
    Integer updateNodeDutyUser(@Param("post") Long post,@Param("dept") Long dept,@Param("user") Long user,@Param("processId") Long processId,@Param("oldUser") Long oldUser);

    /**
     * 删除下一步节点
     * @param id 流程id
     * @param nodeId 步骤id
     */
    void removeNextNode(@Param("id") Long id,@Param("nodeId") Long nodeId);
    /**
     * 删除转派下一节点
     * @param id 流程id
     * @param nodeId 步骤id
     */
    void removeTransferNode(@Param("id") Long id,@Param("nodeId") Long nodeId);

    /**
     * 历史处理过的结点
     * @param id 流程id
     * @param processStep 步骤序号
     * @param stepNode 结点序号
     * @return 步骤结点
     */
    TfProcessNodeL getHistoryMaxNode(@Param("id") Long id,@Param("processStep") Integer processStep,@Param("stepNode") Integer stepNode);

    /**
     * 获取关联并行节点
     * @param id 流程id
     * @param lists 关联步骤
     * @return
     */
    List<TfProcessNodeL> getParallelNode(@Param("id") Long id,@Param("lists") List<Long> lists);

    /**
     * 获取节点列表
     * @param label 标签
     * @param processId 流程ID
     * @return
     */
    List<NodeListVo> getNodeList(@Param("label") Long label, @Param("processId") Long processId);

    /**
     * 获取下一步节点
     * @param processId 流程id
     * @param nodeId 当前节点id
     * @return 下一步流程
     */
    TfProcessNodeL getNextProcessNode(@Param("processId") Long processId,@Param("nodeId") Long nodeId);

    ProcessNodeVo getProcessNodesForReject(Long id);
}
