/*
 * Author: wangjian
 * 生成Service
 */
package com.tofly.workflow.process.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.dto.PNodeDto;
import com.tofly.workflow.process.entity.dto.PNodeSaveDto;
import com.tofly.workflow.process.entity.dto.PNodeUpdateDto;
import com.tofly.workflow.process.entity.vo.NodeListVo;
import com.tofly.workflow.process.entity.vo.ProcessNodeVo;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 流程节点处理信息（操作步骤都有记录）
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
public interface TfProcessNodeLService extends IService<TfProcessNodeL> {

    /**
     * 流程节点数据校验
     *
     * @param processL
     * @param dto 流程信息
     * @param files 文件集合
     * @param edition 版本
     * @return 校验结果
     */
    String nodeSaveCheck(TfProcessL processL, PNodeDto dto, List<MultipartFile> files, Integer edition, Long processId, Long configId);

    /**
     * 保存流程步骤
     * @param processL 流程信息
     * @param pNode 步骤信息
     * @param files 文件集合
     * @return 保存后对象
     */
    TfProcessNodeL nodeSave(TfProcessL processL, PNodeDto pNode, List<MultipartFile> files);


    /**
     * 保存流程步骤
     * @param dto 流程步骤信息
     * @param files 文件
     * @return 保存结果
     */
    Object processNodeSave(PNodeSaveDto dto, List<MultipartFile> files);

    /**
     * 修改流程步骤负责人
     * @param dto 流程步骤修改数据
     * @return 修改结果
     */
    Object processNodeUpdate(PNodeUpdateDto dto);

    /**
     * 根据流程、步骤、节点获取步骤信息
     * @param id 流程Id
     * @param processStep 步骤序号
     * @param stepNode 节点序号
     * @return 步骤节点
     */
    ProcessNodeVo getNodeByProcessIdAndNode(Long id, Integer processStep, Integer stepNode);

    /**
     * 获取节点列表
     * @param label 标签
     * @param processId 流程ID
     * @return 节点数据
     */
    List<NodeListVo> getNodeList(Long label, Long processId);

    /**
     * 流程步骤节点
     * @param processId 流程id
     * @param nodeId 步骤节点
     * @return 步骤数据
     */
    TfProcessNodeL getNextProcessNode(Long processId, Long nodeId);

    ProcessNodeVo getProcessNodes(Long processId);
}
