/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.process.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfTaskP;
import com.tofly.workflow.baseconf.service.TfTaskPService;
import com.tofly.workflow.buss.entity.TfContractW;
import com.tofly.workflow.buss.entity.TfDesignBudgetW;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.TfNowHandleL;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.dto.FsProcessDto;
import com.tofly.workflow.process.entity.dto.PageProcessDto;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.entity.dto.ProcessDto;
import com.tofly.workflow.process.entity.vo.ProcessMaterialVo;
import com.tofly.workflow.process.entity.vo.ProcessNodeVo;
import com.tofly.workflow.process.entity.vo.ProcessVo;
import com.tofly.workflow.process.service.BaseService;
import com.tofly.workflow.process.service.TfNowHandleLService;
import com.tofly.workflow.process.service.TfProcessLService;
import com.tofly.workflow.process.service.TfProcessNodeLService;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.vo.CTypeVo;
import com.tofly.workflow.processconfig.entity.vo.ServiceImplVo;
import com.tofly.workflow.processconfig.service.TfPageControlLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigLService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import com.tofly.workflow.process.mapper.TfProcessLMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 流程信息
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfProcessLService")
public class TfProcessLServiceImpl extends ServiceImpl<TfProcessLMapper, TfProcessL> implements TfProcessLService {
    /**
     * 流程配置
     */
    @Autowired
    private TfProcessConfigLService pConfigService;

    /**
     * 流程节点
     */
    @Autowired
    private TfProcessNodeLService pNodeLService;

    /**
     * 页面空间展示
     */
    @Autowired
    private TfPageControlLService pControlService;

    /**
     * 流程当前处理信息
     */
    @Autowired
    private TfNowHandleLService nHandleService;

    /**
     * 待办任务
     */
    @Autowired
    private TfTaskPService tfTaskPService;

    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private TfProcessNodeLService tfProcessNodeLService;
    /**
     * 流程上报保存
     * @param dto 工单上报信息
     * @param files 文件
     * @return 保存结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object processSave(ProcessDto dto, List<MultipartFile> files) {
        /**数据校验*/
        String check = processSaveCheck(dto,files);
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        //当前节点的业务数据 key为处理的BaseService的子类的名称，value为json格式字符串
        Map<String,String> map = dto.getPNode().getMap();
        //处理业务数据的具体类
        Map<String, BaseService> bss = new HashMap<>();
        //将业务数据json转换为对象
        Map<String,Object> objs = new HashMap<>();
        if(CollUtil.isNotEmpty(map)) {
            //如果业务数据不为空，开始处理业务数据
            for (Map.Entry<String, String> e : map.entrySet()) {
                BaseService bs = SpringUtil.getBean(e.getKey());
                if (bs == null) {
                    return e.getKey() + FlowConstant.BUSINESS_DOES_NOT_EXIST;
                }
                bss.put(e.getKey(), bs);
                //将当前节点的业务数据json转换成对象
                Object obj = bs.getJsonToObject(e.getValue());
                if (obj == null) {
                    obj = e.getValue();
                }
                check = bs.saveDataCheck(obj, files);
                if (StringUtils.isNotEmpty(check)) {
                    return check;
                }
                objs.put(e.getKey(), obj);
            }
        }

        //保存或者更新当前流程的数据信息
        TfProcessL processL = saveProcess(dto);

// 保存当前节点信息
        TfProcessNodeL pn = pNodeLService.nodeSave(processL,dto.getPNode(),files);
// 处理当前步骤的业务数据
        if(CollUtil.isNotEmpty(objs)) {
            ProcessBaseDto pbdto = new ProcessBaseDto();
            pbdto.setId(processL.getId());
            pbdto.setTenantId(processL.getTenantId());
            pbdto.setCategory(processL.getCategory());
            pbdto.setProcessType(processL.getProcessType());
            pbdto.setProcessStep(dto.getPNode().getProcessStep());
            pbdto.setStepNode(dto.getPNode().getStepNode());
            pbdto.setSaveState(dto.getPNode().getSaveState());
            pbdto.setProcessNumber(processL.getProcessNumber());
            pbdto.setConfigId(processL.getConfigId());
            pbdto.setEdition(processL.getEdition());
            pbdto.setRelationId(processL.getRelationId());
            pbdto.setNodeId(pn.getId());
            pbdto.setStartTime(pn.getStartTime());
            pbdto.setEndTime(pn.getEndTime());
            pbdto.setRegisterDept(processL.getRegisterDept());
            pbdto.setRegisterPost(processL.getRegisterPost());
            pbdto.setRegisterStaff(processL.getRegisterStaff());
            pbdto.setRegisterTime(processL.getRegisterTime());
            for (Map.Entry<String, Object> e : objs.entrySet()) {
                BaseService bs = bss.get(e.getKey());
                Object obj = bs.saveData(e.getValue(), files, pbdto);
                if (obj instanceof String) {
                    throw new RuntimeException(obj.toString());
                }
            }
        }
        return true;
    }


    /**
     * 根据ID获取流程数据
     * @param id id
     * @return 流程数据
     */
    @Override
    public ProcessVo getTfProcessLById(Long id) {
        ProcessVo vo = this.baseMapper.getProcessById(id);
        if(vo==null){
            return vo;
        }
        Long configId = vo.getConfigId();
        Integer edition = vo.getEdition();
        Map<String,ServiceImplVo> map =pControlService.getServiceImplByConfig(configId,edition).stream().collect(Collectors.toMap(ServiceImplVo::getNode,r->r));

        if(CollUtil.isNotEmpty(vo.getNodes())&&CollUtil.isNotEmpty(map)){
            for (ProcessNodeVo pn:vo.getNodes()) {
                ServiceImplVo si = map.get(pn.getProcessStep()+"_"+pn.getStepNode());
                if(StringUtils.isNotEmpty(si.getServiceImpls())){
                    Map<String,Object> dtmap = new HashMap<>();
                    for (String str:si.getServiceImpls().split(FlowConstant.CSV)) {
                        BaseService bs = SpringUtil.getBean(str);
                        dtmap.put(str,bs.getBaseDatas(pn.getProcessId(),pn.getId(),vo.getTenantId()));
                    }
                    pn.setDataMap(dtmap);
                }
            }
            Integer nodeType = this.baseMapper.getProcessNodeType(id);
            if (nodeType!=null && nodeType == 3){
                ProcessNodeVo processNodes = tfProcessNodeLService.getProcessNodes(id);
                processNodes.setIsValid((short)1);
                if(vo.getNodes().get(0)!=null &&vo.getNodes().get(0).getId()>processNodes.getId()){

                }else{
                    vo.getNodes().set(0, processNodes);
                }

            }
        }
        List<ProcessNodeVo> nodes = vo.getNodes();
        if (CollUtil.isEmpty(nodes)) {
            return vo;
        }
        if ((StringUtils.isNotBlank(nodes.get(0).getPageCode())
                && !nodes.get(0).getPageCode().equals("contractCheck_addProject")
                && !nodes.get(0).getPageCode().equals("designBudget_addBudget"))
                || StringUtils.isBlank(nodes.get(0).getPageCode())) {
            return vo;
        }
        if (nodes.get(0).getPageCode().equals("contractCheck_addProject")
                && CollUtil.isEmpty((List<TfContractW>) nodes.get(0).getDataMap().get("tfContractWService"))) {
            List<ProcessNodeVo> midNodes = new ArrayList<>();
            nodes.forEach(node -> {
                if (StringUtils.isNotEmpty(node.getPageCode())
                        && node.getPageCode().equals("contractCheck_addProject")
                        && node.getIsValid() != null
                        && node.getIsValid() == (short) 0) {
                    midNodes.add(node);
                }
            });
            if (CollUtil.isNotEmpty(midNodes)) {
                ProcessNodeVo resNode = midNodes.get(0);
                resNode.setId(null);
                resNode.setIsValid((short) 1);
                resNode.setIsValidName("有效");
                nodes.set(0, resNode);
            }
            vo.setNodes(nodes);
            return vo;
        } else if (nodes.get(0).getPageCode().equals("designBudget_addBudget")
                && CollUtil.isEmpty((List<TfDesignBudgetW>) nodes.get(0).getDataMap().get("tfDesignBudgetWService"))) {
            List<ProcessNodeVo> midNodes = new ArrayList<>();
            nodes.forEach(node -> {
                if (StringUtils.isNotEmpty(node.getPageCode())
                        && node.getPageCode().equals("designBudget_addBudget")
                        && node.getIsValid() != null
                        && node.getIsValid() == (short) 0) {
                    midNodes.add(node);
                }
            });
            if (CollUtil.isNotEmpty(midNodes)) {
                ProcessNodeVo resNode = midNodes.get(0);
                resNode.setId(null);
                resNode.setIsValid((short) 1);
                resNode.setIsValidName("有效");
                nodes.set(0, resNode);
            }
            vo.setNodes(nodes);
            return vo;
        } else {
            return vo;
        }
    }

    /**
     * 根据ID获取流程数据
     * @return 流程数据
     */
    @Override
    public Map<Long, List<ProcessNodeVo>> getProcessNodeList(List<Long> processIdList) {
        List<ProcessNodeVo> processNodeVoList = this.baseMapper.getProcessNodeList(processIdList);
        Map<Long, List<ProcessNodeVo>> processVoMap = processNodeVoList.stream().collect(Collectors.groupingBy(ProcessNodeVo::getProcessId));
//        List<ServiceImplVo> serviceImplVoList = pControlService.getServiceImplByConfig(null, null);
//        processVoMap.forEach((k,v) -> {
//            Map<String,ServiceImplVo> map = serviceImplVoList.stream()
//                    .filter(e -> Objects.equals(e.getConfigId(), v.get(0).getConfigId()) && Objects.equals(e.getEdition(), v.get(0).getEdition()))
//                    .collect(Collectors.toMap(ServiceImplVo::getNode,r->r));
//            if(CollUtil.isEmpty(map)) {
//                return;
//            }
//            for (ProcessNodeVo pn:v) {
//                ServiceImplVo si = map.get(pn.getProcessStep() + "_" + pn.getStepNode());
//                if (StringUtils.isEmpty(si.getServiceImpls())) {
//                    continue;
//                }
//                Map<String,Object> dtmap = new HashMap<>();
//                for (String str:si.getServiceImpls().split(FlowConstant.CSV)) {
//                    BaseService bs = SpringUtil.getBean(str);
//                    dtmap.put(str,bs.getBaseDatas(pn.getProcessId(),pn.getId(),v.get(0).getTenantId()));
//                }
//                pn.setDataMap(dtmap);
//            }
//        });
        return processVoMap;
    }


    /**
     * 获取附属流程
     * @param page
     * @param dto 查询条件
     * @return 流程集合
     */
    @Override
    public Page<ProcessVo> getSubsidiaryProcess(Page page, FsProcessDto dto) {
        Page<ProcessVo> vos = this.baseMapper.getFsProcess(page,dto);
        if(CollUtil.isNotEmpty(vos.getRecords())){
            Map<String,Map<String,ServiceImplVo>> cmap = new HashMap();
            for (ProcessVo vo: vos.getRecords()) {
                Long configId = vo.getConfigId();
                Integer edition = vo.getEdition();
                String ckey = configId+"_"+edition;
                Map<String,ServiceImplVo> map = null;
                if(cmap.containsKey(ckey)){
                    map = cmap.get(ckey);
                }else {
                    map =pControlService.getServiceImplByConfig(configId,edition).stream().collect(Collectors.toMap(ServiceImplVo::getNode,r->r));
                    cmap.put(ckey,map);
                }
                if(CollUtil.isNotEmpty(vo.getNodes())&&CollUtil.isNotEmpty(map)){
                    for (ProcessNodeVo pn:vo.getNodes()) {
                        ServiceImplVo si = map.get(pn.getProcessStep()+"_"+pn.getStepNode());
                        if(StringUtils.isNotEmpty(si.getServiceImpls())){
                            Map<String,Object> dtmap = new HashMap<>();
                            for (String str:si.getServiceImpls().split(FlowConstant.CSV)) {
                                BaseService bs = SpringUtil.getBean(str);
                                dtmap.put(str,bs.getBaseDatas(pn.getProcessId(),pn.getId(),vo.getTenantId()));
                            }
                            pn.setDataMap(dtmap);
                        }
                    }
                }
            }
        }
        return vos;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean removeProcessById(Long id) {
        Boolean plb = this.removeById(id);
        if(plb) {
            nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, id));
            tfTaskPService.update(new UpdateWrapper<TfTaskP>().lambda().eq(TfTaskP::getProcessId, id).set(TfTaskP::getStatus, FlowConstant.SHORT_ONE)
                    .set(TfTaskP::getIsDel, FlowConstant.SHORT_ONE));

            List<Long> ids = this.listObjs(new QueryWrapper<TfProcessL>().lambda().select(TfProcessL::getId).eq(TfProcessL::getRelationId,id)
                            .eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO)
                            .and(r->r.eq(TfProcessL::getProcessState,FlowConstant.SHORT_ONE).or().eq(TfProcessL::getProcessState,FlowConstant.SHORT_TWO))
                    ,r->Long.parseLong(String.valueOf(r)));
            if (CollUtil.isNotEmpty(ids)){
                this.update(new UpdateWrapper<TfProcessL>().lambda().in(TfProcessL::getId,ids).set(TfProcessL::getProcessState,FlowConstant.SHORT_THREE)
                        .set(TfProcessL::getIsDel,FlowConstant.SHORT_ONE));
                nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().in(TfNowHandleL::getProcessId,ids));
                tfTaskPService.update(new UpdateWrapper<TfTaskP>().lambda().in(TfTaskP::getProcessId,ids).eq(TfTaskP::getStatus,FlowConstant.SHORT_ZERO)
                        .set(TfTaskP::getStatus,FlowConstant.SHORT_ONE).set(TfTaskP::getIsDel,FlowConstant.SHORT_ONE));
            }

        }
        return plb;
    }

    /**
     * 根据附属流程ID获取工程相关出入库相关信息（第一步骤）
     * @param id 流程Id
     * @return
     */
    @Override
    public ProcessMaterialVo getMaterialNodeById(long id) {
        return this.baseMapper.getMaterialNodeById(id);
    }

    @Override
    public List<Long> getProcessIdWithOutPush() {
        return baseMapper.getProcessIdWithOutPush();
    }

    /**
     * 终止流程
     * @param id 终止申请流程id
     */
    @Override
    public void stopProcess(Long id) {
        TfProcessL pl = this.getById(id);
        if(pl!=null&&pl.getRelationId()!=null&&FlowConstant.SHORT_FOUR==pl.getProcessState() ){
            int count = this.count(new QueryWrapper<TfProcessL>().lambda().eq(TfProcessL::getId,pl.getRelationId())
                    .eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO).and(r->r.eq(TfProcessL::getProcessState,FlowConstant.SHORT_ONE).or()
                            .eq(TfProcessL::getProcessState,FlowConstant.SHORT_TWO)));
            if(count==FlowConstant.INTEGER_ONE){
                this.update(new UpdateWrapper<TfProcessL>().lambda().eq(TfProcessL::getId,pl.getRelationId()).set(TfProcessL::getProcessState,FlowConstant.SHORT_THREE));
                nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId,pl.getRelationId()));
                tfTaskPService.update(new UpdateWrapper<TfTaskP>().lambda().eq(TfTaskP::getProcessId,pl.getRelationId()).eq(TfTaskP::getStatus,FlowConstant.SHORT_ZERO)
                        .set(TfTaskP::getStatus,FlowConstant.SHORT_ONE));
                List<Long> ids = this.listObjs(new QueryWrapper<TfProcessL>().lambda().select(TfProcessL::getId).eq(TfProcessL::getRelationId,pl.getRelationId())
                                .eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO)
                                .and(r->r.eq(TfProcessL::getProcessState,FlowConstant.SHORT_ONE).or().eq(TfProcessL::getProcessState,FlowConstant.SHORT_TWO))
                        ,r->Long.parseLong(String.valueOf(r)));
                if (CollUtil.isNotEmpty(ids)){
                    this.update(new UpdateWrapper<TfProcessL>().lambda().in(TfProcessL::getId,ids).set(TfProcessL::getProcessState,FlowConstant.SHORT_THREE));
                    nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().in(TfNowHandleL::getProcessId,ids));
                    tfTaskPService.update(new UpdateWrapper<TfTaskP>().lambda().in(TfTaskP::getProcessId,ids).eq(TfTaskP::getStatus,FlowConstant.SHORT_ZERO)
                            .set(TfTaskP::getStatus,FlowConstant.SHORT_ONE));
                }
            }
        }

    }

    @Override
    public Page<TfProcessL> pageProcess(Page page, PageProcessDto dto) {
        TfProcessL process = new TfProcessL();
        BeanUtils.copyProperties(dto, process);
        Page<TfProcessL> pg= this.page(page, Wrappers.query(process).lambda().eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO)
                .ge(dto.getStartTime()!=null,TfProcessL::getRegisterTime,dto.getStartTime())
                .le(dto.getEndTime()!=null,TfProcessL::getRegisterTime,dto.getEndTime()).and(StringUtils.isNotEmpty(dto.getContent()),
                        r->r.like(TfProcessL::getProcessNumber,dto.getContent()).or().like(TfProcessL::getName,dto.getContent())));
        return pg;
    }


    /**
     * 保存流程
     * @param dto 流程信息
     * @return 保存对象
     */
    private TfProcessL saveProcess(ProcessDto dto) {
        TfProcessL process = new TfProcessL();
        BeanUtils.copyProperties(dto, process);
        String relationNumber = null;

        if(dto.getRelationId()!=null){
            //如果关联编码ID不为空，根据关联编码ID查询流程编号
            // todo  数据库所有的relationNumber都为null
            relationNumber = this.getObj(new QueryWrapper<TfProcessL>().lambda().eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO)
                            .eq(TfProcessL::getId, dto.getRelationId()).select(TfProcessL::getRelationNumber)
                    ,r->String.valueOf(r));
            process.setRelationNumber(relationNumber);
        }
        if(process.getId()==null){
//            如果流程的id为空
//            根据TF_PROCESS_CONFIG_L的id查询CTypeVo（流程类型，业务类型，流程编码）
            CTypeVo cTypeVo= pConfigService.getConfigType(dto.getConfigId());
//            设置流程编号
            process.setProcessNumber(getProcessNumber(cTypeVo.getBusinessType(),relationNumber,cTypeVo.getProcessCode()));
            process.setIsDel(FlowConstant.SHORT_ZERO);
            process.setCategory(cTypeVo.getProcessType());
            process.setProcessType(cTypeVo.getBusinessType());
            process.setEdition(cTypeVo.getEdition());
//            新增的流程默认状态为草稿
            process.setProcessState(FlowConstant.SHORT_ONE);
            process.setTenantId(SecurityUtils.getTenantId());
//            todo 新增的process为什么需要判断==null，应该覆盖前端传的值
            if(process.getRegisterStaff()==null) process.setRegisterStaff(SecurityUtils.getUserId());
            if(process.getRegisterDept()==null) process.setRegisterDept(CollUtil.isEmpty(SecurityUtils.getUserDeptIds())?null:SecurityUtils.getUserDeptIds().get(0));
            if(process.getRegisterPost()==null)  process.setRegisterPost(CollUtil.isEmpty(SecurityUtils.getUserPostIds())?null:SecurityUtils.getUserPostIds().get(0));
            if(process.getRegisterTime()==null) process.setRegisterTime(new Date());
        } else {
//            如果流程id不为空
            TfProcessL oProcess = this.getById(dto.getId());
            if(oProcess.getRelationId()!=process.getRelationId()){
                CTypeVo cTypeVo= pConfigService.getConfigType(dto.getConfigId());
                process.setProcessNumber(getProcessNumber(cTypeVo.getBusinessType(),relationNumber,cTypeVo.getProcessCode()));
            }
        }
//       todo 事务失效，因该将保存和更新分开
        this.saveOrUpdate(process);
        return this.getById(process.getId());
    }

    /**
     * 流程上报校验
     * @param dto 上报信息
     * @param files 文件集合
     * @return 校验结果
     */
    private String processSaveCheck(ProcessDto dto, List<MultipartFile> files){
        if(CollUtil.isNotEmpty(files)){
            Set<String> names = files.stream().map(MultipartFile::getOriginalFilename).collect(Collectors.toSet());
            if(files.size()!=names.size()){
                return FlowConstant.FILE_NAME_REPEAT;
            }
        }
        Integer edition = null;
        TfProcessL processL=null;
        if(dto.getId()!=null){
            processL = this.getOne(new QueryWrapper<TfProcessL>().lambda().eq(TfProcessL::getId,dto.getId()).eq(TfProcessL::getConfigId,dto.getConfigId())
                    .eq(TfProcessL::getIsDel, FlowConstant.SHORT_ZERO).and(r->r.eq(TfProcessL::getProcessState,FlowConstant.SHORT_ONE).or()
                            .eq(TfProcessL::getProcessState,FlowConstant.SHORT_TWO)));
            if(processL == null){
                return FlowConstant.WORK_ORDER_PROCESS;
            }else {
                edition = processL.getEdition();
            }
        }else {
            edition = pConfigService.getObj(new QueryWrapper<TfProcessConfigL>().lambda().eq(TfProcessConfigL::getId,dto.getConfigId())
                    .eq(TfProcessConfigL::getTenantId, SecurityUtils.getTenantId()).eq(TfProcessConfigL::getIsDel,FlowConstant.SHORT_ZERO)
                    .eq(TfProcessConfigL::getIsMould,FlowConstant.SHORT_ZERO).eq(TfProcessConfigL::getIsValid,FlowConstant.SHORT_ONE)
                    .select(TfProcessConfigL::getEdition),r-> r==null?null:Integer.valueOf(r.toString()));
            if(edition == null){
                return FlowConstant.CONFIG_NOT_EXIST;
            }
        }

        /**流程名称校验*/
        int count = this.count(new QueryWrapper<TfProcessL>().lambda().eq(TfProcessL::getConfigId,dto.getConfigId()).eq(TfProcessL::getName,dto.getName())
                .eq(TfProcessL::getTenantId,SecurityUtils.getTenantId()).eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO)
                .ne(dto.getId()!=null,TfProcessL::getId,dto.getId()));
        if(count != FlowConstant.SHORT_ZERO){
            return FlowConstant.PROCESS_NAME_REPEAT;
        }

        /**判断关联流程*/
        if(dto.getRelationId()!=null){
            count = this.count(new QueryWrapper<TfProcessL>().lambda().eq(TfProcessL::getId,dto.getRelationId())
                    .eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO));
            if(count==FlowConstant.SHORT_ZERO){
                return FlowConstant.WORK_ORDER_PROCESS;
            }
        }
        String check = pNodeLService.nodeSaveCheck(processL,dto.getPNode(),files,edition,dto.getId(),dto.getConfigId());
        if(StringUtils.isNotEmpty(check)){
            return check;
        }

        return null;
    }



    /**
     * 获取编码
     * @param processType 流程配置类型
     * @param rnumber 关联number
     * @param code 流程code
     * @return
     */
    private String getProcessNumber(Long processType, String rnumber, String code){
        if (StringUtils.isEmpty(code)) {
            code = "GDLC";
        }
        code = code.toLowerCase();//转小写
        if(StringUtils.isNotEmpty(rnumber)){
            String key =FlowConstant.PROCESS_CONFIG_REDIS_KEY +  rnumber+code;
            RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
            Long value = counter.incrementAndGet();
            if(value == 1) {
                String number = this.getObj(new QueryWrapper<TfProcessL>().select("MAX(PROCESS_NUMBER)").lambda().eq(TfProcessL::getRelationNumber, rnumber)
                        .eq(TfProcessL::getIsDel, FlowConstant.SHORT_ZERO), (o) -> {
                    return String.valueOf(o);
                });
                if(StringUtils.isNotEmpty(number)){
                    Long num = Long.parseLong(number.substring(number.length() - 2));
                    counter.set(num);
                    counter.expire(FlowConstant.INTEGER_TWENTY_FOUR, TimeUnit.HOURS);
                    value = counter.incrementAndGet();
                }
            }
            return rnumber+code + String.format("%02d", value);
        }else {
            String key = FlowConstant.PROCESS_CONFIG_REDIS_KEY + code + DateUtil.format(new Date(), "yyyyMMdd");
            RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());
            Long value = counter.incrementAndGet();
            if (value == 1) {
                String number = this.baseMapper.getProcessNumber(DateUtil.beginOfDay(new Date()), processType);
                if (StringUtils.isNotEmpty(number)) {
                    Long num = Long.parseLong(number.substring(number.length() - 5));
                    counter.set(num);
                    long betweenDay = DateUtil.between(new Date(), DateUtil.endOfDay(new Date()), DateUnit.MINUTE);
                    counter.expire(betweenDay, TimeUnit.MINUTES);
                    value = counter.incrementAndGet();

                }
            }
            return code + DateUtil.format(new Date(), "yyyyMMdd") + String.format("%04d", value);
        }
    }
}
