/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.process.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.Constant;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.entity.dto.MessageDto;
import com.tofly.common.entity.vo.FileInfo;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.common.utils.POIUtil;
import com.tofly.workflow.baseconf.entity.vo.CalendarVo;
import com.tofly.workflow.baseconf.entity.vo.TaskVo;
import com.tofly.workflow.baseconf.service.TfCalendarPService;
import com.tofly.workflow.buss.entity.TfContractW;
import com.tofly.workflow.buss.service.TfContractWService;
import com.tofly.workflow.buss.service.impl.TfContractWServiceImpl;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.TfNowHandleL;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.TfProcessUpperNodeL;
import com.tofly.workflow.process.entity.dto.*;
import com.tofly.workflow.process.entity.vo.NodeListVo;
import com.tofly.workflow.process.entity.vo.ProcessNodeVo;
import com.tofly.workflow.process.entity.vo.ProcessVo;
import com.tofly.workflow.process.mapper.TfProcessLMapper;
import com.tofly.workflow.process.service.*;
import com.tofly.workflow.processconfig.entity.*;
import com.tofly.workflow.processconfig.entity.vo.ServiceImplVo;
import com.tofly.workflow.processconfig.service.*;
import com.tofly.workflow.utils.NodeLFillUtil;
import com.tofly.workflow.utils.SendTidingsUtil;
import com.tofly.workflow.utils.WorkDayCalculateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import com.tofly.workflow.process.mapper.TfProcessNodeLMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.lang.reflect.Field;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 流程节点处理信息（操作步骤都有记录）
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Slf4j
@Service("tfProcessNodeLService")
public class TfProcessNodeLServiceImpl extends ServiceImpl<TfProcessNodeLMapper, TfProcessNodeL> implements TfProcessNodeLService {
    @Value("${zmrq.flow.tempFile}")
    private String tempFilePath;
    @Autowired
    private TfContractWService tfContractWService;

    /**
     * 流程信息service
     */
    @Autowired
    private TfProcessLService processService;

    /**
     * 流程当前处理service
     */
    @Autowired
    private TfNowHandleLService nHandleService;

    /**
     * 流程配置service
     */
    @Autowired
    private TfProcessConfigLService pConfigService;

    /**
     * 流程步骤service
     */
    @Autowired
    private TfProcessConfigStepLService pCStepService;

    /**
     * 流程步骤处理人员
     */
    @Autowired
    private TfConfigStepUserLService csuserService;

    /**
     * 流程配置信息service
     */
    @Autowired
    private TfProcessConfigInfoLService pCInfoService;

    /**
     * 流程节点上下点关系service
     */
    @Autowired
    private TfProcessUpperNodeLService pUNodeService;

    /**
     * 工作日应用日历
     */
    @Autowired
    private TfCalendarPService calendarService;

    /**
     * 发送消息
     */
    @Autowired
    private SendTidingsUtil sendTidingsUtil;


    /**
     * 页面空间展示
     */
    @Autowired
    private TfPageControlLService pControlService;

    /**
     * 流程指定的历史步骤
     */
    @Autowired
    private TfLinkHistoryStepLService lhStepService;

    /**
     * 填充测试工具
     */
    @Autowired
    private NodeLFillUtil nodeLFillUtil;

    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private TfProcessLMapper tfProcessLMapper;

    /**
     * 流程节点数据校验
     *
     * @param processL
     * @param pdto     流程信息
     * @param files    文件集合
     * @param edition  版本
     * @return 校验结果
     */
    @Override
    public String nodeSaveCheck(TfProcessL processL, PNodeDto pdto, List<MultipartFile> files, Integer edition, Long processId, Long configId) {


        if (pdto == null) {
            return FlowConstant.STEP_NODE_DATE_ERROR;
        }
        if (pdto.getSaveState() == FlowConstant.INTEGER_ZERO) {
            //如果为保存 则不验证
            return null;
        }

        /**文件校验*/
        String fileNonExistent = nodeLFillUtil.checkNodeFile(files, pdto);
        if (fileNonExistent != null) return fileNonExistent;
        /**流程本步骤校验*/
        String check = getThisStepCheck(processId, pdto);
        if (check != null) return check;
        /**流程配置步骤*/
        TfProcessConfigStepL pcs = pCStepService.getOne(new QueryWrapper<TfProcessConfigStepL>().lambda().eq(TfProcessConfigStepL::getConfigId, configId)
                .eq(TfProcessConfigStepL::getEdition, edition).eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO)
                .eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE).eq(TfProcessConfigStepL::getStepOrder, pdto.getProcessStep()));
        if (pcs == null) {
            return FlowConstant.CONFIG_STEP_ERROR;
        }
        if (!pcs.getHandleModel().contains(pdto.getHandleState().toString())) {
            return FlowConstant.NO_PROCESSING_PERMISSION;
        }
        if (pdto.getProcessStep() != FlowConstant.INTEGER_ONE && pdto.getStepNode() == FlowConstant.INTEGER_ONE) {
            /**处理校验流程并行合并是否满足合并条件*/
            String[] steps = pcs.getUpperStep().split("\\|");
            sign:
            for (String step : steps) {
                String[] strs = step.split(",");
                if (strs.length > FlowConstant.INTEGER_ONE) {
                    List<Integer> list = Arrays.asList(strs).stream().filter(r -> StringUtils.isNotEmpty(r)).map(r -> Integer.parseInt(r)).collect(Collectors.toList());
                    List<Integer> uIds = pUNodeService.getUpStepNumber(processId, pdto.getProcessStep(), pdto.getStepNode());
                    if (CollUtil.isNotEmpty(uIds)) {
                        for (Integer uid : uIds) {
                            if (!list.contains(uid)) {
                                continue sign;
                            }
                        }
                    }
                    List<String> names = nHandleService.getStepName(processId, list);
                    if (CollUtil.isNotEmpty(names)) {
                        return "请先处理" + names.stream().collect(Collectors.toList());
                    }
                }
            }
        }
        /**保存并且下一步为空不需要校验下一步*/
        if ((CollUtil.isEmpty(pdto.getNextSteps()) && (pdto.getSaveState() == FlowConstant.INTEGER_ZERO
                /**下一步为空并且处理人不唯一并且不是本步骤最后一个节点*/
                || (pcs.getIsAppointUser() == FlowConstant.SHORT_ZERO && pcs.getNodeNumber() > pdto.getStepNode()))) ||
                /**操作为驳回和终止*/
                pdto.getHandleState() == FlowConstant.INTEGER_THREE || pdto.getHandleState() == FlowConstant.INTEGER_FOUR) {
            return null;
        }
        /**转派 转派人员不能为空并且唯一*/
        if (pdto.getHandleState() == FlowConstant.INTEGER_TWO && CollUtil.isNotEmpty(pdto.getNextSteps()) && pdto.getNextSteps().size() == FlowConstant.INTEGER_ONE) {
            return FlowConstant.TRANSFER_USER_NULL;
        }
        Map<Integer, TfProcessConfigStepL> map = pCStepService.list(new QueryWrapper<TfProcessConfigStepL>().lambda().eq(TfProcessConfigStepL::getConfigId, configId)
                        .eq(TfProcessConfigStepL::getEdition, edition).eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO)
                        .eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE).in(TfProcessConfigStepL::getStepOrder, pcs.getNextStep().split(FlowConstant.CSV)))
                .stream().collect(Collectors.toMap(TfProcessConfigStepL::getStepOrder, r -> r));
        /**校验下一步骤、人员*/
        check = getNextStepCheck(processL, pdto, pcs, map);
        if (check != null) return check;

        /**流程提交信息校验*/
        check = nodeLFillUtil.saveStateCheck(pdto, pcs, map);
        if (check != null) return check;
        return null;
    }


    /**
     * 保存步骤信息
     *
     * @param processL 流程信息
     * @param pNode    步骤信息
     * @param files    文件集合
     * @return 保存后信息
     */
    @Override
    public TfProcessNodeL nodeSave(TfProcessL processL, PNodeDto pNode, List<MultipartFile> files) {
        TfProcessNodeL pn = saveNode(processL, pNode, files);
        return pn;
    }

    /**
     * 保存流程步骤信息
     *
     * @param dto   流程步骤信息
     * @param files 文件
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object processNodeSave(PNodeSaveDto dto, List<MultipartFile> files) {
        /**获取上传文件的文件名称*/
        if (CollUtil.isNotEmpty(files)) {
            Set<String> names = files.stream().map(MultipartFile::getOriginalFilename).collect(Collectors.toSet());
            if (files.size() != names.size()) {
                return FlowConstant.FILE_NAME_REPEAT;
            }
        }

        TfProcessL pl = processService.getById(dto.getProcessId());
        if (pl == null) {
            return FlowConstant.WORK_ORDER_PROCESS;
        }
        //流程终止或完成
        if (FlowConstant.SHORT_THREE.equals(pl.getProcessState()) || FlowConstant.SHORT_FOUR.equals(pl.getProcessState())) {
            return "无法操作，流程已经终止或完成！";
        }

        /**修改本步骤节点数据*/
        if (dto.getId() != null) {
            int count = this.count(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getId, dto.getId()).eq(TfProcessNodeL::getProcessStep, dto.getProcessStep())
                    .eq(TfProcessNodeL::getStepNode, dto.getStepNode()).eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ZERO));
            if (count == FlowConstant.SHORT_ZERO) {
                return FlowConstant.NEXT_WRONG;
            }
        }
        String check = nodeSaveCheck(pl, dto, files, pl.getEdition(), dto.getProcessId(), pl.getConfigId());
        if (StringUtils.isNotEmpty(check)) {
            return check;
        }
        Map<String, BaseService> bss = new HashMap<>();
        Map<String, Object> objs = new HashMap<>();
        if (dto.getHandleState() == FlowConstant.INTEGER_ONE) {
            Map<String, String> map = dto.getMap();
            if (CollUtil.isNotEmpty(map)) {
                for (Map.Entry<String, String> e : map.entrySet()) {
                    BaseService bs = SpringUtil.getBean(e.getKey());
                    if (bs == null) {
                        return e.getKey() + FlowConstant.BUSINESS_DOES_NOT_EXIST;
                    }
                    bss.put(e.getKey(), bs);
                    /**业务数据对象转换 json 转对象*/
                    Object obj = bs.getJsonToObject(e.getValue());
                    if (obj == null) {
                        obj = e.getValue();
                    }
                    /**业务数据校验*/
                    check = bs.saveDataCheck(obj, files);
                    if (StringUtils.isNotEmpty(check)) {
                        return check;
                    }
                    objs.put(e.getKey(), obj);
                }
            }
        }
        TfProcessNodeL pn = nodeSave(pl, dto, files);
        /**保存其它业务数据*/
        if (dto.getHandleState() == FlowConstant.INTEGER_ONE) {
            nodeLFillUtil.saveWorkData(dto, files, pl, bss, objs, pn);
        }
        return true;
    }

    /**
     * 修改流程步骤负责人
     *
     * @param dto 流程步骤修改数据
     * @return 修改结果
     */
    @Override
    public Object processNodeUpdate(PNodeUpdateDto dto) {
        TfProcessL process = processService.getOne(new QueryWrapper<TfProcessL>().lambda().eq(TfProcessL::getId, dto.getProcessId())
                .eq(TfProcessL::getIsDel, FlowConstant.SHORT_ZERO).eq(TfProcessL::getProcessState, FlowConstant.SHORT_TWO));
        if (process == null) {
            return FlowConstant.WORK_ORDER_PROCESS;
        }
        /**下一步节点*/
        TfProcessNodeL pn = this.getOne(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, dto.getProcessId())
                .eq(TfProcessNodeL::getProcessStep, dto.getProcessStep()).eq(TfProcessNodeL::getStepNode, dto.getStepNode())
                .eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE));
        if (pn == null) {
            return FlowConstant.NEXT_WRONG;
        }

        if (!pn.getId().equals(dto.getNodeId())) {
            return FlowConstant.ID_STEP_NOT_MATCH;
        }

        List<Long> nodeids = pUNodeService.getLikeNode(pn.getProcessId(), FlowConstant.INTEGER_ONE, pn.getId()).stream().map(r -> r.getNodeId()).collect(Collectors.toList());
        nodeids.add(pn.getId());
        this.update(new UpdateWrapper<TfProcessNodeL>().lambda().in(TfProcessNodeL::getId, nodeids).set(TfProcessNodeL::getDutyUser, dto.getDutyUser())
                .set(TfProcessNodeL::getDutyPost, dto.getDutyPost()).set(TfProcessNodeL::getDutyDept, dto.getDutyDept()).eq(TfProcessNodeL::getDutyUser, dto.getBillDutyUser())
                .eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE));

        //修改历史节点步骤中的负责人信息
        this.baseMapper.updateNodeDutyUser(dto.getDutyPost(), dto.getDutyDept(), dto.getDutyUser(), dto.getProcessId(), dto.getBillDutyUser());
        //替换更改当前处理步骤中的负责人信息
        nHandleService.updateHandleDutyUser(dto.getDutyPost(), dto.getDutyDept(), dto.getDutyUser(), dto.getProcessId(), dto.getBillDutyUser());

        return true;
    }

    /**
     * 根据流程、步骤、节点获取步骤信息
     *
     * @param id          流程Id
     * @param processStep 步骤序号
     * @param stepNode    节点序号
     * @return
     */
    @Override
    public ProcessNodeVo getNodeByProcessIdAndNode(Long id, Integer processStep, Integer stepNode) {
        TfProcessL processL = processService.getById(id);
        ServiceImplVo vo = pControlService.getNodeService(processL.getConfigId(), processL.getEdition(), processStep, stepNode);
        TfProcessNodeL pn = this.getOne(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, id)
                .eq(TfProcessNodeL::getProcessStep, processStep).eq(TfProcessNodeL::getStepNode, stepNode)
                .eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE).eq(TfProcessNodeL::getIsDel, FlowConstant.SHORT_ZERO));
        if (pn == null) {
            return null;
        }
        ProcessNodeVo pnvo = new ProcessNodeVo();
        BeanUtils.copyProperties(pn, pnvo);
        if (StringUtils.isNotEmpty(vo.getServiceImpls())) {
            Map<String, Object> dtmap = new HashMap<>();
            for (String str : vo.getServiceImpls().split(FlowConstant.CSV)) {
                BaseService bs = SpringUtil.getBean(str);
                dtmap.put(str, bs.getBaseDatas(pn.getProcessId(), pn.getId(), processL.getTenantId()));
            }
            pnvo.setDataMap(dtmap);
        }
        return pnvo;
    }

    /**
     * 获取节点列表
     *
     * @param label     标签
     * @param processId 流程ID
     * @return
     */
    @Override
    public List<NodeListVo> getNodeList(Long label, Long processId) {
        return this.baseMapper.getNodeList(label, processId);
    }

    /**
     * 流程步骤节点
     *
     * @param processId 流程id
     * @param nodeId    步骤节点
     * @return 步骤数据
     */
    @Override
    public TfProcessNodeL getNextProcessNode(Long processId, Long nodeId) {
        return this.baseMapper.getNextProcessNode(processId, nodeId);
    }

    @Override
    public ProcessNodeVo getProcessNodes(Long processId) {
        ProcessVo vo = tfProcessLMapper.getProcessById(processId);
        Long configId = vo.getConfigId();
        Integer edition = vo.getEdition();
        Map<String, ServiceImplVo> map = pControlService.getServiceImplByConfig(configId, edition).stream().collect(Collectors.toMap(ServiceImplVo::getNode, r -> r));
        ProcessNodeVo pn = this.baseMapper.getProcessNodesForReject(processId);
        ServiceImplVo si = map.get(pn.getProcessStep() + "_" + pn.getStepNode());
        if (StringUtils.isNotEmpty(si.getServiceImpls())) {
            Map<String, Object> dtmap = new HashMap<>();
            for (String str : si.getServiceImpls().split(FlowConstant.CSV)) {
                BaseService bs = SpringUtil.getBean(str);
                Object baseDatas = bs.getBaseDatas(pn.getProcessId(), pn.getId(), vo.getTenantId());
                if (baseDatas instanceof List) {
                    for (Object baseData : (List<Object>) baseDatas) {
                        if (ReflectUtil.getField(baseData.getClass(), "id") != null) {
                            ReflectUtil.setFieldValue(baseData, "id", null);
                        }
                    }
                } else {
                    if (ReflectUtil.getField(baseDatas.getClass(), "id") != null) {
                        ReflectUtil.setFieldValue(baseDatas, "id", null);
                    }
                }

                dtmap.put(str, baseDatas);
            }
            pn.setDataMap(dtmap);
        }
        return pn;
    }

    /**
     * 保存步骤信息
     *
     * @param processL 流程ID
     * @param pNode    步骤信息
     * @param files    文件集合
     * @return 保存后信息
     */
    private TfProcessNodeL saveNode(TfProcessL processL, PNodeDto pNode, List<MultipartFile> files) {
        // 根据流程ID和步骤信息查询是否存在相同步骤
        TfProcessNodeL stepNode = this.getOne(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, processL.getId())
                .eq(TfProcessNodeL::getProcessStep, pNode.getProcessStep()).eq(TfProcessNodeL::getStepNode, pNode.getStepNode())
                .eq(TfProcessNodeL::getIsDel, FlowConstant.SHORT_ZERO).eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE)
                .eq(TfProcessNodeL::getIsTemp, FlowConstant.SHORT_ZERO)
                .and(r -> r.eq(TfProcessNodeL::getNodeState, FlowConstant.SHORT_ZERO).or().eq(TfProcessNodeL::getNodeState, FlowConstant.SHORT_ONE)));

        Long deptId = CollUtil.isEmpty(SecurityUtils.getUserDeptIds()) ? null : SecurityUtils.getUserDeptIds().get(0);
        Long postId = CollUtil.isEmpty(SecurityUtils.getUserPostIds()) ? null : SecurityUtils.getUserPostIds().get(0);

        /**本步骤流程配置*/
        TfProcessConfigStepL pcs = pCStepService.getOne(new QueryWrapper<TfProcessConfigStepL>().lambda()
                .eq(TfProcessConfigStepL::getConfigId, processL.getConfigId())
                .eq(TfProcessConfigStepL::getStepOrder, pNode.getProcessStep())
                .eq(TfProcessConfigStepL::getEdition, processL.getEdition())
                .eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE)
                .eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO));
        /**填充流程步骤信息*/
        TfProcessNodeL processNode = nodeLFillUtil.getTfProcessNodeL(processL, pNode, files, stepNode, deptId, postId, pcs);
//页面空间展示
        TfPageControlL bpc = pControlService.getOne(new QueryWrapper<TfPageControlL>().lambda().eq(TfPageControlL::getStepId, pcs.getId())
                .eq(TfPageControlL::getStepNode, processNode.getStepNode()).eq(TfPageControlL::getIsDel, FlowConstant.SHORT_ZERO));
        processNode.setPageCode(bpc.getPageCode());
        /**保存或修改本步骤节点数据*/
        this.saveOrUpdate(processNode);

//     流程配置信息   TF_PROCESS_CONFIG_L
        TfProcessConfigL pc = pConfigService.getOne(new QueryWrapper<TfProcessConfigL>().lambda()
                .eq(TfProcessConfigL::getId, processL.getConfigId())
                .eq(TfProcessConfigL::getIsValid, FlowConstant.SHORT_ONE)
                .eq(TfProcessConfigL::getIsDel, FlowConstant.SHORT_ZERO));
        if (stepNode != null) {
//            如果当前步骤已经存在
            this.baseMapper.removeNextNode(processL.getId(), processNode.getId());
            /**删除下一步操作记录信息*/
            pUNodeService.remove(new QueryWrapper<TfProcessUpperNodeL>().lambda()
                    .eq(TfProcessUpperNodeL::getProcessId, processL.getId())
                    .eq(TfProcessUpperNodeL::getNodeId, processNode.getId())
                    .eq(TfProcessUpperNodeL::getType, FlowConstant.SHORT_ZERO));
            /**删除转派*/
            this.baseMapper.removeTransferNode(processL.getId(), processNode.getId());
            /**删除上一步操作记录信息*/
            pUNodeService.remove(new QueryWrapper<TfProcessUpperNodeL>().lambda().eq(TfProcessUpperNodeL::getProcessId, processL.getId())
                    .eq(TfProcessUpperNodeL::getHandleId, processNode.getId()).eq(TfProcessUpperNodeL::getType, FlowConstant.SHORT_ONE));
        }
        TfProcessL pl = new TfProcessL();

        /**下步骤终结*/
        String nextStep = pcs.getNextStep();
        if (pNode.getHandleState() == FlowConstant.INTEGER_ONE
                && (nextStep.contains(FlowConstant.STRING_MINUS_ONE) || nextStep.contains(FlowConstant.STRING_MINUS_TWO))
                && CollUtil.isEmpty(pNode.getNextSteps())
                && pNode.getStepNode() >= pcs.getCoerceLevel()) {
            //参数传递进来的pNode，如果当前节点的处理状态是通过，并且下一步为-1或-2，并且下一步为空，且当前步骤节点大于或等于配置的coerceLevel
            /**保存状态 0保存，1提交*/
            if (pNode.getSaveState() == FlowConstant.INTEGER_ZERO) {
                /**当前处理是保存信息，不是提交信息，保存流程当前处理信息*/
                saveNowHandle(processL, processNode, pcs, pc);
            } else {
                /**当前处理是提交信息*/
                pl.setId(processL.getId());
//                这三元表达式牛逼 设置已经通过的节点
                pl.setPassingNode(processL.getPassingNode() == null ?
                        new StringBuilder(pNode.getProcessStep()).append(FlowConstant.UNDERLINE).append(pNode.getStepNode()).toString()
                        : new StringBuilder(processL.getPassingNode()).append(FlowConstant.CSV).append(pNode.getProcessStep()).append(FlowConstant.UNDERLINE).append(pNode.getStepNode()).toString());
//统计 当前流程不包含当前节点的情况下已经处理过的节点
                int count = nHandleService.count(new QueryWrapper<TfNowHandleL>().lambda()
                        .eq(TfNowHandleL::getProcessId, processL.getId())
                        .ne(TfNowHandleL::getProcessStep, processNode.getProcessStep()));

                if ((count == FlowConstant.INTEGER_ZERO ||  (processNode.getProcessStep() == FlowConstant.INTEGER_ONE && processNode.getStepNode() == FlowConstant.INTEGER_ONE))
                        && (nextStep.contains(FlowConstant.STRING_MINUS_ONE) || nextStep.contains(FlowConstant.STRING_MINUS_TWO))) {

                    /**
                     *上面的条件判断真牛逼
                     * 条件1： 当前流程已经处理的节点为0 或者 （当前流程已经处理的节点为1，且当前节点为1，且下一步为1）
                     *条件2： 下一步为-1 或者 -2 （终止状态）
                     */
//同时满足上面两个条件  设置流程状态为4处理完成
                    pl.setProcessState(FlowConstant.SHORT_FOUR);

                }
                log.info("更新流程状态");
                processService.updateById(pl);
                /**下一步序号(多个逗号分隔; -1流程结束; -2分支终止)*/
                if (nextStep.contains(FlowConstant.STRING_MINUS_ONE)) {
                    nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                            .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep()));
                } else if (nextStep.contains(FlowConstant.STRING_MINUS_TWO)) {
                    nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                            .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep()).eq(TfNowHandleL::getStepNode, processNode.getStepNode()));
                }
                /**取消待办*/
                sendTidingsUtil.removeLocalTask(processL.getId(), FlowConstant.PROCESS_TABLE_NAME, pNode.getProcessStep(), pNode.getStepNode(), null);
            }
        } else {
            //!(参数传递进来的pNode，如果当前节点的处理状态是通过，并且下一步为-1或-2，并且下一步为空，且当前步骤节点大于或等于配置的coerceLevel)
            Map<Integer, TfProcessConfigStepL> map = null;
            if (pNode.getSaveState() == FlowConstant.INTEGER_ZERO) {
                if (CollUtil.isNotEmpty(pNode.getNextSteps())) {
                    Set<Integer> stepnodes = pNode.getNextSteps().stream().map(r -> r.getProcessStep()).collect(Collectors.toSet());
                    map = pCStepService.list(new QueryWrapper<TfProcessConfigStepL>().lambda()
                                    .eq(TfProcessConfigStepL::getConfigId, processL.getConfigId()).eq(TfProcessConfigStepL::getEdition, processL.getEdition())
                                    .in(TfProcessConfigStepL::getStepOrder, stepnodes)
                                    .eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE).eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO))
                            .stream().collect(Collectors.toMap(TfProcessConfigStepL::getStepOrder, r -> r));
                    /**保存用户保存信息*/
                    saveNextStepALL(processL, pNode, processNode, deptId, postId, map);
                }
                /**保存流程当前处理信息*/
                saveNowHandle(processL, processNode, pcs, pc);
            } else if (pNode.getSaveState() == FlowConstant.INTEGER_ONE) {
                /**驳回的流程节点信息*/
                List<String> stepNodes = null;
                List<TfProcessNodeL> pns = new ArrayList<>();
                if (CollUtil.isNotEmpty(pNode.getNextSteps())) {
                    /**下一步提交*/
                    if (pNode.getHandleState() == FlowConstant.INTEGER_ONE) {
                        Set<Integer> stepnodes = pNode.getNextSteps().stream().map(r -> r.getProcessStep()).collect(Collectors.toSet());
                        map = pCStepService.list(new QueryWrapper<TfProcessConfigStepL>().lambda()
                                        .eq(TfProcessConfigStepL::getConfigId, processL.getConfigId()).eq(TfProcessConfigStepL::getEdition, processL.getEdition())
                                        .in(TfProcessConfigStepL::getStepOrder, stepnodes)
                                        .eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE).eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO))
                                .stream().collect(Collectors.toMap(TfProcessConfigStepL::getStepOrder, r -> r));
                        /**保存提交下一步信息*/
                        pns = saveSubmitNextStepAll(processL, pNode.getNextSteps(), processNode, deptId, postId, map);
                        /**转派*/
                    } else if (pNode.getHandleState() == FlowConstant.INTEGER_THREE) {
                        map = new HashMap<>();
                        map.put(pcs.getStepOrder(), pcs);
                        /**保存步骤转派信息*/
                        TfProcessNodeL pn = nodeLFillUtil.saveStepTransfer(processL, pNode, processNode, deptId, postId, stepNode);
                        this.save(pn);
                        /**修改处理的节点的关联关系*/
                        updateNodeLike(processNode.getId(), pn.getId());
                        pns.add(pn);
                    }
                } else {
                    /**下一步提交*/
                    if (pNode.getHandleState() == FlowConstant.INTEGER_ONE) {
                        List<NextStepDto> nextSteps = null;
                        /**下一步处理 ;当前节点为本步骤最后一个节点，下一步执行其它步骤*/
                        if (pcs.getNodeNumber() == processNode.getStepNode()) {
                            /**下一步操作的所有步骤配置信息,按照并行处理*/
                            List<TfProcessConfigStepL> pcss = pCStepService.list(new QueryWrapper<TfProcessConfigStepL>().lambda().eq(TfProcessConfigStepL::getConfigId, processL.getConfigId())
                                    .eq(TfProcessConfigStepL::getEdition, processL.getEdition()).in(TfProcessConfigStepL::getStepOrder, pcs.getNextStep().split(FlowConstant.CSV))
                                    .eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO).eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE));
                            if (CollUtil.isEmpty(pcss)) {
                                throw new RuntimeException("下一步步骤错误！");
                            }
                            map = pcss.stream().collect(Collectors.toMap(TfProcessConfigStepL::getStepOrder, r -> r));
                            nextSteps = pcss.stream().map(r -> {
                                NextStepDto ns = new NextStepDto();
                                ns.setProcessStep(r.getStepOrder());
                                ns.setStepNode(FlowConstant.INTEGER_ONE);
                                return ns;
                            }).collect(Collectors.toList());
                        } else {
                            /**执行本步骤的下一节点*/
                            map = new HashMap<>();
                            map.put(pcs.getStepOrder(), pcs);
                            nextSteps = new ArrayList<>();
                            NextStepDto ns = new NextStepDto();
                            ns.setProcessStep(pcs.getStepOrder());
                            ns.setStepNode(processNode.getStepNode() + FlowConstant.INTEGER_ONE);
                            nextSteps.add(ns);
                        }
                        /**保存提交下一步信息*/
                        pns = saveSubmitNextStepAll(processL, nextSteps, processNode, deptId, postId, map);

                        /**驳回*/
                    } else if (pNode.getHandleState() == FlowConstant.INTEGER_TWO) {
                        map = new HashMap<>();
                        /**获取驳回到的节点*/
                        List<TfProcessNodeL> npns = getRejectToNodeL(processL, processNode, pcs);
                        for (TfProcessNodeL pn : npns) {
                            if (pn.getProcessStep() == processNode.getProcessStep()) {
                                map.put(pcs.getStepOrder(), pcs);
                            } else {
                                TfProcessConfigStepL npcs = pCStepService.getOne(new QueryWrapper<TfProcessConfigStepL>().lambda()
                                        .eq(TfProcessConfigStepL::getConfigId, processL.getConfigId()).eq(TfProcessConfigStepL::getEdition, processL.getEdition())
                                        .eq(TfProcessConfigStepL::getStepOrder, pn.getProcessStep()).eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO)
                                        .eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE));
                                map.put(npcs.getStepOrder(), npcs);
                            }
                            /**步骤关联关系*/
                            Set<Long> nexts = pUNodeService.getLikeNode(pn.getProcessId(), FlowConstant.INTEGER_ONE, pn.getId()).stream().map(r -> r.getNodeId()).collect(Collectors.toSet());
                            nexts.add(pn.getId());

                            stepNodes = this.baseMapper.getRejectStepNodes(nexts);
                            /**步骤修改为无效被驳回*/
                            this.update(new UpdateWrapper<TfProcessNodeL>().lambda().set(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ZERO).set(TfProcessNodeL::getNodeState, FlowConstant.INTEGER_FIVE)
                                    .eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE).in(TfProcessNodeL::getId, nexts).eq(TfProcessNodeL::getIsDel, FlowConstant.SHORT_ZERO));

                            /**驳回到的步骤节点*/
                            TfProcessNodeL npn = new TfProcessNodeL();
                            BeanUtils.copyProperties(pn, npn);
                            npn.setId(null);
                            npn.setNodeState(FlowConstant.INTEGER_ZERO);
                            npn.setIsValid(FlowConstant.SHORT_ONE);
                            npn.setIsDel(FlowConstant.SHORT_ZERO);
                            npn.setHandleTime(null);
                            npn.setCreateDept(deptId);
                            npn.setCreatePost(postId);
                            npn.setHandleRemark(pNode.getHandleRemark());
                            this.save(npn);
                            /**修改处理的节点的关联关系*/
                            updateNodeLike(pn.getId(), npn.getId());
                            /**把历史节点修改为无效*/
                            if (CollUtil.isNotEmpty(nexts)) {
                                pUNodeService.update(new UpdateWrapper<TfProcessUpperNodeL>().lambda().eq(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ONE)
                                        .and(r -> r.in(TfProcessUpperNodeL::getNodeId, nexts).or().in(TfProcessUpperNodeL::getStepNodeId, nexts)).set(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ZERO));
                                List<TfNowHandleL> nhs = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                                        .in(TfNowHandleL::getNodeId, nexts));
                                nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId()).in(TfNowHandleL::getNodeId, nexts));
                                if (CollUtil.isNotEmpty(nhs)) {
                                    for (TfNowHandleL nh : nhs) {
                                        sendTidingsUtil.removeLocalTask(processL.getId(), FlowConstant.PROCESS_TABLE_NAME, nh.getProcessStep(), nh.getStepNode(), null);
                                    }
                                }
                            }
                            pns.add(npn);
                        }
                    }
                }
                /**清除本步骤的流程当前处理信息*/
                List<TfNowHandleL> dnhs = null;
                /**保存下一步骤流程当前处理信息*/
                List<TfNowHandleL> nhs = new ArrayList<>();
                /**修改流程当前处理信息*/
                dnhs = saveTfNowHandleLS(processL, pNode, processNode, nextStep, stepNodes, map, pns, dnhs, nhs, pl, pNode.getStepNode() >= pcs.getCoerceLevel());
                /**取消消息提醒*/
                if (CollUtil.isNotEmpty(dnhs)) {
                    sendTidingsUtil.removeLocalTask(processL.getId(), FlowConstant.PROCESS_TABLE_NAME, processNode.getProcessStep(), processNode.getStepNode(), SecurityUtils.getUserId());
                }
                sendTidingsUtil.removeLocalTask(processL.getId(), FlowConstant.PROCESS_TABLE_NAME, processNode.getProcessStep(), processNode.getStepNode(), null);

                /**发送消息提醒*/
                if (CollUtil.isNotEmpty(nhs)) {
                    Map<Integer, TfProcessNodeL> pnmap = pns.stream().collect(Collectors.toMap(TfProcessNodeL::getProcessStep, r -> r));
                    sendTidingsUtil.removeLocalTask(processL.getId(), FlowConstant.PROCESS_TABLE_NAME, processNode.getProcessStep(), processNode.getStepNode(), null);
                    for (TfNowHandleL nh : nhs) {
                        TfProcessConfigStepL npcs = map.get(nh.getProcessStep());
                        TfProcessNodeL pn = pnmap.get(nh.getProcessStep());
                        sendTidingsUtil.setMessageRecipient(processL, pc, npcs, pn, processNode.getHandleState());
                    }
                }
            }
        }
        /**保存状态 0保存，1提交*/
        if (pNode.getSaveState().equals(FlowConstant.INTEGER_ONE)) {
            //步骤操作完后推送
            if (bpc != null && bpc.getIsPush() == FlowConstant.SHORT_ONE) {
                TfProcessConfigInfoL pci = pCInfoService.getOne(new QueryWrapper<TfProcessConfigInfoL>().lambda().eq(TfProcessConfigInfoL::getConfigId, processL.getId())
                        .eq(TfProcessConfigInfoL::getEdition, processL.getEdition()).eq(TfProcessConfigInfoL::getIsValid, FlowConstant.SHORT_ONE)
                        .eq(TfProcessConfigInfoL::getIsDel, FlowConstant.SHORT_ZERO));
                StringBuilder sbTopic = new StringBuilder(FlowConstant.REDIS_QUERY_KEY);
                sbTopic.append(pci.getProcessCode().toLowerCase()).append(FlowConstant.UNDERLINE).append(processNode.getProcessStep()).append(FlowConstant.UNDERLINE)
                        .append(processNode.getStepNode());
                StringBuilder value = new StringBuilder(processNode.getProcessId().toString());
                value.append(FlowConstant.UNDERLINE).append(processNode.getId());
                //发送流程ID到redis缓存队列
                redisTemplate.opsForList().leftPush(sbTopic.toString(), value.toString());
                log.info(sbTopic.toString() + "-------" + value.toString());
            }
            //流程完结后推送
            if (pl != null && pl.getProcessState() == FlowConstant.SHORT_FOUR) {
                TfProcessConfigInfoL pci = pCInfoService.getOne(new QueryWrapper<TfProcessConfigInfoL>().lambda().eq(TfProcessConfigInfoL::getConfigId, processL.getConfigId())
                        .eq(TfProcessConfigInfoL::getEdition, processL.getEdition()).eq(TfProcessConfigInfoL::getIsValid, FlowConstant.SHORT_ONE)
                        .eq(TfProcessConfigInfoL::getIsDel, FlowConstant.SHORT_ZERO));
                if (FlowConstant.SHORT_ONE == pci.getIsendTidings()) {
                    StringBuilder sbTopic = new StringBuilder(FlowConstant.REDIS_QUERY_KEY);
                    sbTopic.append(pci.getProcessCode().toLowerCase());
                    //发送流程ID到redis缓存队列
                    redisTemplate.opsForList().leftPush(sbTopic.toString(), processNode.getProcessId().toString());
                    log.info(sbTopic.toString() + "-------" + processNode.getProcessId().toString());
                }
            }
        }
        return processNode;
    }

    /**
     * 保存流程当前处理信息
     *
     * @param processL    流程ID
     * @param processNode 流程节点ID
     * @param pcs         流程步骤节点配置
     * @param pc          流程配置
     */
    private void saveNowHandle(TfProcessL processL, TfProcessNodeL processNode, TfProcessConfigStepL pcs, TfProcessConfigL pc) {
        /**查询本步骤*/
        TfNowHandleL nh = nHandleService.getOne(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep()).eq(TfNowHandleL::getStepNode, processNode.getStepNode()));
        if (nh == null) {
            /**流程当前处理信息*/
            nh = new TfNowHandleL();
            nh.setProcessId(processL.getId());
            nh.setProcessStep(processNode.getProcessStep());
            nh.setStepNode(processNode.getStepNode());
            nh.setStepName(pcs.getStepName());
            nh.setEdition(processL.getEdition());
            nh.setStepId(pcs.getId());
            nh.setLabel(pcs.getLabel());
            nh.setAppLabe(pcs.getAppLabe());
            nh.setTimeLimit(processNode.getTimeLimit());
            Integer timeRemind = pcs.getTimeRemind();
            if (timeRemind != null && processNode.getTimeLimit() != null) {
//                nh.setRemindTime(DateUtil.offsetDay(processNode.getTimeLimit(), -timeRemind));
                nh.setRemindTime(DateUtil.offsetHour(processNode.getTimeLimit(), -timeRemind));
            }
            nh.setHandleStaff(processNode.getDutyUser());
            nh.setHandleDept(processNode.getDutyDept());
            nh.setHandlePost(processNode.getDutyPost());
            nh.setHandleMode(FlowConstant.SHORT_ONE);
            nh.setNodeId(processNode.getId());

            nHandleService.save(nh);
        } else if (nh.getHandleStaff() == null) {
            nh.setHandleStaff(processNode.getDutyUser());
            nh.setHandleDept(processNode.getDutyDept());
            nh.setHandlePost(processNode.getDutyPost());
            nh.setHandleMode(FlowConstant.SHORT_ONE);
            nHandleService.updateById(nh);
            /**取消其它可处理人待办信息，确定为只有本步骤的处理保存人可以处理*/
            sendTidingsUtil.removeLocalTask(processL.getId(), FlowConstant.PROCESS_TABLE_NAME, processNode.getProcessStep(), processNode.getStepNode(), SecurityUtils.getUserId());
        }
    }

    /**
     * 修改流程当前处理信息
     *
     * @param processL    流程信息
     * @param pNode       提交填报的节点信息
     * @param processNode 刚保存的本流程节点信息
     * @param nextStep    下步骤信息
     * @param stepNodes   驳回的流程节点信息
     * @param map         下步步骤流程配置
     * @param pns         下一步流程步骤
     * @param dnhs        清除本步骤的流程当前处理信息
     * @param nhs         保存下一步骤流程当前处理信息
     * @param pl          本流程
     * @return
     */
    private List<TfNowHandleL> saveTfNowHandleLS(TfProcessL processL, PNodeDto pNode, TfProcessNodeL processNode, String nextStep, List<String> stepNodes
            , Map<Integer, TfProcessConfigStepL> map, List<TfProcessNodeL> pns, List<TfNowHandleL> dnhs, List<TfNowHandleL> nhs, TfProcessL pl, Boolean isStepNode) {
        //修改流程基础信息
        List<TfNowHandleL> handles = null;
        /**通过*/
        if (pNode.getHandleState() == FlowConstant.INTEGER_ONE) {
            pl.setId(processL.getId());
            if (pNode.getProcessStep() == FlowConstant.INTEGER_ONE && pNode.getStepNode() == FlowConstant.INTEGER_ONE) {
                pl.setProcessState(FlowConstant.SHORT_TWO);
                pl.setPassingNode("1_1");
            } else {
                pl.setPassingNode(new StringBuilder(processL.getPassingNode()).append(FlowConstant.CSV)
                        .append(pNode.getProcessStep()).append(FlowConstant.UNDERLINE).append(pNode.getStepNode()).toString());
            }
            if (isStepNode && nextStep.contains(FlowConstant.STRING_MINUS_ONE)) {
                pl.setProcessState(FlowConstant.SHORT_FOUR);
            }
            dnhs = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep()).eq(TfNowHandleL::getStepNode, processNode.getStepNode())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
            nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep()).eq(TfNowHandleL::getStepNode, processNode.getStepNode())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
            /**填充流程当前处理信息*/
            if (CollUtil.isNotEmpty(pns) && pns.size() == FlowConstant.INTEGER_ONE) {
                handles = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, pns.get(0).getProcessId())
                        .eq(TfNowHandleL::getProcessStep, pns.get(0).getProcessStep()).eq(TfNowHandleL::getStepNode, pns.get(0).getStepNode()));
                if (CollUtil.isNotEmpty(handles)) {
                    nhs = handles;
                }
            } else if (CollUtil.isNotEmpty(pns) && pns.size() > FlowConstant.INTEGER_ONE) {
                Set<Integer> steps = pns.stream().map(r -> r.getProcessStep()).collect(Collectors.toSet());
                handles = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, pns.get(0).getProcessId())
                        .in(TfNowHandleL::getProcessStep, steps).eq(TfNowHandleL::getStepNode, FlowConstant.INTEGER_ONE));
                if (CollUtil.isNotEmpty(handles)) {
                    nhs = handles;
                }
            }
            if (CollUtil.isEmpty(handles)) {
                nodeLFillUtil.fillNowHandle(processL, map, pns, nhs);
            }
            /**驳回*/
        } else if (pNode.getHandleState() == FlowConstant.INTEGER_TWO) {
            String passingnode = processL.getPassingNode();
            for (String str : stepNodes) {
                passingnode = passingnode.replace(str, "");
            }
            pl.setId(processL.getId());
            pl.setPassingNode(passingnode);
            dnhs = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
            nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
            /**填充流程当前处理信息*/
            nodeLFillUtil.fillNowHandle(processL, map, pns, nhs);
            /**转派*/
        } else if (pNode.getHandleState() == FlowConstant.INTEGER_THREE) {
            dnhs = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep()).eq(TfNowHandleL::getStepNode, processNode.getStepNode())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
            nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getProcessStep, processNode.getProcessStep()).eq(TfNowHandleL::getStepNode, processNode.getStepNode())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
            /**填充流程当前处理信息*/
            nodeLFillUtil.fillNowHandle(processL, map, pns, nhs);
            /**完成*/
        } else if (pNode.getHandleState() == FlowConstant.INTEGER_FOUR) {
            if (!(nextStep.contains(FlowConstant.STRING_MINUS_ONE) || nextStep.contains(FlowConstant.STRING_MINUS_TWO))) {
                pl.setId(processL.getId());
                pl.setProcessState(FlowConstant.SHORT_THREE);
            }
            dnhs = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
            nHandleService.remove(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                    .eq(TfNowHandleL::getEdition, processL.getEdition()));
        }
        if (pl != null && pl.getId() != null) {
            processService.updateById(pl);
        }
        if (CollUtil.isNotEmpty(nhs) && CollUtil.isEmpty(handles)) {
            nHandleService.saveBatch(nhs);
        }
        return dnhs;
    }

    /**
     * 保存提交下一步信息
     *
     * @param processL    流程ID
     * @param nextSteps   下一步处理步骤人员
     * @param processNode 本步骤节点信息
     * @param deptId      部门
     * @param postId      岗位
     * @param map         步骤配置信息
     */
    private List<TfProcessNodeL> saveSubmitNextStepAll(TfProcessL processL, List<NextStepDto> nextSteps, TfProcessNodeL processNode, Long deptId, Long postId, Map<Integer, TfProcessConfigStepL> map) {
        /**保存下一步基础信息*/
        List<TfProcessNodeL> pns = saveNextStep(processL, nextSteps, deptId, postId, map);
        /**填充保存下一步的其它数据信息*/
        for (TfProcessNodeL pn : pns) {
            TfProcessConfigStepL npcs = map.get(pn.getProcessStep());
            TfProcessNodeL hpn = null;
            if (processL.getId() != null && !(pn.getProcessStep() == FlowConstant.INTEGER_ONE && pn.getStepNode() == FlowConstant.INTEGER_ONE)) {
                hpn = this.baseMapper.getHistoryMaxNode(processL.getId(), pn.getProcessStep(), pn.getStepNode());
            }
            if (hpn == null) {
                if (npcs.getTimeLimit() != null) {
                    int limit = npcs.getTimeLimit();
                    Date date = null;
                    String str = "2023-10-17 12:00:00";
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Date parse = null;
                    try {
                        parse = sdf.parse(str);
                    } catch (ParseException e) {
                        throw new RuntimeException(e);
                    }
                    /**完成时限 工作日计算*/
                    Date dateTime = new Date();
                    if (npcs.getDayType() == FlowConstant.SHORT_ONE) {
                        /*Date endDate = DateUtil.offset(dateTime, DateField.DAY_OF_MONTH, (limit / 7 + 1) * 2 + (limit / 30 + 1) * 7);
                        List<CalendarVo> calendars = calendarService.getCalendar(new Date(), endDate, processL.getTenantId());
                        if (CollUtil.isEmpty(calendars)) {
                            calendars = calendarService.getCalendar(new Date(), endDate, null);
                        }
                        List<String> workDays = new ArrayList<>();
                        List<String> restDays = new ArrayList<>();
                        if (CollUtil.isNotEmpty(calendars)) {
                            workDays = calendars.stream().map(r -> {
                                if (r.getType() == FlowConstant.SHORT_ONE) return DateUtil.formatDate(r.getDataTime());
                                return null;
                            }).filter(r -> r != null).collect(Collectors.toList());
                            restDays = calendars.stream().map(r -> {
                                if (r.getType() == FlowConstant.SHORT_ZERO) return DateUtil.formatDate(r.getDataTime());
                                return null;
                            }).filter(r -> r != null).collect(Collectors.toList());
                        }*/
                        // TODO
//                        date = DateUtil.endOfDay(WorkDayCalculateUtils.getDate(dateTime, limit, workDays, restDays));
                        if (processL.getCreateTime().compareTo(parse) < 0) {
                            Date endDate = DateUtil.offset(dateTime, DateField.DAY_OF_MONTH, (limit / 7 + 1) * 2 + (limit / 30 + 1) * 7);
                            List<CalendarVo> calendars = calendarService.getCalendar(new Date(), endDate, processL.getTenantId());
                            if (CollUtil.isEmpty(calendars)) {
                                calendars = calendarService.getCalendar(new Date(), endDate, null);
                            }
                            List<String> workDays = new ArrayList<>();
                            List<String> restDays = new ArrayList<>();
                            if (CollUtil.isNotEmpty(calendars)) {
                                workDays = calendars.stream().map(r -> {
                                    if (r.getType() == FlowConstant.SHORT_ONE)
                                        return DateUtil.formatDate(r.getDataTime());
                                    return null;
                                }).filter(r -> r != null).collect(Collectors.toList());
                                restDays = calendars.stream().map(r -> {
                                    if (r.getType() == FlowConstant.SHORT_ZERO)
                                        return DateUtil.formatDate(r.getDataTime());
                                    return null;
                                }).filter(r -> r != null).collect(Collectors.toList());
                            }
                            date = DateUtil.endOfDay(WorkDayCalculateUtils.getDate(dateTime, limit, workDays, restDays));
                        } else {
                            date = WorkDayCalculateUtils.getDateNew(dateTime, limit, calendarService, processL.getTenantId(), FlowConstant.SHORT_ONE);
                        }
                    } else {
                        if (processL.getCreateTime().compareTo(parse) < 0) {
                            date = DateUtil.endOfDay(DateUtil.offset(date, DateField.DAY_OF_MONTH, limit));
                        } else {
                            date = WorkDayCalculateUtils.getDateNew(dateTime, limit, null, null, FlowConstant.SHORT_ZERO);
                        }
                    }
                    pn.setTimeLimit(date);
                }
                if (npcs.getHandleMode() == FlowConstant.INTEGER_ONE) {
                    /**引用其他流程或本流程其它处理步骤*/
                    TfLinkHistoryStepL lhs = lhStepService.getOne(new QueryWrapper<TfLinkHistoryStepL>().lambda().eq(TfLinkHistoryStepL::getStepId, npcs.getId()));
                    TfProcessNodeL lpn = this.getOne(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, processL.getRelationId())
                            .eq(TfProcessNodeL::getProcessStep, lhs.getHistoryStepOrder()).eq(TfProcessNodeL::getNodeState, FlowConstant.INTEGER_TWO));
                    if (lpn != null) {
                        pn.setDutyPost(lpn.getDutyPost());
                        pn.setDutyDept(lpn.getDutyDept());
                        pn.setDutyUser(lpn.getDutyUser());
                    }
                }
            } else {
                pn.setTimeLimit(hpn.getTimeLimit());
                pn.setDutyPost(hpn.getDutyPost());
                pn.setDutyDept(hpn.getDutyDept());
                pn.setDutyUser(hpn.getDutyUser());
            }
        }
        List<TfProcessNodeL> nodes = null;
        if (pns.size() == FlowConstant.INTEGER_ONE) {
            nodes = this.list(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, pns.get(0).getProcessId())
                    .eq(TfProcessNodeL::getProcessStep, pns.get(0).getProcessStep()).eq(TfProcessNodeL::getStepNode, pns.get(0).getStepNode())
                    .eq(TfProcessNodeL::getIsDel, FlowConstant.SHORT_ZERO).eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE));
            if (CollUtil.isNotEmpty(nodes)) {
                pns = nodes;
            }
        } else if (pns.size() > FlowConstant.INTEGER_ONE) {
            Set<Integer> stepIds = pns.stream().map(r -> r.getProcessStep()).collect(Collectors.toSet());
            nodes = this.list(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, pns.get(0).getProcessId())
                    .in(TfProcessNodeL::getProcessStep, stepIds).eq(TfProcessNodeL::getStepNode, FlowConstant.INTEGER_ONE)
                    .eq(TfProcessNodeL::getIsDel, FlowConstant.SHORT_ZERO).eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE));
            if (CollUtil.isNotEmpty(nodes)) {
                pns = nodes;
            }
        }
        if (CollUtil.isNotEmpty(pns) && pns.size() == FlowConstant.INTEGER_ONE) {
            TfProcessNodeL pn = pns.get(0);
            /**下一步骤节点的步骤不等于本步骤（下一步操为下一步骤的第一个节点）*/
            if (!pn.getProcessStep().equals(processNode.getProcessStep())) {
                /**步骤配置*/
                TfProcessConfigStepL npcs = map.get(pn.getProcessStep());
                /**上一步序号(同组逗号分隔;不同组竖线分隔如：1,2|3)*/
                String[] steps = npcs.getUpperStep().split("\\|");
                /**判断下一步数据是否为临时数据*/
                for (String step : steps) {
                    String[] strs = step.split(",");
                    List<Integer> list = Arrays.asList(strs).stream().filter(r -> StringUtils.isNotEmpty(r)).map(r -> Integer.parseInt(r)).collect(Collectors.toList());
                    if (list.contains(processNode.getProcessStep()) && list.size() > FlowConstant.INTEGER_ONE) {
                        List<TfNowHandleL> nhs = nHandleService.list(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId())
                                .ne(TfNowHandleL::getProcessStep, processNode.getProcessStep()));
                        if (CollUtil.isNotEmpty(nhs)) {
                            List<Integer> psteps = pUNodeService.getUpperProcessStep(processL.getId(), processNode.getId(), 1, 1);
                            boolean istemp = false;
                            sign:
                            for (TfNowHandleL nh : nhs) {
                                List<Integer> lsteps = pUNodeService.getUpperProcessStep(processL.getId(), nh.getNodeId(), 1, 1);
                                for (Integer s : lsteps) {
                                    if (psteps.contains(s)) {
                                        istemp = true;
                                        break sign;
                                    }
                                }
                            }
                            if (istemp) {
                                pn.setIsTemp(FlowConstant.SHORT_ONE);
                            } else {
                                pn.setIsTemp(FlowConstant.SHORT_ZERO);
                            }
                        } else {
                            pn.setIsTemp(FlowConstant.SHORT_ZERO);
                        }
                    }
                }
            }
        }

        if (CollUtil.isNotEmpty(pns) && CollUtil.isEmpty(nodes)) {
            this.saveBatch(pns);
        } else if (CollUtil.isNotEmpty(pns) && pns.size() == FlowConstant.INTEGER_ONE) {
            this.updateBatchById(pns);
        }

        /**保存本步骤下节点关联关系信息*/
        saveNextNodelRelation(processL, processNode, pns, map);
        /**保存下步骤上节点关联关系信息*/
        saveUpNodelRelation(processL, processNode, pns, map);
        return pns;
    }

    /**
     * 获取驳回到节点
     *
     * @param processL    流程信息
     * @param processNode 用户提交的操作步骤信息
     * @return 驳回到的节点
     */
    private List<TfProcessNodeL> getRejectToNodeL(TfProcessL processL, TfProcessNodeL processNode, TfProcessConfigStepL pcs) {
        /**被驳回到的步骤节点*/
        List<TfProcessNodeL> pns = new ArrayList<>();
        TfProcessNodeL pn = null;
        if (pcs.getReturnToStep() == null) {
            TfProcessConfigInfoL pci = pCInfoService.getOne(new QueryWrapper<TfProcessConfigInfoL>().lambda()
                    .eq(TfProcessConfigInfoL::getConfigId, processL.getConfigId()).eq(TfProcessConfigInfoL::getEdition, processL.getEdition())
                    .eq(TfProcessConfigInfoL::getIsValid, FlowConstant.SHORT_ONE).eq(TfProcessConfigInfoL::getIsDel, FlowConstant.SHORT_ZERO));
            List<TfProcessUpperNodeL> puns = pUNodeService.list(new QueryWrapper<TfProcessUpperNodeL>().lambda().eq(TfProcessUpperNodeL::getNodeId, processNode.getId())
                    .eq(TfProcessUpperNodeL::getType, FlowConstant.SHORT_ONE).eq(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ONE));
            if (pci.getReiectMode() == FlowConstant.SHORT_ONE) {
                for (TfProcessUpperNodeL pun : puns) {
                    pn = this.getById(pun.getStepNodeId());
                    pns.add(pn);
                }
            } else if (pci.getReiectMode() == FlowConstant.SHORT_TWO) {
                for (TfProcessUpperNodeL pun : puns) {
                    TfProcessUpperNodeL upun = null;
                    if (processNode.getStepNode() > FlowConstant.INTEGER_ONE) {
                        upun = pUNodeService.getOne(new QueryWrapper<TfProcessUpperNodeL>().lambda().eq(TfProcessUpperNodeL::getNodeId, pun.getStepId())
                                .eq(TfProcessUpperNodeL::getType, FlowConstant.SHORT_ONE).eq(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ONE));
                    }
                    if (upun != null) {
                        pn = this.getById(upun.getStepId());
                    } else {
                        pn = this.getById(pun.getStepId());
                    }
                    pns.add(pn);
                }
            } else {
                throw new RuntimeException("驳回方式错误！");
            }
        } else {
            pn = this.getOne(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, processL.getId())
                    .eq(TfProcessNodeL::getProcessStep, pcs.getReturnToStep()).eq(TfProcessNodeL::getStepNode, FlowConstant.INTEGER_ONE)
                    .eq(TfProcessNodeL::getIsValid, FlowConstant.INTEGER_ONE).eq(TfProcessNodeL::getIsDel, FlowConstant.SHORT_ZERO));
            pns.add(pn);
        }
        return pns;
    }

    /**
     * 修改处理的节点的关联关系
     *
     * @param oldId 修改历史节点
     * @param id    新节点
     */
    private void updateNodeLike(Long oldId, Long id) {
        /**修改本步骤上节点关联关系*/
        List<TfProcessUpperNodeL> onpuns = pUNodeService.list(new QueryWrapper<TfProcessUpperNodeL>().lambda().eq(TfProcessUpperNodeL::getNodeId, oldId)
                .eq(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ONE));
        List<TfProcessUpperNodeL> puns = new ArrayList<>();
        for (TfProcessUpperNodeL opun : onpuns) {
            if (opun.getType() == FlowConstant.INTEGER_ONE) {
                TfProcessUpperNodeL pun = new TfProcessUpperNodeL();
                BeanUtils.copyProperties(opun, pun);
                pun.setId(null);
                pun.setNodeId(id);
                pun.setHandleId(opun.getNodeId());
                puns.add(pun);
            }
            opun.setIsValid(FlowConstant.SHORT_ZERO);
        }
        pUNodeService.updateBatchById(onpuns);
        /**修改上步骤下节点关联关系*/
        List<TfProcessUpperNodeL> oupuns = pUNodeService.list(new QueryWrapper<TfProcessUpperNodeL>().lambda().eq(TfProcessUpperNodeL::getStepNodeId, oldId)
                .eq(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ONE));
        for (TfProcessUpperNodeL oupun : oupuns) {
            if (oupun.getType() == FlowConstant.INTEGER_ZERO) {
                TfProcessUpperNodeL pun = new TfProcessUpperNodeL();
                BeanUtils.copyProperties(oupun, pun);
                pun.setId(null);
                pun.setStepNodeId(id);
                if (oupun.getStepNodeId().equals(oupun.getStepId())) {
                    pun.setStepId(id);
                }
                puns.add(pun);
            }
            oupun.setIsValid(FlowConstant.SHORT_ZERO);
        }
        pUNodeService.updateBatchById(oupuns);
        pUNodeService.saveBatch(puns);
    }

    /**
     * 保存上节点关联关系
     *
     * @param processL    流程ID
     * @param processNode 本步骤节点信息
     * @param pns         下一步集合
     * @param map
     */
    private void saveUpNodelRelation(TfProcessL processL, TfProcessNodeL processNode, List<TfProcessNodeL> pns, Map<Integer, TfProcessConfigStepL> map) {

        List<TfProcessUpperNodeL> puns = new ArrayList<>();
        for (TfProcessNodeL pn : pns) {
            TfProcessConfigStepL pcs = map.get(pn.getProcessStep());
            String[] steps = pcs.getUpperStep().split("\\|");
            List<TfProcessNodeL> upnodes = new ArrayList<>();
            upnodes.add(processNode);
            for (String step : steps) {
                String[] strs = step.split(",");
                List<Long> list = Arrays.asList(strs).stream().filter(r -> StringUtils.isNotEmpty(r)).map(r -> Long.parseLong(r)).collect(Collectors.toList());
                if (list.contains(pn.getProcessStep()) && list.size() > FlowConstant.INTEGER_ONE) {
                    list.remove(pn.getProcessStep());
                    List<TfProcessNodeL> pnss = this.baseMapper.getParallelNode(processL.getId(), list);
                    upnodes.addAll(pnss);
                }
            }
            /**填充保存下一步关系*/
            fillNextNodes(processL, upnodes, puns, pn);
        }
        pUNodeService.saveBatch(puns);
    }

    /**
     * 填充保存下一步关系
     *
     * @param processL 流程
     * @param upnodes  下一步的节点
     * @param puns     下一步的关系
     * @param pn       本流程节点
     */
    private void fillNextNodes(TfProcessL processL, List<TfProcessNodeL> upnodes, List<TfProcessUpperNodeL> puns, TfProcessNodeL pn) {
        List<Long> ids = upnodes.stream().map(r -> r.getId()).collect(Collectors.toList());

        List<TfProcessUpperNodeL> uns = pUNodeService.list(new QueryWrapper<TfProcessUpperNodeL>().lambda()
                .eq(TfProcessUpperNodeL::getProcessId, processL.getId()).in(TfProcessUpperNodeL::getStepNodeId, ids)
                .eq(TfProcessUpperNodeL::getType, FlowConstant.SHORT_ZERO).eq(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ONE));
        Map<Long, TfProcessUpperNodeL> upuns = new HashMap<>();
        for (TfProcessUpperNodeL un : uns) {
            upuns.put(un.getStepNodeId(), un);
        }
        for (TfProcessNodeL processNode : upnodes) {
            TfProcessUpperNodeL upun = upuns.get(processNode.getId());
            TfProcessUpperNodeL pun = new TfProcessUpperNodeL();
            pun.setProcessId(processL.getId());
            pun.setNodeId(pn.getId());
            pun.setStepNodeId(processNode.getId());
            pun.setType(FlowConstant.INTEGER_ONE);
            pun.setHandleId(processNode.getId());
            pun.setIsValid(FlowConstant.SHORT_ONE);
            if (upun != null) {
                pun.setStepId(upun.getStepId());
            } else {
                pun.setStepId(processNode.getId());
            }
            puns.add(pun);
        }
    }

    /**
     * 保存用户保存信息(下一步全量信息)
     *
     * @param processL    流程ID
     * @param pNode       步骤信息
     * @param processNode 本步骤节点信息
     * @param deptId      部门
     * @param postId      岗位
     * @param map
     */
    private void saveNextStepALL(TfProcessL processL, PNodeDto pNode, TfProcessNodeL processNode, Long deptId, Long postId, Map<Integer, TfProcessConfigStepL> map) {
        //创建保存下一步
        /**下一步提交*/
        if (pNode.getHandleState() == FlowConstant.INTEGER_ONE) {
            /**保存下一步基础信息*/
            List<TfProcessNodeL> pns = saveNextStep(processL, pNode.getNextSteps(), deptId, postId, map);
            for (TfProcessNodeL pn : pns) {
                pn.setIsTemp(FlowConstant.SHORT_ONE);
            }
            this.saveBatch(pns);
            /**保存下节点信息*/
            saveNextNodelRelation(processL, processNode, pns, map);
            /**转派*/
        } else if (pNode.getHandleState() == FlowConstant.INTEGER_THREE) {
            /**保存步骤转派信息*/
            TfProcessNodeL pn = nodeLFillUtil.saveStepTransfer(processL, pNode, processNode, deptId, postId, null);
            this.save(pn);
            TfProcessUpperNodeL pun = new TfProcessUpperNodeL();
            pun.setProcessId(processL.getId());
            pun.setNodeId(pn.getId());
            pun.setHandleId(processNode.getId());
            pun.setType(FlowConstant.INTEGER_ONE);
            pun.setIsValid(FlowConstant.SHORT_ONE);
            pUNodeService.save(pun);
        }
    }

    /**
     * 保存下节点关联关系
     *
     * @param processL    流程ID
     * @param processNode 本步骤节点信息
     * @param pns         下一步集合
     * @param map
     */
    private void saveNextNodelRelation(TfProcessL processL, TfProcessNodeL processNode, List<TfProcessNodeL> pns, Map<Integer, TfProcessConfigStepL> map) {

        List<TfProcessUpperNodeL> puns = new ArrayList<>();
        for (TfProcessNodeL pn : pns) {
            TfProcessConfigStepL pcs = map.get(pn.getProcessStep());
            String[] steps = pcs.getUpperStep().split("\\|");
            List<TfProcessNodeL> upnodes = new ArrayList<>();
            upnodes.add(processNode);
            for (String step : steps) {
                String[] strs = step.split(",");
                List<Long> list = Arrays.asList(strs).stream().filter(r -> StringUtils.isNotEmpty(r)).map(r -> Long.parseLong(r)).collect(Collectors.toList());
                if (list.contains(pn.getProcessStep()) && list.size() > FlowConstant.INTEGER_ONE) {
                    list.remove(pn.getProcessStep());
                    /**获取关联并行节点*/
                    List<TfProcessNodeL> pnss = this.baseMapper.getParallelNode(processL.getId(), list);
                    upnodes.addAll(pnss);
                }
            }
            /**填充上节点并行节点关联关系*/
            fillUpNode(processL, upnodes, puns, pn);
        }
        pUNodeService.saveBatch(puns);
    }

    /**
     * 填充上节点并行节点关联关系
     *
     * @param processL 流程ID
     * @param upnodes  上节点结合
     * @param puns     关联节点
     * @param pn       子层节点
     */
    private void fillUpNode(TfProcessL processL, List<TfProcessNodeL> upnodes, List<TfProcessUpperNodeL> puns, TfProcessNodeL pn) {
        List<Long> ids = upnodes.stream().map(r -> r.getId()).collect(Collectors.toList());
        Map<Long, TfProcessUpperNodeL> upuns = pUNodeService.list(new QueryWrapper<TfProcessUpperNodeL>().lambda()
                        .eq(TfProcessUpperNodeL::getProcessId, processL.getId()).in(TfProcessUpperNodeL::getNodeId, ids)
                        .eq(TfProcessUpperNodeL::getType, FlowConstant.SHORT_ZERO).eq(TfProcessUpperNodeL::getIsValid, FlowConstant.SHORT_ONE))
                .stream().collect(Collectors.toMap(TfProcessUpperNodeL::getNodeId, r -> r));
        for (TfProcessNodeL processNode : upnodes) {
            TfProcessUpperNodeL upun = upuns.get(processNode.getId());
            TfProcessUpperNodeL pun = new TfProcessUpperNodeL();
            pun.setProcessId(processL.getId());
            pun.setNodeId(processNode.getId());
            pun.setStepNodeId(pn.getId());
            if (pn.getStepNode() == FlowConstant.INTEGER_ONE) {
                pun.setStepId(pn.getId());
            } else if (pn.getStepNode() >= FlowConstant.INTEGER_TWO) {
                pun.setStepId(processNode.getId());
            } else {
                pun.setStepId(upun.getStepId());
            }
            if (upun != null) {
                pun.setHandleId(upun.getNodeId());
            }
            pun.setType(FlowConstant.INTEGER_ZERO);
            pun.setIsValid(FlowConstant.SHORT_ONE);
            puns.add(pun);
        }
    }

    /**
     * 保存下一步基础信息
     *
     * @param processL  流程ID
     * @param nextSteps 下一步处理步骤人员
     * @param deptId    部门
     * @param postId    岗位
     * @param map       流程步骤配置
     * @return
     */
    private List<TfProcessNodeL> saveNextStep(TfProcessL processL, List<NextStepDto> nextSteps, Long deptId, Long postId, Map<Integer, TfProcessConfigStepL> map) {


        if (CollUtil.isNotEmpty(nextSteps)) {
            List<TfProcessNodeL> pns = new ArrayList<>();
            sign:
            for (NextStepDto ns : nextSteps) {
                TfProcessConfigStepL pcs = map.get(ns.getProcessStep());

                //TODO  如果下一步是工程派工   则将合同标记上已审核
                if("合同签订及缴费".equals(pcs.getStepName())){
                    //将对应的合同 添加已审核
                log.info("添加已审核字样");
                    final List<TfContractW> list = tfContractWService.list(Wrappers.<TfContractW>lambdaQuery().eq(TfContractW::getProcessId, processL.getId()));
                    for (TfContractW tfContractW : list) {
                        final String annex = tfContractW.getContractFile();
                        final JSONArray objects = JSON.parseArray(annex);
                        List<FileInfo> fileInfos = new ArrayList<>();
                        List<FileInfo> pdfFileInfos = new ArrayList<>();
                        for (int i = 0; i < objects.size(); i++) {
                            final JSONObject jsonObject = objects.getJSONObject(i);
                            final FileInfo oldFileInfo = jsonObject.toJavaObject(FileInfo.class);

                            final FileInfo fileInfo = POIUtil.addSign(oldFileInfo.getFilePath(),oldFileInfo);

                            fileInfos.add(fileInfo);


                            String filePathBean = oldFileInfo.getFilePath().substring(0,oldFileInfo.getFilePath().lastIndexOf("/") + 1);
                            String otherName = IdWorker.getIdStr() + ".pdf";
                            String remotePath = filePathBean + otherName;
                            String originPath = fileInfo.getFilePath();
                            TfContractWServiceImpl.doc2pdf(originPath,tempFilePath,remotePath);

                            FileInfo fileInfoPDF = new FileInfo();
                            fileInfoPDF.setFileName(fileInfo.getFileName().substring(0,fileInfo.getFileName().lastIndexOf(Constant.POINT)) + ".pdf");
                            fileInfoPDF.setOtherName(otherName);
                            fileInfoPDF.setFilePath(remotePath);
                            fileInfoPDF.setFileFormat(".pdf");
                            pdfFileInfos.add(fileInfoPDF);

                        }
                        tfContractW.setContractFile(JSON.toJSONString(fileInfos));
                        tfContractW.setContractPdfFile(JSON.toJSONString(pdfFileInfos));
                        tfContractWService.updateById(tfContractW);
                    }
                }


                if (FlowConstant.INTEGER_ONE == ns.getStepNode()) {
                    String[] steps = pcs.getUpperStep().split("\\|");
                    for (String step : steps) {
                        String[] strs = step.split(",");
                        List<Integer> list = Arrays.asList(strs).stream().filter(r -> StringUtils.isNotEmpty(r)).map(r -> Integer.parseInt(r)).collect(Collectors.toList());
                        if (list.contains(ns.getProcessStep()) && list.size() > FlowConstant.INTEGER_ONE) {
                            list.remove(ns.getProcessStep());
                            int count = nHandleService.count(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, processL.getId()).eq(TfNowHandleL::getProcessStep, list));
                            if (count > 0) {
                                continue sign;
                            }
                        }
                    }
                }
                TfPageControlL pc = pControlService.getOne(new QueryWrapper<TfPageControlL>().lambda().eq(TfPageControlL::getStepId, pcs.getId())
                        .eq(TfPageControlL::getStepNode, ns.getStepNode()).eq(TfPageControlL::getIsDel, FlowConstant.SHORT_ZERO));
                TfProcessNodeL pn = new TfProcessNodeL();
                pn.setProcessId(processL.getId());
                pn.setPageCode(pc != null ? pc.getPageCode() : null);
                pn.setProcessStep(ns.getProcessStep());
                pn.setCreatePost(postId);
                pn.setCreateDept(deptId);
                pn.setConfigId(pcs.getConfigId());
                pn.setEdition(pcs.getEdition());
                pn.setLabel(pcs.getLabel());
                pn.setAppLabe(pcs.getAppLabe());
                pn.setNodeState(FlowConstant.INTEGER_ZERO);
                pn.setStepNode(ns.getStepNode());
                pn.setIsOvertime(FlowConstant.SHORT_ZERO);
                pn.setIsValid(FlowConstant.SHORT_ONE);
                pn.setIsDel(FlowConstant.SHORT_ZERO);
                if (ns.getDeptStaff() != null) {
                    pn.setDutyUser(ns.getDeptStaff());
                    pn.setDutyDept(ns.getDept());
                    pn.setDutyPost(ns.getPost());
                }
                pns.add(pn);
            }
            return pns;
        }
        return null;
    }

    /**
     * 校验下一步骤、人员
     *
     * @param pl   工单流程
     * @param pdto 步骤信息
     * @param pcs  本步骤配置
     * @param map  下步骤配置信息
     * @return 校验结果
     */
    private String getNextStepCheck(TfProcessL pl, PNodeDto pdto, TfProcessConfigStepL pcs, Map<Integer, TfProcessConfigStepL> map) {
        if (pdto.getHandleState() == FlowConstant.INTEGER_TWO) {
            if (!pcs.getHandleModel().contains(FlowConstant.STRING_TWO)) {
                return FlowConstant.NO_HANDLE_PURVIEW;
            }
        } else if (pdto.getHandleState() == FlowConstant.INTEGER_THREE) {
            if (!pcs.getHandleModel().contains(FlowConstant.STRING_THREE)) {
                return FlowConstant.NO_HANDLE_PURVIEW;
            }
            if (CollUtil.isEmpty(pdto.getNextSteps()) || pdto.getNextSteps().size() != 1 || pdto.getNextSteps().get(0).getDeptStaff() == null) {
                return FlowConstant.USER_CONFIG_WRONG;
            }
        } else {
            /**校验下一步骤、人员*/
            if (CollUtil.isNotEmpty(pdto.getNextSteps())) {
                List<Integer> steps = new ArrayList<>();
                for (NextStepDto ns : pdto.getNextSteps()) {
                    TfProcessConfigStepL npcs = pcs;
                    Integer stepNode = null;
                    if (steps.contains(ns.getProcessStep())) {
                        return FlowConstant.HANDLE_STEP_REPEAT;
                    } else {
                        steps.add(ns.getProcessStep());
                    }
                    /**通过*/
                    if (pdto.getHandleState() == FlowConstant.INTEGER_ONE) {
                        if (ns.getProcessStep() == null || ns.getStepNode() == null) {
                            return FlowConstant.STEP_NODE_ERROR;
                        }
                        if (ns.getProcessStep() == pdto.getProcessStep()) {
                            if (npcs.getNodeNumber() < ns.getStepNode() || (pdto.getStepNode() + FlowConstant.INTEGER_ONE) != ns.getStepNode()) {
                                return FlowConstant.NODE_NUMBER_ERROR;
                            }
                            if (pdto.getNextSteps().size() > FlowConstant.INTEGER_ONE) {
                                return FlowConstant.NEXT_STEP_NUMBER_ERROR;
                            }
                            if (pcs.getIsAppointUser() == FlowConstant.SHORT_ONE && ns.getDeptStaff() == null) {
                                return FlowConstant.USER_CONFIG_WRONG;
                            }
                        } else {
                            if (!map.containsKey(ns.getProcessStep())) {
                                return FlowConstant.NEXT_STEP_ERROR;
                            }
                            /**判断强制执行等级*/
                            if (pcs.getCoerceLevel() > pdto.getStepNode()) {
                                return FlowConstant.COERCE_NODE_WRONG;
                            }
                            if (pdto.getHandleState() == FlowConstant.INTEGER_ONE && ns.getDept() == null && ns.getPost() == null && ns.getDeptStaff() != null) {
                                return FlowConstant.USER_DEPT_POST_NULL;
                            }
                            npcs = map.get(ns.getProcessStep());
                            if (npcs.getIsAppointUser() == FlowConstant.SHORT_ONE && ns.getDeptStaff() == null) {
                                return FlowConstant.USER_CONFIG_WRONG;
                            }
                            if (npcs.getHandleMode() == FlowConstant.INTEGER_ONE) {
                                if (pl != null && pl.getRelationId() != null) {
                                    TfLinkHistoryStepL lhs = lhStepService.getOne(new QueryWrapper<TfLinkHistoryStepL>().lambda().eq(TfLinkHistoryStepL::getStepId, npcs.getId()));
                                    int count = this.count(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId, pl.getRelationId()).eq(TfProcessNodeL::getNodeState, FlowConstant.INTEGER_TWO)
                                            .eq(TfProcessNodeL::getProcessStep, lhs.getHistoryStepOrder()).eq(TfProcessNodeL::getStepNode, FlowConstant.SHORT_ONE).eq(TfProcessNodeL::getIsValid, FlowConstant.SHORT_ONE));
                                    if (count == FlowConstant.INTEGER_ZERO) {
                                        return FlowConstant.LINK_PROCESS_WRONG;
                                    }
                                } else {
                                    return FlowConstant.LINK_PROCESS_WRONG;
                                }
                            }
                        }
                        stepNode = ns.getStepNode();
                    }
                    /**判断是否有处理权限*/
                    if (ns.getDeptStaff() != null || ns.getDept() != null || ns.getPost() != null) {
                        int count = csuserService.count(new QueryWrapper<TfConfigStepUserL>().lambda().eq(TfConfigStepUserL::getStepId, npcs.getId())
                                .eq(TfConfigStepUserL::getStepNode, stepNode).eq(TfConfigStepUserL::getUserType, FlowConstant.SHORT_ONE)
                                .eq(ns.getDept() != null, TfConfigStepUserL::getDeptId, ns.getDept()).eq(ns.getPost() != null, TfConfigStepUserL::getPostId, ns.getPost())
                                .and(ns.getDeptStaff() != null, r -> r.eq(TfConfigStepUserL::getUserId, ns.getDeptStaff()).or().isNull(TfConfigStepUserL::getUserId))
                                .eq(TfConfigStepUserL::getIsDel, FlowConstant.SHORT_ZERO));
                        if (count != FlowConstant.SHORT_ONE) {
                            return FlowConstant.USER_CONFIG_WRONG;
                        }
                    }
                }
            } else {
                if (pdto.getStepNode() < pcs.getNodeNumber() && pcs.getIsAppointUser() == FlowConstant.SHORT_ONE) {
                    return FlowConstant.NEXT_STEP_NUMBER_ERROR;
                }
                /**本步骤下节点校验*/
                if (pdto.getStepNode() == pcs.getNodeNumber() && StringUtils.isNotEmpty(pcs.getNextStep()) && !pcs.getNextStep().contains("-")) {
                    String[] steps = pcs.getNextStep().split(",");
                    if (steps.length > FlowConstant.INTEGER_ONE && pcs.getNextHandleType() == FlowConstant.SHORT_ZERO) {
                        return FlowConstant.NEXT_STEP_NUMBER_ERROR;
                    }
                    int count = pCStepService.count(new QueryWrapper<TfProcessConfigStepL>().lambda().eq(TfProcessConfigStepL::getConfigId, pcs.getConfigId())
                            .eq(TfProcessConfigStepL::getEdition, pcs.getEdition()).eq(TfProcessConfigStepL::getIsAppointUser, FlowConstant.SHORT_ONE)
                            .eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO)
                            .eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE).in(TfProcessConfigStepL::getStepOrder, steps));
                    if (count > FlowConstant.INTEGER_ZERO) {
                        return FlowConstant.NEXT_STEP_NUMBER_ERROR;
                    }
                }
            }
        }
        return null;
    }

    /**
     * 本步骤信息校验
     *
     * @param id   流程ID
     * @param pdto 步骤信息
     * @return 校验结果
     */
    private String getThisStepCheck(Long id, PNodeDto pdto) {
        if (id == null) {
            if (pdto.getProcessStep() != FlowConstant.INTEGER_ONE || pdto.getStepNode() != FlowConstant.INTEGER_ONE) {
                return FlowConstant.PROCESS_STEP_NUMBER_ERROR;
            }
        } else {
            /**是否有当前处理步骤*/
            TfNowHandleL nh = nHandleService.getOne(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId, id)
                    .eq(TfNowHandleL::getProcessStep, pdto.getProcessStep()).eq(TfNowHandleL::getStepNode, pdto.getStepNode()));
            if (nh == null) {
                return FlowConstant.PROCESS_STEP_NUMBER_ERROR;
            }
            /**校验是否有处理权限*/
            if (nh.getHandleStaff() != null && !nh.getHandleStaff().equals(SecurityUtils.getUserId())) {
                return FlowConstant.NO_PROCESSING_PERMISSION;
            } else if (nh.getHandleStaff() == null) {
                int count = csuserService.count(new QueryWrapper<TfConfigStepUserL>().lambda().eq(TfConfigStepUserL::getStepId, nh.getStepId())
                        .eq(TfConfigStepUserL::getStepNode, nh.getStepNode()).and(r -> r.eq(TfConfigStepUserL::getUserId, SecurityUtils.getUserId())
                                .or(l -> l.in(TfConfigStepUserL::getPostId, SecurityUtils.getUserPostIds()).isNull(TfConfigStepUserL::getUserId)))
                        .eq(TfConfigStepUserL::getUserType, FlowConstant.SHORT_ONE).eq(TfConfigStepUserL::getIsDel, FlowConstant.SHORT_ZERO));
                if (count == FlowConstant.SHORT_ZERO) {
                    return FlowConstant.NO_PROCESSING_PERMISSION;
                }
            }
        }
        return null;
    }


}
