/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.process.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.entity.dto.MessageDto;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.TfNowHandleL;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.TfProcessSuperviseL;
import com.tofly.workflow.process.entity.dto.PSuperviseDto;
import com.tofly.workflow.process.service.TfNowHandleLService;
import com.tofly.workflow.process.service.TfProcessLService;
import com.tofly.workflow.process.service.TfProcessNodeLService;
import com.tofly.workflow.process.service.TfProcessSuperviseLService;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigStepL;
import com.tofly.workflow.processconfig.service.TfProcessConfigLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigStepLService;
import com.tofly.workflow.utils.SendTidingsUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.workflow.process.mapper.TfProcessSuperviseLMapper;

import java.util.List;

/**
 * 流程督办记录信息
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfProcessSuperviseLService")
public class TfProcessSuperviseLServiceImpl extends ServiceImpl<TfProcessSuperviseLMapper, TfProcessSuperviseL> implements TfProcessSuperviseLService {


    /**
     * 流程工单
     */
    @Autowired
    private TfProcessLService processService;
    /**
     * 流程步骤
     */
    @Autowired
    private TfProcessNodeLService pNodeService;

    /**
     * 流程处理信息
     */
    @Autowired
    private TfNowHandleLService nHandleLService;

    /**
     * 流程配置service
     */
    @Autowired
    private TfProcessConfigLService pConfigService;

    /**
     * 流程步骤service
     */
    @Autowired
    private TfProcessConfigStepLService pCStepService;

    /**
     * 发送消息
     */
    @Autowired
    private SendTidingsUtil sendTidingsUtil;

    /**
     * 保存督办信息
     * @param dto 督办信息
     * @return 保存结果
     */
    @Override
    public Object pSuperviseSave(PSuperviseDto dto) {
        String check = pSuperviseCheck(dto);
        if(StringUtils.isNotEmpty(check)){
            return check;
        }

        TfNowHandleL nh = nHandleLService.getOne(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId,dto.getProcessId())
                .eq(TfNowHandleL::getProcessStep,dto.getProcessStep()).eq(TfNowHandleL::getStepNode,dto.getStepNode()));
        TfProcessSuperviseL ps = new TfProcessSuperviseL();
        BeanUtils.copyProperties(dto,ps);
        ps.setHandleDept(nh.getHandleDept());
        ps.setHandlePost(nh.getHandlePost());
        ps.setHandleUser(nh.getHandleStaff());
        ps.setIsDel(FlowConstant.SHORT_ZERO);
        List<Long> depts = SecurityUtils.getUserDeptIds();
        ps.setSuperviseDept(CollUtil.isEmpty(depts) ?null:depts.get(FlowConstant.INTEGER_ONE));
        List<Long> posts = SecurityUtils.getUserPostIds();
        ps.setSupervisePost(CollUtil.isEmpty(posts)?null:posts.get(FlowConstant.INTEGER_ONE));
        ps.setSuperviseUser(SecurityUtils.getUserId());
        ps.setTenantId(SecurityUtils.getTenantId());
        /**发送消息*/
        sendMessage(dto,nh);
        return this.save(ps);
    }

    /**
     * 发送消息
     * @param dto 督办信息
     * @param nh 当前处理信息
     */
    private void sendMessage(PSuperviseDto dto, TfNowHandleL nh) {
        TfProcessL processL = processService.getById(dto.getProcessId());
        TfProcessConfigStepL pcs = pCStepService.getById(nh.getStepId());
        TfProcessConfigL pc = pConfigService.getById(pcs.getConfigId());
        TfProcessNodeL pn = pNodeService.getOne(new QueryWrapper<TfProcessNodeL>().lambda().eq(TfProcessNodeL::getProcessId,dto.getProcessId())
                .eq(TfProcessNodeL::getProcessStep,dto.getProcessStep()).eq(TfProcessNodeL::getStepNode,dto.getStepNode())
                .eq(TfProcessNodeL::getIsValid,FlowConstant.SHORT_ONE));
        sendTidingsUtil.setMessageRecipient(processL,pc,pcs,pn,false);
    }

    /**
     * 校验督办信息
     * @param dto 督办信息
     * @return 校验结果
     */
    private String pSuperviseCheck(PSuperviseDto dto){
        int count = processService.count(new QueryWrapper<TfProcessL>().lambda().eq(TfProcessL::getId,dto.getProcessId())
                .eq(TfProcessL::getIsDel,FlowConstant.SHORT_ZERO)
                .and(r->r.eq(TfProcessL::getProcessState, FlowConstant.SHORT_ONE).or().eq(TfProcessL::getProcessState,FlowConstant.SHORT_TWO)));
        if(count==FlowConstant.INTEGER_ZERO){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        count = nHandleLService.count(new QueryWrapper<TfNowHandleL>().lambda().eq(TfNowHandleL::getProcessId,dto.getProcessId())
                .eq(TfNowHandleL::getProcessStep,dto.getProcessStep()).eq(TfNowHandleL::getStepNode,dto.getStepNode()));
        if(count==FlowConstant.INTEGER_ZERO){
            return FlowConstant.STEP_FINISH_WRONG;
        }
        return null;
    }

}
