/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.processconfig.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.processconfig.entity.dto.CopyConfigDto;
import com.tofly.workflow.processconfig.entity.dto.PCStepLoadDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigLoadDto;
import com.tofly.workflow.processconfig.service.TfProcessConfigLService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.math.BigDecimal;

/**
 * 审批流程配置
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprocessconfigl")
@Api(tags="审批流程配置接口")
public class TfProcessConfigLController extends BaseController {

    private final  TfProcessConfigLService tfProcessConfigLService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dto 查询条件
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultData getTfProcessConfigLPage(Page page, @ModelAttribute PConfigLoadDto dto) {
      return ResultData.ok(tfProcessConfigLService.configLoadPage(page,dto));
    }

    /**
     * 根据租户工程查询流程名称
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="根据租户工程查询流程名称" )
    @GetMapping("/configNames" )
    @ApiOperation(value = "根据租户工程查询流程名称")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "租户id", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "isMould", value = "是否模板0否1是", required = true, dataType = "Short")
    })
    public ResultData getProcessConfigName(Long id,Short isMould){
        return ResultData.ok(tfProcessConfigLService.getProcessConfigName(id,isMould));
    }


    /**
     * 通过id查询审批流程配置
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultData getById(@PathVariable("id" ) Long id) {
      return ResultData.ok(tfProcessConfigLService.getProcessConfigById(id));
    }

    /**
     * 通过id和版本查询历史审批流程配置
     * @param id id
     * @param edition edition
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id和版本查询历史审批流程配置" )
    @GetMapping("/configByIdAnd" )
    @ApiOperation(value = "通过id和版本查询历史审批流程配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "edition", value = "edition", required = false, dataType = "Integer")
    })
    public ResultData getConfigByIdAnd(Long id,Integer edition) {
        return ResultData.ok(tfProcessConfigLService.getConfigByIdAndEdition(id,edition));
    }

    /**
     * 通过租户和标签查询流程名称
     * @param tenantId 租户id
     * @param label 标签id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过租户和标签查询流程名称" )
    @ApiOperation(value = "通过租户和标签查询流程名称")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tenantId", value = "tenantId", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "label", value = "label", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "appLabe", value = "App label", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "processType", value = "processType 流程类型", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "businessType", value = "businessType 业务类型", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "edition", value = "edition", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "processStep", value = "processStep", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "isMould", value = "是否模板0否1是", required = true, dataType = "Short")
    })
    @GetMapping("/configNameInfos")
    public ResultData getConfigName(Long tenantId,Long label,Long appLabe,Long processType,Long businessType,Integer edition,Integer processStep,Short isMould){
        return ResultData.ok(tfProcessConfigLService.getConfigName(tenantId,label, appLabe,processType,businessType,edition,processStep,isMould));
    }

    /**
     * 根据标签、页面和app控件获取步骤和节点
     * @param dto 查询流程配置中的步骤节点
     * @return ResultRespone
     */
    @ToFlyAppLog(title="根据标签、页面和app控件获取步骤和节点" )
    @ApiOperation(value = "根据标签、页面和app控件获取步骤和节点")
    @PostMapping("/stepConfig")
    public ResultData getStepConfig(@RequestBody @Valid PCStepLoadDto dto){
        return ResultData.ok(tfProcessConfigLService.getStepConfig(dto));
    }

    /**
     * 新增审批流程配置
     * @param dto 审批流程配置
     * @return ResultData
     */
    @ToFlyAppLog(title="新增审批流程配置" )
    @ApiOperation(value = "新增审批流程配置")
    @PostMapping
    public ResultData save(@RequestBody @Valid PConfigDto dto) {
        Object obj = tfProcessConfigLService.pConfigSave(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    /**
     * 复制审批流程配置
     * @param dto 复制流程参数
     * @return ResultData
     */
    @ToFlyAppLog(title="复制审批流程配置" )
    @ApiOperation(value = "复制审批流程配置")
    @PostMapping("copysave")
    public ResultData copySave(@RequestBody @Valid CopyConfigDto dto){
        Object obj = tfProcessConfigLService.copyConfigSave(dto);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }


    /**
     * 修改流程是否有效
     * @param id 流程主键
     * @param isValid 是否有效1有效；0无效
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改流程是否有效" )
    @ApiOperation(value = "修改流程是否有效")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "isValid", value = "isValid 是否有效1有效；0无效", required = true, dataType = "Short")
    })
    @GetMapping("/isStart")
    public ResultData updateById(Long id,Short isValid) {
        if(tfProcessConfigLService.updateValidById(id,isValid)){
            return ResultData.ok();
        }else {
            return ResultData.fail();
        }
    }


    /**
     * 通过id删除审批流程配置
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id删除审批流程配置" )
    @ApiOperation(value = "通过ID删除审批流程配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultData removeById(@PathVariable Long id) {
      return ResultData.ok(tfProcessConfigLService.removeById(id));
    }

}
