package com.tofly.workflow.processconfig.controller;

import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.processconfig.entity.dto.ULabelOrServiceDto;
import com.tofly.workflow.processconfig.service.TfProcessConfigStepLService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 审批流程配置步骤
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfprocessconfigStepl")
@Api(tags="审批流程配置步骤接口")
public class TfProcessConfigStepLController extends BaseController {

    private final TfProcessConfigStepLService processConfigStepLService;

    /**
     * 根据租户工程查询流程名称
     * @param id id
     * @return ResultData
     */
    @ToFlyAppLog(title="根据租户工程查询流程名称" )
    @GetMapping("/configNames" )
    @ApiOperation(value = "根据租户工程查询流程名称")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "配置id", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "edition", value = "版本号", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "stepOrder", value = "步骤序号", required = false, dataType = "Integer")
    })
    public ResultData getProcessConfigName(Long id, Integer edition,Integer stepOrder){
        return ResultData.ok(processConfigStepLService.getPCStepUser(id,edition,stepOrder));
    }

    @ToFlyAppLog(title="修改label或service名称" )
    @GetMapping("/updateLabel" )
    @ApiOperation(value = "修改label或service名称")
    public ResultData updateLabelOrService(ULabelOrServiceDto dto){

        return ResultData.ok(processConfigStepLService.updateLabelOrService(dto));
    }



}
