package com.tofly.workflow.processconfig.entity.dto;

import com.tofly.workflow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

/**
 * 流程配置基础信息dto
 */
@Data
@EqualsAndHashCode
public class PConfigInfoDto implements Serializable {
    private static final long serialVersionUID = 1L;
    /**
     * 主流程配置ID
     */
    @ApiModelProperty(value = "主流程配置ID")
    private String mainConfigId;
    /**
     * 流程CODE
     */
    @ApiModelProperty(value = "流程CODE")
    private String processCode;
    /**
     * 部门ID集合；用逗号分隔(预留)
     */
    @ApiModelProperty(value = "部门ID集合；用逗号分隔(预留)")
    private String depts;
    /**
     * 流程处理时限（单位分钟）
     */
    @ApiModelProperty(value = "流程处理时限（单位分钟）")
    private Long timeLimit;

    /**
     * 流程完结推送数据（完结信息推送到队列） 0:不推送;1推送
     */
    @NotNull(message = "isendTidings"+ FlowConstant.ISNOTNUL)
    @Max(value = 1,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 0,message = FlowConstant.DATA_LOWER_LIMIT)
    @ApiModelProperty(value = "流程完结推送数据（完结信息推送到队列） 0:不推送;1推送")
    private Short isendTidings;
    /**
     * 是否催办 0:否；1是（是必须指定督办人）
     */
    @NotNull(message = "isurge"+ FlowConstant.ISNOTNUL)
    @Max(value = 1,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 0,message = FlowConstant.DATA_LOWER_LIMIT)
    @ApiModelProperty(value = "是否催办 0:否；1是（是必须指定督办人）")
    private Short isurge;
    /**
     * 附件id用逗号分隔
     */
    @ApiModelProperty(value = "附件id用逗号分隔")
    private String annex;
    /**
     * 驳回方式1:处理流程驳回；2主流程驳回
     */
    @NotNull(message = "reiectMode"+ FlowConstant.ISNOTNUL)
    @Max(value = 2,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 1,message = FlowConstant.DATA_LOWER_LIMIT)
    @ApiModelProperty(value = "驳回方式1:处理流程驳回；2主流程驳回")
    private Short reiectMode;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 流程督办人员列表
     */
    @Valid
    @ApiModelProperty(value = "流程督办人员列表")
    private List<PConfigStepUserDto> stepUsers;

}
