package com.tofly.workflow.processconfig.entity.dto;

import com.tofly.workflow.common.FlowConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;


/**
 * 流程配置流程dto
 */
@Data
@EqualsAndHashCode
public class PConfigStepDto implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 步骤序号;从1开始
     */
    @NotNull(message = "stepOrder"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "步骤序号;从1开始")
    private Integer stepOrder;
    /**
     * 步骤名称
     */
    @ApiModelProperty(value = "步骤名称")
    private String stepName;
    /**
     * 业务标签(业务菜单ID)
     */
    @ApiModelProperty(value = "业务标签(业务菜单ID)")
    private Long label;

    /**
     * APP业务标签(业务菜单ID)
     */
    @ApiModelProperty(value = "APP业务标签(业务菜单ID)")
    private Long appLabe;

    /**
     * 业务标签(业务菜单ID)
     */
    @ApiModelProperty(value = "业务标签(业务菜单ID)")
    private Long listLabel;

    /**
     * 步骤类型1:填充；2审核
     */
    @NotNull(message = "stepType"+ FlowConstant.ISNOTNUL)
    @Max(value = 2,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 1,message = FlowConstant.DATA_LOWER_LIMIT)
    @ApiModelProperty(value = "步骤类型1:填充；2审核")
    private Short stepType;

    /**
     * 处理方式1:通过;2:驳回;3转派;4归档;(多个用逗分割)
     */
    @NotEmpty(message = "handleModel 不能为空")
    @ApiModelProperty(value = "处理方式1:通过;2:驳回;3转派;4归档;(多个用逗分割)")
    private String handleModel;
    /**
     * 可查看部门多个用逗号分隔
     */
    @ApiModelProperty(value = "可查看部门多个用逗号分隔")
    private String seeDept;
    /**
     * 处理子层级默认1
     */
    @NotNull(message = "nodeNumber"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "处理子层级默认1")
    private Integer nodeNumber;
    /**
     * 强制处理层级默认1 小于等于处理层级
     */
    @NotNull(message = "coerceLevel"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "强制处理层级默认1 小于等于处理层级")
    private Integer coerceLevel;
    /**
     * 上一步序号(同组逗号分隔;不同组竖线分隔如：1,2|3)
     */
    @NotEmpty(message = "upperStep 不能为空")
    @ApiModelProperty(value = "上一步序号(同组逗号分隔;不同组竖线分隔如：1,2|3)")
    private String upperStep;
    /**
     * 下步处理方式（0单一处理;1并行）
     */
    @NotNull(message = "nextHandleType"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "下步处理方式（0单一处理;1并行）")
    private Short nextHandleType;

    /**
     * 下一步序号(多个逗号分隔; -1流程结束; -2分支终止)
     */
    @NotEmpty(message = "nextStep 不能为空")
    @ApiModelProperty(value = "下一步序号(多个逗号分隔; -1流程结束; -2分支终止)")
    private String nextStep;
    /**
     * 处理环节(方式)0自定义;1选择历史步骤
     */
    @NotNull(message = "handleMode"+ FlowConstant.ISNOTNUL)
    @Max(value = 1,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 0,message = FlowConstant.DATA_LOWER_LIMIT)
    @ApiModelProperty(value = "处理环节(方式)0自定义;1选择历史步骤")
    private Integer handleMode;

    /**
     * 是否唯一处理人 1是 0否
     */
    @NotNull(message = "isAppointUser"+ FlowConstant.ISNOTNUL)
    @ApiModelProperty(value = "是否唯一处理人 1是 0否")
    private Short isAppointUser;

    /**
     * 流程处理时限(单位天)
     */
    @ApiModelProperty(value = "流程处理时限(单位时)")
    private Integer timeLimit;
    /**
     * 日类型 0自然日;1工作日
     */
    @ApiModelProperty(value = "日类型 0自然日;1工作日")
    private Short dayType;

    /**
     * 提前提醒时限(天)
     */
    @ApiModelProperty(value = "提前提醒时限(天)")
    private Integer timeRemind;
    /**
     * 驳回到指定步骤序号
     */
    @ApiModelProperty(value = "驳回到指定步骤序号")
    private Integer returnToStep;
    /**
     * 是否允许提交后编辑 1是 0否
     */
    @NotNull(message = "isEdit"+ FlowConstant.ISNOTNUL)
    @Max(value = 1,message = FlowConstant.DATA_UPPER_LIMIT)
    @Min(value = 0,message = FlowConstant.DATA_LOWER_LIMIT)
    @ApiModelProperty(value = "是否允许提交后编辑 1是 0否")
    private Short isEdit;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 流程步骤人员信息表
     */
    @ApiModelProperty(value = "流程步骤人员信息表")
    private List<PConfigStepUserDto> stepUsers;

    /**
     * 流程步骤引用历史步骤
     */
    @ApiModelProperty(value = "流程步骤引用历史步骤")
    private List<LHistoryStepDto> historyStep;

    /**
     * 流程配置步骤层级关联页面
     */
    @Valid
    @ApiModelProperty(value = "流程配置步骤层级关联页面")
    private List<PControlDto> controls;

}
