/*
 * Author: wangjian
 * 生成Service
 */
package com.tofly.workflow.processconfig.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.dto.CopyConfigDto;
import com.tofly.workflow.processconfig.entity.dto.PCStepLoadDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigLoadDto;
import com.tofly.workflow.processconfig.entity.vo.*;

import java.util.List;

/**
 * 审批流程配置
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
public interface TfProcessConfigLService extends IService<TfProcessConfigL> {

    /**
     * 存储流程配置信息
     * @param dto 流程信息
     * @return
     */
    Object pConfigSave(PConfigDto dto);

    /**
     * 根据条件查询流程配置列表
     * @param page 分页
     * @param dto 查询条件
     * @return 分页列表
     */
    Page<PConfigPageVo> configLoadPage(Page page, PConfigLoadDto dto);

    /**
     * 根据配置ID查询配置流程信息
     * @param id 配置ID
     * @return 配置详情
     */
    PConfigVo getProcessConfigById(Long id);

    /**
     * 修改流程配置是否有效状态
     * @param id 流程配置ID
     * @param isValid 是否有效1有效；0无效
     * @return 修改结果
     */
    boolean updateValidById(Long id, Short isValid);

    /**
     * 根据流程ID，获取流程类型及code
     * @param configId 流程配置ID
     * @return 类型信息
     */
    CTypeVo getConfigType(Long configId);

    /**
     * 根据分组查询流程名称
     * @param id 租户id
     * @param isMould 是否模板0否1是
     * @return 流程名称列表
     */
    List<PConfigNameVo> getProcessConfigName(Long id, Short isMould);

    /**
     * 根据租户、标签查询流程名称
     * @param tenantId 租户id
     * @param label 标签
     * @param edition
     * @param isMould 是否模板0否1是
     * @return 流程名称
     */
    List<ConfigNameInfoVo> getConfigName(Long tenantId, Long label, Long appLabe,Long processType,Long businessType, Integer edition,Integer processStep, Short isMould);

    /**
     * 根据控件获取流程配置步骤信息
     * @param dto 查询条件
     * @return 步骤信息
     */
    List<ConfigStepVo> getStepConfig(PCStepLoadDto dto);

    /**
     * 流程配置
     * @param id 流程id
     * @param edition 版本号
     * @return 流程配置信息
     */
    PConfigVo getConfigByIdAndEdition(Long id, Integer edition);

    /**
     * 复制审批流程配置
     * @param dto 查询条件
     * @return
     */
    Object copyConfigSave(CopyConfigDto dto);
}
