/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.processconfig.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.processconfig.entity.TfConfigStepUserL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigInfoL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigStepL;
import com.tofly.workflow.processconfig.entity.dto.PCSUserUpdateDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigStepDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigStepUserDto;
import com.tofly.workflow.processconfig.service.TfConfigStepUserLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigInfoLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigStepLService;
import io.swagger.models.auth.In;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.workflow.processconfig.mapper.TfConfigStepUserLMapper;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 流程配置步骤配置的人员
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfConfigStepUserLService")
public class TfConfigStepUserLServiceImpl extends ServiceImpl<TfConfigStepUserLMapper, TfConfigStepUserL> implements TfConfigStepUserLService {


    /**
     * 配置service
     */
    @Autowired
    private TfProcessConfigLService PConfigService;

    /**
     * 配置基础信息service
     */
    @Autowired
    private TfProcessConfigInfoLService pcinfoService;

    /**
     * 配置步骤service
     */
    @Autowired
    private TfProcessConfigStepLService PCStepService;

    /**
     * 保存流程配置步骤人员
     * @param stepUsers 人员信息
     * @param stepId 关联ID
     * @param edition 版本
     * @param shortTwo 人员类型1普通;2督办
     * @return
     */
    @Override
    public boolean stepUserSave(List<PConfigStepUserDto> stepUsers, Long stepId, Integer edition, Short shortTwo) {
        List<TfConfigStepUserL> list = new ArrayList<>();
        fillConfigStepUser(stepUsers, stepId, edition, shortTwo, list);
        return this.saveBatch(list);
    }

    /**
     * 填充步骤处理人员
     * @param stepUsers 输入处理人员集合
     * @param stepId 步骤ID
     * @param edition 配置版本
     * @param shortTwo 人员类型1普通;2督办
     * @param list 数据集合
     */
    private void fillConfigStepUser(List<PConfigStepUserDto> stepUsers, Long stepId, Integer edition, Short shortTwo, List<TfConfigStepUserL> list) {
        for (PConfigStepUserDto userDto: stepUsers) {
            TfConfigStepUserL stepUser = new TfConfigStepUserL();
            BeanUtils.copyProperties(userDto,stepUser);
            stepUser.setStepId(stepId);
            stepUser.setEdition(edition);
            stepUser.setIsDel(FlowConstant.SHORT_ZERO);
            stepUser.setUserType(shortTwo);
            list.add(stepUser);
        }
    }

    /**
     * 保存流程步骤处理人员
     * @param steps 流程步骤信息
     * @param stepDtos 输入流程步骤信息
     * @param shortTwo 人员类型1普通;2督办
     * @return
     */
    @Override
    public boolean configStepUserSave(Map<Integer, TfProcessConfigStepL> steps, List<PConfigStepDto> stepDtos, Short shortTwo) {
        List<TfConfigStepUserL> list = new ArrayList<>();
        for (PConfigStepDto stepDto:stepDtos) {
            TfProcessConfigStepL pcfs = steps.get(stepDto.getStepOrder());
            if(pcfs==null){
                return false;
            }
            if(pcfs.getHandleMode()==FlowConstant.INTEGER_ZERO) {
                fillConfigStepUser(stepDto.getStepUsers(), pcfs.getId(), pcfs.getEdition(), shortTwo, list);
            }
        }
        if(CollUtil.isNotEmpty(list)){
            return this.saveBatch(list);
        }
        return true;
    }

    /**
     * 修改步骤处理人员
     * @param dto 处理人员信息
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object updateStepUserByconfigId(PCSUserUpdateDto dto) {

        String check = updateUserCheck(dto);
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        TfProcessConfigL config = PConfigService.getById(dto.getConfigId());
        if(config==null||config.getIsDel()==FlowConstant.SHORT_ONE){
            return FlowConstant.CONFIG_NOT_EXIST;
        }

        if(dto.getUserType()==FlowConstant.SHORT_TWO) {
            TfProcessConfigInfoL pci = pcinfoService.getOne(new QueryWrapper<TfProcessConfigInfoL>().lambda()
                    .eq(TfProcessConfigInfoL::getConfigId,config.getId()).eq(TfProcessConfigInfoL::getEdition,config.getEdition())
                    .eq(TfProcessConfigInfoL::getIsDel,FlowConstant.SHORT_ZERO).eq(TfProcessConfigInfoL::getIsValid,FlowConstant.SHORT_ONE));
            this.update(new UpdateWrapper<TfConfigStepUserL>().lambda().eq(TfConfigStepUserL::getStepId, pci.getId())
                    .eq(TfConfigStepUserL::getUserType, dto.getUserType()).eq(TfConfigStepUserL::getIsDel, FlowConstant.SHORT_ZERO)
                    .set(TfConfigStepUserL::getIsDel, FlowConstant.SHORT_ONE));
            return stepUserSave(dto.getStepUsers(),pci.getId(),config.getEdition(),dto.getUserType());
        }else {

            TfProcessConfigStepL pcs = PCStepService.getOne(new QueryWrapper<TfProcessConfigStepL>().lambda().eq(TfProcessConfigStepL::getConfigId,dto.getConfigId())
                    .eq(TfProcessConfigStepL::getId,dto.getStepId()).eq(TfProcessConfigStepL::getIsDel,FlowConstant.SHORT_ZERO)
                    .eq(TfProcessConfigStepL::getIsValid,FlowConstant.SHORT_ONE));
            if(pcs==null){
                return FlowConstant.CONFIG_STEP_ERROR;
            }

            this.update(new UpdateWrapper<TfConfigStepUserL>().lambda().eq(TfConfigStepUserL::getStepId, dto.getStepId())
                    .eq(TfConfigStepUserL::getUserType, dto.getUserType()).eq(TfConfigStepUserL::getIsDel, FlowConstant.SHORT_ZERO)
                    .set(TfConfigStepUserL::getIsDel, FlowConstant.SHORT_ONE));
            return stepUserSave(dto.getStepUsers(),dto.getStepId(),config.getEdition(),dto.getUserType());
        }
    }

    /**
     * 获取步骤处理人员
     * @param configId 流程配置id
     * @param edition 版本
     * @param processStep 步骤编号
     * @param stepNode 节点编号
     * @return
     */
    @Override
    public List<Long> getUserIds(Long configId, Integer edition, Integer processStep, Integer stepNode) {
        List<Long> uIds = this.baseMapper.getStepUsers(configId,edition,processStep,stepNode);
        List<Long> pIds = this.baseMapper.getStepPostUsers(configId,edition,processStep,stepNode);
        if(CollUtil.isNotEmpty(uIds)&&CollUtil.isNotEmpty(pIds)){
            return (List<Long>)CollectionUtils.union(uIds,pIds);
        }else if(CollUtil.isNotEmpty(uIds)){
            return uIds;
        }else if(CollUtil.isNotEmpty(pIds)){
            return pIds;
        }
        return null;
    }

    /**
     * 根据流程配置步骤节点获取处理人员
     * @param configId 流程配置id
     * @param edition 配置版本
     * @param processStep 流程步骤
     * @param stepNode 步骤节点
     * @return
     */
    @Override
    public List<TfConfigStepUserL> getStepNodeUsers(Long configId, Integer edition, Integer processStep, Integer stepNode) {
        return this.baseMapper.getNodeUsers(configId,edition,processStep,stepNode);
    }

    /**
     * 校验修改人员信息
     * @param dto 人员信息集合
     * @return 校验结果
     */
    private String updateUserCheck(PCSUserUpdateDto dto) {
        int count = PConfigService.count(new QueryWrapper<TfProcessConfigL>().lambda().eq(TfProcessConfigL::getId,dto.getConfigId())
                .eq(TfProcessConfigL::getIsDel,FlowConstant.SHORT_ZERO));
        if(count==FlowConstant.INTEGER_ZERO){
            return FlowConstant.DATA_NOTHINGNESS;
        }
        if(dto.getUserType()==FlowConstant.SHORT_ONE){
            TfProcessConfigStepL pcs = PCStepService.getOne(new QueryWrapper<TfProcessConfigStepL>().lambda().eq(TfProcessConfigStepL::getId,dto.getStepId())
                    .eq(TfProcessConfigStepL::getConfigId,dto.getConfigId()));
            Set<Integer> levels = dto.getStepUsers().stream().map(x->x.getStepNode()).collect(Collectors.toSet());
            if(pcs==null||levels.size()!=pcs.getNodeNumber()){
                return FlowConstant.USER_CONFIG_WRONG;
            }
        }
        return null;
    }

}
