/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.processconfig.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.processconfig.entity.TfPageControlL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigStepL;
import com.tofly.workflow.processconfig.entity.dto.PConfigStepDto;
import com.tofly.workflow.processconfig.entity.dto.PControlDto;
import com.tofly.workflow.processconfig.entity.dto.ULabelOrServiceDto;
import com.tofly.workflow.processconfig.entity.vo.PCStepVo;
import com.tofly.workflow.processconfig.entity.vo.PControlVo;
import com.tofly.workflow.processconfig.entity.vo.ServiceImplVo;
import com.tofly.workflow.processconfig.service.TfPageControlLService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.tofly.workflow.processconfig.mapper.TfPageControlLMapper;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 页面空间展示
 *
 * @author xqf
 * @date Tue Aug 09 00:00:00 CST 2022
 */
@Service("tfPageControlLService")
public class TfPageControlLServiceImpl extends ServiceImpl<TfPageControlLMapper, TfPageControlL> implements TfPageControlLService {

    /**
     * 保存流程配置页面
     * @param steps 流程步骤信息
     * @param stepDtos 输入流程配置信息包含页面配置
     * @return
     */
    @Override
    public boolean pageControlSave(Map<Integer, TfProcessConfigStepL> steps, List<PConfigStepDto> stepDtos) {
        List<TfPageControlL> list = new ArrayList<>();
        for (PConfigStepDto stepDto:stepDtos) {
            TfProcessConfigStepL configStep = steps.get(stepDto.getStepOrder());
            if(configStep==null){
                return false;
            }
            List<PControlDto> controls = stepDto.getControls();
            if(CollUtil.isNotEmpty(controls)) {
                for (PControlDto dto : controls) {
                    TfPageControlL control = new TfPageControlL();
                    BeanUtils.copyProperties(dto, control);
                    if(control.getIsPush()==null){
                        control.setIsPush(FlowConstant.SHORT_ZERO);
                    }
                    control.setStepId(configStep.getId());
                    control.setIsDel(FlowConstant.SHORT_ZERO);
                    list.add(control);
                }
            }
        }
        if(CollUtil.isNotEmpty(list)){
            return this.saveBatch(list);
        }
        return true;
    }

    /**
     * 根据配置id获取service
     * @param configId 配置id
     * @param edition 版本id
     * @return 配置service
     */
    @Override
    public List<ServiceImplVo> getServiceImplByConfig(Long configId, Integer edition) {
        return this.baseMapper.getServiceImpl(configId,edition);
    }

    @Override
    public ServiceImplVo getNodeService(Long configId, Integer edition, Integer processStep, Integer stepNode) {
        return this.baseMapper.getNodeService(configId,edition,processStep,stepNode);
    }

    /**
     * 修改label或service名称
     * @param dto 参数
     * @return
     */
    @Override
    public Integer updateLabelOrService(ULabelOrServiceDto dto) {
        return this.baseMapper.updateLabelOrService(dto);
    }

    /**
     * 复制保存流程配置页面
     * @param steps 流程步骤信息
     * @param stepVos 输入流程配置信息包含页面配置
     * @return
     */
    @Override
    public boolean copyPageControlSave(Map<Integer, TfProcessConfigStepL> steps, List<PCStepVo> stepVos) {
        List<TfPageControlL> list = new ArrayList<>();
        for (PCStepVo psvo:stepVos) {
            TfProcessConfigStepL configStep = steps.get(psvo.getStepOrder());
            if(configStep==null){
                return false;
            }
            List<PControlVo> controls = psvo.getControlVos();
            if(CollUtil.isNotEmpty(controls)) {
                for (PControlVo vo : controls) {
                    TfPageControlL control = new TfPageControlL();
                    BeanUtils.copyProperties(vo, control);
                    control.setId(null);
                    control.setStepId(configStep.getId());
                    control.setIsDel(FlowConstant.SHORT_ZERO);
                    list.add(control);
                }
            }
        }
        if(CollUtil.isNotEmpty(list)){
            return this.saveBatch(list);
        }
        return true;
    }
}
