/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.processconfig.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfBaseTypeP;
import com.tofly.workflow.baseconf.service.TfBaseTypePService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.processconfig.entity.TfProcessConfigInfoL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.dto.PConfigInfoDto;
import com.tofly.workflow.processconfig.entity.vo.PCInfoVo;
import com.tofly.workflow.processconfig.service.TfConfigStepUserLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigInfoLService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.workflow.processconfig.mapper.TfProcessConfigInfoLMapper;

/**
 * 流程配置信息
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfProcessConfigInfoLService")
public class TfProcessConfigInfoLServiceImpl extends ServiceImpl<TfProcessConfigInfoLMapper, TfProcessConfigInfoL> implements TfProcessConfigInfoLService {

    /**
     * 基础类型
     */
    @Autowired
    private TfBaseTypePService baseTypeService;

    /**
     * 流程配置的人员
     */
    @Autowired
    private TfConfigStepUserLService cSUserLService;

    /**
     * 配置信息保存校验
     * @param infoDto 配置信息
     * @param isMould
     * @return 校验结果
     */
    @Override
    public String checkConfigInfoSave(PConfigInfoDto infoDto,  Short isMould) {
        if(FlowConstant.SHORT_ONE.equals(infoDto.getIsurge())&& FlowConstant.SHORT_ZERO.equals(isMould)){
            if(CollUtil.isEmpty(infoDto.getStepUsers())){
                return "stepUsers"+FlowConstant.USER_NOT_NULL;
            }
        }
        return null;
    }

    /**
     * 保存流程配置基础信息
     * @param infoDto 流程配置基础信息
     * @param config 流程配置
     * @param size 流程步骤数
     * @return 保存结果
     */
    @Override
    public boolean configInfoSave(PConfigInfoDto infoDto, TfProcessConfigL config, int size) {
        TfProcessConfigInfoL pCInfo = new TfProcessConfigInfoL();
        BeanUtils.copyProperties(infoDto,pCInfo);
        pCInfo.setConfigId(config.getId());
        pCInfo.setEdition(config.getEdition());
        pCInfo.setStepNumber(size);
        //TfBaseTypeP typeP = baseTypeService.getById(config.getBusinessType());
        TfBaseTypeP typeP = baseTypeService.getById(config.getProcessType());
        pCInfo.setProcessCode(typeP.getTypeCode());
        if(config.getIsMould()==FlowConstant.SHORT_ZERO){
            pCInfo.setTenantId(SecurityUtils.getTenantId());
        }
        pCInfo.setIsDel(FlowConstant.SHORT_ZERO);
        pCInfo.setIsValid(FlowConstant.SHORT_ONE);
        if(!this.save(pCInfo)){
            return false;
        }
        /**模板不保存督办人员信息*/
        if(config.getIsMould()==FlowConstant.SHORT_ZERO&&CollUtil.isNotEmpty(infoDto.getStepUsers())) {
            if (!cSUserLService.stepUserSave(infoDto.getStepUsers(), pCInfo.getId(), config.getEdition(), FlowConstant.SHORT_TWO)) {
                return false;
            }
        }
        return true;
    }

    /**
     * 复制保存流程配置基础信息
     * @param infoVo 流程配置基础信息
     * @param config 流程配置
     * @return
     */
    @Override
    public boolean copyConfigInfoSave(PCInfoVo infoVo, TfProcessConfigL config) {
        TfProcessConfigInfoL pCInfo = new TfProcessConfigInfoL();
        BeanUtils.copyProperties(infoVo,pCInfo);
        pCInfo.setId(null);
        pCInfo.setConfigId(config.getId());
        pCInfo.setEdition(config.getEdition());
        pCInfo.setIsValid(FlowConstant.SHORT_ONE);
        pCInfo.setIsDel(FlowConstant.SHORT_ZERO);
        pCInfo.setTenantId(config.getTenantId());
        return this.save(pCInfo);
    }

}
