/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.processconfig.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfBaseTypeP;
import com.tofly.workflow.baseconf.service.TfBaseTypePService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.dto.CopyConfigDto;
import com.tofly.workflow.processconfig.entity.dto.PCStepLoadDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigLoadDto;
import com.tofly.workflow.processconfig.entity.vo.*;
import com.tofly.workflow.processconfig.mapper.TfProcessConfigLMapper;
import com.tofly.workflow.processconfig.service.TfProcessConfigInfoLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigLService;
import com.tofly.workflow.processconfig.service.TfProcessConfigStepLService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 审批流程配置
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfProcessConfigLService")
public class TfProcessConfigLServiceImpl extends ServiceImpl<TfProcessConfigLMapper, TfProcessConfigL> implements TfProcessConfigLService {

    /**
     * 基础类型
     */
    @Autowired
    private TfBaseTypePService baseTypeService;

    /**
     * 流程信息
     */
    @Autowired
    private TfProcessConfigInfoLService pCInfoService;

    /**
     * 流程步骤
     */
    @Autowired
    private TfProcessConfigStepLService pCStepLService;


    /**
     * 存储流程配置信息
     * @param dto 流程配置
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Object pConfigSave(PConfigDto dto) {
        /**流程配置数据校验*/
        String check = checkConfigSave(dto);
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        TfProcessConfigL config = null;
        /**修改和保存流程配置*/
        if(dto.getId()==null) {
            config = new TfProcessConfigL();
            BeanUtils.copyProperties(dto, config);
            if(config.getIsMould()==FlowConstant.SHORT_ZERO){
                config.setTenantId(SecurityUtils.getTenantId());
            }
            TfBaseTypeP typeP = baseTypeService.getById(config.getBusinessType());
            //TfBaseTypeP typeP = baseTypeService.getById(config.getProcessType());
            config.setProcessNumber(typeP.getTypeCode());
            config.setEdition(FlowConstant.INTEGER_ONE);
            config.setIsDel(FlowConstant.SHORT_ZERO);
            config.setIsValid(FlowConstant.SHORT_ONE);
            if (!this.save(config)) {
                return FlowConstant.FAILED;
            }
        } else {
            config = this.getById(dto.getId());
            config.setEdition(config.getEdition()+1);
            if(!this.update(new UpdateWrapper<TfProcessConfigL>().lambda().set(TfProcessConfigL::getEdition,config.getEdition())
                    .eq(TfProcessConfigL::getId,dto.getId()))){
                return FlowConstant.FAILED;
            }
        }
        /**保存流程配置基础信息*/
        if(!pCInfoService.configInfoSave(dto.getInfoDto(),config,dto.getStepDtos().size())){
            throw new RuntimeException(FlowConstant.FAILED);
        }
        /**保存流程配置步骤信息*/
        if(!pCStepLService.configStepSave(dto.getStepDtos(),config)){
            throw new RuntimeException(FlowConstant.FAILED);
        }
        return true;
    }

    /**
     * 根据条件查询流程配置列表
     * @param page 分页
     * @param dto 查询条件
     * @return
     */
    @Override
    public Page<PConfigPageVo> configLoadPage(Page page, PConfigLoadDto dto) {
        return this.baseMapper.getConfigLoadPage(page,dto);
    }

    /**
     * 根据配置ID查询配置流程信息
     * @param id 配置ID
     * @return
     */
    @Override
    public PConfigVo getProcessConfigById(Long id) {
        if(id==null){
            return null;
        }
        return this.baseMapper.getProcessConfigById(id);
    }

    /**
     * 修改流程配置是否有效状态
     * @param id 流程配置ID
     * @param isValid 是否有效1有效；0无效
     * @return
     */
    @Override
    public boolean updateValidById(Long id, Short isValid) {
        return this.update(new UpdateWrapper<TfProcessConfigL>().lambda().eq(TfProcessConfigL::getId,id)
                .eq(TfProcessConfigL::getIsDel,FlowConstant.SHORT_ZERO)
                .set(TfProcessConfigL::getIsValid,isValid));
    }

    /**
     * 根据流程ID，获取流程类型及code
     * @param id 流程配置ID
     * @return
     */
    @Override
    public CTypeVo getConfigType(Long id) {
        return this.baseMapper.getConfigType(id);
    }

    /**
     * 根据分组查询流程名称
     * @param id 租户id
     * @param isMould 是否模板0否1是
     * @return 流程名称列表
     */
    @Override
    public List<PConfigNameVo> getProcessConfigName(Long id, Short isMould) {
        return this.baseMapper.getProcessConfigName(id,isMould);
    }

    /**
     * 根据租户、标签查询流程名称
     * @param tenantId 租户id
     * @param label 标签
     * @param edition 版本
     * @param isMould 是否模板0否1是
     * @return 流程名称
     */
    @Override
    public List<ConfigNameInfoVo> getConfigName(Long tenantId, Long label, Long appLabe,Long processType,Long businessType, Integer edition,Integer processStep, Short isMould) {
        return this.baseMapper.getConfigNameInfo(tenantId,label,appLabe,processType,businessType,edition,processStep,isMould);
    }

    /**
     * 根据控件获取流程配置步骤信息
     * @param dto 查询条件
     * @return 步骤信息
     */
    @Override
    public List<ConfigStepVo> getStepConfig(PCStepLoadDto dto) {
        return this.baseMapper.getStepConfig(dto);
    }

    /**
     * 流程配置
     * @param id 流程id
     * @param edition 版本号
     * @return 流程配置
     */
    @Override
    public PConfigVo getConfigByIdAndEdition(Long id, Integer edition) {
        return this.baseMapper.getConfigByIdAndEdition(id,edition);
    }

    /**
     * 复制审批流程配置
     * @param dto 查询条件
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Object copyConfigSave(CopyConfigDto dto) {
        PConfigVo pvo = this.baseMapper.getProcessConfigById(dto.getId());
        if(pvo==null){
            return FlowConstant.CONFIG_NOT_EXIST;
        }
        int count = this.count(new QueryWrapper<TfProcessConfigL>().lambda().eq(TfProcessConfigL::getBusinessType,pvo.getBusinessType())
                .eq(TfProcessConfigL::getProcessType,pvo.getProcessType()).eq(TfProcessConfigL::getTenantId,dto.getTenantId())
                .eq(TfProcessConfigL::getProcessName,dto.getProcessName()).eq(TfProcessConfigL::getIsValid,FlowConstant.SHORT_ONE)
                .eq(TfProcessConfigL::getIsDel,FlowConstant.SHORT_ZERO));
        if(count!=0){
            return FlowConstant.NAME_EXISTS;
        }
        TfProcessConfigL config = new TfProcessConfigL();
        BeanUtils.copyProperties(pvo, config);
        config.setProcessName(dto.getProcessName());
        config.setTenantId(dto.getTenantId());
        config.setId(null);
        config.setIsDel(FlowConstant.SHORT_ZERO);
        config.setEdition(FlowConstant.INTEGER_ONE);
        config.setIsMould(FlowConstant.SHORT_ZERO);
        if (!this.save(config)) {
            return FlowConstant.FAILED;
        }
        if(!pCInfoService.copyConfigInfoSave(pvo.getInfoVo(),config)){
            throw new RuntimeException(FlowConstant.FAILED);
        }
        if(!pCStepLService.copyConfigStepSave(pvo.getStepVos(),config)){
            throw new RuntimeException(FlowConstant.FAILED);
        }
        return config.getId();
    }

    /**
     * 流程配置流程校验
     * @param dto 流程配置信息
     * @return 校验结果
     */
    private String checkConfigSave(PConfigDto dto){
        if(dto.getId()!=null){
            int count = this.count( new QueryWrapper<TfProcessConfigL>().lambda()
                    .eq(TfProcessConfigL::getId,dto.getId()).eq(TfProcessConfigL::getIsDel,FlowConstant.SHORT_ZERO) );
            if(count ==0){
                return FlowConstant.DATA_NOTHINGNESS;
            }
        } else {
            /**流程配置名称重复*/
            int count = this.count(new QueryWrapper<TfProcessConfigL>().lambda().eq(TfProcessConfigL::getBusinessType,dto.getBusinessType())
                    .eq(TfProcessConfigL::getProcessName,dto.getProcessName()).eq(TfProcessConfigL::getTenantId,SecurityUtils.getTenantId())
                    .eq(TfProcessConfigL::getIsDel,FlowConstant.SHORT_ZERO).eq(TfProcessConfigL::getIsMould,dto.getIsMould()));
            if(count>0){
                return FlowConstant.CONFING_EXISTS;
            }
            count = baseTypeService.count(new QueryWrapper<TfBaseTypeP>().lambda().eq(TfBaseTypeP::getId, dto.getBusinessType())
                    .eq(TfBaseTypeP::getIsDel,FlowConstant.SHORT_ZERO).eq(TfBaseTypeP::getIsValid,FlowConstant.SHORT_ONE));
            if (count == 0) {
                return FlowConstant.BASETYPE_NOT_EXISTS;
            }
        }
        /**配置信息保存校验*/
        String check = pCInfoService.checkConfigInfoSave(dto.getInfoDto(),dto.getIsMould());
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        /**配置步骤信息保存校验*/
        check = pCStepLService.checkStepSave(dto.getStepDtos(),dto.getIsMould());
        if(StringUtils.isNotEmpty(check)){
            return check;
        }

        return null;
    }

}
