/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.workflow.processconfig.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.StrSplitter;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigStepL;
import com.tofly.workflow.processconfig.entity.dto.LHistoryStepDto;
import com.tofly.workflow.processconfig.entity.dto.PConfigStepDto;
import com.tofly.workflow.processconfig.entity.dto.ULabelOrServiceDto;
import com.tofly.workflow.processconfig.entity.vo.PCStepVo;
import com.tofly.workflow.processconfig.service.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.workflow.processconfig.mapper.TfProcessConfigStepLMapper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 流程配置步骤
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@Service("tfProcessConfigStepLService")
public class TfProcessConfigStepLServiceImpl extends ServiceImpl<TfProcessConfigStepLMapper, TfProcessConfigStepL> implements TfProcessConfigStepLService {

    /**
     * 流程配置
     */
    @Autowired
    private TfProcessConfigLService pConfigLService;

    /**
     * 流程步骤节点人员
     */
    @Autowired
    private TfConfigStepUserLService cStepUserLervice;

    /**
     * 流程步骤节点页面
     */
    @Autowired
    private TfPageControlLService pControlService;

    /**
     * 流程步骤关联历史步骤节点
     */
    @Autowired
    private TfLinkHistoryStepLService lHStepService;

    /**
     * 保存流程步骤
     * @param stepDtos 步骤集合
     * @param isMould 是否模板
     * @return 校验结果
     */
    @Override
    public String checkStepSave(List<PConfigStepDto> stepDtos, Short isMould) {
        if(CollUtil.isEmpty(stepDtos)){
            return FlowConstant.STEP_NOT_NULL;
        }
        Map<Integer,PConfigStepDto> map = stepDtos.stream().collect(Collectors.toMap(PConfigStepDto::getStepOrder,x->x));
        if(stepDtos.size()!=map.size()){
            return FlowConstant.STEP_NUMBER_ERROR;
        }
        int size = stepDtos.size() ;
        for (Integer i=1;i<=size;i++){
            if(map.containsKey(i)){
                PConfigStepDto step = map.get(i);

                if(isMould==FlowConstant.SHORT_ZERO&&step.getIsAppointUser().equals(FlowConstant.SHORT_ZERO)&&step.getHandleModel().contains(FlowConstant.SHORT_THREE.toString())){
                    return FlowConstant.NON_UNIQUE_HANDLER;
                }
                /**页面配置节点数大于等于强制节点数*/
                if(step.getNodeNumber()<step.getCoerceLevel()){
                    return i+FlowConstant.COERCE_NODE_WRONG;
                }
                String check = checkStepUser(isMould,step);
                if (StringUtils.isNotEmpty(check)) return i+check;

                /**上步骤判断*/
                String upper = step.getUpperStep();
                if(i==FlowConstant.INTEGER_ZERO){
                    if(StringUtils.isNotEmpty(upper)&&!upper.trim().equals(FlowConstant.FLAG)){
                        return i+FlowConstant.UPPER_WRONG;
                    }
                } else {
                    if(StringUtils.isEmpty(upper)){
                        return i+FlowConstant.UPPER_WRONG;
                    }
                    List<String> strs = StrSplitter.splitByRegex(upper,"[,|]",0,true,true);
                    for (String number : strs) {
                        if (StringUtils.isEmpty(number)||Integer.valueOf(number)<FlowConstant.INTEGER_ZERO
                                || Integer.valueOf(number) > stepDtos.size()){
                            return i+FlowConstant.UPPER_WRONG;
                        }
                    }
                }
                /**下步骤判断*/
                check = checkNextStep(stepDtos.size(), step, upper);
                if (StringUtils.isNotEmpty(check)) return i+check;
                /**被驳回到节点位置*/
                Integer returnToStep = step.getReturnToStep();
                if(returnToStep!=null){
                    if(returnToStep<FlowConstant.SHORT_ONE||returnToStep>=size){
                        return i+FlowConstant.RETURN_STEP_WRONG;
                    }
                    check = isLab(map,returnToStep,step);
                    if(StringUtils.isNotEmpty(check)) {
                        return i + check;
                    }
                }
            }else {
                return FlowConstant.STEP_NUMBER_ERROR;
            }
        }
        return checkStepProcess(map,"",map.get(FlowConstant.INTEGER_ONE));
    }

    /**
     * 保存流程步骤数据
     * @param stepDtos 流程步骤信息
     * @param config 流程信息
     * @return 保存结果
     */
    @Override
    public boolean configStepSave(List<PConfigStepDto> stepDtos, TfProcessConfigL config) {
        Map<Integer,TfProcessConfigStepL> steps = new HashMap<>();
        for (PConfigStepDto stepDto:stepDtos) {
            TfProcessConfigStepL step = new TfProcessConfigStepL();
            BeanUtils.copyProperties(stepDto, step);
            step.setConfigId(config.getId());
            step.setEdition(config.getEdition());
            step.setIsValid(FlowConstant.SHORT_ONE);
            step.setIsEnable(FlowConstant.SHORT_ONE);
            step.setIsDel(FlowConstant.SHORT_ZERO);
            steps.put(step.getStepOrder(),step);
        }
        /**保存流程步骤*/
        if(!this.saveBatch(steps.values())){
            return false;
        }
        /**保存流程步骤配置页面*/
        if(!pControlService.pageControlSave(steps,stepDtos)){
            return false;
        }
        /**模板不保存处理人员信息*/
        if(config.getIsMould()==FlowConstant.SHORT_ZERO) {
            /**保存流程步骤人员*/
            if (!cStepUserLervice.configStepUserSave(steps, stepDtos, FlowConstant.SHORT_ONE)) {
                return false;
            }

            /**保存步骤关联的历史步骤*/
            if (!lHStepService.linkHistoryStepSave(steps, stepDtos)) {
                return false;
            }
        }
        return true;
    }

    /**
     *
     * @param id 流程配置id
     * @param edition 版本号
     * @param stepOrder 步骤号
     * @return
     */
    @Override
    public List<PCStepVo> getPCStepUser(Long id, Integer edition, Integer stepOrder) {
        return this.baseMapper.getPCStepAndUser(id,edition,stepOrder);
    }

    /**
     * 修改label或service名称
     * @param dto 参数
     * @return 修改结果
     */
    @Override
    public Integer updateLabelOrService(ULabelOrServiceDto dto) {
        return pControlService.updateLabelOrService(dto);
    }

    /**
     * 复制保存流程步骤数据
     * @param stepVos 流程步骤信息
     * @param config 流程信息
     * @return
     */
    @Override
    public boolean copyConfigStepSave(List<PCStepVo> stepVos, TfProcessConfigL config) {
        Map<Integer,TfProcessConfigStepL> steps = new HashMap<>();
        for (PCStepVo vo:stepVos) {
            TfProcessConfigStepL step = new TfProcessConfigStepL();
            BeanUtils.copyProperties(vo, step);
            step.setId(null);
            step.setSeeDept(null);
            step.setConfigId(config.getId());
            step.setEdition(config.getEdition());
            step.setIsValid(FlowConstant.SHORT_ONE);
            step.setIsEnable(FlowConstant.SHORT_ONE);
            step.setIsDel(FlowConstant.SHORT_ZERO);
            steps.put(step.getStepOrder(),step);
        }
        /**保存流程步骤*/
        if(!this.saveBatch(steps.values())){
            return false;
        }
        /**保存流程步骤配置页面*/
        if(!pControlService.copyPageControlSave(steps,stepVos)){
            return false;
        }
        return true;
    }


    /**
     * 校验流程步骤
     * @param map 流程步骤集合
     * @param nodes 经过的节点
     * @param step 本步骤
     * @return 校验结果
     */
    private String checkStepProcess(Map<Integer,PConfigStepDto> map,String nodes,PConfigStepDto step){
        nodes = nodes + FlowConstant.CSV + step.getStepOrder();
        List<String> strs = StrSplitter.splitByRegex(step.getNextStep(),"[,]",0,true,true);
        for (String str: strs) {
            if(str.equals(FlowConstant.INTEGER_MINUS_ONE)||str.equals(FlowConstant.INTEGER_MINUS_TWO)){
                continue;
            }
            if(nodes.contains(str)){
                return FlowConstant.NEXT_STEP_LOOP;
            }
            Integer next = Integer.valueOf(str);
            if(map.containsKey(next)){
                PConfigStepDto pcs = map.get(next);
                String check = checkStepProcess(map,nodes,pcs);
                if (StringUtils.isNotEmpty(check)) return check;
            }
        }
        return null;
    }

    /**
     * 校验下一步节点信息
     * @param size 步骤数
     * @param step 本步骤处理信息
     * @param upper 上步骤节点
     * @return 校验结果
     */
    private String checkNextStep(int size , PConfigStepDto step, String upper) {
        String next = step.getNextStep();
        if(StringUtils.isEmpty(next)){
            return FlowConstant.NEXT_WRONG;
        }
        for (String str:next.split(FlowConstant.CSV)) {
            if(StringUtils.isEmpty(str)){
                return FlowConstant.NEXT_WRONG;
            }
            int number = Integer.valueOf(str);
            if((number<FlowConstant.INTEGER_TWO&&number!=FlowConstant.INTEGER_MINUS_ONE&&number!= FlowConstant.INTEGER_MINUS_TWO)||number> size){
                return FlowConstant.NEXT_WRONG;
            }
            if(upper.contains(str)){
                return FlowConstant.NEXT_WRONG;
            }
        }
        return null;
    }

    /**
     * 校验本步骤处理人及步骤节点
     * @param isMould 是否模板
     * @param step 当前步骤信息
     * @return 处理结果
     */
    private String checkStepUser(Short isMould, PConfigStepDto step) {
        if(FlowConstant.SHORT_ZERO.equals(isMould)) {
            /**页面配置节点数等于节点层级数*/
            if(CollUtil.isEmpty(step.getControls())|| step.getControls().size()!= step.getNodeNumber()){
                return FlowConstant.PAGE_CONFIG_WRONG;
            }
            Set<Integer> pagenodes = step.getControls().stream().map(x->x.getStepNode()).collect(Collectors.toSet());
            if(step.getControls().size()!=pagenodes.size()){
                return FlowConstant.PAGE_CONFIG_WRONG;
            }
            Integer controlsMax = pagenodes.stream().max(Integer::compare).get();
            if(controlsMax!= step.getNodeNumber()){
                return FlowConstant.PAGE_CONFIG_WRONG;
            }
            /**处理人员*/
            if(step.getHandleMode().equals(FlowConstant.SHORT_ZERO)) {
                if (CollUtil.isEmpty(step.getStepUsers())) {
                    return FlowConstant.USER_CONFIG_WRONG;
                }
                Set<Integer> usernodes = step.getStepUsers().stream().map(x -> x.getStepNode()).collect(Collectors.toSet());
                if (usernodes.size() != step.getNodeNumber()) {
                    return FlowConstant.USER_CONFIG_WRONG;
                }
                Integer userMax = usernodes.stream().max(Integer::compare).get();
                if (userMax != step.getNodeNumber()) {
                    return FlowConstant.USER_CONFIG_WRONG;
                }
                /**应用历史步骤判断*/
            }else if(step.getHandleMode().equals(FlowConstant.SHORT_ONE)) {
                if (step.getNodeNumber() > FlowConstant.SHORT_ONE) {
                    return FlowConstant.HISTORY_STEP_NODE_ONLY;
                }
                List<LHistoryStepDto> hsteps = step.getHistoryStep();
                if(CollUtil.isEmpty(hsteps)){
                    return FlowConstant.LINK_HISTORY_STEP_NULL;
                }
                List<Long> configids = new ArrayList<>();
                for (LHistoryStepDto hstep:hsteps ) {
                    if(configids.contains(hstep.getHistoryConfigId())){
                        return FlowConstant.LINK_CONFIG_REPEAT;
                    }
                    if (hstep.getHistoryConfigId().equals(FlowConstant.LONG_ZERO)) {
                        if (hstep.getHistoryStepOrder() > step.getStepOrder()) {
                            return FlowConstant.STEP_MAX_I;
                        }
                        if(hsteps.size()>FlowConstant.SHORT_ONE){
                            return FlowConstant.THIS_STEP_NODE_ONLY;
                        }
                    } else {
                        int count = pConfigLService.count(new QueryWrapper<TfProcessConfigL>().lambda()
                                .eq(TfProcessConfigL::getId,hstep.getHistoryConfigId()).eq(TfProcessConfigL::getTenantId, SecurityUtils.getTenantId()));
                        if(count==FlowConstant.INTEGER_ZERO){
                            return FlowConstant.CONFIG_NOT_EXIST;
                        }
                        Short number = this.getObj(new QueryWrapper<TfProcessConfigStepL>().lambda()
                                        .eq(TfProcessConfigStepL::getConfigId, hstep.getHistoryConfigId())
                                        .eq(TfProcessConfigStepL::getStepOrder, hstep.getHistoryStepOrder())
                                        .eq(TfProcessConfigStepL::getIsDel, FlowConstant.SHORT_ZERO).eq(TfProcessConfigStepL::getIsValid, FlowConstant.SHORT_ONE)
                                        .select(TfProcessConfigStepL::getNodeNumber)
                                , r -> {
                                    return Short.valueOf(String.valueOf(r));
                                });
                        if (!FlowConstant.SHORT_ONE.equals(number)) {
                            return FlowConstant.HISTORY_STEP_NODE_ONLY;
                        }
                    }
                    configids.add(hstep.getHistoryConfigId());
                }
            }
        }
        return null;
    }

    /**
     * 驳回判断
     * @param map 处理节点
     * @param returnToStep 驳回到节点编号
     * @param step 当前节点
     * @return 判断结果
     */
    private String isLab(Map<Integer,PConfigStepDto> map,Integer returnToStep,PConfigStepDto step){
        String upper = step.getUpperStep();
        if(upper.contains(returnToStep+"")){
            return null;
        } else if(step.getStepOrder()==FlowConstant.INTEGER_ZERO) {
            return FlowConstant.RETURN_STEP_WRONG;
        } else {
            List<String> strs = StrSplitter.splitByRegex(upper,"[,|]",0,true,true);
            Integer key =Integer.valueOf(strs.get(0));
            if(map.containsKey(key)){
                PConfigStepDto pcs = map.get(key);
                return isLab(map,returnToStep,pcs);
            } else {
                return FlowConstant.RETURN_STEP_WRONG;
            }
        }
    }


}
