package com.tofly.workflow.report.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.workflow.buss.entity.TfChargeW;
import com.tofly.workflow.buss.entity.vo.ProjectBaseVo;
import com.tofly.workflow.buss.service.TfChargeWService;
import com.tofly.workflow.buss.service.TfProjectBaseWService;
import com.tofly.workflow.report.dto.ReportDto;
import com.tofly.workflow.report.mapper.ReportMapper;
import com.tofly.workflow.report.service.ReportService;
import com.tofly.workflow.report.vo.*;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ReportServiceImpl extends ServiceImpl<ReportMapper, SignApplyVo> implements ReportService {

    @Resource
    TfProjectBaseWService tfProjectBaseWService;
    @Resource
    TfChargeWService tfChargeWService;

    @Override
    public Page<SignApplyVo> getSignAndApplyData(Page page, ReportDto reportDto) {
        return baseMapper.getSignAndApplyData(page, reportDto);
    }

    @Override
    public List<SignApplyVo> getSignAndApplyData(ReportDto reportDto) {
        return baseMapper.getSignAndApplyData(reportDto);
    }

    @Override
    public Page<ProjectDispatchVo> getDispatchCount(Page page, ReportDto reportDto) {
        return baseMapper.getDispatchCount(page, reportDto);
    }

    @Override
    public Page<NotInProjectVo> getNotInProjectVo(Page page, ReportDto reportDto) {
        return baseMapper.getNotInProjectVo(page, reportDto);
    }

    @Override
    public Page<PreIncomeVo> preIncomeCount(Page page, ReportDto reportDto) {
        return baseMapper.preIncomeCount(page, reportDto);
    }

    @Override
    public Page<SettlementVo> settlementCount(Page page, ReportDto reportDto) {
        return baseMapper.settlementCount(page, reportDto);
    }

    @Override
    public Page<HasChargeVo> hasChargeCount(Page page, ReportDto reportDto) {
        Page<HasChargeVo> hasChargeVoPage = baseMapper.hasChargeCount(page, reportDto);
        List<HasChargeVo> records = hasChargeVoPage.getRecords();
        for (HasChargeVo record : records) {
            //每条数据添加两个字段  实收总金额和结算总金额
            String typeName = record.getProcessTypeName();
            Long processId = record.getProcessId();
            if ("新安装工程".equals(typeName)) {
                //新安装：实收总金额=合同签订及缴费环节的实收金额+用户结算环节的实收金额
                Long totalAmount = baseMapper.getTotalAmount(processId);
                record.setTotalAmount(totalAmount.toString());
                // 新安装：   包干和按时均取值用户结算环节的项目结算单中的实际结算金额
                ContractVo contractVo = baseMapper.getContract(processId);
                if (Objects.nonNull(contractVo)) {
                    //包干
                    if ("包干结算".equals(contractVo.getSettlementMode())) {
                        record.setSettlementSum((
                                contractVo.getTotalContract()
                                        + contractVo.getDeductExpenses()
                                        - contractVo.getWaterElectricity()) + "");
                    } else {
                        //按时
//                        List<TfChargeW> chargeWList = tfChargeWService.list(Wrappers.lambdaQuery(TfChargeW.class)
//                                .eq(TfChargeW::getProcessId, processId)
//                        );
                        List<TfChargeW> chargeWList =baseMapper.getChargeWList(processId);
                        if (CollectionUtil.isNotEmpty(chargeWList)) {
                            BigDecimal realAmount = chargeWList.stream().map(TfChargeW::getRealAmount).filter(Objects::nonNull).reduce(BigDecimal.ZERO, BigDecimal::add);
                            record.setSettlementSum(realAmount.toString());
                        }
                    }
                }
            } else if ("户内改造工程".equals(typeName)) {
                //户内改造：实收总金额=用户缴费的实收金额+结算办理的实收金额
                Long totalAmount = baseMapper.getTotalAmount(processId);
                record.setTotalAmount(totalAmount.toString());
                //户内改造：结算总金额取值工程结算页面的总价
                ProjectBaseVo projectBaseVo = tfProjectBaseWService.getBaseById(null, processId);
                record.setSettlementSum(projectBaseVo.getBudget().toString());
            }
        }
        return hasChargeVoPage;
    }

    @Override
    public Page<AcceptVo> acceptCount(Page page, ReportDto reportDto) {
        return baseMapper.acceptCount(page, reportDto);
    }

    @Override
    public Page<CompleteVo> completeCount(Page page, ReportDto reportDto) {

        return baseMapper.completeCount(page, reportDto);
    }

    @Override
    public Page<AllRoundVo> allRoundCount(Page page, ReportDto reportDto) {
        Page<AllRoundVo> allRoundVoPage = baseMapper.allRoundCount(page, reportDto);
//        Page<AllRoundVo> allRoundVoPage = new Page<>();
        // 综合报表统计
        List<AllRoundVo> allRoundVos = allRoundCount(reportDto);
        allRoundVoPage.setRecords(allRoundVos);
        return allRoundVoPage;
    }

    /**
     * 1 根据条件统计出属于（市政工程，户内改造工程，新安装工程，管道维修）所有的流程
     * 2 分别统计 总户数（只有新安装和户内适用 取对应类型核定工程户数之和  核定户数可从现场勘察环节取），
     * 登记数，
     * 已缴费数 （取“用户缴费”通过的工程，只有新安装和户内适用），
     * 未开工，已开工，完结归档，未验收，已验收，
     * 已结算（只有新安装和户内适用），
     * 转固（只有市政工程类型）
     *
     * @param reportDto
     * @return
     */
    private List<AllRoundVo> allRoundCount(ReportDto reportDto) {
        Long tenantId = reportDto.getTenantId();
        Date beginTime = reportDto.getBeginTime();
        Date endTime = reportDto.getEndTime();
        List<ProcessType> processTypeList = baseMapper.getProcessTypeList(tenantId, beginTime, endTime);
        if (CollectionUtil.isEmpty(processTypeList)) {
            return null;
        }
        Map<String, List<ProcessType>> processMap = processTypeList.stream().distinct()
                .collect(Collectors.groupingBy(ProcessType::getTypeName));
        List<AllRoundVo> result = new ArrayList<>(processMap.size());
        processMap.forEach((k, v) -> {
            AllRoundVo allRoundVo = new AllRoundVo();
            //工程类型
            allRoundVo.setProcessTypeName(k);
            //设置每一个类型的登记数量
            allRoundVo.setRegisterNumber(v.size() + "");
            //设置每一个类型的已开工数量、未开工数量
            String hasBegin = getHasStep("开工报告", v);
            allRoundVo.setHasBegin(hasBegin);
            allRoundVo.setNoBegin((v.size() - Integer.parseInt(hasBegin)) + "");
            //设置每一个类型的完结归档数量
            String hasComplete = getHasStep("资料归集", v);
            allRoundVo.setCompleteNum(hasComplete);
            //设置每一个类型的已验收数量、未验收数量
            String hasAccept = getHasStep("综合验收", v);
            allRoundVo.setHasAccept(hasAccept);
            allRoundVo.setNoAccept((v.size() - Integer.parseInt(hasAccept)) + "");

            List<Long> processIdList = v.stream().map(ProcessType::getProcessId).collect(Collectors.toList());
            if ("新安装工程".equals(k)) {
             //总户数 已缴费数 已结算
                String totalHouseholdNumber = baseMapper.getHouseHoldNumber(processIdList);
                allRoundVo.setTotalHouseholdNumber(totalHouseholdNumber);
                String hasCharge = getHasStep("合同签订及缴费", v);
                allRoundVo.setHasCharge(hasCharge);
                String hasSettle = getHasStep("用户结算", v);
                allRoundVo.setHasSettle(hasSettle);
            }
            if ("户内改造工程".equals(k)) {
             //总户数 已缴费数 已结算
                String totalHouseholdNumber = baseMapper.getHouseHoldNumber(processIdList);
                allRoundVo.setTotalHouseholdNumber(totalHouseholdNumber);
                String hasCharge = getHasStep("用户缴费", v);
                allRoundVo.setHasCharge(hasCharge);
                String hasSettle = getHasStep("用户结算", v);
                allRoundVo.setHasSettle(hasSettle);

            }
            if ("市政工程".equals(k)) {
                //转固  TF_TRANSFER_FIXED_ASSETS_W
                String toRegularCount = baseMapper.getToRegularCount(processIdList);
                allRoundVo.setToRegularCount(toRegularCount);
            }
            result.add(allRoundVo);
        });
        return result;
    }

    private String getHasStep(String stepName, List<ProcessType> processTypeList) {
        Assert.notBlank(stepName, "统计的步骤名称不能为空");
        List<Long> processIdList = processTypeList.stream().map(ProcessType::getProcessId).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(processIdList)) {
            return "0";
        }
        return baseMapper.getHasStep(stepName, processIdList);
    }


}
