package com.tofly.workflow.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.common.em.FileTableEnum;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.common.service.IFileInfoService;
import com.tofly.common.utils.FileNameCheckUtils;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.TfNowHandleL;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.process.entity.dto.NextStepDto;
import com.tofly.workflow.process.entity.dto.PNodeDto;
import com.tofly.workflow.process.entity.dto.PNodeSaveDto;
import com.tofly.workflow.process.entity.dto.ProcessBaseDto;
import com.tofly.workflow.process.service.BaseService;
import com.tofly.workflow.processconfig.entity.TfProcessConfigStepL;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 步骤节点数据填充工具类型
 */
@Slf4j
@Service("nodeLFillUtil")
public class NodeLFillUtil {

    /**
     * 文件上传
     */
    @Autowired
    private IFileInfoService fileService;

    @Value("${zmrq.ftp.process.node}")
    private String nodePath;

    @Value("${zmrq.ftp.process.signnode}")
    private String signNode;


    /**
     * 流程提交信息校验
     * @param pdto 步骤信息
     * @param pcs 本步骤配置
     * @param map 下步骤配置信息
     * @return 校验结果
     */
    public String saveStateCheck(PNodeDto pdto, TfProcessConfigStepL pcs, Map<Integer, TfProcessConfigStepL> map) {
        if(pdto.getSaveState().equals(FlowConstant.INTEGER_ONE)&& pdto.getHandleState().equals(FlowConstant.SHORT_ONE)){
            /**下一步骤为本步骤节点*/
            if(pcs.getNodeNumber()> pdto.getStepNode()&& pcs.getIsAppointUser()==FlowConstant.SHORT_ONE&&CollUtil.isEmpty(pdto.getNextSteps())){
                return FlowConstant.NEXT_USER_MUST_SPECIFY;
            }
            if(pcs.getCoerceLevel()== pdto.getStepNode()&& pcs.getIsAppointUser()==FlowConstant.SHORT_ONE&& map.size()>FlowConstant.INTEGER_ONE){
                return FlowConstant.NEXT_STEP_MUST_SPECIFY;
            }
            for (TfProcessConfigStepL npcs: map.values()) {
                if(npcs.getIsAppointUser()==FlowConstant.SHORT_ONE){
                    return FlowConstant.NEXT_USER_MUST_SPECIFY;
                }
            }
        }
        return null;
    }


    /**
     * 填充流程当前处理信息
     * @param processL 流程信息
     * @param map 步骤配置信息
     * @param pns 处理步骤节点
     * @param nhs 流程当前处理信息
     */
    public void fillNowHandle(TfProcessL processL, Map<Integer, TfProcessConfigStepL> map, List<TfProcessNodeL> pns, List<TfNowHandleL> nhs) {
        for (TfProcessNodeL pn: pns){
            if(pn.getIsTemp()!=null && pn.getIsTemp()==FlowConstant.SHORT_ONE){
                continue;
            }
            TfNowHandleL nh = new TfNowHandleL();
            nh.setProcessId(processL.getId());
            nh.setNodeId(pn.getId());
            nh.setProcessStep(pn.getProcessStep());
            nh.setStepNode(pn.getStepNode());
            nh.setEdition(processL.getEdition());
            TfProcessConfigStepL npcs = map.get(pn.getProcessStep());
            nh.setStepId(npcs.getId());
            nh.setLabel(npcs.getLabel());
            nh.setAppLabe(npcs.getAppLabe());
            nh.setTimeLimit(pn.getTimeLimit());
            nh.setStepName(npcs.getStepName());
            Integer timeRemind = npcs.getTimeRemind();
            if(timeRemind!=null&&pn.getTimeLimit()!=null){
//                nh.setRemindTime(DateUtil.offsetDay(pn.getTimeLimit(), -timeRemind));
                nh.setRemindTime(DateUtil.offsetHour(pn.getTimeLimit(), -timeRemind));
            }
            if(pn.getDutyUser()!=null){
                nh.setHandleStaff(pn.getDutyUser());
                nh.setHandleDept(pn.getDutyDept());
                nh.setHandlePost(pn.getDutyPost());
                nh.setHandleMode(FlowConstant.SHORT_ONE);
            }
            nhs.add(nh);
        }
    }


    /**
     * 校验节点文件
     * @param files 文件集合
     * @param pdto 节点信息
     * @return 校验结果
     */
    public String checkNodeFile(List<MultipartFile> files, PNodeDto pdto) {
        Map<String, MultipartFile> map = FileNameCheckUtils.getNameFiles(files);
        String check = FileNameCheckUtils.checkNodeFile(map,pdto.getFileName());
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        check = FileNameCheckUtils.checkNodeFile(map,pdto.getSignFileName());
        if(StringUtils.isNotEmpty(check)){
            return check;
        }
        return null;
    }


    /**
     * 保存其它业务数据
     * @param dto 流程步骤信息
     * @param files 文件流数据
     * @param pl 流程信息
     * @param bss 业务baseservice对象
     * @param objs 业务及对应数据
     * @param pn 保存的步骤信息
     */
    public void saveWorkData(PNodeSaveDto dto, List<MultipartFile> files, TfProcessL pl, Map<String, BaseService> bss, Map<String, Object> objs,TfProcessNodeL pn) {
        ProcessBaseDto pbdto = new ProcessBaseDto();
        pbdto.setId(pl.getId());
        pbdto.setTenantId(pl.getTenantId());
        pbdto.setCategory(pl.getCategory());
        pbdto.setProcessType(pl.getProcessType());
        pbdto.setProcessNumber(pl.getProcessNumber());
        pbdto.setProcessStep(dto.getProcessStep());
        pbdto.setSaveState(dto.getSaveState());
        pbdto.setStepNode(dto.getStepNode());
        pbdto.setConfigId(pl.getConfigId());
        pbdto.setEdition(pl.getEdition());
        pbdto.setRelationId(pl.getRelationId());
        pbdto.setNodeId(pn.getId());
        pbdto.setStartTime(pn.getStartTime());
        pbdto.setEndTime(pn.getEndTime());
        pbdto.setRegisterDept(pl.getRegisterDept());
        pbdto.setRegisterPost(pl.getRegisterPost());
        pbdto.setRegisterStaff(pl.getRegisterStaff());
        pbdto.setRegisterTime(pl.getRegisterTime());
        for (Map.Entry<String,Object> e: objs.entrySet()) {
            BaseService bs = bss.get(e.getKey());
            Object obj = bs.saveData(e.getValue(), files,pbdto);
            if(obj instanceof String){
                throw new RuntimeException(obj.toString());
            }
        }
    }


    /**
     * 填充流程步骤信息
     * @param processL 流程id
     * @param pNode 步骤操作数据
     * @param files 文件流信息
     * @param stepNode 历史步骤数据信息
     * @param deptId 部门id
     * @param postId 岗位id
     * @param pcs 步骤配置信息
     * @return 步骤
     */
    public TfProcessNodeL getTfProcessNodeL(TfProcessL processL, PNodeDto pNode, List<MultipartFile> files, TfProcessNodeL stepNode,
                                            Long deptId, Long postId, TfProcessConfigStepL pcs) {
        TfProcessNodeL processNode = new TfProcessNodeL();
        BeanUtils.copyProperties(pNode, processNode);
        processNode.setProcessId(processL.getId());
        processNode.setStepName(pcs.getStepName());
        if(stepNode !=null) {
            processNode.setId(stepNode.getId());
            if(stepNode.getHandleUser()==null) processNode.setHandleUser(SecurityUtils.getUserId());
            if(stepNode.getHandleDept()==null) processNode.setHandleDept(deptId);
            if(stepNode.getHandlePost()==null) processNode.setHandlePost(postId);
            if(stepNode.getDutyUser()==null) processNode.setDutyUser(SecurityUtils.getUserId());
            if(stepNode.getDutyDept()==null) processNode.setDutyDept(deptId);
            if(stepNode.getDutyPost()==null) processNode.setDutyPost(postId);
            if(stepNode.getConfigId()==null) processNode.setConfigId(pcs.getConfigId());
            if(stepNode.getEdition()==null) processNode.setEdition(pcs.getEdition());
            if(stepNode.getLabel()==null) processNode.setLabel(pcs.getLabel());
        }else {
            processNode.setIsDel(FlowConstant.SHORT_ZERO);
            processNode.setIsValid(FlowConstant.SHORT_ONE);
            processNode.setDutyUser(SecurityUtils.getUserId());
            processNode.setDutyDept(deptId);
            processNode.setDutyPost(postId);
            processNode.setHandleUser(SecurityUtils.getUserId());
            processNode.setHandleDept(deptId);
            processNode.setHandlePost(postId);
            processNode.setCreateDept(deptId);
            processNode.setCreatePost(postId);
            processNode.setIsOvertime(FlowConstant.SHORT_ZERO);
            processNode.setConfigId(pcs.getConfigId());
            processNode.setEdition(pcs.getEdition());
            processNode.setLabel(pcs.getLabel());
            processNode.setAppLabe(pcs.getAppLabe());
        }
        processNode.setHandleTime(new Date());

        /**保存文件*/
        if(CollUtil.isNotEmpty(files)) {
            if(CollUtil.isNotEmpty(pNode.getFileName())) {
                log.info("pNode.getFileName():"+pNode.getFileName());
                for (MultipartFile multipartFile : files) {
                    log.info("multipartFile.getOriginalFilename()"+multipartFile.getOriginalFilename());
                }

                List<MultipartFile> fileannex = files.stream().filter(r-> pNode.getFileName().contains(r.getOriginalFilename())).collect(Collectors.toList());
                log.info("pNode.getAnnex():"+pNode.getAnnex());
                log.info("tepNode ==null?null: stepNode.getAnnex():"+(stepNode ==null?null: stepNode.getAnnex()));
                processNode.setAnnex(fileService.uploadFiles(fileannex, pNode.getAnnex(), stepNode ==null?null: stepNode.getAnnex(),nodePath, FileTableEnum.TF_PROCESS_NODE_L.getTableName()));
            }
            if(CollUtil.isNotEmpty(pNode.getSignFileName())){
                List<MultipartFile> fileannex = files.stream().filter(r-> pNode.getSignFileName().contains(r.getOriginalFilename())).collect(Collectors.toList());
                processNode.setSignAnnex(fileService.uploadFiles(fileannex, pNode.getSignAnnex(), stepNode ==null?null: stepNode.getSignAnnex(),signNode, FileTableEnum.TF_PROCESS_NODE_L.getTableName()));
            }
        }
        if(pNode.getSaveState()==FlowConstant.INTEGER_ZERO){
            processNode.setNodeState(FlowConstant.INTEGER_ONE);
        }else {
            processNode.setHandleTime(new Date());
            if(pNode.getHandleState()==FlowConstant.INTEGER_FOUR){
                processNode.setNodeState(FlowConstant.INTEGER_TWO);
            }else {
                processNode.setNodeState(pNode.getHandleState()+FlowConstant.INTEGER_ONE);
            }
            if(pNode.getHandleState()==FlowConstant.INTEGER_TWO|| pNode.getHandleState()==FlowConstant.INTEGER_THREE){
                processNode.setIsValid(FlowConstant.SHORT_ZERO);
            }
            if(stepNode !=null && stepNode.getTimeLimit()!=null){
                if(new Date().getTime()> stepNode.getTimeLimit().getTime()){
                    processNode.setIsOvertime(FlowConstant.SHORT_ONE);
                }
            }
        }
        return processNode;
    }


    /**
     * 保存步骤转派信息
     * @param processL 流程ID
     * @param pNode 步骤信息
     * @param processNode 本步骤节点信息
     * @param deptId 部门
     * @param postId 岗位
     * @param stepNode 存储的历史信息
     * @return
     */
    public TfProcessNodeL saveStepTransfer(TfProcessL processL, PNodeDto pNode, TfProcessNodeL processNode, Long deptId, Long postId, TfProcessNodeL stepNode) {
        TfProcessNodeL pn = new TfProcessNodeL();
        BeanUtils.copyProperties(processNode, pn);
        pn.setId(null);
        pn.setProcessId(processL.getId());
        pn.setIsValid(FlowConstant.SHORT_ONE);
        pn.setIsDel(FlowConstant.SHORT_ZERO);
        NextStepDto ns = pNode.getNextSteps().get(0);
        pn.setDutyUser(ns.getDeptStaff());
        pn.setDutyDept(ns.getDept());
        pn.setDutyPost(ns.getPost());
        pn.setHandleUser(ns.getDeptStaff());
        pn.setHandleDept(ns.getDept());
        pn.setHandlePost(ns.getPost());
        pn.setNodeState(FlowConstant.INTEGER_ZERO);
        pn.setCreatePost(postId);
        pn.setCreateDept(deptId);
        pn.setCondition(null);
        pn.setOpinion(null);
        pn.setHandleTime(null);
        pn.setOutcome(null);
        if(pNode.getSaveState()==FlowConstant.INTEGER_ZERO) {
            pn.setIsTemp(FlowConstant.SHORT_ONE);
        }
        if(stepNode!=null) {
            pn.setTimeLimit(stepNode.getTimeLimit());
        }
        return pn;
    }

}
