package com.tofly.workflow.utils;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.workflow.baseconf.service.TfMatterNumberPService;
import com.tofly.workflow.buss.service.TfAlterDutyWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.common.ForeignConfig;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.service.TfProcessLService;
import com.tofly.workflow.wzyy.service.MaterialOutInService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 监听redis队列，消费流程处理结束后发送的消息
 */
@Slf4j
@Service
public class RedisQueueUtil {
    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 配置文件
     */
    @Autowired
    private ForeignConfig foreignConfig;
    /**
     * 工程流程
     */
    @Autowired
    private TfProcessLService processService;
    /**
     * 小仓库物资数量
     */
    @Autowired
    private TfMatterNumberPService mNumberService;
    /**
     * 负责人变更
     */
    @Autowired
    private TfAlterDutyWService aDutyService;

    /**
     * 同步到用友物料出库、入库数据
     */
    @Autowired
    private MaterialOutInService mOutInService;

    private volatile boolean flag = true;

    private Long popTimeout = 10l;

    //线程池
    private static ExecutorService executorService = new ThreadPoolExecutor(10, 15,
            60L, TimeUnit.SECONDS,
            new SynchronousQueue<Runnable>());

    @PostConstruct
    public void start(){
        executorService.execute(()->{
            try {
                Object message;
                log.info("============启动监听 流程终止  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {
                    log.info("GCZZ::"+FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueGczzKey());
                    message =  redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueGczzKey(), popTimeout, TimeUnit.SECONDS);
                    if(message!=null) {
                        log.info("流程终止 流程id" + message);
                        processService.stopProcess(Long.parseLong(String.valueOf(message).trim()));
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
            }
        });
        executorService.execute(()->{
            try {
                Object message;
                log.info("============启动监听 负责人变更  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {
                    log.info("FZRBG::"+FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueFzrbgKey());
                    message =  redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueFzrbgKey(), popTimeout, TimeUnit.SECONDS);
                    if(message!=null) {
                        Thread.sleep(1000*10);
                        log.info("负责人变更 流程id" + message);
                        aDutyService.mackingAlterDuty(Long.parseLong(String.valueOf(message).trim()));
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
            }
        });


        executorService.execute(()->{
            try {
                Object message;
                log.info("============启动监听 工程物料出库  {}",flag && !Thread.currentThread().isInterrupted());
                String[]keys = foreignConfig.getQueueGcllKey().split(",");
                while(flag && !Thread.currentThread().isInterrupted()) {
                    for (String key:keys) {
                        log.info("GCLL::" + FlowConstant.REDIS_QUERY_KEY + key);
                        message = redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY + key, popTimeout, TimeUnit.SECONDS);
                        if (message != null) {
                            Thread.sleep(1000 * 10);
                            log.info("工程物料出库 " + FlowConstant.REDIS_QUERY_KEY + key+" 流程id:" + message);
                            //用友物料出库
                            mOutInService.materialOut(Long.parseLong(String.valueOf(message).trim()), FlowConstant.SHORT_ONE);
                        }
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
            }
        });

        executorService.execute(()->{
            try {
                Object message;
                log.info("============启动监听 工程物料退料 ,使用出库接口  {}",flag && !Thread.currentThread().isInterrupted());
                String[]keys = foreignConfig.getQueueGctlKey().split(",");
                while(flag && !Thread.currentThread().isInterrupted()) {
                    for (String key:keys) {
                        log.info("GCTL::" + FlowConstant.REDIS_QUERY_KEY + key);
                        message = redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY + key, popTimeout, TimeUnit.SECONDS);
                        if (message != null) {
                            Thread.sleep(1000 * 10);
                            log.info("工程物料退料 " + FlowConstant.REDIS_QUERY_KEY + key +" 流程id:" + message);
                            //用友物料其它入库
                            mOutInService.returnOfMaterial(Long.parseLong(String.valueOf(message).trim()));
                        }
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
            }
        });

        executorService.execute(()->{
            try {
                Object message;
                log.info("============启动监听 小仓库入库(转库)  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {

                    //查找流程中 没有出库记录的流程
                    final List<Long> processIdWithOutPush = processService.getProcessIdWithOutPush();
                   // log.info("ZK-CLLY::"+FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueCllyKey());
                   // message =  redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueCllyKey(), popTimeout, TimeUnit.SECONDS);
                    if(processIdWithOutPush!=null &&processIdWithOutPush.size()>0) {

                        for (Long processId : processIdWithOutPush) {
                            log.info("小仓库入库(转库) 流程id" + processId);
                           // Long processId = Long.parseLong(idWithOutPush);
                            //用友物料出库
                         //  mOutInService.materialOut(processId, FlowConstant.SHORT_EIGHT);

                            //生成获取日期字符串

                          mOutInService.stockTransfer(processId);
                            //本地入库操作，本地小仓库增加审批完成的物料
                            mNumberService.inOrOutMaterial(processId,FlowConstant.SHORT_EIGHT);
                        }

                    }
                    Thread.sleep(1000*100);
                }
            }catch (Exception e){
                log.error(e.getMessage());
            }
        });
        executorService.execute(()->{
            try {
                Object message;
                log.info("============启动监听 小仓库核销  {}",flag && !Thread.currentThread().isInterrupted());
                while(flag && !Thread.currentThread().isInterrupted()) {
                    log.info("HX-CLHX::"+FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueClhxKey());
                    message =  redisTemplate.opsForList().rightPop(FlowConstant.REDIS_QUERY_KEY+foreignConfig.getQueueClhxKey(), popTimeout, TimeUnit.SECONDS);
                    if(message!=null) {
                        //查询

                        Thread.sleep(1000*10);
                        log.info("小仓库核销 流程id" + message);
                        Long processId = Long.parseLong(String.valueOf(message).trim());
                        //用友小仓库出库
                        mOutInService.materialOut(processId,FlowConstant.SHORT_THREE);
                        //本地小仓库出库操作
                        mNumberService.inOrOutMaterial(processId,FlowConstant.SHORT_NINE);
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
            }
        });

    }

}
