package com.tofly.workflow.utils;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tofly.common.apis.SysRemoteService;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.entity.dto.MessageDto;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfTaskP;
import com.tofly.workflow.baseconf.service.TfTaskPService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.TfProcessL;
import com.tofly.workflow.process.entity.TfProcessNodeL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigL;
import com.tofly.workflow.processconfig.entity.TfProcessConfigStepL;
import com.tofly.workflow.processconfig.service.TfConfigStepUserLService;
import com.tofly.workflow.processconfig.service.TfPageControlLService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class SendTidingsUtil {

    /**
     * 系统推送消息和待办
     */
    @Autowired
    private SysRemoteService sysRemoteService;

    /**
     * 待办信息
     */
    @Autowired
    private TfTaskPService taskService;
    /**
     * 流程配置步骤配置的人员
     */
    @Autowired
    private TfConfigStepUserLService cSUserService;


    /**
     * 发送消息及待办信息
     * @param processL 流程信息
     * @param pc 流程配置
     * @param npcs 步骤配置
     */
    public void setMessageRecipient(TfProcessL processL, TfProcessConfigL pc, TfProcessConfigStepL npcs){
        this.setMessageRecipient(processL,pc,npcs,null,false,null);
    }
    /**
     * 发送消息及待办信息
     * @param processL 流程信息
     * @param pc 流程配置
     * @param npcs 步骤配置
     * @param pn 步骤信息
     */
    public void setMessageRecipient(TfProcessL processL, TfProcessConfigL pc, TfProcessConfigStepL npcs, TfProcessNodeL pn,boolean isTask){
        this.setMessageRecipient(processL,pc,npcs,pn,isTask,null);
    }
    /**
     * 发送消息及待办信息
     * @param processL 流程信息
     * @param pc 流程配置
     * @param npcs 步骤配置
     * @param pn 步骤信息
     */
    public void setMessageRecipient(TfProcessL processL, TfProcessConfigL pc, TfProcessConfigStepL npcs, TfProcessNodeL pn){
        this.setMessageRecipient(processL,pc,npcs,pn,true,null);
    }

    /**
     * 发送消息及待办信息
     * @param processL 流程信息
     * @param pc 流程配置
     * @param npcs 步骤配置
     * @param pn 步骤信息
     */
    public void setMessageRecipient(TfProcessL processL, TfProcessConfigL pc, TfProcessConfigStepL npcs, TfProcessNodeL pn,Integer handleState){
        this.setMessageRecipient(processL,pc,npcs,pn,true,handleState);
    }

    /**
     * 发送消息及待办信息
     * @param processL 流程信息
     * @param pc 流程配置
     * @param npcs 步骤配置
     * @param pn 步骤信息
     * @param isTask 是否送待办信息
     * @param handleState 是否送待办信息
     */
    public void setMessageRecipient(TfProcessL processL, TfProcessConfigL pc, TfProcessConfigStepL npcs, TfProcessNodeL pn, boolean isTask,Integer handleState){
        List<Long> usids = new ArrayList<>();
        if(pn.getDutyUser()==null){
            List<Long> uids = cSUserService.getUserIds(npcs.getConfigId(),npcs.getEdition(),pn.getProcessStep(),pn.getStepNode());
            if(CollUtil.isNotEmpty(uids)){
                usids.addAll(uids);
            }
        }else {
            usids.add(pn.getDutyUser());
        }
        for (Long userId:usids) {
            MessageDto message = getMessage(processL, pc, npcs, userId);
            ResultData rd = sysRemoteService.insert(message);
            if (rd.getCode().equals(FlowConstant.STRING_TWO_HUNDRED)) {
                if (isTask) {
                    TfTaskP taskP = new TfTaskP();
                    BeanUtils.copyProperties(message, taskP);
                    taskP.setRouteAppIp(message.getRouteIpApp());
                    taskP.setDescribe(message.getTopic());
                    taskP.setRank(FlowConstant.SHORT_ZERO);
                    taskP.setProcessId(processL.getId());
                    taskP.setConfigId(processL.getConfigId());
                    taskP.setEdition(processL.getEdition());
                    taskP.setStepId(npcs.getId());
                    taskP.setLinkeId(processL.getRelationId() == null ? processL.getId() : processL.getRelationId());
                    taskP.setCategory(processL.getCategory());
                    taskP.setProcessType(processL.getProcessType());
                    taskP.setStartTime(new Date());
                    taskP.setEndTime(pn.getTimeLimit());
                    taskP.setNodeState(pn.getNodeState());
                    taskP.setNodeType(handleState != null && handleState == FlowConstant.INTEGER_TWO ? FlowConstant.SHORT_THREE : npcs.getStepType());
                    taskP.setProcessStep(pn.getProcessStep());
                    taskP.setStepNode(pn.getStepNode());
                    //taskP.setHandleUser(pn.getDutyUser());
                    taskP.setHandleUser(message.getUserPullId());
                    taskP.setTableName(FlowConstant.PROCESS_TABLE_NAME);
                    taskP.setMaxType(FlowConstant.STRING_ELEVEN);
                    taskP.setLabel(npcs.getLabel());
                    taskP.setAppLabe(npcs.getAppLabe());
                    taskP.setNodeName(npcs.getStepName());
                    taskP.setIsDel(FlowConstant.SHORT_ZERO);
                    taskP.setStatus(FlowConstant.SHORT_ZERO);
                    taskService.save(taskP);
                }
            }
        }
    }

    /**
     * 取消待办，标记已处理
     * @param id 流程id
     * @param processTableName 表
     * @param processStep 流程步骤编号
     * @param stepNode 流程步骤节点编号
     * @param userId 用户id 如果传不取消本用户待办
     */
    public void removeLocalTask(Long id, String processTableName, Integer processStep , Integer stepNode,Long userId){
        taskService.update(new UpdateWrapper<TfTaskP>().lambda().eq(TfTaskP::getProcessId,id).eq(TfTaskP::getProcessStep,processStep)
                .eq(TfTaskP::getStatus,FlowConstant.SHORT_ZERO)
                .eq(TfTaskP::getTableName,processTableName).eq(TfTaskP::getStepNode,stepNode).ne(userId!=null,TfTaskP::getHandleUser,userId)
                .set(TfTaskP::getHandleTime,new Date()).set(TfTaskP::getStatus,FlowConstant.SHORT_ONE).set(TfTaskP::getHandleUser,SecurityUtils.getUserId()));

    }

    /**
     * 填充消息提醒
     * @param processL 流程信息
     * @param pc 流程配置信息
     * @param npcs 流程步骤配置信息
     * @return
     */
    private MessageDto getMessage(TfProcessL processL, TfProcessConfigL pc, TfProcessConfigStepL npcs ,Long userId) {
        StringBuilder sb = new StringBuilder("[");
        sb.append(pc.getProcessName()).append("]");
        sb.append(FlowConstant.ADD_HANDLE).append("[").append(npcs.getStepName()).append("]");

        StringBuilder sbip = new StringBuilder(pc.getProcessNumber());
        sbip.append("|").append(npcs.getStepOrder());
        sbip.append(";").append(pc.getBusinessType()).append("|").append(npcs.getStepOrder());
        MessageDto messageDto = new MessageDto();
        messageDto.setTableId(processL.getId());
        messageDto.setMessage(sb.toString());
        messageDto.setTopic(pc.getProcessName()+"_"+npcs.getStepName());
        messageDto.setRouteIp(sbip.toString());
        messageDto.setRouteIpApp(sbip.toString());
        messageDto.setTableName(FlowConstant.PROCESS_TABLE_NAME);
        messageDto.setType(FlowConstant.STRING_ELEVEN);
        messageDto.setTenantId(processL.getTenantId());
        messageDto.setUserPushId(SecurityUtils.getUserId());
        messageDto.setUserPullId(userId);
        return messageDto;
    }

}
