package com.tofly.workflow.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.tofly.workflow.baseconf.entity.vo.CalendarVo;
import com.tofly.workflow.baseconf.service.TfCalendarPService;
import com.tofly.workflow.common.FlowConstant;
import org.apache.commons.lang.time.DateFormatUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 日期计算
 */
public class WorkDayCalculateUtils {

    /**
     * 计算工作日
     * @param currentDate
     * @param days
     * @param workDays
     * @param restDays
     * @return
     */
    public static Date getDate(Date currentDate, int days,List<String> workDays,List<String> restDays) {
        if (days == 0) {
            return currentDate;
        }

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(currentDate);

        int step = days < 0 ? -1 : 1;
        int i = 0;
        int daysAbs = Math.abs(days);
        while (i < daysAbs) {
            calendar.add(Calendar.DATE, step);
            i++;
            if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY
                    || calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {

                // 周六日如果上班就算1个工作日
                if (!workDays.contains(DateFormatUtils.format(calendar.getTime(), "yyyy-MM-dd"))) {
                    i--;
                }
            } else {
                // 周1到周五休息就算1个休息日
                if (restDays.contains(DateFormatUtils.format(calendar.getTime(), "yyyy-MM-dd"))) {
                    i--;
                }
            }
        }
        return calendar.getTime();
    }

    public static Date getDateNew(Date currentDate, int limit, TfCalendarPService calendarService, Long tenantId, int dataType) {
        try {
            Date tempTimeLimit = null;
            List<String> workDays = new ArrayList<>();
            List<String> restDays = new ArrayList<>();
            Calendar instance = Calendar.getInstance();
            if(dataType == FlowConstant.SHORT_ONE) {
                int sunday1 = getSunday(currentDate, currentDate);
                getHoliday(calendarService, tenantId, currentDate, currentDate, workDays, restDays);
                if (sunday1 >= 1 || restDays.size() > 0) {
                    SimpleDateFormat format21 = new SimpleDateFormat("yyyy-MM-dd 09");
                    currentDate = format21.parse(format21.format(currentDate));
                }
            }
            instance.setTime(currentDate);
            if(instance.get(Calendar.HOUR_OF_DAY) != 23){
                instance.add(Calendar.HOUR_OF_DAY, 1);
            }
            // 9-12  1-6
            SimpleDateFormat format = new SimpleDateFormat("HH");
            SimpleDateFormat format2 = new SimpleDateFormat("yyyy-MM-dd HH");
            Integer hour = Integer.valueOf(format.format(instance.getTime()));
            Integer todayHour = 0;
            if(hour < 9){
                todayHour = 8;
            } else if (hour > 9 && hour <= 12) {
                todayHour = 8 - (hour - 9);
            } else if (hour > 12 && hour <= 18) {
                todayHour = 8 - (hour - 9) + 1;
            } else if (hour > 18) {
                todayHour = 0;
            }
            int totalDay = 0;
            if (limit - todayHour <= 0){
                totalDay = 1;
                if(hour < 9){
                    if(9 + limit <= 12){
                        SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd " + (9 + limit));
                        tempTimeLimit = format2.parse(format1.format(currentDate));
                    }else{
                        SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd " + (9 + limit + 1));
                        tempTimeLimit = format2.parse(format1.format(currentDate));
                    }
                } else if (hour > 9 && hour <= 12) {
                    if(hour + limit <= 12){
                        SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd " + (hour + limit));
                        tempTimeLimit = format2.parse(format1.format(currentDate));
                    }else{
                        SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd " + (hour + limit + 1));
                        tempTimeLimit = format2.parse(format1.format(currentDate));
                    }
                } else if (hour > 12 && hour <= 18) {
                    SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd " + (hour + limit));
                    tempTimeLimit = format2.parse(format1.format(currentDate));
                }
            }else{
                totalDay = (limit - todayHour) / 8 + 1;
                instance.add(Calendar.DAY_OF_MONTH, totalDay);
                Date time = instance.getTime();
                SimpleDateFormat format11 = new SimpleDateFormat("yyyy-MM-dd 09:00:00");
                SimpleDateFormat format22 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Calendar instance1 = Calendar.getInstance();
                instance1.setTime(format22.parse(format11.format(time)));
                int i = (limit - todayHour) % 8;
                if(i <= 3){
                    instance1.add(Calendar.HOUR_OF_DAY, i);
                }else{
                    instance1.add(Calendar.HOUR_OF_DAY, i + 1);
                }
                tempTimeLimit = instance1.getTime();
            }
            SimpleDateFormat format33 = new SimpleDateFormat("yyyy-MM-dd");
            Calendar c = Calendar.getInstance();
            c.setTime(tempTimeLimit);
            if(dataType == FlowConstant.SHORT_ONE){
                int sunday = getSunday(format33.parse(format33.format(currentDate)), tempTimeLimit);
                c.add(Calendar.DAY_OF_MONTH, sunday);
                Date endDate = c.getTime();

                getHoliday(calendarService, tenantId, currentDate, endDate, workDays, restDays);

                c.add(Calendar.DAY_OF_MONTH, -workDays.size());
                c.add(Calendar.DAY_OF_MONTH, restDays.size());
            }
            return c.getTime();
        }catch (ParseException e){
            throw new RuntimeException();
        }
    }

    public static int getSunday(Date beginDate, Date endDate){
        Calendar startCalendar = Calendar.getInstance();
        startCalendar.setTime(beginDate); // 设置起始日期

        Calendar endCalendar = Calendar.getInstance();
        endCalendar.setTime(endDate); // 设置结束日期
        int i = 0;
        while (startCalendar.compareTo(endCalendar) <= 0) {
            int year = startCalendar.get(Calendar.YEAR);
            int month = startCalendar.get(Calendar.MONTH) + 1;
            int day = startCalendar.get(Calendar.DAY_OF_MONTH);

            int week = startCalendar.get(Calendar.DAY_OF_WEEK);
            if (week == Calendar.SUNDAY || week == Calendar.SATURDAY) {
                // 是周六日返回true
                i++;
            }

//            System.out.println(year + "-" + month + "-" + day);

            startCalendar.add(Calendar.DAY_OF_MONTH, 1); // 将日期加1
        }
        if (startCalendar.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
            // 是周六日返回true
            i++;
        }
        return i;
    }

    public static void getHoliday(TfCalendarPService calendarService, Long tenantId, Date currentDate, Date endDate, List<String> workDays, List<String> restDays) throws ParseException {
        int i = workDays.size();
        int j = restDays.size();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        List<CalendarVo> calendars = calendarService.getCalendar(sdf.parse(sdf.format(currentDate)), sdf.parse(sdf.format(endDate)), tenantId);
        if (CollUtil.isEmpty(calendars)) {
            calendars = calendarService.getCalendar(sdf.parse(sdf.format(new Date())), sdf.parse(sdf.format(endDate)), null);
        }
        if (CollUtil.isNotEmpty(calendars)) {
            workDays.clear();
            restDays.clear();
            workDays.addAll(calendars.stream().map(r -> {
                if (r.getType() == FlowConstant.SHORT_ONE) return DateUtil.formatDate(r.getDataTime());
                return null;
            }).filter(r -> r != null).collect(Collectors.toList()));
            restDays.addAll(calendars.stream().map(r -> {
                if (r.getType() == FlowConstant.SHORT_ZERO) return DateUtil.formatDate(r.getDataTime());
                return null;
            }).filter(r -> r != null).collect(Collectors.toList()));
        }
        if(j < restDays.size()){
            Calendar ins = Calendar.getInstance();
            ins.setTime(endDate);
            ins.add(Calendar.DAY_OF_MONTH, restDays.size() - j);
            getHoliday(calendarService, tenantId, currentDate, ins.getTime(), workDays, restDays);
        }
    }

}
