package com.tofly.workflow.wzyy.controller;

import cn.hutool.core.thread.ThreadUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.wzyy.entity.YyOrgUnit;
import com.tofly.workflow.wzyy.entity.dto.StockDto;
import com.tofly.workflow.wzyy.service.MaterialBaseService;
import com.tofly.workflow.wzyy.service.YyOrgUnitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 同步用友物资系统基础数据
 * 仓库基础信息
 * 类型基础信息
 * 规格基础信息
 * 物资基础信息
 */
@RestController
@RequestMapping("/material")
@Api(tags="同步用友物资系统基础数据")
@Slf4j
public class MaterialBaseController {

    @Autowired
    private MaterialBaseService materialBaseService;
    @Autowired
    private TfMatterPService tfMatterPService;
    @Autowired
    private YyOrgUnitService yyOrgUnitService;
    /**
     * 拉取用友所有基础物资信息
     * @return
     */
    @GetMapping("/pullYytt")
    @ApiOperation(value = "拉取用友所有基础物资信息")
    public ResultData pullYy(){
        //拉取用友组织信息
        Object obj = materialBaseService.pullYYOrgunit(0);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        ThreadUtil.sleep(5, TimeUnit.SECONDS);
        //拉取用友仓库信息
        obj = materialBaseService.pullYYAccess();
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        ThreadUtil.sleep(5, TimeUnit.SECONDS);
        //拉取用友物料分类信息
        obj = materialBaseService.pullYYClassify(0);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        ThreadUtil.sleep(5, TimeUnit.SECONDS);
        //拉取用友物料档案信息
        obj = materialBaseService.pullYYProduct();
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }


    /**
     * 拉取用友所有基础物资信息
     * @return
     */
    @GetMapping("/pullYyttProduct")
    @ApiOperation(value = "拉取用友物料档案信息")
    public ResultData pullYyttProduct(){
        //拉取用友物料档案信息
        Object  obj = materialBaseService.pullYYProduct();
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }
    /**
     * 获取用友库存信息
     * @return
     */
    @GetMapping("/stock")
    @ApiOperation(value = "获取用友库存信息")
    public ResultData getStockInfo(@Valid StockDto dto){
        Object obj = materialBaseService.getStockInfo(dto,0);
        if(obj instanceof String){
            return ResultData.fail(obj.toString());
        }
        return ResultData.ok(obj);
    }

    @GetMapping("/testYyorg")
    @ApiOperation(value = "测试材料绑定公司")
    public  ResultData getYyMatterPOrg(){
        Page<TfMatterP> page = new Page();
        long current = 1l;

        page.setSize(200);
        while (true){
            page.setCurrent(current);
           // isNotNull(" PRODUCT_APPLY_RANGE_ORG_ID")
            final Page<TfMatterP> pageDatas
                    = tfMatterPService.page(page, Wrappers.<TfMatterP>lambdaQuery().isNotNull(TfMatterP::getProductApplyRangeOrgId));
            final List<TfMatterP> records = pageDatas.getRecords();
             materialBaseService.getYyMatterTenant(null,records);
                log.info(pageDatas.getCurrent()+"===="+pageDatas.getPages());
            if(pageDatas.getCurrent()<=pageDatas.getPages()&&pageDatas.getPages()!=0){
                current++;
            }else{
                break;
            }
        }



        return ResultData.ok("材料绑定公司 拉取成功");
    }

    @GetMapping("/priceHistory")
    @ApiOperation(value = "测试材料绑定公司")
    public  ResultData getYYpriceHistory(){
        Page<TfMatterP> page = new Page();
        long current = 1l;

        page.setSize(200);
        while (true){
            page.setCurrent(current);
            // isNotNull(" PRODUCT_APPLY_RANGE_ORG_ID")
            final Page<TfMatterP> pageDatas
                    = tfMatterPService.page(page, Wrappers.<TfMatterP>lambdaQuery().isNotNull(TfMatterP::getProductApplyRangeOrgId));
            final List<TfMatterP> records = pageDatas.getRecords();
           List<YyOrgUnit>  yyOrgUnits = yyOrgUnitService.list();
            materialBaseService.getYyMatterPrice(null,yyOrgUnits,records);
            if(pageDatas.getCurrent()<=pageDatas.getPages()&&pageDatas.getPages()!=0){
                current++;
            }else{
                break;
            }
        }



        return ResultData.ok("材料绑定公司 拉取成功");
    }


    @GetMapping("/testTypeOrg")
    @ApiOperation(value = "测试材料绑定公司")
    public  ResultData pullWzTypeOverOrg(){

        return ResultData.ok(materialBaseService.pullWzTypeOverOrg(0));
    }
}
