/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.workflow.wzyy.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.workflow.wzyy.entity.YyOrgUnit;
import com.tofly.workflow.wzyy.service.YyOrgUnitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 用友组织信息
 *
 * @author xqf
 * @date Tue Dec 13 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/yyorgunit")
@Api(tags = "用友组织信息接口")
public class YyOrgUnitController {

    private final YyOrgUnitService yyOrgUnitService;

    /**
     * 分页查询
     *
     * @param page      分页对象
     * @param yyOrgUnit 用友组织信息
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultData getYyOrgUnitPage(Page page, @ModelAttribute YyOrgUnit yyOrgUnit) {
        return ResultData.ok(yyOrgUnitService.page(page, Wrappers.query(yyOrgUnit)));
    }


    /**
     * 通过id查询用友组织信息
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultData getById(@PathVariable("id") Long id) {
        return ResultData.ok(yyOrgUnitService.getById(id));
    }


    /**
     * 通过id删除用友组织信息
     *
     * @param id id  
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除用友组织信息")
    @ApiOperation(value = "通过ID删除用友组织信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}")
    public ResultData removeById(@PathVariable Long id) {
        return ResultData.ok(yyOrgUnitService.removeById(id));
    }


}
