package com.tofly.workflow.wzyy.crypto;

import cn.hutool.core.thread.ThreadUtil;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.wzyy.service.MaterialBaseService;
import com.tofly.workflow.wzyy.service.MaterialOutInService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class TimingUtils {

    /**
     * redis
     */
    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 用友物资基础service
     */
    @Autowired
    private MaterialBaseService materialBaseService;


    /**
     * 物料出库、入库数据
     */
    @Autowired
    private MaterialOutInService materialOutInService;

    /**
     * 每日1点执行一次
     */
   @Scheduled(cron = "0 0 1 * * ?")
    public void pullYyData(){
        String key = FlowConstant.REDIS_QUERY_KEY+"yyproduct";
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());

        Long value = counter.incrementAndGet();
        log.info("pullYyData value::"+value);
        if(value == 1) {
            counter.expire(10l, TimeUnit.MINUTES);
        }else {
            log.info("pullYyData 退出。。。。。。。。。。。");
            return;
        }
        //拉取用友组织信息
        Object obj = materialBaseService.pullYYOrgunit(0);
        if(obj instanceof String){
            return;
        }
        ThreadUtil.sleep(5, TimeUnit.SECONDS);
        //拉取用友仓库信息
        obj = materialBaseService.pullYYAccess();
        if(obj instanceof String){
            return;
        }
        ThreadUtil.sleep(5, TimeUnit.SECONDS);
        //拉取用友物料分类信息
        obj = materialBaseService.pullYYClassify(0);
        if(obj instanceof String){
            return;
        }
        ThreadUtil.sleep(5, TimeUnit.SECONDS);
        //拉取用友物料档案信息
        obj = materialBaseService.pullYYProduct();
        if(obj instanceof String){
            return;
        }
        redisTemplate.delete(key);
        log.info("pullYyData 执行完成。。。。。。。。。。。");
    }

    /**
     * 定时同步用友物料出库、退库结果
     */
    @Scheduled(cron = "0 0 */1 * * ?")
    public void pullYyMaterialOutBearFruit(){
        String key = FlowConstant.REDIS_QUERY_KEY+"materialoutbearfruit";
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());

        Long value = counter.incrementAndGet();
        log.info("pullYyMaterialOutBearFruit value::"+value);
        if(value == 1) {
            counter.expire(2l, TimeUnit.MINUTES);
        }else {
            log.info("pullYyMaterialOutBearFruit 退出。。。。。。。。。。。");
            return;
        }
        //定时同步用友物料出库、退库结果
        materialOutInService.getMaterialOutBearFruit(0);
        redisTemplate.delete(key);
        log.info("pullYyMaterialOutBearFruit 执行完成。。。。。。。。。。。");
    }
    /**
     * 定时推送失败的数据
     */
   // @Scheduled(cron = "0 0 */1 * * ?")
    @Scheduled(cron = "0 */10 * * * *")
    public void pushMaterialResult(){
        String key = FlowConstant.REDIS_QUERY_KEY+"materialresult";
        RedisAtomicLong counter = new RedisAtomicLong(key, redisTemplate.getConnectionFactory());

        Long value = counter.incrementAndGet();
        log.info("pushMaterialResult value::"+value);
        if(value == 1) {
            counter.expire(2l, TimeUnit.MINUTES);
        }else {
            log.info("pushMaterialResult 退出。。。。。。。。。。。");
            return;
        }
        //定时推送失败的数据
        materialOutInService.pushMaterial(0);
        redisTemplate.delete(key);
        log.info("pushMaterialResult 执行完成。。。。。。。。。。。");
    }

}
