package com.tofly.workflow.wzyy.service.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.wzyy.common.AccessTokenResponse;
import com.tofly.workflow.wzyy.common.GenericResponse;
import com.tofly.workflow.wzyy.common.JsonUtil;
import com.tofly.workflow.wzyy.common.RequestTool;
import com.tofly.workflow.wzyy.crypto.SignHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 用友token
 */
@Slf4j
@Service
public class AccessTokenServiceImpl {

    //获取token地址
    @Value("${zmrq.flow.wzyy.tokenurl}")
    private String openApiUrl;
    //获取token的api
    @Value("${zmrq.flow.wzyy.api.tokenapi}")
    private String tokenApi;
    //用友api key
    @Value("${zmrq.flow.wzyy.keymap.key}")
    private String openApiKey;
    //用友api 签名
    @Value("${zmrq.flow.wzyy.keymap.secret}")
    private String openApiSecret;

    @Value("${zmrq.flow.wzyy.keymap.tenantid}")
    private String openApiTenantId;

    @Autowired
    private RedisTemplate redisTemplate;


    /**
     * 获取用友用户token,并推送到redis
     * @return
     */
    public String getAccessToken(){
        Object obj = redisTemplate.opsForValue().get(FlowConstant.REDIS_YY_TOKEN_KEY);
        String accessToken =null;
        if(obj==null) {
            try {
                Map<String, String> params = new HashMap<>();
                // 除签名外的其他参数
                params.put("appKey", openApiKey);
//            params.put("suiteKey",openApiKey);
                params.put("timestamp", String.valueOf(System.currentTimeMillis()));
//            params.put("tenantId",openApiTenantId);
                // 计算签名
                String signature = SignHelper.sign(params, openApiSecret);
                params.put("signature", signature);

                // 请求
                String requestUrl = openApiUrl + tokenApi;
                GenericResponse<AccessTokenResponse> response = RequestTool.doGet(requestUrl, params, new TypeReference<GenericResponse<AccessTokenResponse>>() {
                });
                log.info("获取accessToken结果： {}", JsonUtil.object2Json(response));
                if (response.isSuccess()) {
                    AccessTokenResponse data = response.getData();
                    accessToken = data.getAccessToken();
                    redisTemplate.opsForValue().set(FlowConstant.REDIS_YY_TOKEN_KEY,accessToken,60, TimeUnit.MINUTES);
                    return accessToken;
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.error("获取token错误");
            }
        }else {
            return String.valueOf(obj);
        }
        return accessToken;
    }

    /**
     * 清除redis中的用友用户token
     */
    public void delToken(){
        redisTemplate.delete(FlowConstant.REDIS_YY_TOKEN_KEY);
    }
}
