package com.tofly.workflow.wzyy.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.common.mapper.SysCommonMapper;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfBaseTypeP;
import com.tofly.workflow.baseconf.entity.TfMatterAccessP;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.entity.TfMatterPriceP;
import com.tofly.workflow.baseconf.entity.vo.MatterPriceVo;
import com.tofly.workflow.baseconf.service.TfBaseTypePService;
import com.tofly.workflow.baseconf.service.TfMatterAccessPService;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.baseconf.service.TfMatterPricePService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.wzyy.common.HttpClientUtil;
import com.tofly.workflow.wzyy.common.JsonUtil;
import com.tofly.workflow.wzyy.entity.MatterOrg;
import com.tofly.workflow.wzyy.entity.YyOrgUnit;
import com.tofly.workflow.wzyy.entity.dto.*;
import com.tofly.workflow.wzyy.entity.vo.YyStockVo;
import com.tofly.workflow.wzyy.service.MaterialBaseService;
import com.tofly.workflow.wzyy.service.MatterOrgService;
import com.tofly.workflow.wzyy.service.YyOrgUnitService;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
/**
 * 用友物资基础service
 * 仓库基础信息
 * 类型基础信息
 * 规格基础信息
 * 物资基础信息
 */
@Slf4j
@Service
public class MaterialBaseServiceImpl implements MaterialBaseService {

    @Value("${zmrq.flow.wzyy.gatewayurl}")
    private String openApiUrl;
    @Value("${zmrq.flow.wzyy.api.accessapi}")
    private String acessApi;

    @Value("${zmrq.flow.wzyy.api.wztypeapi}")
    private String wztypeapi;
    @Value("${zmrq.flow.wzyy.api.wztypeorgapi}")
    private String wztypeorgapi;
    @Value("${zmrq.flow.wzyy.api.orgapi}")
    private String orgapi;

    @Value("${zmrq.flow.wzyy.api.productapi}")
    private String productapi;
    @Value("${zmrq.flow.wzyy.api.productdetailapi}")
    private String productdetailapi;
    @Value("${zmrq.flow.wzyy.api.stockapi}")
    private String stockapi;
    @Value("${zmrq.flow.wzyy.api.detailapi}")
    private String detailapi;

    @Value("${zmrq.flow.wzyy.datatype}")
    private Long dataType;

    private final Long pageSize = 100l;

    private final Long ORGID = 666666l;
    /**
     * 用友token
     */
    @Autowired
    private AccessTokenServiceImpl accessTokenService;

    @Autowired
    private TfMatterAccessPService mAccessService;
    @Autowired
    private SysCommonMapper sysCommonMapper;

    @Autowired
    private MatterOrgService matterOrgService;
    /**
     * 用友组织
     */
    @Autowired
    private YyOrgUnitService orgUnitService;
    /**
     * 基础类型
     */
    @Autowired
    private TfBaseTypePService bTypeService;
    /**
     * 物资详情
     */
    @Autowired
    private TfMatterPService matterService;

    /**
     * 物资价格
     */
    @Autowired
    private TfMatterPricePService mPriceService;

    /**
     * 拉取用友组织信息
     * @return
     */
    @Override
    public Object pullYYOrgunit(int i) {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            return "获取accessToken失败，请稍后在试";
        }
        Map<String,Object> paramMap = new HashMap<>();
        String url = openApiUrl+orgapi+"?access_token="+accessToken;
        String parentJson = HttpClientUtil.doPost(url,paramMap);
        log.info("信息查询结果:{}",parentJson);
        Map<String,Object> resultMap = JsonUtil.decodeJson(parentJson,Map.class);
        if(CollUtil.isEmpty(resultMap)){
            log.info("拉取用友组织信息数据为空 ");
            return true;
        }
        if(resultMap.get("code") != null && ("200".equals(resultMap.get("code")))) {
            List<Map<String, Object>> datas = (List<Map<String, Object>>) resultMap.get("data");
            if(CollUtil.isNotEmpty(datas)){
                List<YyOrgUnit> units = new ArrayList<>();
                fillOrgUnit(datas, units);
                saveOrgUnit(units);
            }
        }else if(resultMap.get("code") != null && (FlowConstant.TOKEN_ERROR.equals(resultMap.get("code")))&&i<2) {
            accessTokenService.delToken();
            pullYYOrgunit(i++);
        }
        return null;
    }

    /**
     * 保存用友组织信息
     * @param units
     */
    private void saveOrgUnit(List<YyOrgUnit> units) {
        if(CollUtil.isNotEmpty(units)){
            List<Long> yyids = units.stream().map(r->r.getYyId()).collect(Collectors.toList());
            Map<Long,YyOrgUnit> map = orgUnitService.list(new QueryWrapper<YyOrgUnit>().lambda().eq(YyOrgUnit::getIsDel,FlowConstant.SHORT_ZERO)
                    .in(YyOrgUnit::getYyId,yyids)).stream().collect(Collectors.toMap(YyOrgUnit::getYyId,r->r));
            List<YyOrgUnit> up = new ArrayList<>();
            List<YyOrgUnit> in = new ArrayList<>();
            for (YyOrgUnit unit: units) {
                if (map.containsKey(unit.getYyId())){
                    YyOrgUnit ou = map.get(unit.getYyId());
                    unit.setId(ou.getId());
                    BeanUtils.copyProperties(unit,ou);
                    up.add(ou);
                }else {
                    in.add(unit);
                }
            }
            if(CollUtil.isNotEmpty(up)){
                orgUnitService.updateBatchById(up);
            }
            if(CollUtil.isNotEmpty(in)){
                orgUnitService.saveBatch(in);
            }
        }
    }

    /**
     * 填充用友组织信息
     * @param datas
     * @param units
     */
    private void fillOrgUnit(List<Map<String, Object>> datas, List<YyOrgUnit> units) {
        Map<Long,String> tmap = new HashMap<>();
        for (Map<String, Object> map: datas) {
            if(!map.containsKey("orgtype")){
                continue;
            }
            if(1!=Integer.valueOf(String.valueOf(map.get("orgtype")))){
                continue;
            }
            String orgCode = String.valueOf(map.get("code")).trim();
            if(orgCode.matches("^[0-9]*$")&&orgCode.length()>17) {
                Long tId = Long.parseLong(orgCode);
                String tname = null;
                if (tmap.containsKey(tId)) {
                    tname = tmap.get(tId);
                } else {
                    tname = sysCommonMapper.getTenantName(tId);
                    tmap.put(tId, tname);
                }
                YyOrgUnit unit = new YyOrgUnit();
                unit.setOrgLevel(Integer.valueOf(String.valueOf(map.get("level"))));
                unit.setOrgCode(orgCode);
                unit.setTenantId(tId);
                unit.setYyId(Long.parseLong(String.valueOf(map.get("id"))));
                unit.setOrgId(Long.parseLong(String.valueOf(map.get("orgid"))));
                unit.setOrgName(String.valueOf(map.get("name")));
                unit.setEnable(Short.valueOf(String.valueOf(map.get("enable"))));
                unit.setIsDel(FlowConstant.SHORT_ZERO);
                unit.setSysId(String.valueOf(map.get("sysid")));
                units.add(unit);
            }
        }
    }

    /**
     * 拉取用友仓库信息
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object pullYYAccess() {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            return "获取accessToken失败，请稍后在试";
        }
        Map<Long,String> tmap = new HashMap<>();
        reqAllYyAccess(accessToken, tmap,1l);
        return true;
    }

    /**
     * 请求全部用友仓库信息
     * @param accessToken token
     * @param tmap 用友组织与工程公司对应
     * @param index 页数索引
     */
    private void reqAllYyAccess(String accessToken, Map<Long, String> tmap,Long index) {
        Page page = reqAccess(accessToken, tmap,index);
        if(page!=null && page.getPageCount()> page.getPageIndex()){
            reqAccess(accessToken, tmap,page.getPageIndex()+1);
        }
    }

    /**
     * 请求用友仓库信息
     * @param accessToken token
     * @param tmap 用友组织与工程公司对应
     * @param index 页数索引
     * @return 请求的页数
     */
    private Page reqAccess(String accessToken, Map<Long, String> tmap,Long index) {
        Page page = null;
        Map<String,Object> paramMap = new HashMap<>();
        paramMap.put("pageIndex",index);
        paramMap.put("pageSize",pageSize);
        String url = openApiUrl+acessApi+"?access_token="+ accessToken;
        String parentJson = HttpClientUtil.doPost(url,paramMap);
        log.info("信息查询结果:{}",parentJson);
        Map<String,Object> resultMap = JsonUtil.decodeJson(parentJson,Map.class);
        if(CollUtil.isEmpty(resultMap)){
            log.info("拉取仓库数据为空");
            return page;
        }
        if(resultMap.get("code") != null && ("200".equals(resultMap.get("code")))) {
            Map<String, Object> resultDataMap = (Map<String, Object>) resultMap.get("data");
            if(resultDataMap != null && (!resultDataMap.isEmpty())) {
                List<Map<String, Object>> customMapList = (List<Map<String, Object>>) resultDataMap.get("recordList");
                if(customMapList!=null&&customMapList.size()>0){
                    List<TfMatterAccessP> omas = fillMatterAccess(customMapList, tmap);
                    if(CollUtil.isNotEmpty(omas)){
                        Map<Long,TfMatterAccessP> map = mAccessService.list(new QueryWrapper<TfMatterAccessP>().lambda().eq(TfMatterAccessP::getIsValid,FlowConstant.SHORT_ONE)
                                .eq(TfMatterAccessP::getIsDel,FlowConstant.SHORT_ZERO).and(r->{
                                    for (TfMatterAccessP ma : omas) {
                                        r.or(l->l.eq(TfMatterAccessP::getTenantId,ma.getTenantId()).eq(TfMatterAccessP::getYyWzId,ma.getYyWzId()));
                                    }
                                })).stream().collect(Collectors.toMap(TfMatterAccessP::getYyWzId,x->x));
                        List<TfMatterAccessP> up = new ArrayList<>();
                        List<TfMatterAccessP> in = new ArrayList<>();
                        for (TfMatterAccessP ma: omas) {
                            if(map.containsKey(ma.getYyWzId())){
                                TfMatterAccessP ma1 = map.get(ma.getYyWzId());
                                ma.setId(ma1.getId());
                                BeanUtils.copyProperties(ma,ma1);
                                ma1.setAccessType(null);
                                up.add(ma1);
                            }else {
                                in.add(ma);
                            }
                        }
                        if(CollUtil.isNotEmpty(up)){
                            mAccessService.updateBatchById(up);
                        }
                        if(CollUtil.isNotEmpty(in)){
                            mAccessService.saveBatch(in);
                        }
                    }
                }
                page = new Page(Long.valueOf(String.valueOf(resultDataMap.get("pageCount"))),Long.valueOf(String.valueOf(resultDataMap.get("pageIndex"))));
            }
        }
        return page;
    }

    /**
     * 填充仓库信息
     * @param customMapList 用友仓库信息
     * @return 仓库对象
     */
    private List<TfMatterAccessP> fillMatterAccess(List<Map<String, Object>> customMapList,Map<Long,String> tmap) {
        List<TfMatterAccessP> omas = new ArrayList<>();
        for (Map<String, Object> map: customMapList) {
            if(map.containsKey("org_code")){
                String orgCode = String.valueOf(map.get("org_code")).trim();
                if(orgCode.matches("^[0-9]*$")&&orgCode.length()>17){
                    Long tId = Long.parseLong(orgCode);
                    String tname = null;
                    if(tmap.containsKey(tId)){
                        tname = tmap.get(tId);
                    }else {
                        tname = sysCommonMapper.getTenantName(tId);
                        tmap.put(tId,tname);
                    }
                    if(StringUtils.isNotEmpty(tname)){
                        TfMatterAccessP mAccess = new TfMatterAccessP();
                        mAccess.setTenantId(tId);
                        mAccess.setYyWzId(Long.parseLong(String.valueOf(map.get("id"))));
                        mAccess.setYyCode(String.valueOf(map.get("code")));
                        mAccess.setWarehouseName(String.valueOf(map.get("name")));
                        mAccess.setAddress(String.valueOf(map.get("address")));
                        mAccess.setPhone(String.valueOf(map.get("phone")));
                        mAccess.setLinkman(String.valueOf(map.get("linkman")));
                        mAccess.setIsDel(FlowConstant.SHORT_ZERO);
                        mAccess.setIsValid(FlowConstant.SHORT_ONE);
                        mAccess.setDateTime(new Date());
                        if(map.containsKey("pubts")){
                            mAccess.setDateTime(DateUtil.parse(String.valueOf(map.get("pubts"))));
                        }
                        if(map.containsKey("iUsed")) {
                            if("disable".equals(String.valueOf(map.get("iUsed")))){
                                mAccess.setIsValid(FlowConstant.SHORT_ZERO);
                            }
                        }
                        //暂时默认0 出库
                        mAccess.setAccessType(FlowConstant.SHORT_ZERO);
                        omas.add(mAccess);
                    }
                }
            }
        }
        return omas;
    }

    /**
     * 拉取物料分类信息
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Object pullYYClassify(int i) {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            return "获取accessToken失败，请稍后在试";
        }

        Map<String,Object> paramMap = new HashMap<>();
        String url = openApiUrl+wztypeapi+"?access_token="+accessToken;
        String parentJson = HttpClientUtil.doPost(url,paramMap);
        log.info("信息查询结果:{}",parentJson);
        RespClassisfyDto dto = JSON.parseObject(parentJson, RespClassisfyDto.class);
        if("200".equals(dto.getCode())){
            if(CollUtil.isNotEmpty(dto.getData())){
                Map<Long,TfBaseTypeP> map = new HashMap<>();
                List<ClassifyDto> data = dto.getData();
                saveWzType(map,data);
            }
        }else if(FlowConstant.TOKEN_ERROR.equals(dto.getCode())&&i<2){
            accessTokenService.delToken();
            pullYYClassify(i++);
        }
        return true;
    }
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Object  pullWzTypeOverOrg(int i){
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            return "获取accessToken失败，请稍后在试";
        }

        Map<String,Object> paramMap = new HashMap<>();
        paramMap.put("pageIndex",1);
        paramMap.put("pageSize",999);
        String url = openApiUrl+wztypeorgapi+"?access_token="+accessToken;
        String parentJson = HttpClientUtil.doPost(url,paramMap);
        log.info("物资类型信息查询结果:{}",parentJson);
        WzTypeOverOrgDto dto = JSON.parseObject(parentJson, WzTypeOverOrgDto.class);
        if("200".equals(dto.getCode())){
            if(CollUtil.isNotEmpty(dto.getData())){
                 List<WzTypeOverOrg> data = dto.getData();
                 //TODO 更新物资类型里面的org
                for (WzTypeOverOrg datum : data) {
                    final TfBaseTypeP one =
                            bTypeService.
                                    getOne(Wrappers.<TfBaseTypeP>lambdaQuery()
                                            .eq(TfBaseTypeP::getTypeCode, datum.getCode())
                                            .eq (TfBaseTypeP::getIsDel,FlowConstant.SHORT_ZERO)
                                            .eq(TfBaseTypeP::getIsEnd,FlowConstant.SHORT_ONE))
                            ;
                    if(one !=null){
                        one.setOrgCode(datum.getOrgCode());
                        //更新物资类型的区域编码
                        bTypeService.updateById(one);
                    }


                }
            }
        }else if(FlowConstant.TOKEN_ERROR.equals(dto.getCode())&&i<2){
            accessTokenService.delToken();
            pullWzTypeOverOrg(i++);
        }
        return true;
    }

    /**
     * 拉取用友物料档案信息
     * @return
     */
    @Override
  //  @Transactional(rollbackFor = Exception.class)
    public Object pullYYProduct() {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            return "获取accessToken失败，请稍后在试";
        }
        Map<Long,Long> mapType = new HashMap<>();
        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String str = matterService.getObj(new QueryWrapper<TfMatterP>().select("max(pubts)").lambda().isNotNull(TfMatterP::getPubts),r->sf.format(r));
        List<YyOrgUnit> oulist = orgUnitService.list(new QueryWrapper<YyOrgUnit>().lambda().eq(YyOrgUnit::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(YyOrgUnit::getEnable,FlowConstant.SHORT_ONE));
        saveAllProduct(accessToken, mapType,1l,str,oulist);
        return true;
    }

    /**
     * 查询用友库存信息
     * @param dto 查询用友库存条件
     * @return
     */
    @Override
    public Object getStockInfo(StockDto dto,int i) {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            return "获取accessToken失败，请稍后在试";
        }
        Map<String,Object> paramMap = new HashMap<>();
        paramMap.put("bStockStatusDocNotNull",true);
        if(dto.getTenantId()!=null) {
            Long orgId = orgUnitService.getObj(new QueryWrapper<YyOrgUnit>().lambda().eq(YyOrgUnit::getTenantId, dto.getTenantId())
                    .select(YyOrgUnit::getOrgId), r -> Long.valueOf(String.valueOf(r)));
            paramMap.put("org",orgId);
        }
      if(dto.getWarehouse()!=null){
            paramMap.put("warehouse",dto.getWarehouse());
        }
        if(dto.getProduct()!=null){
            paramMap.put("product",dto.getProduct());
        }
        String url = openApiUrl+stockapi+"?access_token="+ accessToken;
        String parentJson = HttpClientUtil.doPost(url, paramMap);
        log.info("信息查询结果:{}",parentJson);
        RespYyStockDto rdto = JSON.parseObject(parentJson,RespYyStockDto.class);
        if(FlowConstant.TOKEN_ERROR.equals(rdto.getCode())&&i<3){
            accessTokenService.delToken();
            getStockInfo(dto,i++);
        }
        if(FlowConstant.STRING_TWO_HUNDRED.equals(rdto.getCode())){
            if (rdto.getData()!=null) {
                List<YyStockVo> datas = rdto.getData();
                if(FlowConstant.SHORT_ONE==dto.getIsPrice()) {
                    Set<Long> productIds = datas.stream().map(YyStockVo::getProduct).collect(Collectors.toSet());

                    List<MatterPriceVo> mpvs = mPriceService.getMatterPriceByMatterIds(productIds, dto.getTenantId()!=null?dto.getTenantId(): SecurityUtils.getTenantId());
                    if (CollUtil.isNotEmpty(mpvs)) {
                        Map<Long, MatterPriceVo> map = mpvs.stream().collect(Collectors.toMap(MatterPriceVo::getYyMatterId, r -> r,(r1,r2)->r1));
                        for (YyStockVo svo : datas) {
                            if (map.containsKey(svo.getProduct())) {
                                MatterPriceVo mpp = map.get(svo.getProduct());
                                svo.setPrice(mpp.getPrice());
                                svo.setInstallPrice(mpp.getInstallPrice());
                                svo.setBatchPrice(mpp.getBatchPrice());
                                svo.setMatterId(mpp.getMatterId());
                                svo.setMatterType(mpp.getMatterType());
                            }
                        }
                    }
                }
                return datas;
            }
        }
        return null;
    }



    /**
     * 保存所有物料档案信息
     * @param accessToken 请求token
     * @param mapType 用友类型与工程类型id对应
     * @param index 请求第几页
     */
    private void saveAllProduct(String accessToken, Map<Long, Long> mapType,Long index,String time,List<YyOrgUnit> oulist) {
        Map<String,Object> paramMap = new HashMap<>();
        paramMap.put("pageIndex",index);
        paramMap.put("pageSize",pageSize);
        if (StringUtils.isNotEmpty(time)) {
            Map<String, Object> simpleMap = new HashMap<>();
            simpleMap.put("pubts",time);
            paramMap.put("simple", simpleMap);
            paramMap.put("detail.stopstatus", false);
           //paramMap.put("code","26300005");
        }
        String url = openApiUrl+productapi+"?access_token="+ accessToken;
        Page page = reqSaveProduct(accessToken,mapType, paramMap, url,oulist);
        if(page!=null && page.getPageCount()> page.getPageIndex()){
            ThreadUtil.sleep(10, TimeUnit.SECONDS);
            saveAllProduct(accessToken, mapType,page.getPageIndex()+1,time,oulist);
        }
    }

    /**
     * 请求保存物料档案信息
     * @param mapType 用友类型与工程类型id对应
     * @param paramMap 请求参数
     * @param url 请求地址
     * @return 请求页数信息
     */
    @Transactional
   public   Page reqSaveProduct(String accessToken,Map<Long, Long> mapType, Map<String, Object> paramMap, String url,List<YyOrgUnit> oulist) { String parentJson = HttpClientUtil.doPost(url, paramMap);
        RespPageProductDto dto = JSON.parseObject(parentJson, RespPageProductDto.class);
        Page page = null;
        if(FlowConstant.STRING_TWO_HUNDRED.equals(dto.getCode())) {
            if (dto.getData()!=null) {
                PageProductDto pdto = dto.getData();
                if(CollUtil.isNotEmpty(pdto.getRecordList())){
                    List<ProductDto> list = pdto.getRecordList();
                    if(CollUtil.isNotEmpty(list)){

                        Set<Long> yyids =  list.stream().map(ProductDto::getYyId).collect(Collectors.toSet());
                        //获取仓库中已经存在材料信息
                        Map<String,TfMatterP> ms = matterService.list(new QueryWrapper<TfMatterP>().lambda().eq(TfMatterP::getIsDel,FlowConstant.SHORT_ZERO)
                                .in(TfMatterP::getYyId,yyids)).stream().collect(Collectors.toMap(TfMatterP::getMatterNumber,r->r));

                        List<TfMatterP> up = new ArrayList<>();
                        List<TfMatterP> in = new ArrayList<>();

                        for (ProductDto p:list) {
                            Long matterType = null;
                            if(mapType.containsKey(p.getYyTypeId())){
                                matterType = mapType.get(p.getYyTypeId());
                            }else {
                                matterType = bTypeService.getObj(new QueryWrapper<TfBaseTypeP>().lambda().eq(TfBaseTypeP::getYyTypeId,p.getYyTypeId()).eq(TfBaseTypeP::getIsDel,FlowConstant.SHORT_ZERO)
                                        .select(TfBaseTypeP::getId),r->{return r==null?null:Long.valueOf(String.valueOf(r));});
                                mapType.put(p.getYyTypeId(),matterType);
                            }
                            String speces = StringUtils.isNotEmpty(p.getModelDescription().getZhcn())?p.getModelDescription().getZhcn():"无";
                            String standards = p.getModel().getZhcn();
                            if(ms.containsKey(p.getMatterNumber())){
                                TfMatterP mp = ms.get(p.getMatterNumber());
                                BeanUtils.copyProperties(p,mp);
                                mp.setSpecs(speces);
                                mp.setMatterName(p.getName().getZhcn());
                                mp.setStandards(StringUtils.isNotEmpty(standards)?standards+speces:speces);
                                mp.setMatterType(matterType);
                                mp.setStockUnit(p.getDetail().getStockUnit());
                                mp.setStockUnitName(p.getDetail().getStockUnitName());
                                up.add(mp);
                            }else {
                                TfMatterP mp = new TfMatterP();
                                BeanUtils.copyProperties(p,mp);
                                mp.setCategory(1l);
                                mp.setSpecs(speces);
                                mp.setMatterName(p.getName().getZhcn());
                                mp.setStandards(StringUtils.isNotEmpty(standards)?standards+speces:speces);
                                mp.setMatterType(matterType);
                                mp.setIsDel(FlowConstant.SHORT_ZERO);
                                mp.setStockUnit(p.getDetail().getStockUnit());
                                mp.setStockUnitName(p.getDetail().getStockUnitName());
                                in.add(mp);
                            }
                        }
                       List<TfMatterP> tms = new ArrayList<>();
                        Map<Long,Integer > map = new HashMap<>();
                        Set<Long> idSet = new HashSet<>();
                        for (TfMatterP tfMatterP : up) {
                             Integer idNum = map.get(tfMatterP.getId());
                            if(idNum==null){
                                idNum=1;
                                map.put(tfMatterP.getId(),idNum);
                            }else{
                                map.put(tfMatterP.getId(),++idNum);
                            }
                            if(idNum>1){
                                idSet.add(tfMatterP.getId());
                            }
                        }

                        System.out.println(JSON.toJSONString(idSet));
                      if(CollUtil.isNotEmpty(up)){
                            matterService.updateBatchById(up);
                            tms.addAll(up);
                        }
                        if(CollUtil.isNotEmpty(in)){
                            matterService.saveBatch(in);
                            tms.addAll(in);
                        }
                       if(CollUtil.isNotEmpty(tms)){
                            getYyMatterTenant(accessToken, tms);
                        }

                        if(CollUtil.isNotEmpty(tms)){
                            getYyMatterPrice(accessToken,oulist, tms);
                        }
                    }
                }
                page = new Page(pdto.getPageCount(),pdto.getPageIndex());
            }
        }
        return page;
    }

    /**
     * 获取材料组织绑定信息
     * @param accessToken
     * @param tms
     */
    @Override
    public void getYyMatterTenant(String accessToken,  List<TfMatterP> tms) {
         accessToken = accessTokenService.getAccessToken();

        List<MatterOrg> matterOrgList = new ArrayList<>();
            for (TfMatterP tm: tms) {
                Map<String,Object> paramMap = new HashMap<>();
                paramMap.put("id",tm.getYyId());
                String url = openApiUrl+productdetailapi+"?access_token="+ accessToken +"&id="+tm.getYyId()+"&productApplyRangeId="+tm.getProductApplyRangeId();
                log.info("请求参数："+url);
                String parentJson = HttpClientUtil.doGet(url);
                log.info("材料组织绑定信息::"+parentJson);

                RespProductDetailTenantDto dto = JSON.parseObject(parentJson, RespProductDetailTenantDto.class);
                if(FlowConstant.STRING_TWO_HUNDRED.equals(dto.getCode())) {
                     ProductDetailTenantDto data = dto.getData();

                    final boolean remove =
                            matterOrgService.remove(Wrappers.<MatterOrg>lambdaQuery()
                            .eq(MatterOrg::getYyId, tm.getYyId()));
                    for (ProductOrg p : data.getProductOrgs()) {

                        MatterOrg  matterOrg = new MatterOrg();
                        matterOrg.setMatterId(tm.getId());
                        matterOrg.setOrgId(p.getOrgId());
                        matterOrg.setOrgCode(p.getOrgCode());
                        matterOrg.setYyId(tm.getYyId());
                        matterOrgList.add(matterOrg);
                    }


                }
                ThreadUtil.sleep(2, TimeUnit.SECONDS);
            }

            matterOrgService.saveBatch(matterOrgList,200);
   }


    @Override
    public void getYyMatterPrice(String accessToken,List<YyOrgUnit> oulist, List<TfMatterP> tms) {
       if(StringUtils.isEmpty(accessToken)){
           accessToken = accessTokenService.getAccessToken();
       }
        String url = openApiUrl+detailapi+"?access_token="+ accessToken;
        List<TfMatterPriceP> mps = new ArrayList<>();
        for (YyOrgUnit you: oulist) {
            for (TfMatterP tm: tms) {
                Map<String,Object> paramMap = new HashMap<>();
                paramMap.put("id",tm.getYyId());
                paramMap.put("orgCode",you.getTenantId());
                String parentJson = HttpClientUtil.doPost(url, paramMap);
                RespProductDto dto = JSON.parseObject(parentJson, RespProductDto.class);
                if(FlowConstant.STRING_TWO_HUNDRED.equals(dto.getCode())) {
                    ProductDto pDto = dto.getData();
                    ProductDetailDto dDto = pDto.getDetail();
                    if(dDto != null){
                        TfMatterPriceP mpp = new TfMatterPriceP();
                        BeanUtils.copyProperties(dDto,mpp);
                        mpp.setYyMatterId(tm.getYyId());
                        mpp.setMatterId(tm.getId());
                        mpp.setIsValid(FlowConstant.SHORT_ONE);
                        mpp.setIsDel(FlowConstant.SHORT_ZERO);
                        mpp.setTenantId(you.getTenantId());
                        mps.add(mpp);
                        mPriceService.update(new UpdateWrapper<TfMatterPriceP>().lambda().set(TfMatterPriceP::getIsValid,FlowConstant.SHORT_ZERO)
                                .set(TfMatterPriceP::getIsDel,FlowConstant.SHORT_ONE).eq(TfMatterPriceP::getYyMatterId,tm.getYyId())
                                .eq(TfMatterPriceP::getTenantId,you.getTenantId()));
                    }
                }
                ThreadUtil.sleep(2, TimeUnit.SECONDS);
            }
        }
        if(CollUtil.isNotEmpty(mps)){
            mPriceService.saveBatch(mps);
        }
    }

    @AllArgsConstructor
    @NoArgsConstructor
    @Data
    class Page{
        Long pageCount;
        Long  pageIndex;
    }

    /**
     * 递归保存物料类型
     * @param map 物料类型map
     * @param data 用友物料类型数据
     */
    private void saveWzType(Map<Long, TfBaseTypeP> map, List<ClassifyDto> data) {
        map.putAll(saveYyWzType(map,data));
        List<ClassifyDto> dtos = new ArrayList<>();
        for (ClassifyDto dto:data) {
            if(CollUtil.isNotEmpty(dto.getChildren())){
                dtos.addAll(dto.getChildren());
            }
        }
        if(CollUtil.isNotEmpty(dtos)) {
            saveWzType(map, dtos);
        }
    }

    /**
     * 保存物料类型
     * @param tbmap 物料类型map
     * @param data 用友物料类型数据
     * @return
     */
    private Map<Long,TfBaseTypeP> saveYyWzType(Map<Long, TfBaseTypeP> tbmap,List<ClassifyDto> data){
        Set<Long> ids = data.stream().map(ClassifyDto::getYyTypeId).collect(Collectors.toSet());
        Map<Long,TfBaseTypeP> map =bTypeService.list(new QueryWrapper<TfBaseTypeP>().lambda().eq(TfBaseTypeP::getIsDel,FlowConstant.SHORT_ZERO)
                .in(TfBaseTypeP::getYyTypeId,ids)).stream().collect(Collectors.toMap(TfBaseTypeP::getYyTypeId,r->r,(r1,r2)->r1));
        List<TfBaseTypeP> bts = new ArrayList<>();
        Map<Long,TfBaseTypeP> in = new HashMap<>();
        Map<Long,TfBaseTypeP> up = new HashMap<>();
        for (ClassifyDto fdto:data){
            Long yyParentId = fdto.getYyParentId();
            Long prarentId = null;
            if(yyParentId != null&& tbmap.containsKey(yyParentId)){
                prarentId = tbmap.get(yyParentId).getId();
            }
            if(map.containsKey(fdto.getYyTypeId())&&!up.containsKey(fdto.getYyTypeId())){
                TfBaseTypeP bt = map.get(fdto.getYyTypeId());
                BeanUtils.copyProperties(fdto,bt);
                bt.setParentId(prarentId);
                bt.setIsValid(bt.getIsValid()==FlowConstant.SHORT_ZERO?FlowConstant.SHORT_ONE:FlowConstant.SHORT_ZERO);
                up.put(fdto.getYyTypeId(),bt);
            }else if(!map.containsKey(fdto.getYyTypeId())&&!in.containsKey(fdto.getYyTypeId())){
                TfBaseTypeP bt = new TfBaseTypeP();
                BeanUtils.copyProperties(fdto,bt);
                bt.setIsDel(FlowConstant.SHORT_ZERO);
                bt.setParentId(prarentId);
                bt.setIsValid(bt.getIsValid()==FlowConstant.SHORT_ZERO?FlowConstant.SHORT_ONE:FlowConstant.SHORT_ZERO);
                bt.setDataType(dataType);
                in.put(fdto.getYyTypeId(),bt);
            }
        }
        if(CollUtil.isNotEmpty(up)){
            bTypeService.updateBatchById(up.values());
        }
        if(CollUtil.isNotEmpty(in)){
            bTypeService.saveBatch(in.values());
        }
        bts.addAll(up.values());
        bts.addAll(in.values());
        return bts.stream().collect(Collectors.toMap(TfBaseTypeP::getYyTypeId,r->r));
    }



}
