package com.tofly.workflow.wzyy.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.workflow.baseconf.entity.TfMatterP;
import com.tofly.workflow.baseconf.service.TfMatterPService;
import com.tofly.workflow.buss.entity.vo.MaterialItemVo;
import com.tofly.workflow.buss.service.TfProjectMaterialWService;
import com.tofly.workflow.common.FlowConstant;
import com.tofly.workflow.process.entity.vo.ProcessMaterialVo;
import com.tofly.workflow.process.service.TfProcessLService;
import com.tofly.workflow.wzyy.common.HttpClientUtil;
import com.tofly.workflow.wzyy.common.JsonUtil;
import com.tofly.workflow.wzyy.entity.MaterialCharacter;
import com.tofly.workflow.wzyy.entity.MaterialItemOutIn;
import com.tofly.workflow.wzyy.entity.MaterialOutIn;
import com.tofly.workflow.wzyy.entity.YyOrgUnit;
import com.tofly.workflow.wzyy.entity.dto.*;
import com.tofly.workflow.wzyy.mapper.MaterialOutInMapper;
import com.tofly.workflow.wzyy.service.MaterialCharacterService;
import com.tofly.workflow.wzyy.service.MaterialItemOutInService;
import com.tofly.workflow.wzyy.service.MaterialOutInService;
import com.tofly.workflow.wzyy.service.YyOrgUnitService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 用友物料出库、入库数据
 */
@Slf4j
@Service
public class MaterialOutInServiceImpl extends ServiceImpl<MaterialOutInMapper, MaterialOutIn>  implements MaterialOutInService {

    @Value("${zmrq.flow.wzyy.gatewayurl}")
    private String openApiUrl;
    /**
     * 出库保存
     */
    @Value("${zmrq.flow.wzyy.api.materialoutapi}")
    private String materialoutapi;

    /**
     * 其它入库
     */
    @Value("${zmrq.flow.wzyy.api.otherStockapi}")
    private String otherStockApi;

    /**
     * 转库
     */
    @Value("${zmrq.flow.wzyy.api.storetransferapi}")
    private String storetransferApi;

    /**
     * 物料出库结果查询
     */
    @Value("${zmrq.flow.wzyy.api.outbearfruitapi}")
    private String outBearFruitApi;

    /**
     * 物料其它入库结果查询（退库）
     */
    @Value("${zmrq.flow.wzyy.api.inbearfruitapi}")
    private String inBearFruitApi;

    /**
     * 转库结果查询
     */
    @Value("${zmrq.flow.wzyy.api.zkbearfruitapi}")
    private String zkbearfruitApi;

    /**
     * 出库列表查询
     */
    @Value("${zmrq.flow.wzyy.api.outboundlistapi}")
    private String outboundListApi;


    /**
     * 其它入库列表查询
     */
    @Value("${zmrq.flow.wzyy.api.otherinboundlistapi}")
    private String otherInboundListApi;

    /**
     * 用友token
     */
    @Autowired
    private AccessTokenServiceImpl accessTokenService;
    /**
     * 用友组织
     */
    @Autowired
    private YyOrgUnitService orgUnitService;

    /**
     * 物资详情列表
     */
    @Autowired
    private MaterialItemOutInService mItemOutInService;

    /**
     * 用友物料出入库特征信息
     */
    @Autowired
    private MaterialCharacterService characterService;

    /**
     * 工程材料(设计预算、领料出库)
     */
    @Autowired
    private TfProjectMaterialWService pmaterialService;

    /**
     * 流程信息
     */
    @Autowired
    private TfProcessLService processService;

    /**
     * 工程用友物料出库
     * @param parseLong
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void materialOut(long parseLong,Short mType) {

        ProcessMaterialVo pm = processService.getMaterialNodeById(parseLong);
        if(pm==null||pm.getWarehouse()==null){
            log.error("出库流程："+parseLong+" 未获取到出库流程或未获取到用友仓库id");
            return;
        }
        YyOrgUnit yyOrg = orgUnitService.getOne(new QueryWrapper<YyOrgUnit>().lambda().eq(YyOrgUnit::getTenantId, pm.getTenantId())
                .eq(YyOrgUnit::getIsDel, FlowConstant.SHORT_ZERO));
        if(yyOrg==null){
            log.error("出库流程："+parseLong+" 未获取到用友对应组织信息请先维护");
            return;
        }
        if(FlowConstant.SHORT_ONE!=mType&&FlowConstant.SHORT_THREE!=mType){
            log.error("mtype:赋值异常：："+mType);
            return;
        }
        Date date = new Date();
        MaterialOutDto outDto = new MaterialOutDto();
        outDto.setOrg(yyOrg.getOrgId());
        outDto.setVouchdate(pm.getDataTime());
        outDto.setFactoryOrg(yyOrg.getOrgId());
        outDto.setWarehouse(pm.getWarehouse());
        outDto.setBustype(FlowConstant.OUT_BUSTYPE);
        HeadItemDto headItem = new HeadItemDto();
        headItem.setDefine1(pm.getProcessNumber());
        headItem.setDefine2(pm.getStaffName());
        outDto.setHeadItem(headItem);
        outDto.set_status(FlowConstant.INSERT);
        //自定义项特征
        OutDefineDto defineDto = new OutDefineDto();
        //租户id用友不允许用 ytenant ytenantid
        //defineDto.setYtenant(String.valueOf(pm.getTenantId()));
        defineDto.setGcxz(String.valueOf(pm.getUserType()));
//        defineDto.setXm001(String.valueOf(pm.getRelationId()));
//        defineDto.setXm001Name(pm.getProjectName());
        //defineDto.setXmgly(String.valueOf(pm.getDutyUser()));
        defineDto.setXmglyName(pm.getDutyUserNAME());
        defineDto.setA001(pm.getStaffName());
        defineDto.setA02(pm.getProcessNumber());
        defineDto.setXmx(pm.getProjectName());
        defineDto.setXmid(String.valueOf(pm.getRelationId()));
        defineDto.setXmglyx(pm.getDutyUserNAME());
        outDto.setMaterialOutDefineCharacter(defineDto);
        //流程物料及数量信息
        List<MaterialItemVo> pms = pmaterialService.getMaterialOutIn(pm.getProcessId(),pm.getNodeId(), mType==1?FlowConstant.SHORT_THREE:FlowConstant.SHORT_NINE);
        if(CollUtil.isEmpty(pms)){
            log.error("出库流程："+parseLong+"未查询到出库物料！");
            return;
        }
        List<MaterialItemOutDto> itemOutDtos = new ArrayList<>();
        for (MaterialItemVo vo:pms) {
            MaterialItemOutDto itemOut = new MaterialItemOutDto();
            BeanUtils.copyProperties(vo,itemOut);
            itemOut.set_status(FlowConstant.INSERT);
            itemOut.setInvExchRate(FlowConstant.INTEGER_ONE);
            itemOut.setContactsQuantity(vo.getQty());
            itemOutDtos.add(itemOut);
        }
        outDto.setMaterOuts(itemOutDtos);
        List<MaterialItemOutIn> itemios = new ArrayList<>();
        MaterialOutIn outIn = saveYyOutInDb(pm.getProcessId(),pm.getRelationId(),outDto,itemios,mType,pm.getTenantId());
        Map<String,Object> remap = new HashMap<>();
        remap.put("data", outDto);
        pushMaterialOutToYy( remap, itemios, outIn,FlowConstant.INTEGER_ONE,materialoutapi,0);
    }

    /**
     * 工程用友物料退库
     * @param parseLong
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void returnOfMaterial(long parseLong) {
        ProcessMaterialVo pm = processService.getMaterialNodeById(parseLong);
        if(pm==null||pm.getWarehouse()==null){
            log.error("退库流程："+parseLong+" 未获取到出库流程或未获取到用友仓库id");
            return;
        }
        YyOrgUnit yyOrg = orgUnitService.getOne(new QueryWrapper<YyOrgUnit>().lambda().eq(YyOrgUnit::getTenantId, pm.getTenantId())
                .eq(YyOrgUnit::getIsDel, FlowConstant.SHORT_ZERO));
        if(yyOrg==null){
            log.error("退库流程："+parseLong+" 未获取到用友对应组织信息请先维护");
            return;
        }
        Date date = new Date();
        MaterialRetDto retDto = new MaterialRetDto();
        retDto.setOrg(yyOrg.getOrgId());
        retDto.setVouchdate(pm.getDataTime());
        retDto.setFactoryOrg(yyOrg.getOrgId());
        retDto.setWarehouse(pm.getWarehouse());
        retDto.setBustype(FlowConstant.RET_BUSTYPE);
        HeadItemDto headItem = new HeadItemDto();
        headItem.setDefine1(pm.getProcessNumber());
        headItem.setDefine2(pm.getStaffName());
        retDto.setHeadItem(headItem);
        retDto.set_status(FlowConstant.INSERT);
        retDto.setResubmitCheckKey(String.valueOf(parseLong));
        retDto.setAccountOrg(yyOrg.getOrgCode());
        //自定义项特征
        OutDefineDto defineDto = new OutDefineDto();
        // 租户id用友不允许用 ytenant ytenantid
        //defineDto.setYtenant(String.valueOf(pm.getTenantId()));
        defineDto.setGcxz(String.valueOf(pm.getUserType()));
//        defineDto.setXm001(String.valueOf(pm.getRelationId()));
//        defineDto.setXm001Name(pm.getProjectName());
        //defineDto.setXmgly(String.valueOf(pm.getDutyUser()));
        defineDto.setXmglyName(pm.getDutyUserNAME());
        defineDto.setA001(pm.getStaffName());
        defineDto.setA02(pm.getProcessNumber());
        defineDto.setXmx(pm.getProjectName());
        defineDto.setXmid(String.valueOf(pm.getRelationId()));
        defineDto.setXmglyx(pm.getDutyUserNAME());
        retDto.setOthInRecordDefineCharacter(defineDto);
        //工程物料及数量信息
        List<MaterialItemVo> pms = pmaterialService.getMaterialOutIn(pm.getProcessId(),pm.getNodeId(),FlowConstant.SHORT_FIVE);
        if(CollUtil.isEmpty(pms)){
            log.error("退库流程："+parseLong+"未查询到退库物料！");
            return;
        }
        List<MaterialItemRetDto> itemOutDtos = new ArrayList<>();
        for (MaterialItemVo vo:pms) {
            MaterialItemRetDto itemOut = new MaterialItemRetDto();
            BeanUtils.copyProperties(vo,itemOut);
            itemOut.set_status(FlowConstant.INSERT);
            itemOut.setInvExchRate(FlowConstant.INTEGER_ONE);
            itemOut.setContactsQuantity(vo.getQty());
            itemOut.setSubQty(vo.getQty());
            itemOut.setUnitExchangeType(FlowConstant.SHORT_ONE);
            itemOutDtos.add(itemOut);
        }
        retDto.setOthInRecords(itemOutDtos);
        List<MaterialItemOutIn> itemios = new ArrayList<>();
        MaterialOutIn outIn = saveYyOutInDb(pm.getProcessId(),pm.getRelationId(),retDto, itemios,FlowConstant.SHORT_ZERO,pm.getTenantId());

        pushMaterialRetToYy(retDto, itemios, outIn,0);
    }

    /**
     * 物料转库
     * @param parseLong 小仓库附属流程Id
     */
    @Override
    public void stockTransfer(long parseLong) {
        ProcessMaterialVo pm = processService.getMaterialNodeById(parseLong);
        if(pm==null||pm.getWarehouse()==null||pm.getOutWarehouse()==null){
            log.error("出库流程："+parseLong+" 未获取到出库流程或未获取到用友仓库id");
            return;
        }
        YyOrgUnit yyOrg = orgUnitService.getOne(new QueryWrapper<YyOrgUnit>().lambda().eq(YyOrgUnit::getTenantId, pm.getTenantId())
                .eq(YyOrgUnit::getIsDel, FlowConstant.SHORT_ZERO));
        if(yyOrg==null){
            log.error("出库流程："+parseLong+" 未获取到用友对应组织信息请先维护");
            return;
        }
        StockTransferDto retDto = new StockTransferDto();
        retDto.setOrg(yyOrg.getOrgId());
        retDto.setVouchdate(pm.getDataTime());
        retDto.setInWarehouse(pm.getWarehouse());
        retDto.setOutWarehouse(pm.getOutWarehouse());
        retDto.setBusinesstype(FlowConstant.ZK_BUSTYPE);
        retDto.set_status(FlowConstant.INSERT);
        HeadItemDto headItem = new HeadItemDto();
        headItem.setDefine1(pm.getProcessNumber());
        headItem.setDefine2(pm.getStaffName());
        retDto.setStoreTransCustomItem(headItem);
        //工程物料及数量信息
        List<MaterialItemVo> pms = pmaterialService.getMaterialOutIn(pm.getProcessId(),pm.getNodeId(),FlowConstant.SHORT_EIGHT);
        if(CollUtil.isEmpty(pms)){
            log.error("出库流程："+parseLong+"未查询到出库物料！");
            return;
        }
        List<MaterialItemRetDto> itemRetDtos = new ArrayList<>();
        for (MaterialItemVo vo:pms) {
            MaterialItemRetDto itemRetDto = new MaterialItemRetDto();
            BeanUtils.copyProperties(vo, itemRetDto);
            itemRetDto.set_status(FlowConstant.INSERT);
            itemRetDto.setInvExchRate(FlowConstant.INTEGER_ONE);
           itemRetDto.setProduct(vo.getProduct());
           itemRetDto.setContactsQuantity(vo.getQty());
            itemRetDto.setSubQty(vo.getQty());
           itemRetDtos.add(itemRetDto);
        }
        retDto.setDetails(itemRetDtos);
        List<MaterialItemOutIn> itemios = new ArrayList<>();
        MaterialOutIn outIn = saveStockMaterial(pm.getProcessId(),pm.getRelationId(),retDto, itemRetDtos, itemios,pm.getTenantId());
        Map<String,Object> remap = new HashMap<>();
        remap.put("data", retDto);
        pushTransferMaterialOutToYy( remap, itemios, outIn,FlowConstant.INTEGER_TWO,storetransferApi,0);
    }

    /**
     * 存储转库物料信息（小仓库入库）
     * @param retDto 出库、入库信息
     * @param itemRetDtos 物料详情
     * @param itemios 保存后物料详情
     */
    private MaterialOutIn saveStockMaterial(Long processId,Long relationId,StockTransferDto retDto, List<MaterialItemRetDto> itemRetDtos, List<MaterialItemOutIn> itemios,Long tenantId) {
        MaterialOutIn outIn = new MaterialOutIn();
        BeanUtils.copyProperties(retDto, outIn);
        outIn.setProcessId(processId);
        outIn.setRelationId(relationId);
        outIn.setTenantId(tenantId);
        outIn.setWarehouse(retDto.getInWarehouse());
        outIn.setBustype(retDto.getBusinesstype());
        outIn.setProcessNumber(retDto.getStoreTransCustomItem().getDefine1());
        outIn.setUserName(retDto.getStoreTransCustomItem().getDefine2());
        outIn.setMType(FlowConstant.SHORT_TWO);
        outIn.setIsDel(FlowConstant.SHORT_ZERO);
        outIn.setIsFinish(FlowConstant.SHORT_ZERO);
        outIn.setPushNumber(FlowConstant.INTEGER_ONE);
        this.save(outIn);
        saveMaterialItem(itemios,outIn.getId(), itemRetDtos);
        return outIn;
    }


    /**
     * 推送物料退库数据到用友
     * @param retDto 推送到用友数据
     * @param itemios 本地推送物料详情
     * @param outIn 本地物料数据
     */
    private void pushMaterialRetToYy(MaterialRetDto retDto, List<MaterialItemOutIn> itemios, MaterialOutIn outIn,int i) {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            log.error("退库流程："+ outIn.getProcessId() +"获取accessToken失败，请稍后在试  ");
            return;
        }
        //修改为出库接口  otherStockApi  materialoutapi
        String url = openApiUrl+materialoutapi+"?access_token="+ accessToken;
        //将  retDto 转换为 MaterialOutDto
        final MaterialOutDto outDto = toOutDto(retDto);
        outDto.setBustype(FlowConstant.OUT_BUSTYPE);
        Map<String,Object> remap = new HashMap<>();
        remap.put("data", outDto);
        log.info("退库请求参数："+JSON.toJSONString(remap));
        String parentJson = HttpClientUtil.doPost(url, remap);
        log.info("退库信息查询结果:{}",parentJson);
        /**退料返回结果*/
        RespMaterialRetDto dto = JSON.parseObject(parentJson,RespMaterialRetDto.class);
        log.info("退库转换后的{}",JSON.toJSONString(dto));
        if(dto!=null&&FlowConstant.STRING_TWO_HUNDRED.equals(dto.getCode())) {  outIn.setYyRes(FlowConstant.SHORT_ONE);
            if(dto.getData()!=null) {
                outIn.setSucessCount(dto.getData().getSucessCount());
                outIn.setFailCount(dto.getData().getFailCount());
                outIn.setMessages(dto.getData().getMessages());

                if(CollUtil.isNotEmpty(dto.getData().getInfos())) {
                    for (InOutInfoDto infoDto:dto.getData().getInfos()) {

                            fillInMaterialId(itemios, outIn, infoDto, infoDto.getMaterOuts());

                    }
                }
            }
            this.updateById(outIn);
            mItemOutInService.updateBatchById(itemios);
        }else if(dto != null && FlowConstant.TOKEN_ERROR.equals(dto.getCode())&&i<4){
            accessTokenService.delToken();
            pushMaterialRetToYy(retDto,itemios,outIn,i++);
        }else {
            this.update(new UpdateWrapper<MaterialOutIn>().lambda().eq(MaterialOutIn::getId, outIn.getId()).set(MaterialOutIn::getMessages,dto.getMessage())
                    .set(MaterialOutIn::getYyRes,FlowConstant.SHORT_ZERO));
        }
    }

    private MaterialOutDto toOutDto(MaterialRetDto retDto) {
        MaterialOutDto outDto = new MaterialOutDto();
        BeanUtils.copyProperties(retDto,outDto);
        //特征项
        final OutDefineDto othInRecordDefineCharacter = retDto.getOthInRecordDefineCharacter();
        OutDefineDto outDefineDto = new OutDefineDto();
        BeanUtils.copyProperties(othInRecordDefineCharacter,outDefineDto);
        outDto.setMaterialOutDefineCharacter(outDefineDto);
        //单子表

        final List<MaterialItemRetDto> othInRecords = retDto.getOthInRecords();

        List<MaterialItemOutDto> materOuts = new ArrayList<>();
        othInRecords.forEach(f->{

            MaterialItemOutDto materialItemOutDto = new MaterialItemOutDto();
            BeanUtils.copyProperties(f,materialItemOutDto);
            //材料为负数
            materialItemOutDto.setQty(-f.getQty());
            materOuts.add(materialItemOutDto);

        });
        outDto.setMaterOuts(materOuts);


        return  outDto;

    }

    /**
     * 推送物料出库数据到用友
     * @param remap 推送到用友数据
     * @param itemios 本地推送物料详情
     * @param outIn 本地物料数据
     */
    private void pushMaterialOutToYy(Map<String,Object> remap, List<MaterialItemOutIn> itemios, MaterialOutIn outIn,int type,String yyapi,int i) {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            log.error("出库流程："+ outIn.getProcessId() +"获取accessToken失败，请稍后在试  ");
            return;
        }
        String url = openApiUrl+yyapi+"?access_token="+ accessToken;
        log.info("出库请求参数:{}",JSON.toJSONString(remap));
        String parentJson = HttpClientUtil.doPost(url, remap);
        log.info("信息查询结果:{}",parentJson);
        /**出库结果*/
        RespMaterialOutDto dto = JSON.parseObject(parentJson, RespMaterialOutDto.class);
        if(dto!=null&&FlowConstant.STRING_TWO_HUNDRED.equals(dto.getCode())) {
            outIn.setYyRes(FlowConstant.SHORT_ONE);
            if(dto.getData()!=null) {
                outIn.setSucessCount(dto.getData().getSucessCount());
                outIn.setFailCount(dto.getData().getFailCount());
                outIn.setMessages(dto.getData().getMessages());

                if(CollUtil.isNotEmpty(dto.getData().getInfos())) {
                    for (InOutInfoDto infoDto:dto.getData().getInfos()) {
                        if(1==type) {
                            fillOutMaterialId(itemios, outIn, infoDto, infoDto.getMaterOuts());
                        }else if(2==type){
                            if (outIn.getWarehouse().equals(infoDto.getInWarehouse())) {
                                outIn.setYyId(infoDto.getId());
                                List<YyMaterOutDto> materOuts = infoDto.getDetails();
                                fillItems(itemios, materOuts);
                            }
                        }
                    }
                }
            }
            this.updateById(outIn);
            mItemOutInService.updateBatchById(itemios);
        }else if(dto != null && FlowConstant.TOKEN_ERROR.equals(dto.getCode())&&i<4){
            accessTokenService.delToken();
            pushMaterialOutToYy(remap,itemios,outIn,type,yyapi,i++);
        }else {
            this.update(new UpdateWrapper<MaterialOutIn>().lambda().eq(MaterialOutIn::getId, outIn.getId()).set(MaterialOutIn::getYyRes,FlowConstant.SHORT_ZERO));
        }
    }
    /**
     * 推送转库数据到用友
     * @param remap 推送到用友数据
     * @param itemios 本地推送物料详情
     * @param outIn 本地物料数据
     */
    private void pushTransferMaterialOutToYy(Map<String,Object> remap, List<MaterialItemOutIn> itemios, MaterialOutIn outIn,int type,String yyapi,int i) {
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            log.error("转库流程："+ outIn.getProcessId() +"获取accessToken失败，请稍后在试  ");
            return;
        }
        String url = openApiUrl+yyapi+"?access_token="+ accessToken;
        log.info("转库请求参数:{}",JSON.toJSONString(remap));
        String parentJson = HttpClientUtil.doPost(url, remap);
        log.info("转库信息查询结果:{}",parentJson);
        /**出转库结果*/
        RespMaterialOutDto dto = JSON.parseObject(parentJson, RespMaterialOutDto.class);
        if(dto!=null&&FlowConstant.STRING_TWO_HUNDRED.equals(dto.getCode())) {
            outIn.setYyRes(FlowConstant.SHORT_ONE);
            if(dto.getData()!=null) {
                outIn.setSucessCount(dto.getData().getSucessCount());
                outIn.setFailCount(dto.getData().getFailCount());
                outIn.setMessages(dto.getData().getMessages());
                if(CollUtil.isNotEmpty(dto.getData().getInfos())) {
                    for (InOutInfoDto infoDto:dto.getData().getInfos()) {
                        if(0==type) {
                            fillOutMaterialId(itemios, outIn, infoDto, infoDto.getMaterOuts());
                        }else if(1==type){
                            if (outIn.getWarehouse().equals(infoDto.getInWarehouse())) {
                                outIn.setYyId(infoDto.getId());
                                List<YyMaterOutDto> materOuts = infoDto.getDetails();
                                fillItems(itemios, materOuts);
                            }
                        }
                    }
                }
            }
            this.updateById(outIn);
            mItemOutInService.updateBatchById(itemios);
        }else if(dto != null && FlowConstant.TOKEN_ERROR.equals(dto.getCode())&&i<4){
            accessTokenService.delToken();
            pushTransferMaterialOutToYy(remap,itemios,outIn,type,yyapi,i++);
        }else {
            this.update(new UpdateWrapper<MaterialOutIn>().lambda().eq(MaterialOutIn::getId, outIn.getId()).set(MaterialOutIn::getYyRes,FlowConstant.SHORT_ZERO));
        }
    }
    /**
     * 用友出入库记录，填充用友返回的id
     * @param itemios 用友出入库物料信息详情
     * @param outIn 用友出入库记录
     * @param infoDto 用友接口返回信息
     * @param materOuts2 用友接口返回物料详情信息
     */
    private void fillOutMaterialId(List<MaterialItemOutIn> itemios, MaterialOutIn outIn, InOutInfoDto infoDto, List<YyMaterOutDto> materOuts2) {
      //出库

        if (outIn.getWarehouse().equals(infoDto.getWarehouse())||infoDto.getWarehouse().equals(outIn.getOutWarehouse())) {
            outIn.setYyId(infoDto.getId());
            outIn.setCode(infoDto.getCode());
            List<YyMaterOutDto> materOuts = materOuts2;
            fillItems(itemios, materOuts);
        }
    }

    /**
     * 退料返回参数填充
     * @param itemios 用友出入库物料信息详情
     * @param outIn 用友出入库记录
     * @param infoDto 用友接口返回信息
     * @param materOuts2 用友接口返回物料详情信息
     */
    private void fillInMaterialId(List<MaterialItemOutIn> itemios, MaterialOutIn outIn, InOutInfoDto infoDto, List<YyMaterOutDto> materOuts2) {
        //出库

        if (outIn.getWarehouse().equals(infoDto.getWarehouse())||infoDto.getWarehouse().equals(outIn.getOutWarehouse())) {
            outIn.setYyId(infoDto.getId());
            outIn.setCode(infoDto.getCode());
            List<YyMaterOutDto> materOuts = materOuts2;
            fillItems(itemios, materOuts);
        }
    }

    /**
     * 用友返回数据填充工程出入库物料子项
     * @param itemios 用友出入库物料信息详情
     * @param materOuts 用友接口返回物料详情信息
     */
    private void fillItems(List<MaterialItemOutIn> itemios, List<YyMaterOutDto> materOuts) {
        if (CollUtil.isNotEmpty(materOuts)) {
            Map<Long, YyMaterOutDto> map = materOuts.stream().collect(Collectors.toMap(YyMaterOutDto::getProduct, r -> r));
            for (MaterialItemOutIn item : itemios) {
                if (map.containsKey(item.getProduct())) {
                    YyMaterOutDto outdto = map.get(item.getProduct());
                    item.setYyId(outdto.getId());
                    item.setYyMainId(outdto.getMainid());
                    item.setYyRes(FlowConstant.SHORT_ONE);
                }
            }
        }
    }

    /**
     * 存储推送用友物料出入库信息到本地数据表
     * @param yyDto 仓库信息
     * @param itemios 存储后详情
     * @return 存储后物料信息
     */
    private MaterialOutIn saveYyOutInDb(Long processId,Long relationId,MaterialYyDto yyDto,List<MaterialItemOutIn> itemios,Short mType,Long tenantId) {
        MaterialOutIn outIn = new MaterialOutIn();
        List itemOutDtos = null;
        OutDefineDto defineDto = null;
        if(yyDto instanceof MaterialOutDto) {
            MaterialOutDto outDto = (MaterialOutDto) yyDto;
            BeanUtils.copyProperties(outDto, outIn);
            itemOutDtos = outDto.getMaterOuts();
            defineDto = outDto.getMaterialOutDefineCharacter();
        }else if(yyDto instanceof MaterialRetDto){
            MaterialRetDto retDto = (MaterialRetDto)yyDto;
            BeanUtils.copyProperties(retDto, outIn);
            outIn.setCheckKey(retDto.getResubmitCheckKey());
            itemOutDtos = retDto.getOthInRecords();
            defineDto = retDto.getOthInRecordDefineCharacter();
        }
        outIn.setProcessId(processId);
        outIn.setRelationId(relationId);
        outIn.setTenantId(tenantId);
        outIn.setProcessNumber(yyDto.getHeadItem().getDefine1());
        outIn.setUserName(yyDto.getHeadItem().getDefine2());
        outIn.setMType(mType);
        outIn.setIsDel(FlowConstant.SHORT_ZERO);
        outIn.setIsFinish(FlowConstant.SHORT_ZERO);
        outIn.setPushNumber(FlowConstant.INTEGER_ONE);
        this.save(outIn);
        if(defineDto!=null){
            MaterialCharacter character = new MaterialCharacter();
            BeanUtils.copyProperties(defineDto, character);
            character.setMOiId(outIn.getId());
            characterService.save(character);
        }
        saveMaterialItem(itemios, outIn.getId(), itemOutDtos);
        return outIn;
    }

    /**
     * 保存物料存储子项
     * @param itemios 存储后子项
     * @param mId 物料入库id
     * @param itemOutDtos 存储前子项列表
     */
    private void saveMaterialItem(List<MaterialItemOutIn> itemios, Long mId, List itemOutDtos) {
        for (Object obj: itemOutDtos) {
            MaterialItemOutIn itemOutIn = new MaterialItemOutIn();
            if(obj instanceof MaterialItemRetDto){
                MaterialItemRetDto item = (MaterialItemRetDto) obj;
                BeanUtils.copyProperties(item, itemOutIn);
                itemOutIn.setExchangeType(item.getUnitExchangeType());
                itemOutIn.setCq(item.getContactsQuantity());
            }else if(obj instanceof MaterialItemOutDto) {
                MaterialItemOutDto item = (MaterialItemOutDto) obj;
                BeanUtils.copyProperties(item, itemOutIn);
                itemOutIn.setCq(item.getContactsQuantity());
            }
            itemOutIn.setMOiId(mId);
            itemOutIn.setIsDel(FlowConstant.SHORT_ZERO);
            itemios.add(itemOutIn);
        }
        mItemOutInService.saveBatch(itemios);
    }


    /**
     * 定时同步用友物料出库、退库结果
     */
    public void getMaterialOutBearFruit(int i){
        Map<Long,MaterialOutIn> mos =this.list(new QueryWrapper<MaterialOutIn>().lambda().isNotNull(MaterialOutIn::getYyId).eq(MaterialOutIn::getIsDel,FlowConstant.SHORT_ZERO)
                .eq(MaterialOutIn::getIsFinish,FlowConstant.SHORT_ZERO)
                .isNotNull(MaterialOutIn::getCode)
                .select(MaterialOutIn::getYyId,MaterialOutIn::getId,MaterialOutIn::getMType))
                .stream().collect(Collectors.toMap(MaterialOutIn::getYyId,r->r,(r1,r2)->r1));

        if(CollUtil.isNotEmpty(mos)) {
            String accessToken = accessTokenService.getAccessToken();
            if (accessToken == null || "".equals(accessToken)) {
                log.warn("获取accessToken失败，请稍后在试  ");
                return ;
            }
            String outUrl = openApiUrl+outBearFruitApi+"?access_token="+ accessToken;
            String inUrl = openApiUrl+inBearFruitApi+"?access_token="+ accessToken;
            String zkUrl = openApiUrl+zkbearfruitApi+"?access_token="+ accessToken;
            List<MaterialOutIn> up = new ArrayList<>();
            for (Long id : mos.keySet()) {
                Map<String,Object> map = new HashMap<>();
                map.put("id",id);
                MaterialOutIn mo = mos.get(id);
                String url = mo.getMType()==FlowConstant.SHORT_ONE||mo.getMType()==FlowConstant.SHORT_THREE?outUrl:mo.getMType()==FlowConstant.SHORT_ZERO?inUrl:mo.getMType()==FlowConstant.SHORT_TWO?zkUrl:null;
                if(StringUtils.isNotEmpty(url)) {
                    String parentJson = HttpUtil.get(url, map);
                    log.info("信息查询结果:{}", parentJson);
                    RespLoadBearFruitDto dto = JSON.parseObject(parentJson, RespLoadBearFruitDto.class);
                    if (dto != null && FlowConstant.STRING_TWO_HUNDRED.equals(dto.getCode())) {
                        if (dto.getData() != null && dto.getData().getId() != null) {
                            if (id.equals(dto.getData().getId()) && FlowConstant.SHORT_ONE == dto.getData().getStatus()) {
                                mo.setIsFinish(FlowConstant.SHORT_ONE);
                                up.add(mo);
                            }
                        } else {
                            mo.setIsFinish(FlowConstant.SHORT_TWO);
                            up.add(mo);
                        }
                    }else if(dto != null && FlowConstant.TOKEN_ERROR.equals(dto.getCode())&&i<4){
                        accessTokenService.delToken();
                        getMaterialOutBearFruit(i++);
                    }
                }
            }
            if(CollUtil.isNotEmpty(up)){
                this.updateBatchById(up);
            }
        }
    }


    /**
     * 定时推送失败的数据
     */
    public void pushMaterial(int i){
        List<MaterialOutIn> mos =this.list(new QueryWrapper<MaterialOutIn>().lambda()
                .isNull(MaterialOutIn::getYyId).eq(MaterialOutIn::getIsDel,FlowConstant.SHORT_ZERO)
                .le(MaterialOutIn::getPushNumber,20).isNull(MaterialOutIn::getCode));
        if(CollUtil.isNotEmpty(mos)) {
            String accessToken = accessTokenService.getAccessToken();
            if (accessToken == null || "".equals(accessToken)) {
                log.warn("获取accessToken失败，请稍后在试  ");
                return ;
            }
            for (MaterialOutIn material:mos) {
                Short mtype = material.getMType();
                material.setPushNumber(material.getPushNumber()+1);
                /**推送用友物料出库、核销（出库）*/
                if(FlowConstant.SHORT_ONE==mtype||FlowConstant.SHORT_THREE==mtype){
                    MaterialOutDto dto = new MaterialOutDto();
                    BeanUtils.copyProperties(material, dto);
                    HeadItemDto headItemDto = new HeadItemDto();
                    headItemDto.setDefine1(material.getProcessNumber());
                    headItemDto.setDefine2(material.getUserName());
                    dto.setHeadItem(headItemDto);
                    List<MaterialItemOutDto> items = new ArrayList<>();
                    for (MaterialItemOutIn itemOutIn:material.getItem()) {
                        MaterialItemOutDto itemdto = new MaterialItemOutDto();
                        BeanUtils.copyProperties(itemOutIn, itemdto);
                        itemdto.setContactsQuantity(itemOutIn.getCq());
                        items.add(itemdto);
                    }
                    if(material.getMCharacter()!=null){
                        OutDefineDto defineDto = new OutDefineDto();
                        BeanUtils.copyProperties( material.getMCharacter(), defineDto) ;
                        dto.setMaterialOutDefineCharacter(defineDto);
                    }
                    dto.setMaterOuts(items);
                    dto.set_status(FlowConstant.INSERT);
                    Map<String,Object> remap = new HashMap<>();
                    remap.put("data", dto);
                    pushMaterialOutToYy( remap, material.getItem(), material,FlowConstant.INTEGER_ONE,materialoutapi,0);
                    /**推送用友物料入库*/
                }else if(FlowConstant.SHORT_ZERO==mtype){
                    MaterialRetDto dto = new MaterialRetDto();
                    BeanUtils.copyProperties(material, dto);
                    dto.setResubmitCheckKey(material.getCheckKey());
                    HeadItemDto headItemDto = new HeadItemDto();
                    headItemDto.setDefine1(material.getProcessNumber());
                    headItemDto.setDefine2(material.getUserName());
                    dto.setHeadItem(headItemDto);
                    dto.set_status(FlowConstant.INSERT);
                    List<MaterialItemRetDto> items = getMaterialItemRetDtos(material);
                    if(material.getMCharacter()!=null){
                        OutDefineDto defineDto = new OutDefineDto();
                        BeanUtils.copyProperties( material.getMCharacter(),defineDto);
                        dto.setOthInRecordDefineCharacter(defineDto);
                    }
                    dto.setOthInRecords(items);
                    pushMaterialRetToYy(dto, material.getItem(), material,0);
                    /**推送用友物料转库库（A库转B库）*/
                }else if(FlowConstant.SHORT_TWO==mtype){
                    StockTransferDto dto = new StockTransferDto();
                    BeanUtils.copyProperties(material, dto);
                    dto.setInWarehouse(material.getWarehouse());
                    dto.setBusinesstype(material.getBustype());
                    HeadItemDto headItemDto = new HeadItemDto();
                    headItemDto.setDefine1(material.getProcessNumber());
                    headItemDto.setDefine2(material.getUserName());
                    dto.setStoreTransCustomItem(headItemDto);
                    dto.set_status(FlowConstant.INSERT);
                    List<MaterialItemRetDto> items = getMaterialItemRetDtos(material);
                    dto.setDetails(items);
                    Map<String,Object> remap = new HashMap<>();
                    remap.put("data", dto);
                    pushMaterialOutToYy( remap, material.getItem(), material,FlowConstant.INTEGER_TWO,storetransferApi,0);
                }
            }
        }
    }

    /**
     * 用友出库工程物料信息（价格等）
     * @param dto 查询条件
     * @return 工程提交到用友的物料信息
     */
    @Override
    public PageOutListDto getYyOutMaterial(YyProjectMaterialDto dto) {
        Map<String,Object> remap = new HashMap<>();
        if(dto.getPageIndex()!=null){
            remap.put("pageIndex",dto.getPageIndex());
        }
        if(dto.getPageSize()!=null){
            remap.put("pageSize",dto.getPageSize());
        }
        if(dto.getBeginTime()!=null){
            remap.put("open_vouchdate_begin",DateUtil.format(dto.getBeginTime(),"yyyy-MM-dd HH:mm:ss"));
        }
        if(dto.getEndTime()!=null){
            remap.put("open_vouchdate_end",DateUtil.format(dto.getEndTime(),"yyyy-MM-dd HH:mm:ss"));
        }
        fillOutBoundFactor("materialOutDefineCharacter.",dto, remap);
        return getOutMaterial(remap,FlowConstant.INTEGER_ZERO);
    }

    /**
     * 用友出库aip 调用
     * @param remap 条件
     * @param i 数量
     * @return
     */
    private PageOutListDto getOutMaterial(Map<String,Object> remap,int i){
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            log.error("获取工程中用友物资，获取accessToken失败，请稍后在试  ");
            return null;
        }
        String url = openApiUrl+outboundListApi+"?access_token="+ accessToken;
        String parentJson = HttpClientUtil.doPost(url, remap);
        log.info("信息查询结果:{}",parentJson);
        RespOutListDto rdto = JSON.parseObject(parentJson, RespOutListDto.class);
        if(rdto!=null&&FlowConstant.STRING_TWO_HUNDRED.equals(rdto.getCode())) {
            return rdto.getData();
        }else if(rdto != null && FlowConstant.TOKEN_ERROR.equals(rdto.getCode())&&i<2){
            accessTokenService.delToken();
            getOutMaterial(remap,i++);
        }
        return null;
    }



    /**
     * 用友入库工程物料信息（价格等）
     * @param dto 查询条件
     * @return 工程提交到用友的物料信息
     */
    @Override
    public PageInListDto getYyInMaterial(YyProjectMaterialDto dto) {
        Map<String,Object> remap = new HashMap<>();
        if(dto.getPageIndex()!=null){
            remap.put("pageIndex",dto.getPageIndex());
        }
        if(dto.getPageSize()!=null){
            remap.put("pageSize",dto.getPageSize());
        }
        if(dto.getBeginTime()!=null){
            remap.put("open_hopeReceiveDate_begin",DateUtil.format(dto.getBeginTime(),"yyyy-MM-dd HH:mm:ss"));
        }
        if(dto.getEndTime()!=null){
            remap.put("open_hopeReceiveDate_end",DateUtil.format(dto.getEndTime(),"yyyy-MM-dd HH:mm:ss"));
        }
        fillOutBoundFactor("othInRecordDefineCharacter.",dto, remap);
        return getInMaterial(remap,FlowConstant.INTEGER_ZERO);
    }

    @Override
    public String getYyCode(String id) {

        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            log.warn("获取accessToken失败，请稍后在试  ");
            return  "";
        }
        String outUrl = openApiUrl+outBearFruitApi+"?access_token="+ accessToken;
        Map<String,Object> map = new HashMap<>();
        map.put("id",id);
        String parentJson = HttpUtil.get(outUrl, map);
        log.info("信息查询结果:{}", parentJson);
        RespLoadBearFruitDto dto = JSON.parseObject(parentJson, RespLoadBearFruitDto.class);
        if (dto != null) {
           if(dto.getData()!=null){
               return  dto.getData().getCode();
           }else{
               return "";
           }

        }else{
            return "";
        }

    }

    @Override
    public List<Long> findNotPushTlInfo() {
        return baseMapper.findNotPushTlInfo();
    }

    /**
     * 用友入库aip 调用
     * @param remap 条件
     * @param i 次数
     * @return
     */
    private PageInListDto getInMaterial(Map<String,Object> remap,int i){
        String accessToken = accessTokenService.getAccessToken();
        if (accessToken == null || "".equals(accessToken)) {
            log.error("获取工程中用友物资，获取accessToken失败，请稍后在试  ");
            return null;
        }
        String url = openApiUrl+otherInboundListApi+"?access_token="+ accessToken;
        String parentJson = HttpClientUtil.doPost(url, remap);
        log.info("信息查询结果:{}",parentJson);
        RespInListDto rdto = JSON.parseObject(parentJson,RespInListDto.class);
        if(rdto!=null&&FlowConstant.STRING_TWO_HUNDRED.equals(rdto.getCode())) {
            return rdto.getData();
        }else if(rdto != null && FlowConstant.TOKEN_ERROR.equals(rdto.getCode())&&i<2){
            accessTokenService.delToken();
            getOutMaterial(remap,i++);
        }
        return null;
    }

    /**
     * 填充查询出库条件
     * @param dto
     * @param remap
     */
    private void fillOutBoundFactor(String simple,YyProjectMaterialDto dto, Map<String, Object> remap) {
        List<SimpleDto> sds = new ArrayList<>();
        if(StringUtils.isNotEmpty(dto.getProjectName())){
            SimpleDto sdto = new SimpleDto();
            sdto.setField(simple+"xmx");
            sdto.setOp("eq");
            sdto.setValue1(dto.getProjectName());
            sds.add(sdto);
        }
        if(StringUtils.isNotEmpty(dto.getDutyName())){
            SimpleDto sdto = new SimpleDto();
            sdto.setField(simple+"xmglyx");
            sdto.setOp("eq");
            sdto.setValue1(dto.getDutyName());
            sds.add(sdto);
        }
        if(dto.getProcessId()!=null){
            SimpleDto sdto = new SimpleDto();
            sdto.setField(simple+"xmid");
            sdto.setOp("eq");
            sdto.setValue1(String.valueOf(dto.getProcessId()));
            sds.add(sdto);
        }
        if(StringUtils.isNotEmpty(dto.getPickerName())){
            SimpleDto sdto = new SimpleDto();
            sdto.setField(simple+"A001");
            sdto.setOp("eq");
            sdto.setValue1(dto.getPickerName());
            sds.add(sdto);
        }
        if(CollUtil.isNotEmpty(sds)) {
            remap.put("simpleVOs", sds);
        }
    }

    /**
     * 填充获取推送至用友的物料详情信息
     * @param material 本地物料详情信息
     * @return
     */
    private List<MaterialItemRetDto> getMaterialItemRetDtos(MaterialOutIn material) {
        List<MaterialItemRetDto> items = new ArrayList<>();
        for (MaterialItemOutIn itemOutIn : material.getItem()) {
            MaterialItemRetDto itemdto = new MaterialItemRetDto();
            BeanUtils.copyProperties(itemOutIn, itemdto);
            itemdto.setContactsQuantity(itemOutIn.getCq());
            itemdto.setUnitExchangeType(itemOutIn.getExchangeType());
            itemdto.set_status(FlowConstant.INSERT);
            items.add(itemdto);
        }
        return items;
    }

}
